<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\I18n\Time;

/**
 * AnnualComponentTrackings Controller
 *
 * @property \App\Model\Table\AnnualComponentTrackingsTable $AnnualComponentTrackings
 *
 * @method \App\Model\Entity\AnnualComponentTracking[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class AnnualComponentTrackingsController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();
        $this->loadComponent('RequestHandler');

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
            ],
            'listeners' => [
                'Crud.Api'
            ]
        ]);

	#load model
		$this->loadModel("Groups");
		$this->loadModel("GroupUsers");
		$this->loadModel("SubAssemblies");
		$this->loadModel("SubAssemblyProducts");
		$this->loadModel("Products");
		$this->loadModel("AnnualPlans");
        $this->loadModel("AnnualQuarterPlans");
        $this->loadModel("AnnualAssemblyTrackings");
        $this->loadModel("AnnualPlannedProducts");
        $this->loadModel("AnnualProductTrackings");
        $this->loadModel("AnnualComponentTrackings");
        $this->loadModel("AnnualComponentStatusTrackings");
        $this->loadModel("ComponentRejectionReasons");
        $this->loadModel("ComponentProducts");
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #session data
        $session        = $this->getRequest()->getSession();
        $collapseData   = $session->read('collapseData');
    #get current login user id
        $id                         = $this->Auth->user('id');
        $user_role_id               = $this->Auth->user('user_role_id');
    #get global configuration from config/app.php
        $assemblyStatus             = Configure::read('TrackSheetAssemblyStatus');
        $componentStatus            = Configure::read('TrackSheetComponentStatus');
        $trackSheetProductStatus    = Configure::read('TrackSheetProductStatus');
        $trackSheetSpareStatus      = Configure::read('TrackSheetSpareStatus');
        $drawingPath                = Configure::read('DrawingPath');
    #variable declaration
        $productId                  = $this->request->getQuery('productId');
        $subAssemblyId              = $this->request->getQuery('subAssembly');
        $groupId                    = $this->request->getQuery('groupId');
        $productNo                  = $this->request->getQuery('productNo');
        $subAssemblies              = [];
    #check data null
        $productId                  = ( (isset($productId) && !empty($productId)) ? $productId : null );
        $subAssemblyId              = ( (isset($subAssemblyId) && !empty($subAssemblyId)) ? $subAssemblyId : null );
        $groupId                    = ( (isset($groupId) && ($groupId!="") ? $groupId : null ));
        $productNo                  = ( (isset($productNo) && !empty($productNo)) ? $productNo : null );

    #get product details if product id is empty
        if(empty($productId) || $productId==null){
            $productDetails = $this->Products->find('all')->where(['is_active' => 1])->first();
            $productId      = $productDetails->id;
        }
        else{
        #product details
            $productDetails = $this->Products->get($productId);
        }

    #get group users
        $groupUsers         = $this->GroupUsers->find()->where(['user_id'=>$id, 'is_active' => 1])->first();

    #standard products
        $standardProducts   = $this->Products->find()
                                ->select([
                                    'id', 
                                    'product_name', 
                                    'is_eventual_product'
                                ])
                                ->where(['id' => $productId])
                                ->hydrate(false)
                                ->toArray();
    #eventual products
        $eventualProducts   = $this->Products->find('all')
                                ->select([
                                    'id'                        => 'EventualProducts.id',
                                    'product_name'              => 'EventualProducts.product_name',
                                    'is_eventual_product'       => 'EventualProducts.is_eventual_product',
                                ])
                                ->join([
                                    'EventualProducts' => [
                                        'table'         => 'products',
                                        'type'          => 'inner',
                                        'conditions'    => ['EventualProducts.parent_id=Products.id']
                                    ]
                                ])
                                ->where(['Products.id' => $productId, 'Products.is_active' => 1])->hydrate(false)->toArray();

    #merge standard and eventual products get product tracking sheet
        $products = array_merge($standardProducts, $eventualProducts);

    #check sub assembly empty
        if(!empty($products)){
        #variable declaration
            $productTrackingData    = [];
            $eventualData           = [];

        #eventual products loop data
            foreach($products AS $key => $rows){
            #sub assembly where condition
                if(!empty($subAssemblyId)){
                #product assembly
                    $productAssmblyWhere = ['SubAssemblyProducts.sub_assembly_id' => $subAssemblyId, 'SubAssemblyProducts.product_id' => $rows['id'], 'SubAssemblyProducts.is_active' => 1];
                }
                else{
                #product assembly
                    $productAssmblyWhere = ['SubAssemblyProducts.product_id' => $rows['id'], 'SubAssemblyProducts.is_active'     => 1];
                }

            #check group id null
                if($user_role_id==4){
                #prepare where condition dynamically based on product and group
                    $assemblyTrackGroupWhere    = ['AnnualAssemblyTrackings.product_id' => $rows['id'], 'ComponentGroups.group_id' => $groupUsers->group_id];
                    $componentTrackGroupWhere   = ['ComponentProducts.product_id'       => $rows['id'], 'ComponentGroups.group_id' => $groupUsers->group_id];
                }
                elseif($groupId==null){
                #get group list
                    $groups = $this->Groups->find()->select(['id'])->where(['is_active' => 1])->order(['group_name'])->first()->toArray();
                #prepare where condition dynamically based on group
                    $assemblyTrackGroupWhere    = ['AnnualAssemblyTrackings.product_id' => $rows['id'], 'ComponentGroups.group_id' => $groups['id']];
                    $componentTrackGroupWhere   = ['ComponentProducts.product_id'       => $rows['id'], 'ComponentGroups.group_id' => $groups['id']];
                    $groupId = $groups['id'];
                }
                elseif($groupId==0){
                #prepare where condition dynamically based on product
                    $assemblyTrackGroupWhere    = ['AnnualAssemblyTrackings.product_id' => $rows['id']];
                    $componentTrackGroupWhere   = ['ComponentProducts.product_id'       => $rows['id']];
                }
                else{
                #prepare where condition dynamically based on product and group
                    $assemblyTrackGroupWhere    = ['AnnualAssemblyTrackings.product_id' => $rows['id'], 'ComponentGroups.group_id' => $groupId];
                    $componentTrackGroupWhere   = ['ComponentProducts.product_id'       => $rows['id'], 'ComponentGroups.group_id' => $groupId];
                }

            #eventual product name
                $productTrackingData[$key]['product_name']          = $rows['product_name'];
                $productTrackingData[$key]['is_eventual_product']   = $rows['is_eventual_product'];

            #get standard products/eventual products trakcing table for tracking sheet table UI header product number (eg:- 1, 2, 3, 4)
                $productTrackingData[$key]['header']   = $this->AnnualProductTrackings->find('all')
                                                            ->where([
                                                                'AnnualProductTrackings.product_id' => $rows['id'], 
                                                                'AnnualProductTrackings.status!=4',
                                                                'AnnualProductTrackings.is_active'  => 1,
                                                            ])
                                                            ->order(['AnnualProductTrackings.product_number' => 'ASC'])
                                                            ->toArray();

            #get tracking sheet component details based on sub assembly and components
            #group components based on sub assembly
            #list all sub assembly and components in tracking sheet
                $productTrackingData[$key]['assembly']  = $this->SubAssemblyProducts->find('all')
                                                            ->contain([
                                                                'SubAssemblies' => [
                                                                #assembly tracking information
                                                                    'AnnualAssemblyTrackings' => function($query) use($assemblyTrackGroupWhere){
                                                                    #get assembly tracking information based on component tracking, component product and component group
                                                                    #assembly tracking filter based on group
                                                                        return $query
                                                                        ->select([
                                                                            'id'                    => 'AnnualAssemblyTrackings.id',
                                                                            'sub_assembly_id'       => 'AnnualAssemblyTrackings.sub_assembly_id',
                                                                            'product_number'        => 'AnnualAssemblyTrackings.product_number',
                                                                            'total_component'       => 'AnnualAssemblyTrackings.total_component',
                                                                            'completed_component'   => 'AnnualAssemblyTrackings.completed_component',
                                                                            'status'                => 'AnnualAssemblyTrackings.status',
                                                                            'status_updated_on'     => 'AnnualAssemblyTrackings.status_updated_on',
                                                                            'description'           => 'AnnualAssemblyTrackings.description',
                                                                            'total'                 => 'COUNT(AnnualComponentTrackings.status)',
                                                                            'completed'             => 'SUM(if(AnnualComponentTrackings.status=9, 1, 0))',
                                                                            'group_id'              => 'ComponentGroups.group_id',
                                                                        ])
                                                                        ->join([
                                                                            'AnnualProductTrackings' => [
                                                                                'table'         => 'annual_product_trackings',
                                                                                'type'          => 'inner',
                                                                                'conditions'    => ['AnnualProductTrackings.id=AnnualAssemblyTrackings.annual_product_tracking_id', 'AnnualProductTrackings.status!=4', 'AnnualProductTrackings.is_active' => 1]
                                                                            ],
                                                                            'AnnualComponentTrackings' => [
                                                                                'table'         => 'annual_component_trackings',
                                                                                'type'          => 'inner',
                                                                                'conditions'    => ['AnnualComponentTrackings.annual_assembly_tracking_id=AnnualAssemblyTrackings.id', 'AnnualComponentTrackings.is_active' => 1]
                                                                            ],
                                                                            'ComponentProducts' => [
                                                                                'table'         => 'component_products',
                                                                                'type'          => 'inner',
                                                                                'conditions'    => ['ComponentProducts.id=AnnualComponentTrackings.component_product_id', 'ComponentProducts.is_active' => 1]
                                                                            ],
                                                                            'ComponentGroups' => [
                                                                                'table'         => 'component_groups',
                                                                                'type'          => 'inner',
                                                                                'conditions'    => ['ComponentGroups.component_id=ComponentProducts.component_id', 'ComponentGroups.is_active' => 1]
                                                                            ],
                                                                        ])
                                                                        ->where($assemblyTrackGroupWhere)
                                                                        ->group(['AnnualComponentTrackings.annual_assembly_tracking_id']);
                                                                    },
                                                                #annual component tracking
                                                                    'ComponentAssemblies' => function($query) use($rows, $componentTrackGroupWhere){
                                                                    #get component tracking information based on component product, component group and assembly tracking
                                                                    #component tracking filter based on group and product assembly
                                                                        return $query
                                                                        ->select([
                                                                            'product_number'                    => 'AnnualComponentTrackings.product_number',
                                                                            'sub_assembly_id'                   => 'ComponentAssemblies.sub_assembly_id',
                                                                            'component_id'                      => 'Components.id',
                                                                            'component_name'                    => 'Components.component_name',
                                                                            'drawing_path'                      => 'Components.drawing_path',
                                                                            'drawing_file'                      => 'Components.drawing_file',
                                                                            'drawing_reference_no'              => 'Components.drawing_reference_no',
                                                                            'component_product_id'              => 'ComponentProducts.id',
                                                                            'number_of_component'               => 'ComponentProducts.number_of_component',
                                                                            'spare_count'                       => 'ComponentProducts.spare_count',
                                                                            'spare_required'                    => 'ComponentProducts.spare_required',
                                                                            'total_spare_completed_components'  => 'ComponentProducts.total_completed_components',
                                                                            'total_spare_rejected_components'   => 'ComponentProducts.total_rejected_components',
                                                                            'start_product_no'                  => 'ComponentProducts.start_product_no',
                                                                            'end_product_no'                    => 'ComponentProducts.end_product_no',
                                                                            'spare_status'                      => 'ComponentProducts.status',
                                                                            'spare_description'                 => 'ComponentProducts.description',
                                                                            'status_on'                         => 'ComponentProducts.status_on',
                                                                            'spare_delays'                      => 'DATEDIFF(NOW(),ComponentProducts.status_on)',
                                                                            'annual_product_tracking_id'        => 'AnnualComponentTrackings.id',
                                                                            'status'                            => 'AnnualComponentTrackings.status',
                                                                            'status_updated_on'                 => 'AnnualComponentTrackings.status_updated_on',
                                                                            'description'                       => 'AnnualComponentTrackings.description',
                                                                            'total_completed_components'        => 'AnnualComponentTrackings.total_completed_components',
                                                                            'total_rejected_components'         => 'AnnualComponentTrackings.total_rejected_components',
                                                                            'is_eventual_product'               => 'AnnualComponentTrackings.is_eventual_product',
                                                                            'is_eventual_mapped'                => 'AnnualComponentTrackings.is_eventual_mapped',
                                                                            'delays'                            => 'DATEDIFF(NOW(),AnnualComponentTrackings.status_updated_on)',
                                                                            'component_completed'               => '(SELECT COUNT(*) FROM `annual_component_trackings` WHERE `component_product_id`=ComponentProducts.`id` AND `status`=9 GROUP BY `component_product_id`)',
                                                                            'component_progress_count'          => '(SELECT COUNT(*) FROM `annual_component_trackings` WHERE `component_product_id`=ComponentProducts.`id` AND (`status`>0 AND `status`!=14) GROUP BY `component_product_id`)',
                                                                        ])
                                                                        ->join([
                                                                            'ComponentProducts' => [
                                                                                'table'         => 'component_products',
                                                                                'type'          => 'INNER',
                                                                                'conditions'    => ['ComponentProducts.component_id=ComponentAssemblies.component_id', 'ComponentProducts.is_active' => 1]
                                                                            ],
                                                                            'ComponentGroups'   => [
                                                                                'table'         => 'component_groups',
                                                                                'type'          => 'INNER',
                                                                                'conditions'    => ['ComponentGroups.component_id=ComponentProducts.component_id', 'ComponentGroups.is_active' => 1]
                                                                            ],
                                                                            'AnnualComponentTrackings' => [
                                                                                'table'         => 'annual_component_trackings',
                                                                                'type'          => 'INNER',
                                                                                'conditions'    => ['AnnualComponentTrackings.component_product_id=ComponentProducts.id', 'AnnualComponentTrackings.is_active' => 1]
                                                                            ],
                                                                            'AnnualAssemblyTrackings' => [
                                                                                'table'         => 'annual_assembly_trackings',
                                                                                'type'          => 'INNER',
                                                                                'conditions'    => ['AnnualAssemblyTrackings.id=AnnualComponentTrackings.annual_assembly_tracking_id', 'AnnualAssemblyTrackings.sub_assembly_id=ComponentAssemblies.sub_assembly_id', 'AnnualAssemblyTrackings.is_active' => 1]
                                                                            ],
                                                                            'AnnualProductTrackings' => [
                                                                                'table'         => 'annual_product_trackings',
                                                                                'type'          => 'INNER',
                                                                                'conditions'    => ['AnnualProductTrackings.id=AnnualAssemblyTrackings.annual_product_tracking_id', 'AnnualProductTrackings.status!=4', 'AnnualProductTrackings.is_active' => 1]
                                                                            ],
                                                                            'Components' => [
                                                                                'table'         => 'components',
                                                                                'type'          => 'INNER',
                                                                                'conditions'    => ['Components.id=ComponentProducts.component_id', 'Components.is_active' => 1]
                                                                            ],
                                                                        ])
                                                                        ->where($componentTrackGroupWhere)
                                                                        ->order([
                                                                            'ComponentProducts.component_id', 
                                                                            'AnnualComponentTrackings.product_number', 
                                                                            'AnnualComponentTrackings.is_eventual_product' => 'ASC'
                                                                        ])
                                                                        ->group(['AnnualComponentTrackings.component_product_id', 'AnnualComponentTrackings.annual_assembly_tracking_id', 'AnnualComponentTrackings.product_number']);
                                                                    },
                                                                ],
                                                            ])
                                                            ->where($productAssmblyWhere)
                                                            ->order(['SubAssemblies.sub_assembly_code'])
                                                            ->toArray();
            }
        }
    #get product list for select option field
		$products = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->where(['is_active' => 1, 'is_eventual_product' => 0]);

    #sub assembly list
        if($groupId==0 || $groupId==null){
            $subAssemblyList = $this->SubAssemblies
                                    ->find('list', ['keyField' => 'id', 'valueField' => 'sub_assembly_name'])
                                    ->join([
                                        'SubAssemblyProducts' => [
                                            'table'         => 'sub_assembly_products',
                                            'type'          => 'INNER',
                                            'conditions'    => ['SubAssemblyProducts.sub_assembly_id=SubAssemblies.id', 'SubAssemblyProducts.is_active' => 1]
                                        ],
                                    ])
                                    ->where(['SubAssemblies.is_active' => 1, 'SubAssemblyProducts.product_id' => $productId])
                                    ->order(['SubAssemblies.sub_assembly_code' => 'ASC']);
        }
        else{
            $subAssemblyList = $this->SubAssemblies
                                    ->find('list', ['keyField' => 'id', 'valueField' => 'sub_assembly_name'])
                                    ->join([
                                        'SubAssemblyProducts' => [
                                            'table'         => 'sub_assembly_products',
                                            'type'          => 'INNER',
                                            'conditions'    => ['SubAssemblyProducts.sub_assembly_id=SubAssemblies.id', 'SubAssemblyProducts.is_active' => 1]
                                        ],
                                        'ComponentAssemblies' => [
                                            'table'         => 'component_assemblies',
                                            'type'          => 'INNER',
                                            'conditions'    => ['ComponentAssemblies.sub_assembly_id=SubAssemblyProducts.sub_assembly_id', 'ComponentAssemblies.is_active' => 1]
                                        ],
                                        'ComponentGroups' => [
                                            'table'         => 'component_groups',
                                            'type'          => 'INNER',
                                            'conditions'    => ['ComponentGroups.component_id=ComponentAssemblies.component_id', 'ComponentGroups.is_active' => 1]
                                        ],
                                    ])
                                    ->where(['SubAssemblies.is_active' => 1, 'SubAssemblyProducts.product_id' => $productId, 'ComponentGroups.group_id' => $groupId])
                                    ->order(['SubAssemblies.sub_assembly_code' => 'ASC']);
        }

	#get groups list for select option field
		$groups = $this->Groups->find('list', ['keyField' => 'id', 'valueField' => 'group_name'])->where(['is_active' => 1]);

    #get rejection reasons
		$componentRejectionReasons = $this->ComponentRejectionReasons->find('list', ['keyField' => 'reasons', 'valueField' => 'reasons'])->where(['is_active' => 1]);

    #set result for view
        $this->set(compact(['trackSheetProductStatus', 'trackSheetSpareStatus', 'drawingPath', 'assemblyStatus', 'componentStatus', 'productDetails', 'subAssemblyId', 'groupId', 'productNo', 'productTrackingData', 'products', 'subAssemblyList', 'groups', 'componentRejectionReasons', 'collapseData']));
    }

    /**
     * update product status
     *
     */
    public function updateComponentTrackingStatus()
    {
    #get current login user id
        $uid = $this->Auth->user('id');
    #variable declaration
        $this->result   = [];
        $data           = [];
        $statusCode     = true;
        $title          = "";
        $message        = "";
    #check method
		if($this->request->is(['post', 'put'])){
        #variable declaration
            $status     = $this->request->data('status');
            $is_undo    = $this->request->data('is_undo');
            $user_id    = $this->request->data('user_id');
        #description
            if( $status==5 || $status==6 || $status==7 || $status==11 || $status==12 || $status==13 ){
                $qc_rejection_reason    = $this->request->data('qc-rejection-reason');
                $description            = ( ($qc_rejection_reason=='Others') ? $this->request->data('description') : $qc_rejection_reason );
            }
            else{
                $description            = $this->request->data('description');
            }
            $total_completed_components = $this->request->data('total_completed_components');
            $total_rejected_components  = $this->request->data('total_rejected_components');
        #find all data by array of id's
            $annualComponentTrackings = $this->AnnualComponentTrackings
                                            ->find('all',[
                                                'conditions'=>[
                                                    'id IN'     => $this->request->data('ids'),
                                                ]
                                            ]);
            $annualComponentTracking    = $annualComponentTrackings->first()->toArray();
        #check rejected count
            if($status==5 || $status==6 || $status==11 || $status==12){
                $total_completed_components = $total_completed_components;
                $total_rejected_components  = $total_rejected_components;
            }
            elseif($status==7 || $status==8){
            #find assembly rejected or not
                $annualComponentStatusTrackings = $this->AnnualComponentStatusTrackings
                                                    ->find()
                                                    ->where([
                                                        'annual_component_tracking_id IN' => $this->request->data('ids'), 'status IN' => [11, 13, 14]
                                                    ])
                                                    ->toArray();
            #set status
                if(!empty($annualComponentStatusTrackings)){
                    $total_completed_components = $annualComponentTracking["total_completed_components"];
                    $total_rejected_components  = 0;
                }
                else{
                    $total_completed_components = 0;
                    $total_rejected_components  = 0;
                }
            }
            elseif($status==13 || $status==14){
                $total_completed_components = $annualComponentTracking["total_components"];
                $total_rejected_components  = 0;
            }
            else{
                $total_completed_components = $annualComponentTracking["total_completed_components"];
                $total_rejected_components  = $annualComponentTracking["total_rejected_components"];
            }

        #check is undo
            if($is_undo==1){
                $annualComponentStatusTrackings = $this->AnnualComponentStatusTrackings->find()
                                                    ->where([
                                                        'annual_component_tracking_id IN' => $this->request->data('ids'), 
                                                        'is_last_updated !=' => 1, 
                                                        'is_undo !=' => 1, 
                                                        'status IN' => [5,6,11,12]
                                                    ])
                                                    ->order(['created' => 'DESC'])
                                                    ->first();
            #checks status
                if(!empty($annualComponentStatusTrackings)){
                    $status                     = $annualComponentStatusTrackings['status'];
                    $description                = $annualComponentStatusTrackings['description'];
                    $total_completed_components = $annualComponentStatusTrackings['total_completed_components'];
                    $total_rejected_components  = $annualComponentStatusTrackings['total_rejected_components'];
                }
                else{
                    $total_completed_components = 0;
                    $total_rejected_components  = 0;
                }
            }

        #check selected data valid for update given status
            if(!empty($annualComponentTrackings)>0){
            #update query
                $annualComponentTrackings = $this->AnnualComponentTrackings->find()
                                                ->select([
                                                    'id'                            => 'id',
                                                    'status'                        => $status,
                                                    'description'                   => "'".$description."'",
                                                    'status_updated_on'             => "'".Time::now()."'",
                                                    'total_completed_components'    => $total_completed_components,
                                                    'total_rejected_components'     => $total_rejected_components,
                                                    'modified_by'                   => $uid,
                                                    'is_undo'                       => $is_undo,
                                                ])
                                                ->where(['id IN' => $this->request->data('ids')]);
            #prepare update data
                $updateData = $annualComponentTrackings->hydrate(false)->toArray();
            #patch entity update data
                $entity = $this->AnnualComponentTrackings->patchEntities($annualComponentTrackings, $updateData);
                $this->AnnualComponentTrackings->saveMany($entity);
            #save many check result true/false
                    $statusCode = true;
                    $title      = "Alert";
                    $message    = "Status Updated as Completed";
            }
            else{
                $statusCode = false;
                $title      = "Alert";
                $message    = "Status Updation Failed";
            }
        }
        else{
            $statusCode = false;
            $title      = "Alert";
            $message    = "Status Updation Failed";
        }
    #prepare response data
        $returnData = array( "status" => $statusCode, 'title' => $title, "message" => $message, "data" => $data );
    #return data
        $this->set([
            'data'          => $returnData,
            '_serialize'    => 'data',
        ]);
        $this->RequestHandler->renderAs($this, 'json');
    }

    /**
     * update component product spare maintain status
     *
     */
    public function updateComponentSpareTrackingStatus()
    {
    #get current login user id
        $uid = $this->Auth->user('id');
    #global variable declaration
        $data       = [];
        $status     = true;
        $title      = "";
        $message    = "";
    #check method
		if($this->request->is(['post', 'put'])){
            $status = $this->request->data('status');
        #description
            if( $status==5 || $status==6 || $status==7 ){
                $qcSpareRejectionReason = $this->request->data('qc-spare-rejection-reason');
                $description            = ( ($qcSpareRejectionReason=='Others') ? $this->request->data('description') : $qcSpareRejectionReason );
            }
            else{
                $description            = $this->request->data('description');
            }
        #variable declaration
            $total_completed_components = $this->request->data('total_completed_components');
            $total_rejected_components  = $this->request->data('total_rejected_components');
        #update component traking status with accept & reject count
            $componentProducts = $this->ComponentProducts
                                    ->get($this->request->data('id'),[
                                        'conditions'=>[
                                            'id' => $this->request->data('id'),
                                        ]
                                    ]);
        #check rejected count
            if($status==5 || $status==6){
                $total_completed_components = $total_completed_components;
                $total_rejected_components  = $total_rejected_components;
            }
            elseif($status==7 || $status==8){
                $total_completed_components = $componentProducts->total_completed_components;
                $total_rejected_components  = 0;
            }
            else{
                $total_completed_components = $componentProducts->total_completed_components;
                $total_rejected_components  = $componentProducts->total_rejected_components;
            }
        #check selected data valid for update given status
            if(!empty($componentProducts)){
            #prepare value
                $this->result = [];
                $this->result['status']                     = $status;
                $this->result['description']                = $description;
                $this->result['status_on']                  = Time::now();
                $this->result['total_completed_components'] = $total_completed_components;
                $this->result['total_rejected_components']  = $total_rejected_components;
            #set spare remaining count after spare qc completed`
                if($status==7 || $status==8){
                    $this->result['spare_remaning'] = ($componentProducts['spare_remaning']+$componentProducts['spare_required']);
                }
            #find query
                $componentProducts = $this->ComponentProducts->get($this->request->data('id'));
            #patch entity
                $entity = $this->ComponentProducts->patchEntity($componentProducts, $this->result);
            #check values updated
                if($this->ComponentProducts->save($entity)){
                #save many check result true/false
                    $status     = true;
                    $title      = "Alert";
                    $message    = "Status Updated as Completed";
                }
            }
            else{
                $status     = false;
                $title      = "Alert";
                $message    = "Status Updation Failed";
            }
        }
        else{
            $status     = false;
            $title      = "Alert";
            $message    = "Status Updation Failed";
        }
    #prepare response data
        $returnData = array( "status" => $status, 'title' => $title, "message" => $message, "data" => $data );
    #return data
        $this->set([
            'data'          => $returnData,
            '_serialize'    => 'data',
        ]);
        $this->RequestHandler->renderAs($this, 'json');
    }

    /**
     * update product tracking status
     *
     */
    public function updateProductTrackingStatus()
    {
    #find query
        $annualProductTrackings = $this->AnnualProductTrackings->get($this->request->data('product_id'));
    #set values to update
        $this->data                 = [];
        $this->data['status']       = 4;
        $this->data['description']  = $this->request->data('description');
    #patch entity
        $entity = $this->AnnualProductTrackings->patchEntity($annualProductTrackings, $this->data);
    #check values updated
        if($this->AnnualProductTrackings->save($entity)){
        #get eventual products
            $eventualProducts = $this->Products
                                    ->find()
                                    ->select(['id'])
                                    ->where([
                                        'parent_id' => $annualProductTrackings['product_id'], 
                                        'is_active' => 1
                                    ])
                                    ->hydrate(false)
                                    ->extract('id')
                                    ->toArray();
        #checks not empty
            if(!empty($eventualProducts)){
            #find query
                $annualProductTrackings = $this->AnnualProductTrackings->find()
                                            ->where([ 
                                                'annual_quarter_plan_id'    => $annualProductTrackings['annual_quarter_plan_id'], 
                                                'product_id IN'             => $eventualProducts, 
                                                'product_number'            => $annualProductTrackings['product_number'],
                                            ]);
                $annualProductTrackingsData = $annualProductTrackings
                                                ->select([
                                                    'id'            => 'AnnualProductTrackings.id',
                                                    'status'        => 4,
                                                    'description'   => "'".$this->data['description']."'",
                                                ])->hydrate(false)
                                                ->toArray();
            #patch entity
                $entity = $this->AnnualProductTrackings->patchEntities($annualProductTrackings, $annualProductTrackingsData);
            #save many
                $this->AnnualProductTrackings->saveMany($entity);
            }
            
        #return result
            $status     = true;
            $title      = "Alert";
            $message    = "Status Updated as Completed";
        }
        else{
            $status     = false;
            $title      = "Alert";
            $message    = "Status Updation Failed";
        }
    #prepare response data
        $returnData = array( "status" => $status, 'title' => $title, "message" => $message );
    #return data
        $this->set([
            'data'          => $returnData,
            '_serialize'    => 'data',
        ]);
        $this->RequestHandler->renderAs($this, 'json');
    }

    /**
     * get component tracking information
     *
    */
    public function componentTrackingInfo()
    {
    #check method
        if($this->request->is(['post', 'put']))
        {
        #request data
            $ids = $this->request->data('ids');
            $annualComponentTrackings = $this->AnnualComponentTrackings->find()
                                            ->select([
                                               'group_name'                     => 'Groups.group_name',
                                               'component_name'                 => 'Components.component_name',
                                               'number_of_component'            => 'SUM(ComponentProducts.number_of_component)',
                                               'component_product_id'           => 'AnnualComponentTrackings.component_product_id',
                                               'total_completed_components'     => 'AnnualComponentTrackings.total_completed_components',
                                               'total_rejected_components'      => 'AnnualComponentTrackings.total_rejected_components',
                                               'status'                         => 'AnnualComponentTrackings.status',
                                               'total_components_to_complete'   => 'ComponentProducts.number_of_component-AnnualComponentTrackings.total_completed_components',
                                            ])
                                            ->join([
                                                'ComponentProducts' => [
                                                    'table'         => 'component_products',
                                                    'type'          => 'inner',
                                                    'conditions'    => ['ComponentProducts.id=AnnualComponentTrackings.component_product_id']
                                                ],
                                                'ComponentGroups' => [
                                                    'table'         => 'component_groups',
                                                    'type'          => 'inner',
                                                    'conditions'    => ['ComponentProducts.component_id=ComponentGroups.component_id']
                                                ],
                                                'Groups' => [
                                                    'table'         => 'groups',
                                                    'type'          => 'inner',
                                                    'conditions'    => ['Groups.id=ComponentGroups.group_id']
                                                ],
                                                'Components' => [
                                                    'table'         => 'components',
                                                    'type'          => 'inner',
                                                    'conditions'    => ['Components.id=ComponentProducts.component_id']
                                                ],
                                            ])
                                            ->contain([
                                                'ComponentProducts'
                                            ])
                                            ->where(['AnnualComponentTrackings.id IN' => $ids, 'AnnualComponentTrackings.is_active' => 1])
                                            ->group(['ComponentProducts.component_id'])
                                            ->first();
        #prepare response data
            $returnData = array( "status" => null, 'title' => null, "message" => null, "data" => $annualComponentTrackings );
        #return data
            $this->set([
                'data'          => $returnData,
                '_serialize'    => 'data',
            ]);
            $this->RequestHandler->renderAs($this, 'json');
        }
    }

    /**
     * get component spare tracking information
     *
    */
    public function componentSpareTrackingInfo()
    {
    #check method
        if($this->request->is(['post', 'put']))
        {
        #request data
            $id = $this->request->data('id');
            $componentProducts = $this->ComponentProducts->find()
                                    ->select([
                                       'group_name'                 => 'Groups.group_name',
                                       'component_name'             => 'Components.component_name',
                                       'component_product_id'       => 'ComponentProducts.id',
                                       'spare_count'                => 'ComponentProducts.spare_count',
                                       'spare_remaning'             => 'ComponentProducts.spare_remaning',
                                       'spare_required'             => 'ComponentProducts.spare_required',
                                       'total_completed_components' => 'ComponentProducts.total_completed_components',
                                       'total_rejected_components'  => 'ComponentProducts.total_rejected_components',
                                    ])
                                    ->join([
                                        'ComponentGroups' => [
                                            'table'         => 'component_groups',
                                            'type'          => 'inner',
                                            'conditions'    => ['ComponentProducts.component_id=ComponentGroups.component_id']
                                        ],
                                        'Groups' => [
                                            'table'         => 'groups',
                                            'type'          => 'inner',
                                            'conditions'    => ['Groups.id=ComponentGroups.group_id']
                                        ],
                                        'Components' => [
                                            'table'         => 'components',
                                            'type'          => 'inner',
                                            'conditions'    => ['Components.id=ComponentProducts.component_id']
                                        ],
                                    ])
                                    ->where(['ComponentProducts.id' => $id, 'ComponentProducts.is_active' => 1])
                                    ->first();
        #prepare response data
            $returnData = array( "status" => null, 'title' => null, "message" => null, "data" => $componentProducts );
        #return data
            $this->set([
                'data'          => $returnData,
                '_serialize'    => 'data',
            ]);
            $this->RequestHandler->renderAs($this, 'json');
        }
    }

    /**
     * get component tracking information
     *
    */
    public function collapseExpandState()
    {
    #variable declaration
        $sessionCollapseData = [];
    #session data
        $session = $this->getRequest()->getSession();
    #get post data
        $collapseData   = $this->request->data('collapseData');
        $type           = $this->request->data('type');
    #session data
        if(!empty($session->read('collapseData'))){
            $sessionCollapseData = $session->read('collapseData');
        }
    #checks empty
        if( !empty($sessionCollapseData) ){
        #checks type
            if($type==1){
                $sessionCollapseData = array_merge($sessionCollapseData, $collapseData);
            }
            elseif($type==0){
                $sessionCollapseData = array_diff($sessionCollapseData, $collapseData);
            }
        }
        else{
            $sessionCollapseData = array_merge($sessionCollapseData, $collapseData);
        }
    #set session
        $sessionCollapseData = array_unique($sessionCollapseData);
        $session->write('collapseData', $sessionCollapseData);
    #prepare response data
        $returnData = array( "status" => null, 'title' => null, "message" => null, "data" => 'success' );
    #return data
        $this->set([
            'data'          => $returnData,
            '_serialize'    => 'data',
        ]);
        $this->RequestHandler->renderAs($this, 'json');
    }

    /**
     * get component rejection reason
     *
    */
    public function getRejectionReason()
    {
    #get post data
        $type = $this->request->data('type');
    #find data
        $componentRejectionReasons = $this->ComponentRejectionReasons->find()->where(['type' => $type]);
    #prepare response data
        $returnData = array( "status" => null, 'title' => null, "message" => null, "data" => $componentRejectionReasons );
    #return data
        $this->set([
            'data'          => $returnData,
            '_serialize'    => 'data',
        ]);
        $this->RequestHandler->renderAs($this, 'json');
    }
}
