<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;

/**
 * AnnualPlannedCapacities Controller
 *
 * @property \App\Model\Table\AnnualPlannedCapacitiesTable $AnnualPlannedCapacities
 *
 * @method \App\Model\Entity\AnnualPlannedCapacity[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class AnnualPlannedCapacitiesController extends AppController
{

    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

        $this->loadComponent('RequestHandler');

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'Crud.View',
            ],
        ]);
		
	#load model
		$this->loadModel('Products');
		$this->loadModel('Groups');
		$this->loadModel('Machines');
		$this->loadModel('AnnualPlans');
		$this->loadModel('AnnualPlannedProducts');

    #Datatable configuration
		$this->DataTables->createConfig('AnnualPlans')
			->column('Sno.'					        , ['label' => 'Sno.', 'class' => 'text-center', 'database' => false])
			->column('AnnualPlans.financial_year'	, ['label' => 'Financial Year'])
			->column('AnnualPlans.month_from'	    , ['label' => 'Month From'])
			->column('AnnualPlans.month_to'	        , ['label' => 'Month To'])
			->column('AnnualPlans.is_active'		, ['label' => 'Status', 'class' => 'text-center'])
			->column('AnnualPlans.id'				, ['label' => 'Actions', 'class' => 'text-center'])
            ->queryOptions([
                'conditions' => ['status' => 1, 'is_active' => 1],
            ]);
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
        $this->DataTables->setViewVars('AnnualPlans');
    }

    /**
     * view method
     *
     * @param string|null $id Annual Planned Capacity id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function view($id = null)
    {
    #variable declaration
        $productId  = $this->request->getQuery('product_id');
        $groupId    = $this->request->getQuery('group_id');
    #check data null
        $groupId  = ( (isset($groupId) && !empty($groupId)) ? $groupId : null );
    #check product id empty
        if(!isset($productId) || empty($productId) || $productId==null){
            $productId  = $this->Products->find()->select(['id'])->where(['is_eventual_product' => 0, 'is_active' => 1])->first();
            $productId  = $productId->id;
        }
    #get product details if product id is empty
        if($groupId==null){
            $machineWhere = [ 'ComponentProducts.product_id' => $productId, 'AnnualPlannedCapacities.annual_plan_id' => $id ];
        }
        else{
        #product details
            $machineWhere = [ 'ComponentProducts.product_id' => $productId, 'ComponentGroups.group_id' => $groupId, 'AnnualPlannedCapacities.annual_plan_id' => $id ];
        }
    #before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) use($groupId, $machineWhere, $productId) {
		#get capacite plan based on machines
			$machines = $this->Machines->find('all')
            ->contain([
				'AnnualPlannedCapacities' => function($q) use($machineWhere){
					return $q->select([
                        'id'                    => 'ComponentProcesses.id',
                        'process_id'            => 'ComponentProcesses.process_id',
                        'average_time_taken'    => 'ComponentProcesses.average_time_taken',
                        'set_time_for_batch'    => 'ComponentProcesses.set_time_for_batch',
                        'annual_plan_id'        => 'AnnualPlannedCapacities.annual_plan_id',
                        'product_id'            => 'AnnualPlannedCapacities.annual_plan_id',
                        'component_id'          => 'AnnualPlannedCapacities.component_id',
                        'machine_id'            => 'AnnualPlannedCapacities.machine_id',
                        'total_process'         => 'COUNT(AnnualPlannedCapacities.process_id)',
                        'average_process_time'  => 'SUM(AnnualPlannedCapacities.total_time)',
                        'number_of_component'   => 'ComponentProducts.number_of_component',
                        'batch_quantity'        => 'Components.batch_quantity',
					])
					->enableAutofields(true)
					->contain(['AnnualPlans', 'Products', 'Components', 'Processes' ])
					->join([
						'ComponentProcesses' => [
							'table'         => 'component_processes',
							'type'          => 'INNER',
							'conditions'    => [
                                                'ComponentProcesses.component_id = AnnualPlannedCapacities.component_id', 'ComponentProcesses.machine_id=AnnualPlannedCapacities.machine_id', 
                                                'ComponentProcesses.process_id = AnnualPlannedCapacities.process_id', 
                                                'AnnualPlannedCapacities.tag IN' => [0,2]
                                            ]
						],
                        'ComponentGroups' => [
							'table'         => 'component_groups',
							'type'          => 'INNER',
							'conditions'    => [ 'ComponentGroups.component_id = ComponentProcesses.component_id' ]
						],
                        'ComponentProducts' => [
							'table'         => 'component_products',
							'type'          => 'INNER',
							'conditions'    => [ 'ComponentProducts.component_id = ComponentGroups.component_id' ]
						],
                        'Components' => [
							'table'         => 'components',
							'type'          => 'INNER',
							'conditions'    => [ 'Components.id = ComponentProducts.component_id' ]
						],
					])
                    ->where($machineWhere)
                    ->group([
                        'AnnualPlannedCapacities.annual_plan_id', 
                        'AnnualPlannedCapacities.product_id', 
                        'AnnualPlannedCapacities.component_id', 
                        'AnnualPlannedCapacities.machine_id'
                    ])
                    ->order(['AnnualPlannedCapacities.tag' => 'ASC']);
				}
			])->toArray();

        #tag
            $tag = Configure::read('tag');

        #get product list
            $products = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->where(['is_active' => 1, 'is_eventual_product' => 0]);

        #get groups list
            $groups	  = $this->Groups->find('list', ['keyField' => 'id', 'valueField' => 'group_name'])->where(['is_active' => 1]);

		#set values to view
			$this->set(compact(['tag', 'products', 'groups', 'machines', 'productId', 'groupId']));
		});

    #return
		return $this->Crud->execute();
    }

    /**
     * freez products
     *
     * @return \Cake\Http\Response|void
     */
    public function getMachineComponentProcess()
    {
    #check method
        if($this->request->is(['post', 'put']))
		{
        #get
            $annual_plan_id     = $this->request->data('annual_plan_id');
            $product_id         = $this->request->data('product_id');
            $machine_id         = $this->request->data('machine_id');
            $component_id       = $this->request->data('component_id');
        #get eventual mapped components product id
            $annualPlannedCapacities = $this->AnnualPlannedCapacities->find()
            ->select([
                'process_name'          => 'Processes.process_name',
                'batch_quantity'        => 'Components.batch_quantity',
                'average_time_taken'    => 'ComponentProcesses.average_time_taken',
                'set_time_for_batch'    => 'ComponentProcesses.set_time_for_batch',
            ])
            ->join([
                'ComponentProducts' => [
                    'table'         => 'component_products',
                    'type'          => 'INNER',
                    'conditions'    => [ 'ComponentProducts.component_id = AnnualPlannedCapacities.component_id' ]
                ],
                'ComponentProcesses' => [
                    'table'         => 'component_processes',
                    'type'          => 'INNER',
                    'conditions'    => [
                                        'ComponentProcesses.component_id = ComponentProducts.component_id', 'ComponentProcesses.machine_id=AnnualPlannedCapacities.machine_id', 
                                        'ComponentProcesses.process_id = AnnualPlannedCapacities.process_id', 
                                        'AnnualPlannedCapacities.tag IN' => [0,2]
                                    ]
                ],
                'Processes' => [
                    'table'         => 'processes',
                    'type'          => 'INNER',
                    'conditions'    => [ 'Processes.id = ComponentProcesses.process_id' ]
                ],
                'Components' => [
                    'table'         => 'components',
                    'type'          => 'INNER',
                    'conditions'    => [ 'Components.id = ComponentProcesses.component_id' ]
                ],
            ])
            ->where([
                'AnnualPlannedCapacities.annual_plan_id'    => $annual_plan_id,
                'AnnualPlannedCapacities.product_id'        => $product_id,
                'AnnualPlannedCapacities.machine_id'        => $machine_id,
                'AnnualPlannedCapacities.component_id'      => $component_id,
            ]);

        #prepare response data
            $returnData = array( "status_code" => 200, 'data' => $annualPlannedCapacities );
        #return data
            $this->set([
            'data'          => $returnData,
            '_serialize'    => 'data',
            ]);
            $this->RequestHandler->renderAs($this, 'json');
        }
    }
}
