<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;

/**
 * AnnualPlans Controller
 *
 * @property \App\Model\Table\AnnualPlansTable $AnnualPlans
 *
 * @method \App\Model\Entity\AnnualPlan[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class AnnualPlansController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'componentRawMaterialReport' => ['className' => 'Crud.Index'],
                'add' => ['className' => 'Crud.Add', 'relatedModels' => false],
                'Crud.Edit',
                'freezCapacityPlan' => ['className' => 'Crud.Edit', 'relatedModels' => false],
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);

	#load model
        $this->loadModel('Groups');
		$this->loadModel('AnnualPlans');
		$this->loadModel('AnnualPlannedProducts');
		$this->loadModel('Products');
		$this->loadModel('Machines');
		$this->loadModel('Components');
        $this->loadModel('RawMaterials');
		$this->loadModel('AnnualPlannedCapacities');
		$this->loadModel('AnnualQuarterPlans');
		$this->loadModel('SubAssemblyProducts');
        $this->loadModel('AnnualAssemblyTrackings');
		$this->loadModel('AnnualComponentTrackings');
		$this->loadModel('AnnualQuarterPlannedProducts');
        $this->loadModel('ComponentProducts');
        $this->loadModel('PurchaseIndents');
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #return data
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id Annual Plan id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#product details
		$Products   = $this->Products->find('all')
                                    ->select([
                                        'last_product_no'   => 'AnnualPlannedProducts.last_product_no',
                                        'product_quantity'  => 'AnnualPlannedProducts.product_quantity',
                                    ])
                                    ->join([
                                        'AnnualPlannedProducts' => [
                                            'type'			=> 'inner',
                                            'table'			=> 'annual_planned_products',
                                            'conditions' 	=> ['AnnualPlannedProducts.product_id=Products.id']
                                        ],
                                    ])
                                    ->where([
                                        'Products.is_eventual_product'          => 0, 
                                        'Products.is_active'                    => 1, 
                                        'AnnualPlannedProducts.annual_plan_id'  => $id
                                    ])
                                    ->enableAutofields(true)->toArray();
		$annualPlannedProducts  = $this->AnnualPlannedProducts->find('all')->where(['annual_plan_id' => $id, 'is_active' => 1]);
		$annualPlan  			= $this->AnnualPlans->get($id);

	#get financial years
		$FinancialYears = Configure::read('FinancialYears');
		$this->set(compact('annualPlan', 'FinancialYears', 'Products', 'annualPlannedProducts'));
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function componentRawMaterialReport()
    {
    #find
        $rawMaterials       = $this->RawMaterials->find('list')->where(['is_active' => 1]);
        $annualPlanOptions  = $this->AnnualPlans->find('list', ['keyField' => 'id', 'valueField' => 'financial_year'])->where(['is_active' => 1])->order(['id' => 'DESC']);
        $groups             = $this->Groups->find('list', ['keyField' => 'id', 'valueField' => 'group_name'])->where(['is_active' => 1])->order(['group_name' => 'ASC']);
        $annualPlanFilter   = $this->AnnualPlans->find()->where(['is_active' => 1])->order(['id' => 'DESC'])->first();

    #set result
        $this->set(compact('rawMaterials', 'groups', 'annualPlanOptions'));

    #before find
        $this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) use($annualPlanFilter){
        #query
            $event->subject()->query
                ->select([
                    'component_name'                    => 'Components.component_name',
                    'material_name'                     => 'RawMaterials.material_name',
                    'raw_material_shape'                => 'ComponentRawMaterials.raw_material_shape',
                    'min_cut_length'                    => 'ComponentRawMaterials.min_cut_length',
                    'outer_dia'                         => 'ComponentRawMaterials.outer_dia',
                    'inner_dia'                         => 'ComponentRawMaterials.inner_dia',
                    'raw_material_thickness'            => 'ComponentRawMaterials.raw_material_thickness',
                    'raw_material_width'                => 'ComponentRawMaterials.raw_material_width',
                    'raw_material_length'               => 'ComponentRawMaterials.raw_material_length',
                    'total_purchase_received_quantity'  => 'SUM(PurchaseOrderComponents.grn_quantity)',
                    'total_completed_component'         => '(SELECT SUM(ACP.`total_components`) FROM `component_products` AS CP INNER JOIN `annual_component_trackings` AS ACP ON ACP.`component_product_id`=CP.`id` AND ACP.`status`=14 WHERE CP.`component_id`=ComponentRawMaterials.`component_id` AND CP.`is_active`=1 GROUP BY CP.`component_id`)',
                ])
                ->join([
                    'AnnualPlannedProducts' => [
                        'table'     => 'annual_planned_products',
                        'type'		=> 'INNER',
                        'conditions'=> ['AnnualPlannedProducts.annual_plan_id=AnnualPlans.id']
                    ],
                    'ComponentProducts' => [
                        'table'     => 'component_products',
                        'type'		=> 'INNER',
                        'conditions'=> ['ComponentProducts.product_id=AnnualPlannedProducts.product_id']
                    ],
                    'Components' => [
                        'table'     => 'components',
                        'type'		=> 'INNER',
                        'conditions'=> ['Components.id=ComponentProducts.component_id']
                    ],
                    'ComponentRawMaterials' => [
                        'table'     => 'component_raw_materials',
                        'type'		=> 'INNER',
                        'conditions'=> ['ComponentRawMaterials.component_id=Components.id']
                    ],
                    'RawMaterials' => [
                        'table'     => 'raw_materials',
                        'type'		=> 'INNER',
                        'conditions'=> ['RawMaterials.id=ComponentRawMaterials.raw_material_id']
                    ],
                    'PurchaseOrderComponents' => [
                        'table'     => 'purchase_order_components',
                        'type'		=> 'LEFT',
                        'conditions'=> ['PurchaseOrderComponents.component_id=ComponentRawMaterials.component_id', 'PurchaseOrderComponents.raw_material_id=ComponentRawMaterials.raw_material_id']
                    ],
                ])
                ->where([
                    'AnnualPlannedProducts.annual_plan_id' => $annualPlanFilter['id']
                ])
                ->group([
                    'ComponentRawMaterials.component_id', 
                    'ComponentRawMaterials.raw_material_id', 
                ]);
        });

    #return data
        return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
	#product details
		$Products = $this->Products->find('all')->where(['is_eventual_product' => 0, 'is_active' => 1]);
    #get annual planned product details
        $annualQuarterPlannedProducts = $this->AnnualQuarterPlannedProducts->find()
                                            ->select([
                                                'product_id'        => 'MAX(AnnualQuarterPlannedProducts.product_id)',
                                                'last_product_no'   => '(MAX(AnnualQuarterPlannedProducts.last_product_no)+AnnualQuarterPlannedProducts.product_quantity+AnnualQuarterPlannedProducts.product_buffer_quantity)',
                                            ])
                                            ->group(['AnnualQuarterPlannedProducts.product_id'])
                                            ->hydrate(false)
                                            ->toArray();
	#get financial years
		$FinancialYears = Configure::read('FinancialYears');
		$this->set(compact('FinancialYears', 'Products', 'annualQuarterPlannedProducts'));

	#check method
		$this->Crud->on('afterSave', function(\Cake\Event\Event $event){
		#get last insert id
			$annualPlanId = $event->subject()->entity->id;
        #check annual plan id empty
            if(!empty($annualPlanId)){
            #machines
                $machines = $this->Machines->find('all')
                                ->select([
                                    'batch_quantity'                => 'Components.batch_quantity',
                                    'annual_plan_id'				=> 'AnnualPlans.id',
                                    'product_id' 					=> 'Products.id', 
                                    'component_id' 					=> 'Components.id', 
                                    'process_id' 					=> 'Processes.id', 
                                    'machine_id' 					=> 'Machines.id',
                                    'tag'							=> 'Components.tag',
                                    'total_components'              => 'ROUND(((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component))',
                                    'total_batches'                 => 'ROUND((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component)/Components.batch_quantity))',
                                    'total_batch_set_time'          => 'ROUND(((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component)/Components.batch_quantity) * ComponentProcesses.set_time_for_batch)/60)',
                                    'total_process_time'            => 'ROUND((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component) * ComponentProcesses.average_time_taken)/60)',
                                    'total_time'                    => 'ROUND((((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component)/Components.batch_quantity) * ComponentProcesses.set_time_for_batch)/60)+((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component) * ComponentProcesses.average_time_taken)/60))',
                                    'in_component_total'            => 'if((Components.tag=0 || Components.tag=2), ROUND(((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component)), 0)',
                                    'out_component_total'           => 'if(Components.tag=1, ((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component), 0)',
                                    'total_in_process_time'         => 'if(Components.tag=0, ROUND((((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component)/Components.batch_quantity) * ComponentProcesses.set_time_for_batch)/60)+((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component) * ComponentProcesses.average_time_taken)/60)), 0)',
                                    'total_out_process_time'        => 'if(Components.tag=1, ROUND((((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component)/Components.batch_quantity) * ComponentProcesses.set_time_for_batch)/60)+((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component) * ComponentProcesses.average_time_taken)/60)), 0)',
                                ])
                                ->join(
                                    [
                                        'ComponentProcesses' => [
                                            'type'			=> 'INNER',
                                            'table'			=> 'component_processes',
                                            'conditions' 	=> ['ComponentProcesses.machine_id=Machines.id']
                                        ],
                                        'MachineProcesses' => [
                                            'type'			=> 'INNER',
                                            'table'			=> 'machine_processes',
                                            'conditions' 	=> ['MachineProcesses.machine_id=ComponentProcesses.machine_id', 'MachineProcesses.process_id=ComponentProcesses.process_id']
                                        ],
                                        'Processes' => [
                                            'type'			=> 'INNER',
                                            'table'			=> 'processes',
                                            'conditions' 	=> ['Processes.id=ComponentProcesses.process_id']
                                        ],
                                        'Components'	=>[
                                            'type'			=> 'INNER',
                                            'table' 		=> 'components',
                                            'conditions' 	=> ['Components.id=ComponentProcesses.component_id']
                                        ],
                                        'ComponentProducts'	=>[
                                            'type'			=> 'INNER',
                                            'table' 		=> 'component_products',
                                            'conditions' 	=> ['ComponentProducts.component_id=Components.id']
                                        ],
                                        'Products'	=>[
                                            'type'			=> 'INNER',
                                            'table' 		=> 'products',
                                            'conditions' 	=> ['Products.id=ComponentProducts.product_id']
                                        ],
                                        'AnnualPlans' => [
                                            'type'			=> 'INNER',
                                            'table'			=> 'annual_plans',
                                            'conditions' 	=> ['AnnualPlans.id'=>$annualPlanId]
                                        ],
                                        'AnnualPlannedProducts' => [
                                            'type'			=> 'INNER',
                                            'table'			=> 'annual_planned_products',
                                            'conditions' 	=> ['AnnualPlannedProducts.product_id=Products.id', 'AnnualPlannedProducts.annual_plan_id=AnnualPlans.id']
                                        ],
                                ])
                                ->toArray();
            #result
                $this->result = [];
            #array recursive
                array_walk_recursive($machines, function($result, $key){
                #array
                    $this->result[$key]['annual_plan_id']       = $result['annual_plan_id'];
                    $this->result[$key]['product_id']           = $result['product_id'];
                    $this->result[$key]['component_id']         = $result['component_id'];
                    $this->result[$key]['process_id']           = $result['process_id'];
                    $this->result[$key]['machine_id']           = $result['machine_id'];
                    $this->result[$key]['total_components']     = $result['total_components'];
                    $this->result[$key]['total_process_time']   = $result['total_process_time'];
                    $this->result[$key]['total_batches']        = $result['total_batches'];
                    $this->result[$key]['total_batch_set_time'] = $result['total_batch_set_time'];
                    $this->result[$key]['total_time']           = $result['total_time'];
                    $this->result[$key]['in_components']        = 0;
                    $this->result[$key]['out_components']       = 0;
                    $this->result[$key]['tag']                  = $result['tag'];
                });

            #insert data
                $entities 	= $this->AnnualPlannedCapacities->newEntities($this->result);
                $result		= $this->AnnualPlannedCapacities->saveMany($entities);
            #return
                return $this->redirect(
                    ['action' => 'capacityPlanView', $annualPlanId]
                );
            }
		});

    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Annual Plan id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
    #product details
		$Products   = $this->Products->find('all')
                                    ->select([
                                        'last_product_no'   => 'AnnualPlannedProducts.last_product_no',
                                        'product_quantity'  => 'AnnualPlannedProducts.product_quantity',
                                    ])
                                    ->join([
                                        'AnnualPlannedProducts' => [
                                            'type'			=> 'inner',
                                            'table'			=> 'annual_planned_products',
                                            'conditions' 	=> ['AnnualPlannedProducts.product_id=Products.id']
                                        ],
                                    ])
                                    ->where([
                                        'Products.is_eventual_product'          => 0, 
                                        'Products.is_active'                    => 1, 
                                        'AnnualPlannedProducts.annual_plan_id'  => $id
                                    ])
                                    ->enableAutofields(true)->toArray();
		$annualPlannedProducts  = $this->AnnualPlannedProducts->find('all')->where(['annual_plan_id' => $id, 'is_active' => 1]);

	#get financial years
		$FinancialYears = Configure::read('FinancialYears');
		$this->set(compact('FinancialYears', 'Products', 'annualPlannedProducts'));

    #before save
        $this->Crud->on('beforeSave', function(\Cake\Event\Event $event){
        #Delete capacities plans 
            $del_entity = $this->AnnualPlannedCapacities
                            ->find('all',['conditions'=>['AnnualPlannedCapacities.annual_plan_id'=>$event->subject()->entity->id],'fields'=>['id']]);
        #check empty entity
            if(!empty($del_entity)){
                $this->AnnualPlannedCapacities->deleteAll(['AnnualPlannedCapacities.annual_plan_id'=>$event->subject()->entity->id]);
                $this->AnnualPlannedProducts->deleteAll(['AnnualPlannedProducts.annual_plan_id'=>$event->subject()->entity->id]);
            }
        });

	#after save
		$this->Crud->on('afterSave', function(\Cake\Event\Event $event){
		#get last insert id
			$annualPlanId = $event->subject()->entity->id;
		#machines
			$machines = $this->Machines->find('all')
                                ->select([
                                    'batch_quantity'                => 'Components.batch_quantity',
                                    'annual_plan_id'				=> 'AnnualPlans.id',
                                    'product_id' 					=> 'Products.id', 
                                    'component_id' 					=> 'Components.id', 
                                    'process_id' 					=> 'Processes.id', 
                                    'machine_id' 					=> 'Machines.id',
                                    'tag'							=> 'Components.tag',
                                    'total_components'              => 'ROUND(((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component))',
                                    'total_batches'                 => 'ROUND((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component)/Components.batch_quantity))',
                                    'total_batch_set_time'          => 'ROUND(((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component)/Components.batch_quantity) * ComponentProcesses.set_time_for_batch)/60)',
                                    'total_process_time'            => 'ROUND((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component) * ComponentProcesses.average_time_taken)/60)',
                                    'total_time'                    => 'ROUND((((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component)/Components.batch_quantity) * ComponentProcesses.set_time_for_batch)/60)+((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component) * ComponentProcesses.average_time_taken)/60))',
                                    'in_component_total'            => 'if((Components.tag=0 || Components.tag=2), ROUND(((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component)), 0)',
                                    'out_component_total'           => 'if(Components.tag=1, ((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component), 0)',
                                    'total_in_process_time'         => 'if(Components.tag=0, ROUND((((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component)/Components.batch_quantity) * ComponentProcesses.set_time_for_batch)/60)+((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component) * ComponentProcesses.average_time_taken)/60)), 0)',
                                    'total_out_process_time'        => 'if(Components.tag=1, ROUND((((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component)/Components.batch_quantity) * ComponentProcesses.set_time_for_batch)/60)+((((AnnualPlannedProducts.product_quantity+AnnualPlannedProducts.product_buffer_quantity) * ComponentProducts.number_of_component) * ComponentProcesses.average_time_taken)/60)), 0)',
                                ])
                                ->join([
                                    'ComponentProcesses' => [
                                        'type'			=> 'inner',
                                        'table'			=> 'component_processes',
                                        'conditions' 	=> ['ComponentProcesses.machine_id=Machines.id']
                                    ],
                                    'MachineProcesses' => [
                                        'type'			=> 'inner',
                                        'table'			=> 'machine_processes',
                                        'conditions' 	=> ['MachineProcesses.machine_id=ComponentProcesses.machine_id', 'MachineProcesses.process_id=ComponentProcesses.process_id']
                                    ],
                                    'Processes' => [
                                        'type'			=> 'inner',
                                        'table'			=> 'processes',
                                        'conditions' 	=> ['Processes.id=ComponentProcesses.process_id']
                                    ],
                                    'Components'	=>[
                                        'type'			=> 'inner',
                                        'table' 		=> 'components',
                                        'conditions' 	=> ['Components.id=ComponentProcesses.component_id']
                                    ],
                                    'ComponentProducts'	=>[
                                        'type'			=> 'inner',
                                        'table' 		=> 'component_products',
                                        'conditions' 	=> ['ComponentProducts.component_id=Components.id']
                                    ],
                                    'Products'	=>[
                                        'type'			=> 'inner',
                                        'table' 		=> 'products',
                                        'conditions' 	=> ['Products.id=ComponentProducts.product_id']
                                    ],
                                    'AnnualPlans' => [
                                        'type'			=> 'inner',
                                        'table'			=> 'annual_plans',
                                        'conditions' 	=> ['AnnualPlans.id'=>$annualPlanId]
                                    ],
                                    'AnnualPlannedProducts' => [
                                        'type'			=> 'inner',
                                        'table'			=> 'annual_planned_products',
                                        'conditions' 	=> ['AnnualPlannedProducts.product_id=Products.id', 'AnnualPlannedProducts.annual_plan_id=AnnualPlans.id']
                                    ],
                                ])
                                ->toArray();

		#result
			$this->result = [];
		#array recursive
			array_walk_recursive($machines, function($result, $key){
			#prepare data to insert multiple
				$this->result[$key]['annual_plan_id']       = $result['annual_plan_id'];
				$this->result[$key]['product_id']           = $result['product_id'];
				$this->result[$key]['component_id']         = $result['component_id'];
				$this->result[$key]['process_id']           = $result['process_id'];
				$this->result[$key]['machine_id']           = $result['machine_id'];
				$this->result[$key]['total_components']     = $result['total_components'];
				$this->result[$key]['total_process_time']   = $result['total_process_time'];
				$this->result[$key]['total_batches']        = $result['total_batches'];
				$this->result[$key]['total_batch_set_time'] = $result['total_batch_set_time'];
				$this->result[$key]['total_time']           = $result['total_time'];
				$this->result[$key]['in_components']        = 0;
				$this->result[$key]['out_components']       = 0;
				$this->result[$key]['tag']                  = $result['tag'];
			});
		#insert data
			$entities 	= $this->AnnualPlannedCapacities->newEntities($this->result);
			$result		= $this->AnnualPlannedCapacities->saveMany($entities);
        #return
            return $this->redirect(
                ['action' => 'capacityPlanView', $annualPlanId]
            );
		});

    #return
		return $this->Crud->execute();
    }

    /**
     * get capacity plan
     *
     * @param string|null $id Annual Planned Capacity id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function capacityPlanView($id = null)
    {
        
    #variable declaration
        $productId  = $this->request->data('product_id');
        $groupId    = $this->request->data('group_id');
    #check data null
        $groupId  = ( (isset($groupId) && !empty($groupId)) ? $groupId : null );
    #check product id empty
        if(!isset($productId) || empty($productId) || $productId==null){
            $productId  = $this->Products->find()->select(['id'])->where(['is_eventual_product' => 0, 'is_active' => 1])->first();
            $productId  = $productId->id;
        }
    #get product details if product id is empty
        if($groupId==null){
            $machineWhere = [ 'ComponentProducts.product_id' => $productId, 'AnnualPlannedCapacities.annual_plan_id' => $id ];
        }
        else{
        #product details
            $machineWhere = [ 'ComponentProducts.product_id' => $productId, 'ComponentGroups.group_id' => $groupId, 'AnnualPlannedCapacities.annual_plan_id' => $id ];
        }

    #get capacite plan based on machines
        $machines = $this->Machines->find('all')
                        ->contain([
                            'AnnualPlannedCapacities' => function($q) use($machineWhere){
                                return $q->select([
                                    'id'                    => 'ComponentProcesses.id',
                                    'process_id'            => 'ComponentProcesses.process_id',
                                    'average_time_taken'    => 'ComponentProcesses.average_time_taken',
                                    'set_time_for_batch'    => 'ComponentProcesses.set_time_for_batch',
                                    'annual_plan_id'        => 'AnnualPlannedCapacities.annual_plan_id',
                                    'product_id'            => 'AnnualPlannedCapacities.annual_plan_id',
                                    'component_id'          => 'AnnualPlannedCapacities.component_id',
                                    'machine_id'            => 'AnnualPlannedCapacities.machine_id',
                                    'total_process'         => 'COUNT(AnnualPlannedCapacities.process_id)',
                                    'average_process_time'  => 'SUM(AnnualPlannedCapacities.total_time)',
                                    'number_of_component'   => 'ComponentProducts.number_of_component',
                                    'batch_quantity'        => 'Components.batch_quantity',
                                ])
                                ->enableAutofields(true)
                                ->contain(['AnnualPlans', 'Products', 'Components', 'Processes' ])
                                ->join([
                                    'ComponentProcesses' => [
                                        'table'         => 'component_processes',
                                        'type'          => 'INNER',
                                        'conditions'    => [
                                                            'ComponentProcesses.component_id = AnnualPlannedCapacities.component_id', 'ComponentProcesses.machine_id=AnnualPlannedCapacities.machine_id', 
                                                            'ComponentProcesses.process_id = AnnualPlannedCapacities.process_id', 
                                                            'AnnualPlannedCapacities.tag IN' => [0,2]
                                                        ]
                                    ],
                                    'ComponentGroups' => [
                                        'table'         => 'component_groups',
                                        'type'          => 'INNER',
                                        'conditions'    => [ 'ComponentGroups.component_id = ComponentProcesses.component_id' ]
                                    ],
                                    'ComponentProducts' => [
                                        'table'         => 'component_products',
                                        'type'          => 'INNER',
                                        'conditions'    => [ 'ComponentProducts.component_id = ComponentGroups.component_id' ]
                                    ],
                                    'Components' => [
                                        'table'         => 'components',
                                        'type'          => 'INNER',
                                        'conditions'    => [ 'Components.id = ComponentProducts.component_id' ]
                                    ],
                                ])
                                ->where($machineWhere)
                                ->group([
                                    'AnnualPlannedCapacities.annual_plan_id', 
                                    'AnnualPlannedCapacities.product_id', 
                                    'AnnualPlannedCapacities.component_id', 
                                    'AnnualPlannedCapacities.machine_id'
                                ])
                                ->order(['AnnualPlannedCapacities.tag' => 'ASC']);
                            }
                        ])
                        ->toArray();

    #get product list
        $products = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->where(['is_active' => 1, 'is_eventual_product' => 0]);

    #get groups list
        $groups	  = $this->Groups->find('list', ['keyField' => 'id', 'valueField' => 'group_name'])->where(['is_active' => 1]);

    #annual plan
        $annualPlans = $this->AnnualPlans->get($id);

    #confriguration files
        $tag                    = Configure::read('tag');
        $QuaterDuration         = Configure::read('QuaterDuration');
        $machineRunningDays     = Configure::read('machineRunningDays');
        $machineRunningMonths   = Configure::read('machineRunningMonths');

    #set values to view
        $this->set(compact(['id', 'annualPlans', 'tag', 'products', 'groups', 'machines', 'productId', 'groupId', 'machineRunningDays', 'machineRunningMonths']));
    }

    /**
     * freez method
     *
     * @param string|null $id Annual Planned Capacity id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function freezCapacityPlan($id = null)
    {
    #tag
        $QuaterDuration = Configure::read('QuaterDuration');
    #check method
		if($this->request->is(['post', 'put']))
		{
        #get id from array
			$ids = array_column($this->request->data('annual_planned_capacities'), 'id');
		#find all data by array of id's
			$annualPlannedCapacities = $this->AnnualPlannedCapacities->find('all',[
				'conditions'=>[
					'id IN' => $ids
				]
			]);
		#update multiple records
			$entity = $this->AnnualPlannedCapacities->patchEntities($annualPlannedCapacities, $this->request->data('annual_planned_capacities'));
			$this->AnnualPlannedCapacities->saveMany($entity);
		#Get Planned Products
            $annualPlannedProducts = $this->AnnualPlannedProducts->find('all')->where(['AnnualPlannedProducts.annual_plan_id' => $id])->toArray();
            
        #BOF  purchase indent 04062019
            #Toupdate  purchase Indent
            foreach ($annualPlannedProducts as $pKey => $products) {
                $total_product_count = $products['product_quantity'] + $products['product_buffer_quantity'];
            #prepare component products
                $components = $this->ComponentProducts->find()
                                    ->select(['Components.component_name',
                                        'Components.id',
                                        'Components.buying_frequency',
                                        'Components.tag',
                                        'ComponentGroups.id',
                                        'Components.component_name',
                                        'Groups.id',
                                        'Groups.group_name',
                                        'AnnualPlannedCapacities.tag',
                                        'AnnualPlannedCapacities.total_components',
                                        'AnnualPlannedCapacities.in_components',
                                        'AnnualPlannedCapacities.out_components',

                                    ])
                                    ->join([
                                        'Components'              => [
                                            'table'      => 'components',
                                            'type'       => 'inner',
                                            'conditions' => [
                                                'Components.id =ComponentProducts.component_id',
                                            ]],
                                        'ComponentGroups'         => [
                                            'table'      => 'component_groups',
                                            'type'       => 'inner',
                                            'conditions' => [
                                                'ComponentGroups.component_id = ComponentProducts.component_id',
                                            ]],
                                        'Groups'                  => [
                                            'table'      => 'groups',
                                            'type'       => 'inner',
                                            'conditions' => [
                                                'Groups.id =ComponentGroups.group_id',
                                            ]],
                                        'AnnualPlannedCapacities' => [
                                            'table'      => 'annual_planned_capacities',
                                            'type'       => 'left',
                                            'conditions' => [
                                                'AnnualPlannedCapacities.component_id =ComponentProducts.component_id'
                                                , 'AnnualPlannedCapacities.annual_plan_id' => $id,
                                            ]],

                                    ])
                                    ->distinct(['ComponentProducts.component_id'])
                                    ->enableAutofields(true)
                                    ->where(['ComponentProducts.product_id' => $products['product_id']])
                                    ->toArray();

                $purchase_ident_array = [];
            #loop components
                foreach ($components as $key => $value)
                {
                    $temp_tag                       = $value->Components['tag'];
                    $temp_group                     = $value->Groups['id'];
                    $temp_buying_freq               = $value->Components['buying_frequency'];
                    $data['group_name']             = $value->Groups['group_name'];
                    $data['group_id']               = $value->Groups['id'];
                    $data['component_id']           = $value->component_id;
                    $data['number_of_component']    = $value->number_of_component;
                    $data['buying_frequency']       = $temp_buying_freq;
                #check tag
                    if ($temp_tag == 2) {
                        $planned_capacity_tag       = $value->AnnualPlannedCapacities['tag'];
                        $planned_capacity_in_total  = $value->AnnualPlannedCapacities['in_components'];
                        $planned_capacity_out_total = $value->AnnualPlannedCapacities['out_components'];
                        if (!empty($planned_capacity_tag)) {
                            if (!empty($planned_capacity_in_total)) {

                                $data['total_quantity'] = round(($planned_capacity_in_total) / $temp_buying_freq);

                                $data['tag']                                               = 0;
                                $purchase_ident_array[$temp_group][0][$temp_buying_freq][] = $data;
                            }
                            if (!empty($planned_capacity_out_total)) {
                                $data['total_quantity']                                    = round(($planned_capacity_out_total) / $temp_buying_freq);
                                $data['tag']                                               = 1;
                                $purchase_ident_array[$temp_group][1][$temp_buying_freq][] = $data;
                            }
                        }
                    } else {
                        $data['total_quantity']                                            = round(($value->number_of_component * $total_product_count) / $temp_buying_freq);
                        $data['tag']                                                       = $value->Components['tag'];
                        $purchase_ident_array[$temp_group][$temp_tag][$temp_buying_freq][] = $data;
                    }

                }
                $purchase_idents       = [];
                $purchase_indent_array = [];
                $keys                  = 0;
                #Group wise data
                foreach ($purchase_ident_array as $key => $group_value) {
                #tag wise data
                    foreach ($group_value as $tag_value_key => $buying_value) {
                    #buying frequency
                        foreach ($buying_value as $buying_key => $components) {
                            for ($i = 0; $i < $buying_key; $i++) {
                                $pi_components = [];
                                foreach ($components as $components_key => $components_value) {
                                    $purchase_indent_array[$keys]['annual_plan_id']     = $products['annual_plan_id'];
                                    $purchase_indent_array[$keys]['product_id']         = $products['product_id'];
                                    $purchase_indent_array[$keys]['group_id']           = $components_value['group_id'];
                                    $purchase_indent_array[$keys]['purchase_intent_no'] = $components_value['group_name'] . date('Yds').mt_rand(10,100);
                                    $purchase_indent_array[$keys]['tag']                = $components_value['tag'];
                                    $purchase_indent_array[$keys]['is_adhoc']           = 0;
                                    $purchase_indent_array[$keys]['status']             = 0;
                                    $pi_components[$components_key]['component_id']     = $components_value['component_id'];
                                    $pi_components[$components_key]['is_active']        = 1;
                                    $pi_components[$components_key]['total_quantity']   = $components_value['total_quantity'];
                                }
                                $purchase_indent_array[$keys]['purchase_indent_components'] = $pi_components;
                                $keys = $keys + 1;
                            }
                        }
                    }
                }
                $entities = $this->PurchaseIndents->newEntities($purchase_indent_array);
                $result = $this->PurchaseIndents->saveMany($entities);
            }
        #END of purchase indent
        #check annaual planned products empty
            if(!empty($annualPlannedProducts))
            {
            #variable declaration
                $this->result       = [];
                $last_product_no    = [];
            #split products to quarter wise
                for( $i = 0 ; $i < $QuaterDuration ; $i++ )
                {
                    $this->result[$i]['annual_plan_id']  = $id;
                    $this->result[$i]['duration']        = 'Quarter '.($i+1);
                    $this->result[$i]['status']          = 0;
                    $this->result[$i]['is_active']       = 1;
                #quarter plan products
                    foreach($annualPlannedProducts AS $pKey => $products)
                    {
                    #split quarter plan products
                        $quarterWiseProductQuantity = $this->splitEqualParts($products['product_quantity'], $QuaterDuration);
                        $quarterWiseBufferQuantity  = $this->splitEqualParts($products['product_buffer_quantity'], $QuaterDuration);
                    #set last product number
                        if($i==0){
                            $last_product_no[$products->product_id] = $products->last_product_no;
                        }
                        else{
                            $last_product_no[$products->product_id] = $last_product_no[$products->product_id] + ($quarterWiseProductQuantity[$i]+$quarterWiseBufferQuantity[$i]);
                        }
                    #prepare data for insert
                        $this->result[$i]['annual_quarter_planned_products'][$pKey]['product_id']                  = $products->product_id;
                        $this->result[$i]['annual_quarter_planned_products'][$pKey]['product_quantity']            = $quarterWiseProductQuantity[$i];
                        $this->result[$i]['annual_quarter_planned_products'][$pKey]['product_buffer_quantity']     = $quarterWiseBufferQuantity[$i];
                        $this->result[$i]['annual_quarter_planned_products'][$pKey]['product_requested_quantity']  = 0;
                        $this->result[$i]['annual_quarter_planned_products'][$pKey]['product_total_quantity']      = 0;
                        $this->result[$i]['annual_quarter_planned_products'][$pKey]['last_product_no']             = $last_product_no[$products->product_id];
                        $this->result[$i]['annual_quarter_planned_products'][$pKey]['is_active']                   = 1;
                    }
                }
            #save Annual Planned Products
                $quarterPlanEntities  = $this->AnnualQuarterPlans->newEntities($this->result);
                if($this->AnnualQuarterPlans->saveMany($quarterPlanEntities)){
                #update query
                    $this->AnnualPlans->updateAll(
                        ['status' => 1],
                        ['id' => $id]
                    );
                }
            }
        #return
            return $this->redirect(
                ['controller' => 'AnnualQuarterPlans', 'action' => 'view', $id]
            );
		}
    }
}
