<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\I18n\Time;

/**
 * AnnualQuarterPlans Controller
 *
 * @property \App\Model\Table\AnnualQuarterPlansTable $AnnualQuarterPlans
 *
 * @method \App\Model\Entity\AnnualQuarterPlan[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class AnnualQuarterPlansController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'prepareQuarterPlan' => ['className' => 'Crud.Edit'],
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Api'
            ]
        ]);

    #load models
        $this->loadModel('Groups');
        $this->loadModel('AnnualPlans');
        $this->loadModel('Products');
        $this->loadModel('SubAssemblyProducts');
        $this->loadModel('AnnualAssemblyTrackings');
        $this->loadModel('AnnualProductTrackings');
        $this->loadModel('AnnualComponentTrackings');
        $this->loadModel('AnnualQuarterPlannedProducts');
        $this->loadModel('AnnualPlannedProducts');
        $this->loadModel('ComponentProducts');
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #paginate
        $annualPlans = $this->paginate($this->AnnualPlans->find());
    #return data
        $this->set(compact(['annualPlans']));
    }

    /**
     * View method
     *
     * @param string|null $id Annual Quarter Plan id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#get quarter plan based on annual plan
        $annualQuarterPlans = $this->AnnualPlans->find()
                                ->contain([
                                    'AnnualPlannedProducts' => [
                                        'Products' => [
                                            'AnnualQuarterPlannedProducts' => function($query) use($id){
                                                return $query
                                                            ->select([
                                                                'id'                                => 'AnnualQuarterPlans.id',
                                                                'status'                            => 'AnnualQuarterPlans.status',
                                                                'duration'                          => 'AnnualQuarterPlans.duration',
                                                                'annual_quarter_planned_product_id' => 'AnnualQuarterPlannedProducts.id',
                                                                'product_id'                        => 'AnnualQuarterPlannedProducts.product_id',
                                                                'product_quantity'                  => 'AnnualQuarterPlannedProducts.product_quantity',
                                                                'product_buffer_quantity'           => 'AnnualQuarterPlannedProducts.product_buffer_quantity',
                                                                'product_requested_quantity'        => 'AnnualQuarterPlannedProducts.product_requested_quantity',
                                                                'product_total_quantity'            => 'AnnualQuarterPlannedProducts.product_total_quantity',
                                                                'last_product_no'                   => 'AnnualQuarterPlannedProducts.last_product_no',
                                                            ])
                                                            ->join([
                                                                'AnnualQuarterPlans' => [
                                                                    'table'         => 'annual_quarter_plans',
                                                                    'type'          => 'inner',
                                                                    'conditions'    => ['AnnualQuarterPlans.id=AnnualQuarterPlannedProducts.annual_quarter_plan_id']
                                                                ],
                                                                'AnnualPlans' => [
                                                                    'table'         => 'annual_plans',
                                                                    'type'          => 'inner',
                                                                    'conditions'    => ['AnnualPlans.id=AnnualQuarterPlans.annual_plan_id']
                                                                ],
                                                            ])
                                                            ->where([
                                                                'AnnualPlans.id' => $id
                                                            ]);
                                            }
                                        ]
                                    ]
                                ])
                                ->where([ 'AnnualPlans.id' => $id, 'AnnualPlans.is_active' => 1 ])
                                ->first();
    #set data
        $this->set(compact('annualQuarterPlans'));
    }

    /**
     * prepare quarter plan method
     *
     * @return \Cake\Http\Response|void
     */
    public function prepareQuarterPlan($id = null)
    {
    #get app.php global configured data
        $groupDelay         = Configure::read('GroupDelay');
        $quarterMonths      = Configure::read('QuarterMonths');
        $quarterWorkingDays = Configure::read('QuarterWorkingDays');
        $startMonth         = array(1 => 3, 2 => 6, 3 => 9, 4 => 12);
    #get post data
        $editData           = $this->request->data('product_quantity');
        $annualQuarterPlans = $this->AnnualQuarterPlans->get($id);
    #before find
		$this->Crud->on('afterSave', function(\Cake\Event\Event $event) use($id, $annualQuarterPlans, $quarterWorkingDays, $groupDelay, $startMonth, $editData) {
        #data edit quarter plan edit
            $this->data = [];
            foreach($editData AS $key => $rows){
                $this->data[] = $rows['id'];
            }
        #find given values from quater planned products
            $annualQuarterPlannedProducts = $this->AnnualQuarterPlannedProducts->find()->where(['id IN' => $this->data]);
        #loop data
            $this->data = [];
            foreach($annualQuarterPlannedProducts AS $key => $rows){
                $this->data[$key]['id']               = $rows->id;
                $this->data[$key]['product_quantity'] = $editData[$rows->id]['product_quantity'];
            }

        #patch entity update data
            $entity = $this->AnnualQuarterPlannedProducts->patchEntities($annualQuarterPlannedProducts, $this->data);
        #checks product quantity updated
            if($this->AnnualQuarterPlannedProducts->saveMany($entity)){
            #update annual product quantity
                $annualPlannedProducts = $this->AnnualPlannedProducts->find()
                                            ->select([
                                                'id'                => 'AnnualPlannedProducts.id',
                                                'product_quantity'  => 'SUM(AnnualQuarterPlannedProducts.product_quantity)',
                                            ])
                                            ->join([
                                                'AnnualQuarterPlans' => [
                                                    'table'     => 'annual_quarter_plans',
                                                    'type'		=> 'INNER',
                                                    'conditions'=> ['AnnualQuarterPlans.annual_plan_id=AnnualPlannedProducts.annual_plan_id']
                                                ],
                                                'AnnualQuarterPlannedProducts' => [
                                                    'table'     => 'annual_quarter_planned_products',
                                                    'type'		=> 'INNER',
                                                    'conditions'=> ['AnnualQuarterPlannedProducts.annual_quarter_plan_id=AnnualQuarterPlans.id', 'AnnualQuarterPlannedProducts.product_id=AnnualPlannedProducts.product_id']
                                                ],
                                            ])
                                            ->where(['AnnualPlannedProducts.annual_plan_id' => $annualQuarterPlans['annual_plan_id']])
                                            ->group(['AnnualQuarterPlannedProducts.product_id']);
                $annualPlannedProductsData = $annualPlannedProducts->hydrate(false)->toArray();
            #patch entity update data
                $entity = $this->AnnualPlannedProducts->patchEntities($annualPlannedProducts, $annualPlannedProductsData);
                $this->AnnualPlannedProducts->saveMany($entity);

            #edit product number series
                foreach($annualQuarterPlannedProducts AS $key => $rows){
                #find current record for update serial number
                    $currentRecords = $this->AnnualQuarterPlannedProducts->find()
                                                        ->where([
                                                           'product_id' => $rows->product_id,
                                                           'id'  => $rows->id,
                                                        ])
                                                        ->hydrate(false)
                                                        ->first();
                #find next record for update serial number
                    $nextRecords = $this->AnnualQuarterPlannedProducts->find()
                                                        ->where([
                                                           'product_id' => $rows->product_id,
                                                           'id >'  => $rows->id,
                                                        ])
                                                        ->order([
                                                            'AnnualQuarterPlannedProducts.id' => 'ASC'
                                                        ])
                                                        ->hydrate(false)
                                                        ->extract('id')
                                                        ->toArray();
                #check next records empty
                    if(!empty($nextRecords)){
                    #find records
                        $updateProductNumbers = $this->AnnualQuarterPlannedProducts->find()->where(['AnnualQuarterPlannedProducts.id IN' => $nextRecords]);
                    #check conditions
                        #variable declaration
                            $this->data     = [];
                            $lastProductNo  = 0;
                        #loop data
                            foreach($updateProductNumbers AS $lKey=>$lRows){
                                $this->data[$lKey]['id'] = $lRows->id;
                            #checks key
                                if($lKey==0){
                                    $lastProductNo = $currentRecords['last_product_no']+$currentRecords['product_quantity'];
                                    $this->data[$lKey]['last_product_no']  = $lastProductNo;
                                }
                                else{
                                    $lastProductNo =  $lastProductNo+$lRows->product_quantity;
                                    $this->data[$lKey]['last_product_no']  = $lastProductNo;
                                }
                            }
                        #patch entity update data
                            $entity = $this->AnnualQuarterPlannedProducts->patchEntities($updateProductNumbers, $this->data);
                            $this->AnnualQuarterPlannedProducts->saveMany($entity);
                    }
                }

            #get standard product data
                $quarterStandardPlanedProducts = $this->AnnualQuarterPlannedProducts->find('all')
                                                    ->select([
                                                        'duration_plan_id'              => 'AnnualQuarterPlans.id',
                                                        'duration'                      => 'AnnualQuarterPlans.duration',
                                                        'id'                            => 'AnnualQuarterPlannedProducts.id',
                                                        'annual_quarter_plan_id'        => 'AnnualQuarterPlannedProducts.annual_quarter_plan_id',
                                                        'product_quantity'              => 'AnnualQuarterPlannedProducts.product_quantity',
                                                        'product_buffer_quantity'       => 'AnnualQuarterPlannedProducts.product_buffer_quantity',
                                                        'product_requested_quantity'    => 'AnnualQuarterPlannedProducts.product_requested_quantity',
                                                        'product_total_quantity'        => 'AnnualQuarterPlannedProducts.product_total_quantity',
                                                        'is_active'                     => 'AnnualQuarterPlannedProducts.is_active',
                                                        'last_product_no'               => 'AnnualQuarterPlannedProducts.last_product_no',
                                                        'product_id'                    => 'Products.id',
                                                        'is_eventual_product'           => 'Products.is_eventual_product',
                                                        'total_assembly'                => 'COUNT(SubAssemblyProducts.id)',
                                                    ])
                                                    ->contain(['AnnualQuarterPlans'])
                                                    ->join([
                                                        'Products' => [
                                                            'table'     => 'products',
                                                            'type'		=> 'INNER',
                                                            'conditions'=> ['Products.id=AnnualQuarterPlannedProducts.product_id']
                                                        ],
                                                        'SubAssemblyProducts' => [
                                                            'table'     => 'sub_assembly_products',
                                                            'type'		=> 'INNER',
                                                            'conditions'=> ['SubAssemblyProducts.product_id=Products.id']
                                                        ],
                                                    ])
                                                    ->where([
                                                        'AnnualQuarterPlannedProducts.annual_quarter_plan_id' => $event->subject->entity->id
                                                    ])
                                                    ->group(['SubAssemblyProducts.product_id'])->toArray();

            #get eventual product details based on standard product
                $quarterEventualPlanedProducts = $this->AnnualQuarterPlannedProducts->find('all')
                                                    ->select([
                                                        'duration_plan_id'              => 'AnnualQuarterPlans.id',
                                                        'duration'                      => 'AnnualQuarterPlans.duration',
                                                        'id'                            => 'AnnualQuarterPlannedProducts.id',
                                                        'annual_quarter_plan_id'        => 'AnnualQuarterPlannedProducts.annual_quarter_plan_id',
                                                        'product_quantity'              => 'AnnualQuarterPlannedProducts.product_quantity',
                                                        'product_buffer_quantity'       => 'AnnualQuarterPlannedProducts.product_buffer_quantity',
                                                        'product_requested_quantity'    => 'AnnualQuarterPlannedProducts.product_requested_quantity',
                                                        'product_total_quantity'        => 'AnnualQuarterPlannedProducts.product_total_quantity',
                                                        'is_active'                     => 'AnnualQuarterPlannedProducts.is_active',
                                                        'last_product_no'               => 'AnnualQuarterPlannedProducts.last_product_no',
                                                        'product_id'                    => 'Products.id',
                                                        'is_eventual_product'           => 'Products.is_eventual_product',
                                                        'total_assembly'                => 'COUNT(SubAssemblyProducts.id)',
                                                    ])
                                                    ->contain(['AnnualQuarterPlans'])
                                                    ->join([
                                                        'Products' => [
                                                            'table'     => 'products',
                                                            'type'		=> 'INNER',
                                                            'conditions'=> ['Products.parent_id=AnnualQuarterPlannedProducts.product_id']
                                                        ],
                                                        'SubAssemblyProducts' => [
                                                            'table'     => 'sub_assembly_products',
                                                            'type'		=> 'INNER',
                                                            'conditions'=> ['SubAssemblyProducts.product_id=Products.id']
                                                        ],
                                                    ])
                                                    ->where([
                                                        'AnnualQuarterPlannedProducts.annual_quarter_plan_id' => $event->subject->entity->id
                                                    ])
                                                    ->group(['SubAssemblyProducts.product_id'])->toArray();

            #merge standard and eventual products
                $quarterPlannedProducts = array_merge($quarterStandardPlanedProducts, $quarterEventualPlanedProducts);

            #loop product details
                foreach($quarterPlannedProducts AS $aqpKey => $aqpRows)
                {
                #total products
                    $totalQuarterProducts   = $aqpRows->product_quantity + $aqpRows->product_buffer_quantity;
                    $expectedNoOfDays       = ceil($quarterWorkingDays/$totalQuarterProducts);
                    $month                  = $startMonth[$aqpRows->duration_plan_id];
                    $startDate              = date('Y').'-'.$month.'-'.'01';
                    $endDate                = $startDate;
                #prepare to insert products
                    $this->result = [];
                    for($i = 1 ; $i <= $totalQuarterProducts ; $i++)
                    {
                    #machine end date
                        $endDate = date('Y-m-d', strtotime("+".$expectedNoOfDays." day", strtotime($endDate)));
                    #prepare data
                        $this->result[$i]['annual_quarter_plan_id'] = $event->subject->entity->id;
                        $this->result[$i]['product_id']             = $aqpRows->product_id;
                        $this->result[$i]['product_number']         = $aqpRows->last_product_no+$i;
                        $this->result[$i]['total_assembly']         = $aqpRows->total_assembly;
                        $this->result[$i]['completed_assembly']     = 0;
                        $this->result[$i]['status']                 = 0;
                        $this->result[$i]['expected_delivery_date'] = $endDate;
                        $this->result[$i]['is_eventual_product']    = $aqpRows->is_eventual_product;
                        $this->result[$i]['is_eventual_mapped']     = 0;
                        $this->result[$i]['is_active']              = 1;
                    }
                #patch entity
                    $entity  = $this->AnnualProductTrackings->newEntities($this->result);
                #check insertion success in product tracking
                    if($this->AnnualProductTrackings->saveMany($entity, array('callbacks' => false))){
                    #product assembly
                        $assemblyProduct = $this->AnnualProductTrackings->find()
                                                ->select([
                                                    'annual_quarter_plan_id'        => $event->subject->entity->id,
                                                    'sub_assembly_id'               => 'SubAssemblyProducts.sub_assembly_id',
                                                    'annual_product_tracking_id'    => 'AnnualProductTrackings.id',
                                                    'product_id'                    => 'AnnualProductTrackings.product_id',
                                                    'product_number'                => 'AnnualProductTrackings.product_number',
                                                    'total_component'               => '(SELECT COUNT(*) FROM `component_assemblies` AS CA INNER JOIN `component_products` AS CP ON CP.`component_id`=CA.`component_id` INNER JOIN `components` AS C ON C.`id`=CP.`component_id` AND C.`is_active`=1 WHERE CA.`sub_assembly_id`=SubAssemblyProducts.`sub_assembly_id` AND CP.`product_id`=SubAssemblyProducts.`product_id` AND ( (CP.`start_product_no`=0 AND CP.`end_product_no`=0) OR (CP.`start_product_no`<=AnnualProductTrackings.product_number AND CP.`end_product_no`>=AnnualProductTrackings.product_number) ) GROUP BY CA.`sub_assembly_id`)',
                                                    'completed_component'           => 0,
                                                    'status'                        => 0,
                                                ])
                                                ->join([
                                                    'SubAssemblyProducts' => [
                                                        'table'     => 'sub_assembly_products',
                                                        'type'		=> 'INNER',
                                                        'conditions'=> ['SubAssemblyProducts.product_id=AnnualProductTrackings.product_id', 'SubAssemblyProducts.is_active' => 1]
                                                    ],
                                                ])
                                                ->where([
                                                    'AnnualProductTrackings.annual_quarter_plan_id' => $event->subject->entity->id, 
                                                    'AnnualProductTrackings.product_id'             => $aqpRows->product_id, 
                                                    'AnnualProductTrackings.is_active'              => 1
                                                ])
                                                ->order([
                                                    'SubAssemblyProducts.sub_assembly_id'   => 'ASC',
                                                    'AnnualProductTrackings.product_number' => 'ASC'
                                                ])
                                                ->hydrate(false)
                                                ->toArray();

                    #patch entity
                        $entity  = $this->AnnualAssemblyTrackings->newEntities($assemblyProduct);
                    #check query executed
                        if($this->AnnualAssemblyTrackings->saveMany($entity, array('callbacks' => false)))
                        {
                        #prepare select data product component tracking
                            $productsComponentTracking = $this->AnnualAssemblyTrackings->find()
                                                            ->select([
                                                               'annual_quarter_plan_id'         => $event->subject->entity->id,
                                                               'annual_assembly_tracking_id'    => 'AnnualAssemblyTrackings.id',
                                                               'component_product_id'           => 'ComponentProducts.id',
                                                               'product_number'                 => 'AnnualAssemblyTrackings.product_number',
                                                               'total_components'               => 'ComponentProducts.number_of_component',
                                                               'total_completed_components'     => 0,
                                                               'total_rejected_components'      => 0,
                                                               'status'                         => 0,
                                                               'is_eventual_product'            => $aqpRows->is_eventual_product,
                                                               'is_eventual_mapped'             => 0,
                                                            ])
                                                            ->join([
                                                                'ComponentProducts' => [
                                                                    'table'     => 'component_products',
                                                                    'type'		=> 'INNER',
                                                                    'conditions'=> [ 'ComponentProducts.product_id=AnnualAssemblyTrackings.product_id' ]
                                                                ],
                                                                'ComponentAssemblies' => [
                                                                    'table'     => 'component_assemblies',
                                                                    'type'		=> 'INNER',
                                                                    'conditions'=> [ 'ComponentAssemblies.sub_assembly_id=AnnualAssemblyTrackings.sub_assembly_id', 'ComponentAssemblies.component_id=ComponentProducts.component_id' ]
                                                                ],
                                                            ])
                                                            ->where([
                                                                'AnnualAssemblyTrackings.annual_quarter_plan_id'    => $event->subject->entity->id, 
                                                                'AnnualAssemblyTrackings.product_id'                => $aqpRows->product_id, 
                                                                'AnnualAssemblyTrackings.is_active'                 => 1
                                                             ])
                                                            ->hydrate(false)
                                                            ->toArray();
                        #prepare entity
                            $pEntity  = $this->AnnualComponentTrackings->newEntities($productsComponentTracking);
                        #save data
                            if($this->AnnualComponentTrackings->saveMany($pEntity, array('callbacks' => false))){
                            #get data
                                $componentProducts = $this->ComponentProducts->find()
                                                        ->select([
                                                            'id'             => 'id',   
                                                            'spare_required' => '(spare_count-spare_remaning)'
                                                        ])
                                                        ->where([
                                                            'product_id'    => $aqpRows->product_id, 
                                                            'is_active'     => 1
                                                        ])
                                                        ->hydrate(false)
                                                        ->toArray();
                            #prepare data (patch entity)
                                $entity = $this->ComponentProducts->patchEntities($componentProducts, $componentProducts);
                            #save data
                                $this->ComponentProducts->saveMany($entity);
                            }
                        }
                    }
                }
            #return
                return $this->redirect(
                    ['action' => 'eventualProducts', $event->subject->entity->id]
                );
            }
		});
    #return
		return $this->Crud->execute();
    }

    /**
     * eventual products
     *
     * @return \Cake\Http\Response|void
     */
    public function eventualProducts($id = null)
    {
        $productId = $this->request->getQuery('product_id');
        $productId = ( (isset($productId) && !empty($productId)) ? $productId : null );

    #product details
        $productsList = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])
                            ->where([
                                'is_eventual_product'   => 0, 
                                'is_active'             => 1
                            ]);

    #get product information
        if(empty($productId) || $productId==null){
        #find first product
            $products = $this->Products->find()
                            ->where([
                                'is_eventual_product'   => 0, 
                                'is_active'             => 1
                            ])->first();
        #get first product id
            $productId = $products->id;
        }

    #products numbers with tracking data
        $annualProductTrackings   = $this->Products->find()
                                        ->contain([
                                            'AnnualProductTrackings' => function($query) use($id){
                                                return $query
                                                            ->where([
                                                                'AnnualProductTrackings.annual_quarter_plan_id'    => $id,
                                                                'AnnualProductTrackings.is_eventual_product'       => 1,
                                                            ])
                                                            ->group(['AnnualProductTrackings.product_number'])
                                                            ->order(['AnnualProductTrackings.product_number' => 'ASC', 'AnnualProductTrackings.is_eventual_mapped' => 'DESC']);
                                            }
                                        ])
                                        ->where([
                                            'Products.parent_id'            => $productId,
                                            'Products.is_eventual_product'  => 1, 
                                            'Products.is_active'            => 1
                                        ])
                                        ->group(['Products.parent_id'])
                                        ->first();

    #get quarter details
        $annualQuarterPlans = $this->AnnualQuarterPlans->get($id);

    #total products for quarter
        $monthWiseProducts = 0;
        if(!empty($annualProductTrackings->annual_product_trackings)){
            $monthWiseProducts  = $this->splitEqualParts(count($annualProductTrackings->annual_product_trackings), 3);
        }

    #get global configuration from config/app.php
        $quarterMonths = Configure::read('QuarterMonths');

    #set data
        $this->set(compact('id', 'annualQuarterPlans', 'productId', 'productsList', 'annualProductTrackings', 'monthWiseProducts', 'quarterMonths'));
    }

    /**
     * eventual component mapping
     *
     * @return \Cake\Http\Response|void
     */
    public function eventualComponentMapping($id = null)
    {
    #get eventual component details
        $annualProductTrackings = $this->AnnualProductTrackings->get($id);
        $productId              = $annualProductTrackings['product_id'];
        $productNumber          = $annualProductTrackings['product_number'];
        $annualQuarterPlanId    = $annualProductTrackings['annual_quarter_plan_id'];

    #product details
        $products               = $this->Products->get($productId);
        $parentId               = $products['parent_id'];
        $masterProduct          = $this->Products->get($parentId);

    #groups
        $groups = $this->Groups->find()
                        ->contain([
                            'ComponentGroups' => function($query) use($parentId){
                                return $query
                                            ->select([
                                                'group_id'      => 'ComponentGroups.group_id',
                                                'product_id'    => 'Products.id',
                                                'product_name'  => 'Products.product_name',
                                                'description'   => 'Products.description',
                                            ])
                                            ->join([
                                                'ComponentProducts' => [
                                                    'table'     => 'component_products',
                                                    'type'		=> 'INNER',
                                                    'conditions'=> [ 'ComponentProducts.component_id=ComponentGroups.component_id' ]
                                                ],
                                                'Products' => [
                                                    'table'     => 'products',
                                                    'type'		=> 'INNER',
                                                    'conditions'=> [ 'Products.id=ComponentProducts.product_id' ]
                                                ],
                                            ])
                                            ->where(['Products.parent_id' => $parentId])
                                            ->group(['ComponentGroups.group_id', 'Products.id']);
                            }
                        ])->where(['is_active' => 1]);

    #product components
        $componentProducts = $this->Products->find()
                                ->select([
                                    'component_product_id'  => 'ComponentProducts.id',
                                    'component_group_id'    => 'ComponentGroups.id',
                                    'product_id'            => 'Products.id',
                                    'group_id'              => 'ComponentGroups.group_id',
                                    'component_id'          => 'Components.id',
                                    'component_name'        => 'Components.component_name',
                                ])
                                ->join([
                                    'ComponentProducts' => [
                                        'table'     => 'component_products',
                                        'type'		=> 'INNER',
                                        'conditions'=> [ 'ComponentProducts.product_id=Products.id',  'ComponentProducts.is_active' => 1 ]
                                    ],
                                    'ComponentGroups' => [
                                        'table'     => 'component_groups',
                                        'type'		=> 'INNER',
                                        'conditions'=> [ 'ComponentGroups.component_id=ComponentProducts.component_id',  'ComponentGroups.is_active' => 1 ]
                                    ],
                                    'Components' => [
                                        'table'     => 'components',
                                        'type'		=> 'INNER',
                                        'conditions'=> [ 'Components.id=ComponentGroups.component_id',  'Components.is_active' => 1]
                                    ],
                                ])
                                ->group(['ComponentGroups.component_id', 'ComponentGroups.group_id'])
                                ->where(['Products.parent_id' => $parentId])->toArray();

    #mapped eventual components to the selected product number
        $mappedComponentProducts = $this->Products->find()
                                        ->select([
                                            'component_product_id' => 'ComponentProducts.id'
                                        ])
                                        ->join([
                                            'ComponentProducts' => [
                                                'table'     => 'component_products',
                                                'type'		=> 'INNER',
                                                'conditions'=> [ 'ComponentProducts.product_id=Products.id' ]
                                            ],
                                            'AnnualComponentTrackings' => [
                                                'table'     => 'annual_component_trackings',
                                                'type'		=> 'INNER',
                                                'conditions'=> [ 'AnnualComponentTrackings.component_product_id=ComponentProducts.id' ]
                                            ],
                                        ])
                                        ->where([
                                            'Products.parent_id'                                => $parentId,
                                            'Products.is_active'                                => 1,
                                            'AnnualComponentTrackings.annual_quarter_plan_id'   => $annualQuarterPlanId,
                                            'AnnualComponentTrackings.product_number'           => $productNumber,
                                            'AnnualComponentTrackings.is_eventual_mapped'       => 1,
                                        ])->hydrate(false)->extract('component_product_id')->toArray();

    #array flip
        $mappedComponentProducts = array_flip($mappedComponentProducts);

    #set data
        $this->set(compact('annualProductTrackings', 'groups', 'componentProducts', 'mappedComponentProducts', 'masterProduct', 'productNumber'));
    }

    /**
     * map eventual components
     *
     * @return \Cake\Http\Response|void
     */
    public function mapEventualComponents()
    {
    #check method
        if($this->request->is(['post', 'put']))
		{
        #get
            $annual_quarter_plan_id = $this->request->data('annual_quarter_plan_id');
            $product_number         = $this->request->data('product_number');
            $component_products     = $this->request->data('component_products');

        #check not empty
            if(!empty($component_products)){
            #update product as eventual mapped
                $componentProducts      = $this->ComponentProducts->find()
                                            ->select([
                                                'id'            => 'id',
                                                'product_id'    => 'product_id',
                                            ])
                                            ->where([
                                                'id IN' => $component_products
                                            ])
                                            ->hydrate(false);

            #extract id from selected data
                $components = $componentProducts->extract('id')->toArray(); 
                $products   = $componentProducts->extract('product_id')->toArray();
            }

        #remove mapped eventual before edit
            $annualProductTracking = $this->AnnualProductTrackings
                                        ->updateAll(
                                            ["is_eventual_mapped" => 0],
                                            ['annual_quarter_plan_id' => $annual_quarter_plan_id, 'product_number' => $product_number, 'is_eventual_product' => 1]
                                        );
        #check not empty
            if(!empty($component_products)){
            #update all annual product tracking as eventual
                $annualProductTracking = $this->AnnualProductTrackings
                                            ->updateAll(
                                                ["is_eventual_mapped" => 1],
                                                ['annual_quarter_plan_id' => $annual_quarter_plan_id, 'product_number' => $product_number]
                                            );
            }

        #remove mapped eventual before edit
            $annualProductTracking = $this->AnnualComponentTrackings
                                        ->updateAll(
                                            ["is_eventual_mapped" => 0],
                                            ['annual_quarter_plan_id' => $annual_quarter_plan_id, 'product_number' => $product_number, 'is_eventual_product' => 1]
                                        );
        #check not empty
            if(!empty($component_products)){
            #update all annual component tracking as eventual
                $annualProductTracking = $this->AnnualComponentTrackings
                                            ->updateAll(
                                                ["is_eventual_mapped" => 1],
                                                ["component_product_id IN " => $components, 'annual_quarter_plan_id' => $annual_quarter_plan_id, 'product_number' => $product_number]
                                            );
            }
        #return
            return $this->redirect(
                ['action' => 'eventualProducts', $annual_quarter_plan_id]
            );
        }
    }

    /**
     * freez products
     *
     * @return \Cake\Http\Response|void
     */
    public function freezeEventualProducts($id = null)
    {
    #check method
        if($this->request->is(['post', 'put']))
		{
        #get
            $annual_quarter_plan_id = $this->request->data('annual_quarter_plan_id');
            $eventualProducts       = $this->request->data('eventual');            
        #loop data
            foreach($eventualProducts AS $key=>$rows)
            {
                if(isset($rows['is_eventual_mapped']))
                {
                #unset eventual product in products tracking
                    $annualProductTrackings = $this->AnnualProductTrackings
                                                ->updateAll(
                                                    ["is_eventual_mapped" => 0],
                                                    [   
                                                        "annual_quarter_plan_id"=> $annual_quarter_plan_id, 
                                                        "product_id"            => $rows['product_id'] 
                                                    ]
                                                );
                #set eventual product in products tracking
                    $annualProductTrackings = $this->AnnualProductTrackings
                                                ->updateAll(
                                                    ["is_eventual_mapped" => 1],
                                                    [ 
                                                        "annual_quarter_plan_id"=> $annual_quarter_plan_id, 
                                                        "product_id"            => $rows['product_id'], 
                                                        'product_number IN'     => $rows['is_eventual_mapped'] 
                                                    ]
                                                );
                #unset eventual product in component products
                    $annualComponentTrackings = $this->AnnualComponentTrackings
                                                    ->updateAll(
                                                        ["is_eventual_mapped" => 0],
                                                        [ 
                                                            "annual_quarter_plan_id" => $annual_quarter_plan_id, 
                                                            "component_product_id IN (SELECT `id` FROM `component_products` WHERE component_products.`product_id`=".$rows['product_id'].")", 
                                                        ]
                                                    );
                #set eventual selected product in component products
                    $annualComponentTrackings = $this->AnnualComponentTrackings
                                                    ->updateAll(
                                                        ["is_eventual_mapped" => 1],
                                                        [ 
                                                            "annual_quarter_plan_id"=> $annual_quarter_plan_id,
                                                            "product_number IN"     => $rows['is_eventual_mapped'],                            
                                                            "component_product_id IN (SELECT `id` FROM `component_products` WHERE component_products.`product_id`=".$rows['product_id'].")", 
                                                        ]
                                                    );
                }
            }
        }
    #return
        return $this->redirect(
            ['action' => 'eventualProducts', $id]
        );
    }
}
