<?php
namespace App\Controller;

use App\Controller\AppController;

/**
 * ComponentAssemblies Controller
 *
 * @property \App\Model\Table\ComponentAssembliesTable $ComponentAssemblies
 *
 * @method \App\Model\Entity\ComponentAssembly[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class ComponentAssembliesController extends AppController
{

    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
        ]);
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
        $this->paginate = [
            'contain' => ['Components', 'SubAssemblies']
        ];
        $componentAssemblies = $this->paginate($this->ComponentAssemblies);

        $this->set(compact('componentAssemblies'));
    }

    /**
     * View method
     *
     * @param string|null $id Component Assembly id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Components', 'SubAssemblies'
			]);
		});

	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $components = $this->ComponentAssemblies->Components->find('list', ['limit' => 200]);
        $subAssemblies = $this->ComponentAssemblies->SubAssemblies->find('list', ['limit' => 200]);
	#set result values
		$this->set(compact('components', 'subAssemblies'));
    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Component Assembly id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $components = $this->ComponentAssemblies->Components->find('list', ['limit' => 200]);
        $subAssemblies = $this->ComponentAssemblies->SubAssemblies->find('list', ['limit' => 200]);
	#set result values
		$this->set(compact('components', 'subAssemblies'));
    #return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Component Assembly id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
    #return
		return $this->Crud->execute();
    }
}
