<?php
namespace App\Controller;

use App\Controller\AppController;

/**
 * ComponentProducts Controller
 *
 * @property \App\Model\Table\ComponentProductsTable $ComponentProducts
 *
 * @method \App\Model\Entity\ComponentProduct[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class ComponentProductsController extends AppController
{

    /**
     * initialize method
     */
    public function initialize()
    {
	//initialization
		parent::initialize();

	//CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
        ]);
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
        $this->paginate = [
            'contain' => ['Components', 'Products']
        ];
        $componentProducts = $this->paginate($this->ComponentProducts);

        $this->set(compact('componentProducts'));
    }

    /**
     * View method
     *
     * @param string|null $id Component Product id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	//before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Components', 'Products', 'AnnualPlannedCapacities'
			]);
		});

	//return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
    //return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Component Product id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
    //return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Component Product id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
         $this->Crud->on('afterDelete', function(\Cake\Event\Event $event) {
        if (!$event->getSubject()->success) {
            $this->log("Delete failed for entity " . $event->getSubject()->id);
            }
            return  $this->redirect($this->referer());
        });
	//return
		return $this->Crud->execute();
    }
}