<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\I18n\Time;

/**
 * ComponentRawMaterials Controller
 *
 * @property \App\Model\Table\ComponentRawMaterialsTable $ComponentRawMaterials
 *
 * @method \App\Model\Entity\ComponentRawMaterial[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class ComponentRawMaterialsController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);

    #load model
        $this->loadModel('Components');
        $this->loadModel('RawMaterials');
        $this->loadModel('AnnualPlans');
        $this->loadModel('Vendors');
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #active status tag
        $activeStatus  = Configure::read('activeStatus');
    #find selection option
        $this->set(compact('activeStatus'));

    #before find
        $this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) {
            $event->subject()->query->contain([
                'Components', 'RawMaterials'
            ]);
        });

    #return data
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id Component Raw Material id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Components', 'RawMaterials'
			]);
		});

	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
    #find query
      $components           = $this->Components->find('list',['keyField' => 'id', 'valueField' => 'component_name'])->where(['is_active' => 1]);
      $rawMaterials         = $this->RawMaterials->find('list',['keyField' => 'id', 'valueField' => 'material_name'])->where(['is_active' => 1]);
      $unitOfMeasurement    = ['Number' => 'Number', 'MM' => 'MM'];
      $rawMaterialShape     = ['Round' => 'Round', 'Pipe' => 'Pipe', 'Flate' => 'Flate', 'Profile' =>'Profile'];
      $buyFrequency         = [1 => 'Monthly', 1 => 'Bi-monthly', 3 => 'Quarterly', 4 => 'Half yearly', 5 => 'Yearly'];
      $vendors              = $this->Vendors->find('list', ['keyField' => 'id', 'valueField' => 'vendor_name'])->where(['is_active' => 1]);
    #set result
      $this->set(compact('components', 'rawMaterials', 'rawMaterialShape', 'unitOfMeasurement', 'buyFrequency', 'vendors'));

    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Component Raw Material id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
    #find query
      $components           = $this->Components->find('list',['keyField' => 'id', 'valueField' => 'component_name'])->where(['is_active' => 1]);
      $rawMaterials         = $this->RawMaterials->find('list',['keyField' => 'id', 'valueField' => 'material_name'])->where(['is_active' => 1]);
      $unitOfMeasurement    = ['Number' => 'Number', 'MM' => 'MM'];
      $rawMaterialShape     = ['Round' => 'Round', 'Pipe' => 'Pipe', 'Flate' => 'Flate', 'Profile' =>'Profile'];
      $buyFrequency         = [1 => 'Monthly', 1 => 'Bi-monthly', 3 => 'Quarterly', 4 => 'Half yearly', 5 => 'Yearly'];
      $vendors              = $this->Vendors->find('list', ['keyField' => 'id', 'valueField' => 'vendor_name'])->where(['is_active' => 1]);
    #set result
      $this->set(compact('components', 'rawMaterials', 'rawMaterialShape', 'unitOfMeasurement', 'buyFrequency', 'vendors'));

    #return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Component Raw Material id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
	#return
		return $this->Crud->execute();
    }
}
