<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\I18n\Time;

/**
 * Components Controller
 *
 * @property \App\Model\Table\ComponentsTable $Components
 *
 * @method \App\Model\Entity\Component[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class ComponentsController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();
        $this->loadComponent('RequestHandler');

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'edit' => 'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);

    #load model
        $this->loadModel('Processes');
        $this->loadModel('Products');
        $this->loadModel('SubAssemblies');
        $this->loadModel('Groups');
        $this->loadModel('Vendors');
        $this->loadModel('Machines');
        $this->loadModel('MachineProcesses');
        $this->loadModel('ComponentProducts');
        $this->loadModel('ComponentProcesses');
        $this->loadModel('ComponentAssemblies');
        $this->loadModel('AnnualProductTrackings');
        $this->loadModel('AnnualAssemblyTrackings');
        $this->loadModel('AnnualComponentTrackings');
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #process tag
        $componentTag   = Configure::read('tag');
    #active status tag
        $activeStatus   = Configure::read('activeStatus');
    #find selection option
        $products       = $this->Products->find('list')->where(['is_active' => 1]);
        $subAssemblies  = $this->SubAssemblies->find('list', ['keyField' => 'id', 'valueField' => 'sub_assembly_name'])->where(['is_active' => 1]);
        $groups         = $this->Groups->find('list')->where(['is_active' => 1]);
        $this->set(compact('activeStatus', 'products', 'subAssemblies', 'groups', 'componentTag'));

    #before find
		$this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Products', 'Processes', 'ComponentProducts', 'ComponentAssemblies', 'ComponentGroups' => ['Groups']
			])->order([
                'component_name' => 'ASC'
            ]);
		});

    #return data
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id Component id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
        $tags  = Configure::read('tag');
    #set result for view
        $this->set(compact('tags'));

	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Products', 'Processes', 'ComponentProcesses' => ['Machines', 'Processes'],
                'ComponentAssemblies' => function($query){
                    return $query->select([
                        'component_id'      => 'ComponentAssemblies.component_id',
                        'sub_assembly_name' => 'SubAssemblies.sub_assembly_name',
                    ])->contain(['SubAssemblies'])->group(['SubAssemblies.id']);
                }, 
                'ComponentGroups' => function($query){
                    return $query->select([
                        'component_id'  => 'ComponentGroups.component_id',
                        'group_name'    => 'Groups.group_name',
                    ])->contain(['Groups'])->group(['Groups.id']);
                } 
			]);
		});

	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $in_out_tags      = Configure::read('tag');
     #get select option data
        $process          = $this->Processes->find('list', ['keyField' => 'id', 'valueField' => 'process_name'])->where(['is_active' => 1]);
        $products         = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->where(['is_active' => 1]);
        $sub_assemblies   = $this->SubAssemblies->find('list', ['keyField' => 'id', 'valueField' => 'sub_assembly_name'])->where(['is_active' => 1]);
        $groups           = $this->Groups->find('list', ['keyField' => 'id', 'valueField' => 'group_name'])->where(['is_active' => 1]);
        $vendors          = $this->Vendors->find('list', ['keyField' => 'id', 'valueField' => 'vendor_name'])->where(['is_active' => 1]);
        $buyer_frequency  = [1 => 'Monthly', 2 => 'Bimonthly', 3 => 'Quarterly', 4 => 'Half Quarterly', 5 => 'Yearly'];
    #set result for view
        $this->set(compact('process','products','in_out_tags','raw_material','sub_assemblies', 'groups', 'buyer_frequency','vendors'));

    #after save
		$this->Crud->on('afterSave', function(\Cake\Event\Event $event) {
        #get data
            $is_need_tracking_sheet = $this->request->data('is_need_tracking_sheet');
        #check conditions
            if($is_need_tracking_sheet==1){
            #get component id
                $componentId = $event->subject()->entity->id;
            #get product id
                $componentProducts = $this->ComponentProducts->find()->select(['product_id'])->where(['ComponentProducts.component_id' => $componentId, 'ComponentProducts.is_active' => 1])->hydrate(false)->extract('product_id')->toArray();
            #get sub assembly id
                $componentAssemblies = $this->ComponentAssemblies->find()->select(['sub_assembly_id'])->where(['ComponentAssemblies.component_id' => $componentId, 'ComponentAssemblies.is_active' => 1])->hydrate(false)->extract('sub_assembly_id')->toArray();

            #find assembly tracking by sub assembly and product id
                $annualAssemblyTrackings = $this->AnnualAssemblyTrackings
                                                ->find()
                                                ->join([
                                                    'AnnualProductTrackings' => [
                                                        'table'     => 'annual_product_trackings',
                                                        'type'		=> 'INNER',
                                                        'conditions'=> ['AnnualProductTrackings.id=AnnualAssemblyTrackings.annual_product_tracking_id']
                                                    ],
                                                ])
                                                ->where([
                                                    'AnnualAssemblyTrackings.product_id IN'         => $componentProducts,
                                                    'AnnualAssemblyTrackings.sub_assembly_id IN'    => $componentAssemblies,
                                                    'AnnualProductTrackings.status !='              => 4,
                                                ]);
            #prepare temprory array
                $tempAnnualAssemblyTrackings = $annualAssemblyTrackings->toArray();
            #checks empty
                if(!empty($tempAnnualAssemblyTrackings)){
                #prepare data
                    $annualAssemblyTrackingsData = $annualAssemblyTrackings
                                                        ->select([
                                                            'id'                => 'AnnualAssemblyTrackings.id',
                                                            'status'            => '(IF(AnnualAssemblyTrackings.total_component=AnnualAssemblyTrackings.completed_component, 0, AnnualAssemblyTrackings.status))',
                                                            'total_component'   => '(SELECT COUNT(*) FROM `component_assemblies` AS CA INNER JOIN `component_products` AS CP ON CP.`component_id`=CA.`component_id` INNER JOIN `components` AS C ON C.`id`=CP.`component_id` AND C.`is_active`=1 WHERE CA.`sub_assembly_id`=AnnualAssemblyTrackings.`sub_assembly_id` AND CP.`product_id`=AnnualAssemblyTrackings.`product_id`  AND ( (CP.`start_product_no`=0 AND CP.`end_product_no`=0) OR (CP.`start_product_no`<=AnnualProductTrackings.product_number AND CP.`end_product_no`>=AnnualProductTrackings.product_number) ) GROUP BY CA.`sub_assembly_id`)',
                                                        ])
                                                        ->hydrate(false)
                                                        ->toArray();
                #patch entity update data
                    $entity = $this->AnnualAssemblyTrackings->patchEntities($annualAssemblyTrackings, $annualAssemblyTrackingsData);
                #checkes data updated or not
                    if($this->AnnualAssemblyTrackings->saveMany($entity)){
                    #prepare select data product component tracking
                        $productsComponentTracking = $this->AnnualAssemblyTrackings->find()
                                                        ->select([
                                                           'annual_quarter_plan_id'         => 'AnnualAssemblyTrackings.annual_quarter_plan_id',
                                                           'annual_assembly_tracking_id'    => 'AnnualAssemblyTrackings.id',
                                                           'component_product_id'           => 'ComponentProducts.id',
                                                           'product_number'                 => 'AnnualAssemblyTrackings.product_number',
                                                           'total_components'               => 'ComponentProducts.number_of_component',
                                                           'total_completed_components'     => 0,
                                                           'total_rejected_components'      => 0,
                                                           'status'                         => 0,
                                                           'is_eventual_product'            => 'Products.is_eventual_product',
                                                           'is_eventual_mapped'             => 0,
                                                        ])
                                                        ->join([
                                                            'AnnualProductTrackings' => [
                                                                'table'     => 'annual_product_trackings',
                                                                'type'		=> 'INNER',
                                                                'conditions'=> [ 'AnnualProductTrackings.id=AnnualAssemblyTrackings.annual_product_tracking_id' ]
                                                            ],
                                                            'ComponentProducts' => [
                                                                'table'     => 'component_products',
                                                                'type'		=> 'INNER',
                                                                'conditions'=> [ 'ComponentProducts.product_id=AnnualAssemblyTrackings.product_id' ]
                                                            ],
                                                            'ComponentAssemblies' => [
                                                                'table'     => 'component_assemblies',
                                                                'type'		=> 'INNER',
                                                                'conditions'=> [ 'ComponentAssemblies.sub_assembly_id=AnnualAssemblyTrackings.sub_assembly_id', 'ComponentAssemblies.component_id=ComponentProducts.component_id' ]
                                                            ],
                                                            'Products' => [
                                                                'table'     => 'products',
                                                                'type'		=> 'INNER',
                                                                'conditions'=> [ 'Products.id=ComponentProducts.product_id' ]
                                                            ],
                                                        ])
                                                        ->where([
                                                            'AnnualProductTrackings.status !='              => 4,
                                                            'AnnualAssemblyTrackings.product_id IN'         => $componentProducts, 
                                                            'AnnualAssemblyTrackings.sub_assembly_id IN'    => $componentAssemblies, 
                                                            'AnnualAssemblyTrackings.is_active'             => 1,
                                                            'ComponentProducts.component_id'                => $componentId,
                                                         ])
                                                        ->hydrate(false)
                                                        ->toArray();
                    #check component products empty
                        if(!empty($productsComponentTracking)){
                        #prepare entity for insert component tracking data insert
                            $pEntity = $this->AnnualComponentTrackings->newEntities($productsComponentTracking);
                        #save data
                            if($eResult = $this->AnnualComponentTrackings->saveMany($pEntity)){
                            #get total components list
                                $annualAssemblyTrackings = $this->AnnualAssemblyTrackings->find()
                                                                ->join([
                                                                    'AnnualProductTrackings' => [
                                                                        'table'     => 'annual_product_trackings',
                                                                        'type'		=> 'INNER',
                                                                        'conditions'=> [ 'AnnualProductTrackings.id=AnnualAssemblyTrackings.annual_product_tracking_id' ]
                                                                    ],
                                                                ])
                                                                ->where([
                                                                    'AnnualProductTrackings.status !='             => 4,
                                                                    'AnnualAssemblyTrackings.sub_assembly_id IN'   => $componentAssemblies, 
                                                                    'AnnualAssemblyTrackings.product_id IN'        => $componentProducts,
                                                                ]);
                            }
                        }
                    }
                }
                else{
                #find data
                    $annualProductTrackings = $this->AnnualProductTrackings
                                                ->find()
                                                ->where([
                                                    'AnnualProductTrackings.product_id IN'  => $componentProducts, 
                                                    'AnnualProductTrackings.status !='      => 4
                                                ]);
                    $tempAnnualProductTrackings = $annualProductTrackings;
                #checks not empty
                    if(!empty($tempAnnualProductTrackings)){
                    #get values for update
                        $annualProductTrackingsData = $annualProductTrackings
                                                        ->select([
                                                            'id'                => 'AnnualProductTrackings.id',
                                                            'total_assembly'    => '(AnnualProductTrackings.total_assembly+1)',
                                                            'status'            => 'IF(AnnualProductTrackings.status=1, 0, AnnualProductTrackings.status)',
                                                        ])->hydrate(false)->toArray();
                    #patch entity update data
                        $entity = $this->AnnualProductTrackings->patchEntities($annualProductTrackings, $annualProductTrackingsData);
                    #checkes data updated or not
                        if($this->AnnualProductTrackings->saveMany($entity)){
                        #product assembly
                            $assemblyProduct = $this->AnnualProductTrackings->find()
                                                    ->select([
                                                        'annual_quarter_plan_id'        => 'AnnualProductTrackings.annual_quarter_plan_id',
                                                        'sub_assembly_id'               => 'SubAssemblyProducts.sub_assembly_id',
                                                        'annual_product_tracking_id'    => 'AnnualProductTrackings.id',
                                                        'product_id'                    => 'AnnualProductTrackings.product_id',
                                                        'product_number'                => 'AnnualProductTrackings.product_number',
                                                        'total_component'               => '(SELECT COUNT(*) FROM `component_assemblies` AS CA INNER JOIN `component_products` AS CP ON CP.`component_id`=CA.`component_id` WHERE CA.`sub_assembly_id`=SubAssemblyProducts.`sub_assembly_id` AND CP.`product_id`=SubAssemblyProducts.`product_id` GROUP BY CA.`sub_assembly_id`)',
                                                        'completed_component'           => 0,
                                                        'status'                        => 0,
                                                    ])
                                                    ->join([
                                                        'SubAssemblyProducts' => [
                                                            'table'     => 'sub_assembly_products',
                                                            'type'		=> 'INNER',
                                                            'conditions'=> [
                                                                'SubAssemblyProducts.product_id=AnnualProductTrackings.product_id', 
                                                                'SubAssemblyProducts.is_active'             => 1,
                                                                'SubAssemblyProducts.sub_assembly_id IN'    => $componentAssemblies,
                                                            ]
                                                        ],
                                                    ])
                                                    ->where([
                                                        'AnnualProductTrackings.product_id IN'      => $componentProducts, 
                                                        'AnnualProductTrackings.is_active'          => 1,
                                                        'AnnualProductTrackings.status !='          => 4
                                                    ])
                                                    ->order([
                                                        'SubAssemblyProducts.sub_assembly_id'   => 'ASC',
                                                        'AnnualProductTrackings.product_number' => 'ASC'
                                                    ])
                                                    ->hydrate(false)
                                                    ->toArray();
                        #patch entity
                            $entity  = $this->AnnualAssemblyTrackings->newEntities($assemblyProduct);
                        #check query executed
                            if($this->AnnualAssemblyTrackings->saveMany($entity, array('callbacks' => false)))
                            {
                            #prepare select data product component tracking
                                $productsComponentTracking = $this->AnnualAssemblyTrackings->find()
                                                                ->select([
                                                                   'annual_quarter_plan_id'         => 'AnnualAssemblyTrackings.annual_quarter_plan_id',
                                                                   'annual_assembly_tracking_id'    => 'AnnualAssemblyTrackings.id',
                                                                   'component_product_id'           => 'ComponentProducts.id',
                                                                   'product_number'                 => 'AnnualAssemblyTrackings.product_number',
                                                                   'total_components'               => 'ComponentProducts.number_of_component',
                                                                   'total_completed_components'     => 0,
                                                                   'total_rejected_components'      => 0,
                                                                   'status'                         => 0,
                                                                   'is_eventual_product'            => 'Products.is_eventual_product',
                                                                   'is_eventual_mapped'             => 0,
                                                                ])
                                                                ->join([
                                                                    'AnnualProductTrackings' => [
                                                                        'table'     => 'annual_product_trackings',
                                                                        'type'		=> 'INNER',
                                                                        'conditions'=> [ 'AnnualProductTrackings.id=AnnualAssemblyTrackings.annual_product_tracking_id' ]
                                                                    ],
                                                                    'ComponentProducts' => [
                                                                        'table'     => 'component_products',
                                                                        'type'		=> 'INNER',
                                                                        'conditions'=> [ 'ComponentProducts.product_id=AnnualAssemblyTrackings.product_id' ]
                                                                    ],
                                                                    'ComponentAssemblies' => [
                                                                        'table'     => 'component_assemblies',
                                                                        'type'		=> 'INNER',
                                                                        'conditions'=> [ 'ComponentAssemblies.sub_assembly_id=AnnualAssemblyTrackings.sub_assembly_id', 'ComponentAssemblies.component_id=ComponentProducts.component_id' ]
                                                                    ],
                                                                    'Products' => [
                                                                        'table'     => 'products',
                                                                        'type'		=> 'INNER',
                                                                        'conditions'=> [ 'Products.id=ComponentProducts.product_id' ]
                                                                    ],
                                                                ])
                                                                ->where([
                                                                    'AnnualProductTrackings.status !='              => 4,
                                                                    'AnnualAssemblyTrackings.product_id IN'         => $componentProducts, 
                                                                    'AnnualAssemblyTrackings.sub_assembly_id IN'    => $componentAssemblies, 
                                                                    'AnnualAssemblyTrackings.is_active'             => 1,
                                                                    'ComponentProducts.component_id'                => $componentId,
                                                                 ])
                                                                ->hydrate(false)
                                                                ->toArray();
                             #check empty
                                if(!empty($productsComponentTracking)){
                                #prepare entity
                                    $pEntity = $this->AnnualComponentTrackings->newEntities($productsComponentTracking);
                                #save data
                                    if($eResult = $this->AnnualComponentTrackings->saveMany($pEntity)){

                                    }
                                }
                            }
                        }
                    }
                }
            }
        });

    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Component id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $in_out_tags      = Configure::read('tag');
    #get select option data
        $process          = $this->Processes->find('list', ['keyField' => 'id', 'valueField' => 'process_name'])->where(['is_active' => 1]);
        $products         = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->where(['is_active' => 1]);
        $sub_assemblies   = $this->SubAssemblies->find('list', ['keyField' => 'id', 'valueField' => 'sub_assembly_name'])->where(['is_active' => 1]);
        $groups           = $this->Groups->find('list', ['keyField' => 'id', 'valueField' => 'group_name'])->where(['is_active' => 1]);
        $vendors          = $this->Vendors->find('list', ['keyField' => 'id', 'valueField' => 'vendor_name'])->where(['is_active' => 1]);
        $buyer_frequency  = [1 => 'Monthly', 2 => 'Bimonthly', 3 => 'Quarterly', 4 => 'Half Quarterly', 5 => 'Yearly'];
    #set result for view
        $this->set(compact('process', 'products', 'in_out_tags', 'raw_material', 'sub_assemblies', 'groups', 'buyer_frequency', 'vendors', 'machineProcesses'));
    #before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Processes' => ['MachineProcesses' => ['Machines']] , 'Products', 'ComponentGroups', 'ComponentAssemblies', 'SubAssemblies'
			]);
		});

    #return
        return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Component id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function getProcessMachines()
    {
    #get post data
        $process_id = $this->request->data('process_id');
    #find query
        $machineProcesses = $this->MachineProcesses->find()->select(['id' => 'Machines.id', 'machine_name' => 'Machines.machine_name'])->contain(['Machines'])->where(['process_id' => $process_id, 'MachineProcesses.is_active' => 1])->toArray();
    #prepare response data
        $returnData = array( 'status' => true, 'data' => $machineProcesses );
    #return data
        $this->set([
            'data'          => $returnData,
            '_serialize'    => 'data',
        ]);
        $this->RequestHandler->renderAs($this, 'json');
    }

    /**
     * Delete method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
    #return
		return $this->Crud->execute();
    }

    /**
     * add component to tracking sheet method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function componentToTrackingSheet($id = null)
    {
    #checks id
        if($id!=null){
            $componentId = $id;
        #get product id
            $componentProducts = $this->ComponentProducts->find()->select(['product_id'])->where(['ComponentProducts.component_id' => $componentId, 'ComponentProducts.is_active' => 1])->hydrate(false)->extract('product_id')->toArray();
        #get sub assembly id
            $componentAssemblies = $this->ComponentAssemblies->find()->select(['sub_assembly_id'])->where(['ComponentAssemblies.component_id' => $componentId, 'ComponentAssemblies.is_active' => 1])->hydrate(false)->extract('sub_assembly_id')->toArray();

        #find assembly tracking by sub assembly and product id
            $annualAssemblyTrackings = $this->AnnualAssemblyTrackings
                                            ->find()
                                            ->join([
                                                'AnnualProductTrackings' => [
                                                    'table'     => 'annual_product_trackings',
                                                    'type'		=> 'INNER',
                                                    'conditions'=> ['AnnualProductTrackings.id=AnnualAssemblyTrackings.annual_product_tracking_id']
                                                ],
                                            ])
                                            ->where([
                                                'AnnualAssemblyTrackings.product_id IN'         => $componentProducts,
                                                'AnnualAssemblyTrackings.sub_assembly_id IN'    => $componentAssemblies,
                                                'AnnualProductTrackings.status !='              => 4,
                                            ]);
        #prepare temprory array
            $tempAnnualAssemblyTrackings = $annualAssemblyTrackings->toArray();
        #checks empty
            if(!empty($tempAnnualAssemblyTrackings)){
            #prepare data
                $annualAssemblyTrackingsData = $annualAssemblyTrackings
                                                    ->select([
                                                        'id'                => 'AnnualAssemblyTrackings.id',
                                                        'status'            => '(IF(AnnualAssemblyTrackings.total_component=AnnualAssemblyTrackings.completed_component, 0, AnnualAssemblyTrackings.status))',
                                                        'total_component'   => '(SELECT COUNT(*) FROM `component_assemblies` AS CA INNER JOIN `component_products` AS CP ON CP.`component_id`=CA.`component_id` INNER JOIN `components` AS C ON C.`id`=CP.`component_id` AND C.`is_active`=1 WHERE CA.`sub_assembly_id`=AnnualAssemblyTrackings.`sub_assembly_id` AND CP.`product_id`=AnnualAssemblyTrackings.`product_id`  AND ( (CP.`start_product_no`=0 AND CP.`end_product_no`=0) OR (CP.`start_product_no`<=AnnualProductTrackings.product_number AND CP.`end_product_no`>=AnnualProductTrackings.product_number) ) GROUP BY CA.`sub_assembly_id`)',
                                                    ])
                                                    ->hydrate(false)
                                                    ->toArray();

            #patch entity update data
                $entity = $this->AnnualAssemblyTrackings->patchEntities($annualAssemblyTrackings, $annualAssemblyTrackingsData);
            #checkes data updated or not
                if($this->AnnualAssemblyTrackings->saveMany($entity)){
                #prepare select data product component tracking
                    $productsComponentTracking = $this->AnnualAssemblyTrackings->find()
                                                    ->select([
                                                       'annual_quarter_plan_id'         => 'AnnualAssemblyTrackings.annual_quarter_plan_id',
                                                       'annual_assembly_tracking_id'    => 'AnnualAssemblyTrackings.id',
                                                       'component_product_id'           => 'ComponentProducts.id',
                                                       'product_number'                 => 'AnnualAssemblyTrackings.product_number',
                                                       'total_components'               => 'ComponentProducts.number_of_component',
                                                       'total_completed_components'     => 0,
                                                       'total_rejected_components'      => 0,
                                                       'status'                         => 0,
                                                       'is_eventual_product'            => 'Products.is_eventual_product',
                                                       'is_eventual_mapped'             => 0,
                                                    ])
                                                    ->join([
                                                        'AnnualProductTrackings' => [
                                                            'table'     => 'annual_product_trackings',
                                                            'type'		=> 'INNER',
                                                            'conditions'=> [ 'AnnualProductTrackings.id=AnnualAssemblyTrackings.annual_product_tracking_id' ]
                                                        ],
                                                        'ComponentProducts' => [
                                                            'table'     => 'component_products',
                                                            'type'		=> 'INNER',
                                                            'conditions'=> [ 'ComponentProducts.product_id=AnnualAssemblyTrackings.product_id' ]
                                                        ],
                                                        'ComponentAssemblies' => [
                                                            'table'     => 'component_assemblies',
                                                            'type'		=> 'INNER',
                                                            'conditions'=> [ 'ComponentAssemblies.sub_assembly_id=AnnualAssemblyTrackings.sub_assembly_id', 'ComponentAssemblies.component_id=ComponentProducts.component_id' ]
                                                        ],
                                                        'Products' => [
                                                            'table'     => 'products',
                                                            'type'		=> 'INNER',
                                                            'conditions'=> [ 'Products.id=ComponentProducts.product_id' ]
                                                        ],
                                                    ])
                                                    ->where([
                                                        'AnnualProductTrackings.status !='              => 4,
                                                        'AnnualAssemblyTrackings.product_id IN'         => $componentProducts, 
                                                        'AnnualAssemblyTrackings.sub_assembly_id IN'    => $componentAssemblies, 
                                                        'AnnualAssemblyTrackings.is_active'             => 1,
                                                        'ComponentProducts.component_id'                => $componentId,
                                                     ])
                                                    ->hydrate(false)
                                                    ->toArray();
                #check component products empty
                    if(!empty($productsComponentTracking)){
                    #prepare entity for insert component tracking data insert
                        $pEntity = $this->AnnualComponentTrackings->newEntities($productsComponentTracking);
                    #save data
                        if($eResult = $this->AnnualComponentTrackings->saveMany($pEntity)){
                        #get total components list
                            $annualAssemblyTrackings = $this->AnnualAssemblyTrackings->find()
                                                            ->join([
                                                                'AnnualProductTrackings' => [
                                                                    'table'     => 'annual_product_trackings',
                                                                    'type'		=> 'INNER',
                                                                    'conditions'=> [ 'AnnualProductTrackings.id=AnnualAssemblyTrackings.annual_product_tracking_id' ]
                                                                ],
                                                            ])
                                                            ->where([
                                                                'AnnualProductTrackings.status !='             => 4,
                                                                'AnnualAssemblyTrackings.sub_assembly_id IN'   => $componentAssemblies, 
                                                                'AnnualAssemblyTrackings.product_id IN'        => $componentProducts,
                                                            ]);
                        #redirect page
                            return $this->redirect(['action' => 'index']);
                        }
                    }
                }
            }
            else{
            #find data
                $annualProductTrackings = $this->AnnualProductTrackings->find()
                                            ->where([
                                                'AnnualProductTrackings.product_id IN'  => $componentProducts, 
                                                'AnnualProductTrackings.status !='      => 4
                                            ]);
                $tempAnnualProductTrackings = $annualProductTrackings;
            #checks not empty
                if(!empty($tempAnnualProductTrackings)){
                #get values for update
                    $annualProductTrackingsData = $annualProductTrackings
                                                    ->select([
                                                        'id'                => 'AnnualProductTrackings.id',
                                                        'total_assembly'    => '(AnnualProductTrackings.total_assembly+1)',
                                                        'status'            => 'IF(AnnualProductTrackings.status=1, 0, AnnualProductTrackings.status)',
                                                    ])
                                                    ->hydrate(false)
                                                    ->toArray();
                #patch entity update data
                    $entity = $this->AnnualProductTrackings->patchEntities($annualProductTrackings, $annualProductTrackingsData);
                #checkes data updated or not
                    if($this->AnnualProductTrackings->saveMany($entity)){
                    #product assembly
                        $assemblyProduct = $this->AnnualProductTrackings->find()
                                                ->select([
                                                    'annual_quarter_plan_id'        => 'AnnualProductTrackings.annual_quarter_plan_id',
                                                    'sub_assembly_id'               => 'SubAssemblyProducts.sub_assembly_id',
                                                    'annual_product_tracking_id'    => 'AnnualProductTrackings.id',
                                                    'product_id'                    => 'AnnualProductTrackings.product_id',
                                                    'product_number'                => 'AnnualProductTrackings.product_number',
                                                    'total_component'               => '(SELECT COUNT(*) FROM `component_assemblies` AS CA INNER JOIN `component_products` AS CP ON CP.`component_id`=CA.`component_id` WHERE CA.`sub_assembly_id`=SubAssemblyProducts.`sub_assembly_id` AND CP.`product_id`=SubAssemblyProducts.`product_id` GROUP BY CA.`sub_assembly_id`)',
                                                    'completed_component'           => 0,
                                                    'status'                        => 0,
                                                ])
                                                ->join([
                                                    'SubAssemblyProducts' => [
                                                        'table'     => 'sub_assembly_products',
                                                        'type'		=> 'INNER',
                                                        'conditions'=> [
                                                            'SubAssemblyProducts.product_id=AnnualProductTrackings.product_id', 
                                                            'SubAssemblyProducts.is_active'             => 1,
                                                            'SubAssemblyProducts.sub_assembly_id IN'    => $componentAssemblies,
                                                        ]
                                                    ],
                                                ])
                                                ->where([
                                                    'AnnualProductTrackings.product_id IN'      => $componentProducts, 
                                                    'AnnualProductTrackings.is_active'          => 1,
                                                    'AnnualProductTrackings.status !='          => 4
                                                ])
                                                ->order([
                                                    'SubAssemblyProducts.sub_assembly_id'   => 'ASC',
                                                    'AnnualProductTrackings.product_number' => 'ASC'
                                                ])
                                                ->hydrate(false)
                                                ->toArray();
                    #patch entity
                        $entity  = $this->AnnualAssemblyTrackings->newEntities($assemblyProduct);
                    #check query executed
                        if($this->AnnualAssemblyTrackings->saveMany($entity, array('callbacks' => false)))
                        {
                        #prepare select data product component tracking
                            $productsComponentTracking = $this->AnnualAssemblyTrackings->find()
                                                            ->select([
                                                               'annual_quarter_plan_id'         => 'AnnualAssemblyTrackings.annual_quarter_plan_id',
                                                               'annual_assembly_tracking_id'    => 'AnnualAssemblyTrackings.id',
                                                               'component_product_id'           => 'ComponentProducts.id',
                                                               'product_number'                 => 'AnnualAssemblyTrackings.product_number',
                                                               'total_components'               => 'ComponentProducts.number_of_component',
                                                               'total_completed_components'     => 0,
                                                               'total_rejected_components'      => 0,
                                                               'status'                         => 0,
                                                               'is_eventual_product'            => 'Products.is_eventual_product',
                                                               'is_eventual_mapped'             => 0,
                                                            ])
                                                            ->join([
                                                                'AnnualProductTrackings' => [
                                                                    'table'     => 'annual_product_trackings',
                                                                    'type'		=> 'INNER',
                                                                    'conditions'=> [ 'AnnualProductTrackings.id=AnnualAssemblyTrackings.annual_product_tracking_id' ]
                                                                ],
                                                                'ComponentProducts' => [
                                                                    'table'     => 'component_products',
                                                                    'type'		=> 'INNER',
                                                                    'conditions'=> [ 'ComponentProducts.product_id=AnnualAssemblyTrackings.product_id' ]
                                                                ],
                                                                'ComponentAssemblies' => [
                                                                    'table'     => 'component_assemblies',
                                                                    'type'		=> 'INNER',
                                                                    'conditions'=> [ 'ComponentAssemblies.sub_assembly_id=AnnualAssemblyTrackings.sub_assembly_id', 'ComponentAssemblies.component_id=ComponentProducts.component_id' ]
                                                                ],
                                                                'Products' => [
                                                                    'table'     => 'products',
                                                                    'type'		=> 'INNER',
                                                                    'conditions'=> [ 'Products.id=ComponentProducts.product_id' ]
                                                                ],
                                                            ])
                                                            ->where([
                                                                'AnnualProductTrackings.status !='              => 4,
                                                                'AnnualAssemblyTrackings.product_id IN'         => $componentProducts, 
                                                                'AnnualAssemblyTrackings.sub_assembly_id IN'    => $componentAssemblies, 
                                                                'AnnualAssemblyTrackings.is_active'             => 1,
                                                                'ComponentProducts.component_id'                => $componentId,
                                                             ])
                                                            ->hydrate(false)
                                                            ->toArray();
                         #check empty
                            if(!empty($productsComponentTracking)){
                            #prepare entity
                                $pEntity = $this->AnnualComponentTrackings->newEntities($productsComponentTracking);
                            #save data
                                if($eResult = $this->AnnualComponentTrackings->saveMany($pEntity)){
                                #return
                                    return $this->redirect(
                                        ['action' => 'index']
                                    );
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
