<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\I18n\Time;

/**
 * GoodsReceivedNotes Controller
 *
 * @property \App\Model\Table\GoodsReceivedNotesTable $GoodsReceivedNotes
 *
 * @method \App\Model\Entity\GoodsReceivedNote[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class GoodsReceivedNotesController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
        ]);

       
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {

        $user_id=$this->Auth->User('id');
        $grn=$this->GoodsReceivedNotes->find('all')->toArray();
        
        $grn=$this->GoodsReceivedNotes->find('all')->contain(['PurchaseOrders'=>['Vendors']])
            ->where(['GoodsReceivedNotes.created_by' => $user_id]);
        $goodsReceivedNotes = $this->paginate($grn);
        $this->set(compact('goodsReceivedNotes'));
    }

    /**
     * View method
     *
     * @param string|null $id Goods Received Note id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
   public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) use($id){
			$event->subject()->query
            ->select(['Users.username','Users.user_full_name'])
            ->contain([
                'PurchaseOrders'=>[
                    'Vendors'
                ],
                 'GoodsReceivedComponents'=>[
                    'PurchaseOrderComponents'=>[
                        'Components',
                        'RawMaterials',
                        'Processes',
                        'StoreComponents'
                    ]
                ]
			])->join([
                    'Users' => [
                        'table'      => 'users',
                        'type'       => 'left',
                        'conditions' => [
                            'Users.id = GoodsReceivedNotes.created_by',
                        ]],
                ])
            ->enableAutoFields(true);
		});
	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $this->loadModel('PurchaseOrders');
    #return


        #check user role for GRN po numbers
        if($this->Auth->User('user_role_id') == 7){
            $purchase_orders=$this->PurchaseOrders->find('list',[
                                'keyField' => 'id',
                                'valueField' => 'purchase_order_no'
                            ])->where(['PurchaseOrders.is_store_order' =>1]);
        }else{
            #find purchase order list based on current logged in user group
          $purchase_orders=$this->PurchaseOrders->find('list',[
                                'keyField' => 'id',
                                'valueField' => 'purchase_order_no'
                            ])            
                            ->join([
                                'JobOrders' => [
                                    'table' => 'job_orders',
                                    'type' => 'LEFT',
                                    'conditions' => 'JobOrders.id = PurchaseOrders.job_order_id'
                                ],
                                'PurchaseIndents' => [
                                    'table' => 'purchase_indents',
                                    'type' => 'LEFT',
                                    'conditions' => 'PurchaseIndents.id = PurchaseOrders.purchase_indent_id'
                                ],
                                'Groups' => [
                                    'table' => 'groups',
                                    'type' => 'INNER',
                                    'conditions' => 'Groups.id = JobOrders.group_id OR Groups.id = PurchaseIndents.group_id'
                                ],
                                'GroupUsers' => [
                                    'table' => 'group_users',
                                    'type' => 'INNER',
                                    'conditions' => 'GroupUsers.group_id = Groups.id'
                                ]

                            ])
                            ->where([
                                'PurchaseOrders.status NOT IN'=>[1,4,5] ,
                                'GroupUsers.user_id'=>$this->Auth->User('id')

                            ]);
        }
        
        
        $this->set(compact(['purchase_orders']));


        $this->Crud->on('beforeSave', function(\Cake\Event\Event $event){
            
            $entity=$event->subject->entity;
            if(!empty($entity->goods_received_components)){

                foreach ($entity->goods_received_components as $key => $goods_received_components) {
                    if(empty($goods_received_components->received_quantity)){
                        unset($entity->goods_received_components[$key]);
                    }
                }
            }

            if(empty($entity->goods_received_components)){
                $this->Flash->error(__('Could not create GRN .Please Fill Atleast One Component'));
                return $this->redirect(['action'=>'add']);
            }else{
                $entity->is_active=1; 
            }
            


        });

        $this->Crud->on('afterSave', function(\Cake\Event\Event $event){
              if($event->getSubject()->success){
                if(!empty($event->getSubject()->entity->goods_received_components)){
                    $po_id = $event->getSubject()->entity->goods_received_components[0]->goods_received_note_id;
                    $grn_no = "GRN".DATE('Y').$po_id; // GRN Number - GRN201901 GRN+YERAR+Runing number
                    $goods_received_note_model = $this->loadModel('GoodsReceivedNotes');
                    $goods_received_note =  $goods_received_note_model->get($po_id);
                    $goods_received_note->grn_no = $grn_no;
                    $goods_received_note->status =1;
                    $goods_received_note_model->save($goods_received_note);
                }
                
            }
        });
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Goods Received Note id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
    #return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Goods Received Note id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
	#return
		return $this->Crud->execute();
    }

    /**
     * get purchase order
     */
    public function getGrnPurchaseOrders()
    {

        $this->loadModel('PurchaseOrders');
        if ($this->request->is('ajax') || $this->request->is('post')) {
            $purchase_order_id=$this->request->data['purchase_order_id'];

            $purchase_orders=$this->PurchaseOrders->find('all')->contain(['Vendors','PurchaseOrderComponents'=>['Components','RawMaterials','Processes','StoreComponents']])->where(['PurchaseOrders.id' =>$purchase_order_id])->first();

            if(!empty($purchase_orders)){
                 $this->set(compact('purchase_orders'));
            }else{
                $this->autoRender = false;
            }

        }

    }
}
