<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\I18n\Time;

/**
 * Groups Controller
 *
 * @property \App\Model\Table\GroupsTable $Groups
 *
 * @method \App\Model\Entity\Group[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class GroupsController extends AppController
{
  public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);
    
    #load model
        $this->loadModel('Users');
    }


    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #active status tag
        $activeStatus  = Configure::read('activeStatus');
    #find selection option
        $this->set(compact('activeStatus'));

    #before find
		$this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Users'
			])
            ->order([
                'group_name' => 'ASC'
            ]);
		});

    #return data
        return $this->Crud->execute();
    }

   /**
     * View method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
    #before find
        $this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
            $event->subject()->query->contain([
                'ComponentGroups', 'Users'
            ]);
        });
    #return
        return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $users = $this->Users->find('list')->where(['user_role_id IN' => 4, 'is_active' => 1])->toArray();
    #set result
        $this->set(compact('users'));
    #return
        return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $users = $this->Users->find('list')->where(['user_role_id IN' => 4, 'is_active' => 1])->toArray();
    #set result
        $this->set(compact('users'));
    #Crud before find
        $this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
            $event->subject()->query->contain([
                'Users' => function($query){
                    return $query->where(['Users.is_active' => 1]);
                },
            ]);
        });

    #return
        return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
    #return
	    return $this->Crud->execute();
    }
}
