<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;

/**
 * IntergroupDispatchNotes Controller
 *
 * @property \App\Model\Table\IntergroupDispatchNotesTable $IntergroupDispatchNotes
 *
 * @method \App\Model\Entity\IntergroupDispatchNote[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class IntergroupDispatchNotesController extends AppController
{

    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
        ]);
    
    #load model
        $this->loadModel('GroupUsers');
        $this->loadModel('Components');
        $this->loadModel('ComponentGroups');
        $this->loadModel('ComponentProducts');
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #get global configuration from config/app.php
        $intergroupStatus = Configure::read('intergroupStatus');
    #get user auth id
        $id = $this->Auth->user('id');
    #find user group
        $groupUsers = $this->GroupUsers->find()->where(['user_id' => $id])->first();
    #get inter group notes
        $intergroupDispatchNotesReceived    = $this->IntergroupDispatchNotes->find()->contain(['Products', 'Groups', 'Components', 'Processes'])->where(['IntergroupDispatchNotes.is_active' => 1, 'group_id' => $groupUsers['group_id']])->toArray();
        $intergroupDispatchNotesForwarded   = $this->IntergroupDispatchNotes->find()->contain(['Products', 'Groups', 'Components', 'Processes'])->where(['IntergroupDispatchNotes.is_active' => 1, 'from_group' => $groupUsers['group_id']])->toArray();
    #set data
        $this->set(compact('intergroupStatus', 'intergroupDispatchNotesReceived', 'intergroupDispatchNotesForwarded'));
    }

    /**
     * View method
     *
     * @param string|null $id Intergroup Dispatch Note id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
    #get global configuration from config/app.php
        $intergroupStatus = Configure::read('intergroupStatus');
    #set result values
		$this->set(compact('intergroupStatus'));

	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Products', 'Groups', 'Components', 'Processes'
			]);
		});

	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
    #get user auth id
        $id = $this->Auth->user('id');
    #find user group
        $groupUsers = $this->GroupUsers->find()->where(['user_id' => $id, 'is_active' => 1])->first();
    #select options
        $products       = $this->IntergroupDispatchNotes->Products->find('list');
        $groups         = $this->IntergroupDispatchNotes->Groups->find('list')->where(['Groups.id !=' => $groupUsers['group_id'], 'Groups.is_active' => 1]);
	#set result values
		$this->set(compact('products', 'groups', 'components', 'processes', 'groupUsers'));
    
    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Intergroup Dispatch Note id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
    #get user auth id
        $id = $this->Auth->user('id');
    #find user group
        $groupUsers = $this->GroupUsers->find()->where(['user_id' => $id, 'is_active' => 1])->first();
    #select options
        $products       = $this->IntergroupDispatchNotes->Products->find('list');
        $groups         = $this->IntergroupDispatchNotes->Groups->find('list')->where(['Groups.id !=' => $groupUsers['group_id'], 'Groups.is_active' => 1]);
	#set result values
		$this->set(compact('products', 'groups', 'components', 'processes', 'groupUsers'));

    #return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Intergroup Dispatch Note id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
    #return
		return $this->Crud->execute();
    }

    /**
     * get component name
     *
    */
    public function getComponentsForInterGroup()
    {
    #get current login user id
        $uid = $this->Auth->user('id');
    #find user group
        $groupUsers = $this->GroupUsers->find()->where(['user_id' => $uid, 'is_active' => 1])->first();
    #get post data
        $product_id = $this->request->data('product_id');
        $group_id   = $groupUsers['group_id'];
    #find data
        $components = $this->ComponentProducts->find()
        ->select([
            'id'                => 'Components.id',
            'component_name'    => 'Components.component_name',
        ])
        ->join([
            'ComponentGroups' => [
                'table'         => 'component_groups',
                'type'          => 'INNER',
                'conditions'    => ['ComponentGroups.component_id=ComponentProducts.component_id', 'ComponentGroups.is_active' => 1]
            ],
            'Components' => [
                'table'         => 'components',
                'type'          => 'INNER',
                'conditions'    => ['Components.id=ComponentGroups.component_id', 'Components.is_active' => 1, 'Components.tag' => 0]
            ]
        ])
        ->where(['ComponentProducts.product_id' => $product_id, 'ComponentGroups.group_id' => $group_id, 'ComponentProducts.is_active' => 1]);
    #prepare response data
        $returnData = array( "status" => null, 'title' => null, "message" => null, "data" => $components );
    #return data
        $this->set([
            'data'          => $returnData,
            '_serialize'    => 'data',
        ]);
        $this->RequestHandler->renderAs($this, 'json');
    }

    /**
     * get component process
     *
    */
    public function getProcessForInterGroup()
    {
    #get current login user id
        $uid = $this->Auth->user('id');
    #find user group
        $groupUsers = $this->GroupUsers->find()->where(['user_id' => $uid, 'is_active' => 1])->first();
    #get post data
        $product_id     = $this->request->data('product_id');
        $group_id       = $groupUsers['group_id'];
        $component_id   = $this->request->data('component_id');
    #find data
        $components = $this->ComponentProducts->find()
        ->select([
            'id'            => 'Processes.id',
            'process_name'  => 'Processes.process_name',
        ])
        ->join([
            'ComponentProcesses' => [
                'table'         => 'component_processes',
                'type'          => 'inner',
                'conditions'    => ['ComponentProcesses.component_id=ComponentProducts.component_id', 'ComponentProcesses.is_active' => 1]
            ],
            'Processes' => [
                'table'         => 'processes',
                'type'          => 'inner',
                'conditions'    => ['Processes.id=ComponentProcesses.process_id', 'Processes.is_active' => 1]
            ]
        ])
        ->where(['ComponentProducts.product_id' => $product_id, 'ComponentProducts.component_id' => $component_id, 'ComponentProducts.is_active' => 1]);
    #prepare response data
        $returnData = array( "status" => null, 'title' => null, "message" => null, "data" => $components );
    #return data
        $this->set([
            'data'          => $returnData,
            '_serialize'    => 'data',
        ]);
        $this->RequestHandler->renderAs($this, 'json');
    }

    /**
     * update intergroup status
     *
    */
    public function updateInterGroupStatus()
    {
    #get post data
        $id      = $this->request->data('id');
        $status  = $this->request->data('status');
    #prepare data
        $this->result           = [];
        $this->result['id']     = $id;
        $this->result['status'] = $status;
    #find data
        $intergroupDispatchNotes = $this->IntergroupDispatchNotes->get($id);
    #patch entity
        $entity = $this->IntergroupDispatchNotes->patchEntity($intergroupDispatchNotes, $this->result);
    #check values updated
        if($this->IntergroupDispatchNotes->save($entity)){
        #save many check result true/false
            $status     = true;
            $title      = "Alert";
            $message    = "Status Updated as Completed";
        }
        else{
            $status     = false;
            $title      = "Alert";
            $message    = "Failed to update status";
        }
    #prepare response data
        $returnData = array( "status" => $status, 'title' => $title, "message" => $message );
    #return data
        $this->set([
            'data'          => $returnData,
            '_serialize'    => 'data',
        ]);
        $this->RequestHandler->renderAs($this, 'json');
    }
}
