<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\I18n\Time;

/**
 * Machines Controller
 *
 * @property \App\Model\Table\MachinesTable $Machines
 *
 * @method \App\Model\Entity\Machine[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class MachinesController extends AppController
{
    /**
     * initialize method
    */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);

    #Datatable configuration
        $findOptions = [
            'conditions' =>
            ['OR' =>
                [
                    ['Ads.id LIKE' => '%mercedes%'],
                    ['Ads.name LIKE' => '%mercedes%'],
                ],
            ],
            'fields'     => [
                'id',
                'name',
            ],
        ];
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #active status tag
        $activeStatus  = Configure::read('activeStatus');
    #find selection option
        $machineTypes   = $this->Machines->MachineTypes->find('list', ['limit' => 200])->where(['is_active' => 1])->order(['MachineTypes.machine_type_name' => 'ASC']);
        $groups         = $this->Machines->Groups->find('list', ['limit' => 200])->where(['is_active' => 1])->order(['Groups.group_name' => 'ASC']);
    #set result
        $this->set(compact('activeStatus', 'machineTypes', 'groups'));

    #before find
		$this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Groups', 'MachineTypes'
			])->order([
                'machine_name' => 'ASC'
            ]);
		});

    #return data
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id Machine id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'MachineTypes', 'Groups', 'AnnualPlannedCapacities', 'AnnualPlannedMachines', 'ComponentProcesses', 'MachineProcesses'
			]);
		});

	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $machineTypes   = $this->Machines->MachineTypes->find('list')->where(['is_active' => 1])->order(['MachineTypes.machine_type_name' => 'ASC']);
        $groups         = $this->Machines->Groups->find('list')->where(['is_active' => 1])->order(['Groups.group_name' => 'ASC']);
        $shiftPerDay      = [ 1 => 1, 2 => 2, 3 => 3 ];
    #set result
        $this->set(compact('machineTypes','groups','shiftPerDay'));
        
    #return
        return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Machine id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $machineTypes   = $this->Machines->MachineTypes->find('list')->where(['is_active' => 1])->order(['MachineTypes.machine_type_name' => 'ASC']);
        $groups         = $this->Machines->Groups->find('list')->where(['is_active' => 1])->order(['Groups.group_name' => 'ASC']);
        $shiftPerDay    = [1 => 1, 2 => 2, 3 => 3];
    #set result
        $this->set(compact('machineTypes','groups','shiftPerDay'));
    #return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Machine id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
	#return
		return $this->Crud->execute();
    }
}
