<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\I18n\Time;

/**
 * Processes Controller
 *
 * @property \App\Model\Table\ProcessesTable $Processes
 *
 * @method \App\Model\Entity\Process[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class ProcessesController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);

    #load model
        $this->loadModel('Machines');
        $this->loadModel('Groups');
        $this->loadModel('Vendors');
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
        $processTag     = Configure::read('tag');
        $activeStatus   = Configure::read('activeStatus');
    #find selection option
        $this->set(compact('activeStatus', 'processTag'));

    #before find
		$this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'MachineProcesses' => ['Machines']
			])->order([
                'process_name' => 'ASC'
            ]);
		});

    #return data
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
    #prepare tag
        $processTag = Configure::read('tag');
    #set result
        $this->set(compact(['processTag']));

    #CRUD before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
        #query contain
            $event->subject()->query->contain([
            'AnnualPlannedCapacities', 'ComponentProcesses', 'MachineProcesses', 'VendorProcesses'
            ]);
        });

    #return
		return $this->Crud->execute();
    }

   /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $groups         = $this->Groups->find('list')->where(['is_active' => 1])->order(['group_name' => 'ASC']);
        $machines       = $this->Machines->find('list')->where(['is_active' => 1])->order(['machine_name' => 'ASC']);
        $vendors        = $this->Vendors->find('list', ['keyField' => 'id', 'valueField' => 'vendor_name'])->where(['is_active' => 1])->order(['vendor_name' => 'ASC']);
        $processTags    = Configure::read('tag');
    #set result
        $this->set(compact(['groups','machines', 'processTags','vendors']));

    #return
        return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
      $groups       = $this->Groups->find('list')->where(['is_active' => 1])->order(['group_name' => 'ASC']);
      $machines     = $this->Machines->find('list')->where(['is_active' => 1])->order(['machine_name' => 'ASC']);
      $vendors      = $this->Vendors->find('list', ['keyField' => 'id', 'valueField' => 'vendor_name'])->where(['is_active' => 1])->order(['vendor_name' => 'ASC']);
      $processTags  = Configure::read('tag');
    #set result
      $this->set(compact(['groups', 'machines', 'processTags', 'vendors']));

    #CRUD before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
        #query contain
            $event->subject()->query->contain([
				'MachineProcesses'
			]);
        });

    #return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
    #return
	  	return $this->Crud->execute();
    }
}
