<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Cache\Cache;
use Cake\Core\Configure;
use Cake\I18n\Time;

/**
 * Products Controller
 *
 * @property \App\Model\Table\ProductsTable $Products
 *
 * @method \App\Model\Entity\Product[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class ProductsController extends AppController
{
    /*
    *initialize function
    */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);

	#load model
		$this->loadModel('Groups');
		$this->loadModel('SubAssemblies');
		$this->loadModel('MachineTypes');
		$this->loadModel('Machines');
		$this->loadModel('Processes');
		$this->loadModel('MachineProcesses');
		$this->loadModel('Components');
		$this->loadModel('ComponentProducts');
		$this->loadModel('ComponentProcesses');
		$this->loadModel('ComponentGroups');
		$this->loadModel('ComponentAssemblies');
		$this->loadModel('SubAssemblyProducts');
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
     #active status tag
        $activeStatus  = Configure::read('activeStatus');
    #find selection option
        $parentProducts = $this->Products->find('list')->where(['is_eventual_product' => 0, 'is_active' => 1]);
        $this->set(compact('activeStatus', 'parentProducts'));

    #before find
        $this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) {
            $event->subject()->query
            ->select([
                'id'                    => 'Products.id',
                'product_name'          => 'Products.product_name',
                'description'           => 'Products.description',
                'photo'                 => 'Products.photo',
                'dir'                   => 'Products.dir',
                'parent_id'             => 'Products.parent_id',
                'is_eventual_product'   => 'Products.is_eventual_product',
                'is_active'             => 'Products.is_active',
                'created_by'            => 'Products.created_by',
                'created'               => 'Products.created',
                'modified_by'           => 'Products.modified_by',
                'modified'              => 'Products.modified',
                'parent_product_name'   => 'EventualProducts.product_name',
            ])
            ->join([
                'EventualProducts' => [
                    'table'         => 'products',
                    'type'          => 'LEFT',
                    'conditions'    => ['EventualProducts.id=Products.parent_id']
                ]
            ])
            ->order(['Products.product_name' => 'ASC']);
        });

    #return data
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id Product id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'AnnualPlannedCapacities', 'AnnualPlannedProducts', 'AnnualProductTrackings', 'ComponentProducts'
			]);
		});

	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $parentProducts = $this->Products->find('list')->where(['is_eventual_product' => 0, 'is_active' => 1]);
        $this->set(compact('parentProducts'));
    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Product id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $parentProducts = $this->Products->find('list')->where(['is_eventual_product' => 0, 'is_active' => 1, 'id !=' => $id]);
        $this->set(compact('parentProducts'));
    #return
        return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Product id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
    #return
		return $this->Crud->execute();
    }

	/**
     * import method
     *
     */
    public function import()
    {
	#variable declaration
		$importData	= [];

	#check request method
		if($this->request->is('post'))
		{
			if(file_exists($_FILES['import_file']['tmp_name']) || is_uploaded_file($_FILES['import_file']['tmp_name']))
			{
            #import data
				$importData = $this->Products->importCsv($this->request->data['import_file']['tmp_name'], [
					'product_name',
					'group_name',
					'sub_assembly_code',
					'sub_assembly',
					'component_name',
					'drawing_no',
					'npm',
					'tag',
					'standard_product',
					'batch_quantity',
                    'is_spare',
					'minimum_spare',
					'gst',
					'machine_type',
					'machine_number',
					'process',
					'set_time',
					'actual_time',
					'parent_product',
				]);
            #variable declaration
				$this->result = [];
			#foreach
				foreach( $importData AS $key => $rows ){
					$importData[$key]['product_name'] 		= ( empty($rows['product_name']) ? $importData[$key-1]['product_name'] : $rows['product_name'] );
					$importData[$key]['group_name'] 		= ( empty($rows['group_name']) ? $importData[$key-1]['group_name'] : $rows['group_name'] );
					$importData[$key]['sub_assembly'] 		= ( empty($rows['sub_assembly']) ? $importData[$key-1]['sub_assembly'] : $rows['sub_assembly'] );
					$importData[$key]['sub_assembly_code']  = ( empty($rows['sub_assembly_code']) ? $importData[$key-1]['sub_assembly_code'] : $rows['sub_assembly_code'] );
					$importData[$key]['component_name'] 	= ( empty($rows['component_name']) ? $importData[$key-1]['component_name'] : $rows['component_name'] );
					$importData[$key]['drawing_no'] 		= ( empty($rows['drawing_no']) ? $importData[$key-1]['drawing_no'] : $rows['drawing_no'] );
					$importData[$key]['npm'] 				= ( empty($rows['npm']) ? $importData[$key-1]['npm'] : $rows['npm'] );
					$importData[$key]['tag'] 				= ( empty($rows['tag']) ? $importData[$key-1]['tag'] : $rows['tag'] );
					$importData[$key]['standard_product'] 	= ( empty($rows['standard_product']) ? $importData[$key-1]['standard_product'] : $rows['standard_product'] );
					$importData[$key]['batch_quantity'] 	= ( empty($rows['batch_quantity']) ? $importData[$key-1]['batch_quantity'] : $rows['batch_quantity'] );
                    $importData[$key]['is_spare'] 		    = ( ($rows['is_spare']==NULL || $rows['is_spare']=="") ? $importData[$key-1]['is_spare'] : $rows['is_spare'] );
                    $importData[$key]['minimum_spare']      = ( ($rows['minimum_spare']==NULL && $rows['minimum_spare']=="") ? $importData[$key-1]['minimum_spare'] : $rows['minimum_spare'] );
                    $importData[$key]['gst'] 				= ( ($rows['gst']==NULL && $rows['gst']=="") ? $importData[$key-1]['gst'] : $rows['gst'] );
                #checks out components
                    if($rows['tag']!='OUT'){
                        $importData[$key]['machine_type'] 		= ( empty($rows['machine_type']) ? $importData[$key-1]['machine_type'] : $rows['machine_type'] );
                        $importData[$key]['machine_number'] 	= ( empty($rows['machine_number']) ? $importData[$key-1]['machine_number'] : $rows['machine_number'] );
                        $importData[$key]['process'] 			= ( empty($rows['process']) ? $importData[$key-1]['process'] : $rows['process'] );
                        $importData[$key]['set_time']	 		= ( empty($rows['set_time']) ? 0 : $rows['set_time'] );
                        $importData[$key]['actual_time'] 		= ( empty($rows['actual_time']) ? 0 : $rows['actual_time'] );
                        $importData[$key]['parent_product']     = ( empty($rows['parent_product']) ? 0 : $rows['parent_product'] );
                    }
				}
			}
		#delete cache
			Cache::clear(true);
		#set cache
			Cache::write('importData', $importData);
			return $this->redirect(['action' => 'confirm']);
		}
    }

	/**
     * import confirm method
     *
     */
    public function confirm()
    {
	#set value
		$importData = Cache::read('importData');
	#check method
		if($this->request->is('post'))
		{
		#array each
			foreach($importData AS $key => $rows){
			#product name
				if(!empty($rows['product_name'])){
				#check data
					$product_name = strtolower($rows['product_name']);
					$pData = $this->Products->find()->where(['product_name' => $product_name, 'is_active' => 1]);
				#check duplication
					if($pData->count()==0){
						$data['product_name']           = $rows['product_name'];
						$data['is_eventual_product']	= ( strtolower($rows['standard_product']) == 'yes' ? 0 : 1 );
                    #set eventual product parent id
                        if($data['is_eventual_product']==1){
                            $products = $this->Products->find()->where(['product_name' => $rows['parent_product']])->first();
                            $data['parent_id']  = $products->id;
                        }
					#prepare to insert
						$insertData 	= $this->Products->newEntity($data);
						$result 		= $this->Products->save($insertData);
					#get last insert id
						$productId		= $result->id;
					}else{
					#get id
						$pData			= $pData->first();
						$productId		= $pData->id;
					}
				}

			#group name
				if(!empty($rows['group_name'])){
				#convert string
					$group_name = strtolower($rows['group_name']);
				#check data
					$gData = $this->Groups->find()->where(['group_name' => $group_name, 'is_active' => 1]);
				#check duplication
					if($gData->count()==0){
						$data				= [];
						$data['group_name']	= $rows['group_name'];
					#prepare to insert
						$insertData 		= $this->Groups->newEntity($data);
						$result 			= $this->Groups->save($insertData);
					#get last insert id
						$groupId			= $result->id;
					}
					else{
					#get id
						$gData				= $gData->first();
						$groupId			= $gData->id;
					}
				}

			#sub assembly
				if(!empty($rows['sub_assembly'])){
				#convert string
					$sub_assembly_code = strtolower($rows['sub_assembly_code']);
				#check data
					$saData = $this->SubAssemblies->find()->where(['sub_assembly_code' => $sub_assembly_code, 'is_active' => 1]);
				#check duplication
					if($saData->count()==0){
						$data						= [];
						$data['sub_assembly_name']	= $rows['sub_assembly'];
						$data['sub_assembly_code']	= $rows['sub_assembly_code'];
					#prepare to insert
						$insertData 	= $this->SubAssemblies->newEntity($data);
						$result 		= $this->SubAssemblies->save($insertData);
					#get last insert id
						$subAssemblyId	= $result->id;
					}
					else{
					#get id
						$saData			= $saData->first();
						$subAssemblyId	= $saData->id;
					}
				}

            #subassembly products
                if(!empty($productId) && !empty($subAssemblyId)){
				#check data
					$sapData = $this->SubAssemblyProducts->find()->where(['sub_assembly_id' => $subAssemblyId, 'product_id' => $productId, 'is_active' => 1]);
				#check duplication
					if($sapData->count()==0){
						$data						= [];
						$data['sub_assembly_id']	= $subAssemblyId;
						$data['product_id']	        = $productId;
						$data['is_active']	        = 1;
					#prepare to insert
						$insertData 	= $this->SubAssemblyProducts->newEntity($data);
						$result 		= $this->SubAssemblyProducts->save($insertData);
					#get last insert id
						$subAssemblyProductId	= $result->id;
					}
					else{
					#get id
						$sapData			    = $sapData->first();
						$subAssemblyProductId	= $sapData->id;
					}
                }

			#components
				if(!empty($rows['component_name'])){
				#convert string
					$component_name         = strtolower($rows['component_name']);
					$drawing_reference_no   = str_replace('/', '_', $rows['drawing_no']);
				#check data
					$cData = $this->Components->find()->where(['component_name' => $component_name, 'drawing_reference_no' => $drawing_reference_no, 'is_active' => 1]);
				#check duplication
					if($cData->count()==0){
						$data							= [];
						$data['component_name']			= $rows['component_name'];
						$data['component_code']			= $rows['component_name'];
						$data['drawing_reference_no']	= $drawing_reference_no;
						$data['component_cost']			= 0;
						$data['component_gst']			= 0;
						$data['batch_quantity']			= $rows['batch_quantity'];
						$data['po_turnaround_time']     = 0;
						$data['buying_frequency']       = 1;
						$data['drawing_file']           = $drawing_reference_no.'.pdf';
						$data['tag']					= ( ($rows['tag']=='IN' ? 0 : ($rows['tag']=='OUT' ? 1 : 2) ) );
						$data['is_active']				= 1;
					#prepare to insert
						$insertData 	= $this->Components->newEntity($data);
						$result 		= $this->Components->save($insertData);
						$componentId	= $result->id;
					}
					else{
					#get id
						$cData			= $cData->first();
						$componentId	= $cData->id;
					}
				}
            #component products
				if(!empty($componentId) && !empty($productId)){
				#check data
					$cpData = $this->ComponentProducts->find("all")->where(['component_id' => $componentId, 'product_id' => $productId, 'is_active' => 1]);
				#check duplication
					if($cpData->count()==0){
						$data						            = [];
						$data['component_id']		            = $componentId;
						$data['product_id']			            = $productId;
						$data['number_of_component']            = $rows['npm'];
						$data['spare_count']                    = $rows['minimum_spare'];
						$data['spare_required']                 = $rows['minimum_spare'];
						$data['spares_taken']                   = 0;
                        $data['spare_remaning']                 = 0;
						$data['start_product_no']	            = 0;
						$data['end_product_no']		            = 0;
						$data['total_completed_components']		= 0;
						$data['total_rejected_components']		= 0;
						$data['status']		                    = 0;
					#prepare to insert
						$insertData 				            = $this->ComponentProducts->newEntity($data);
						$result 					            = $this->ComponentProducts->save($insertData);
					#get last insert id
						$componentProductId			            = $result->id;
					}
				}

            #component groups
				if(!empty($componentId) && !empty($groupId)){
				#check data
					$cgData = $this->ComponentGroups->find("all")->where(['component_id' => $componentId, 'group_id' => $groupId, 'is_active' => 1]);
				#check duplication
					if($cgData->count()==0){
						$data						= [];
						$data['component_id']		= $componentId;
						$data['group_id']			= $groupId;
					#prepare to insert
						$insertData 				= $this->ComponentGroups->newEntity($data);
						$result 					= $this->ComponentGroups->save($insertData);
					}
				}

			#component assembly
				if(!empty($componentId) && !empty($subAssemblyId)){
				#check data
					$caData = $this->ComponentAssemblies->find("all")->where(['component_id' => $componentId, 'sub_assembly_id' => $subAssemblyId, 'is_active' => 1]);
				#check duplication
					if($caData->count()==0){
						$data						= [];
						$data['component_id']		= $componentId;
						$data['sub_assembly_id']	= $subAssemblyId;
					#prepare to insert
						$insertData 				= $this->ComponentAssemblies->newEntity($data);
						$result 					= $this->ComponentAssemblies->save($insertData);
					#get last insert id
						$componentAssemblyId		= $result->id;
					}
				}

        #check in or out
                if($rows['tag']!='OUT'){
                #machine types
                    if(!empty($rows['machine_type'])){
                    #convert string
                        $machine_type = strtolower($rows['machine_type']);
                    #check data
                        $mtData = $this->MachineTypes->find()->where(['machine_type_name' => $machine_type, 'is_active' => 1]);
                    #check duplication
                        if($mtData->count()==0){
                            $data                       = [];
                            $data['machine_type_name']  = $rows['machine_type'];
                            $data['machine_type_code']	= $rows['machine_type'];
                            $data['is_active']	        = 1;
                        #prepare to insert
                            $insertData 	= $this->MachineTypes->newEntity($data);
                            $result 		= $this->MachineTypes->save($insertData);
                        #get last insert id
                            $machineTypeId	= $result->id;
                        }
                        else{
                        #get id
                            $mtData			= $mtData->first();
                            $machineTypeId	= $mtData->id;
                        }
                    }

                #machines
                    if(!empty($rows['machine_number']) && !empty($machineTypeId) && !empty($groupId)){
                    #convert string
                        $machine_number = strtolower($rows['machine_number']);
                    #check data
                        $mData = $this->Machines->find()->where(['machine_name' => $machine_number, 'is_active' => 1]);
                    #check duplication
                        if($mData->count()==0){
                            $data								= [];
                            $data['machine_type_id']			= $machineTypeId;
                            $data['group_id']					= $groupId;
                            $data['machine_name']				= $rows['machine_number'];
                            $data['machine_code']				= $rows['machine_number'];
                            $data['machine_hours_per_shift']	= 8;
                            $data['machine_shift_per_day']		= 2;
                            $data['machine_downtime_per_month']	= 24;
                        #prepare to insert
                            $insertData 	= $this->Machines->newEntity($data);
                            $result 		= $this->Machines->save($insertData);
                        #get last insert id
                            $machineId	= $result->id;
                        }
                        else{
                        #get id
                            $mData		= $mData->first();
                            $machineId	= $mData->id;
                        }
                    }

                #processes
                    if(!empty($rows['process'])){
                    #convert string
                        $process = strtolower($rows['process']);
                    #check data
                        $pData = $this->Processes->find()->where(['process_name' => $process, 'is_active' => 1]);
                    #check duplication
                        if($pData->count()==0){
                            $data					= [];
                            $data['process_name']	= $rows['process'];
                            $data['process_tag']	= 0;
                        #prepare to insert
                            $insertData 	= $this->Processes->newEntity($data);
                            $result 		= $this->Processes->save($insertData);
                        #get last insert id
                            $processId		= $result->id;
                        }
                        else{
                        #get id
                            $pData		= $pData->first();
                            $processId	= $pData->id;
                        }
                    }

                #machine processes
                    if( !empty($machineId) && !empty($processId) ){
                    #check data
                        $mpData = $this->MachineProcesses->find("all")->where(['machine_id' => $machineId, 'process_id' => $processId, 'is_active' => 1]);
                    #check duplication
                        if($mpData->count()==0){
                            $data					= [];
                            $data['machine_id']		= $machineId;
                            $data['process_id']		= $processId;
                        #prepare to insert
                            $insertData 			= $this->MachineProcesses->newEntity($data);
                            $result 				= $this->MachineProcesses->save($insertData);
                        #get last insert id
                            $machineProcessId		= $result->id;
                        }
                    }

                #component process
                    if(!empty($componentId) && !empty($machineId) && !empty($processId)){
                    #check data
                        $cpData = $this->ComponentProcesses->find("all")->where(['component_id' => $componentId, 'machine_id' => $machineId, 'process_id' => $processId, 'is_active' => 1]);
                    #check duplication
                        if($cpData->count()==0){
                            $data						= [];
                            $data['component_id']		= $componentId;
                            $data['machine_id']			= $machineId;
                            $data['process_id']			= $processId;
                            $data['average_time_taken']	= $rows['actual_time'];
                            $data['set_time_for_batch']	= $rows['set_time'];
                        #prepare to insert
                            $insertData 				= $this->ComponentProcesses->newEntity($data);
                            $result 					= $this->ComponentProcesses->save($insertData);
                        }
                    }
                }
			}
		#clear cache
			Cache::clear(true);
			return $this->redirect(['action' => 'index']);
		}
	#set time
		$this->set(compact('importData'));
    }
}
