<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;

/**
 * PurchaseIndents Controller
 *
 * @property \App\Model\Table\PurchaseIndentsTable $PurchaseIndents
 *
 * @method \App\Model\Entity\PurchaseIndent[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class PurchaseIndentsController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
    #initialization
        parent::initialize();

    #CRUD operations
        $this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
        ]);

        $this->Crud->mapAction('rawMaterials', [
            'className'     => 'Crud.Index',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('outComponent', [
            'className'     => 'Crud.Index',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('outComponent', [
            'className'     => 'Crud.Index',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('addAdhoc', [
            'className'     => 'Crud.Add',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('editAdhoc', [
            'className'     => 'Crud.Edit',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('adhocOutComponents', [
            'className'     => 'Crud.Add',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('adhocRawMaterials', [
            'className'     => 'Crud.Add',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('approveAdhoc', [
            'className'     => 'Crud.Edit',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('rejectAdhoc', [
            'className'     => 'Crud.Edit',
            'relatedModels' => true,
        ]);
        
        $this->Crud->mapAction('viewAdhocOutComponents', [
            'className'     => 'Crud.View',
            'relatedModels' => true,
        ]);
        $this->Crud->mapAction('viewAdhocRawMaterials', [
            'className'     => 'Crud.View',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('listAdhocRawMaterials', [
            'className'     => 'Crud.Index',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('listAdhocOutComponents', [
            'className'     => 'Crud.Index',
            'relatedModels' => true,
        ]);

    #loadModels
        $this->loadModel('Products');
        $this->loadModel('ComponentProducts');
        $this->loadModel('Components');
        $this->loadModel('Users');
        $this->loadModel('GroupUsers');
        $this->loadModel('RawMaterials');
        $this->loadModel('Groups');
        $this->loadModel('PurchaseIndents');
        $this->loadModel('AnnualPlans');
        $this->loadModel('AnnualPlannedProducts');
        $this->loadModel('AnnualQuarterPlans');
        $this->loadModel('AnnualQuarterPlannedProducts');

    #app.php config
        $purchase_indent_status = Configure::read('purchase_indent');
        $this->set(compact(['purchase_indent_status']));
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #session
        $user_id = $this->Auth->User('id');
        $purchaseIndentsData = $this->PurchaseIndents->find('all')->contain(['AnnualQuarterPlans', 'Products', 'Groups'])->where(['PurchaseIndents.is_adhoc' => 1]);
    #set view
        $this->set(compact(['purchaseIndentsData', 'purchase_indent_status']));
    #return
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id Purchase Indent id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
    #app.php config
        $purchase_indent_status = Configure::read('purchase_indent');
    #before find
        $purchase_indent = $this->PurchaseIndents->get($id);
    #before find
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) use ($purchase_indent) {
            $product_id = $purchase_indent->product_id;
        #query
            $event->subject->query->contain([
                'PurchaseOrders', 
                'PurchaseIndentComponents' => [
                    'Components' => [
                        'ComponentRawMaterials' => ['RawMaterials'], 
                        'ComponentProducts' => function ($q) use ($product_id) {
                            return $q->where(['ComponentProducts.product_id' => $product_id]);
                        },
                    ]
                ]
            ]);
        });
    #set view
        $this->set(compact(['purchase_indent_status']));
    #return
        return $this->Crud->execute();
    }

    public function viewAdhocOutComponents($id = null)
    {
    #app.php config
        $purchase_indent_status = Configure::read('purchase_indent');
    #before find
        $purchase_indent = $this->PurchaseIndents->get($id);
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) use ($purchase_indent) {
            $product_id = $purchase_indent->product_id;
            $event->subject->query->contain([
                'PurchaseOrders',
                'PurchaseIndentComments',
                'PurchaseIndentComponents' => [
                    'Components' => [
                        'ComponentRawMaterials' => ['RawMaterials'], 
                        'ComponentProducts' => function ($q) use ($product_id) {
                                return $q->where(['ComponentProducts.product_id' => $product_id]);
                        },
                    ]
                ]
            ]);
        });
    #set view
        $this->set(compact(['purchase_indent_status']));
    #return
        return $this->Crud->execute();
    }

    public function viewAdhocRawMaterials($id = null)
    {
    #app.php config
        $purchase_indent_status = Configure::read('purchase_indent');
    #before find
        $purchase_indent = $this->PurchaseIndents->get($id);
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) use ($purchase_indent) {
            $product_id = $purchase_indent->product_id;
            $event->subject->query->contain(['PurchaseOrders','PurchaseIndentComments','PurchaseIndentComponents' => ['Components','RawMaterials']]);
        });
    #set view
        $this->set(compact(['purchase_indent_status']));
    #return
        return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
    #return
        return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Purchase Indent id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
    #app.php config
        $purchase_indent_status = Configure::read('purchase_indent');
        $purchase_indent        = $this->PurchaseIndents->get($id);
    #get quarter plan based on annual plan
        $annualPlans            = $this->AnnualPlans->find()
                                    ->contain([
                                        'AnnualPlannedProducts' => [
                                            'Products' => [
                                                'AnnualQuarterPlannedProducts' => function($query) use($purchase_indent){
                                                    return $query
                                                                ->select([
                                                                    'id'                                => 'AnnualQuarterPlans.id',
                                                                    'status'                            => 'AnnualQuarterPlans.status',
                                                                    'duration'                          => 'AnnualQuarterPlans.duration',
                                                                    'annual_quarter_planned_product_id' => 'AnnualQuarterPlannedProducts.id',
                                                                    'product_id'                        => 'AnnualQuarterPlannedProducts.product_id',
                                                                    'product_quantity'                  => 'AnnualQuarterPlannedProducts.product_quantity',
                                                                    'product_buffer_quantity'           => 'AnnualQuarterPlannedProducts.product_buffer_quantity',
                                                                    'product_requested_quantity'        => 'AnnualQuarterPlannedProducts.product_requested_quantity',
                                                                    'product_total_quantity'            => 'AnnualQuarterPlannedProducts.product_total_quantity',
                                                                    'last_product_no'                   => 'AnnualQuarterPlannedProducts.last_product_no',
                                                                ])
                                                                ->join([
                                                                    'AnnualQuarterPlans' => [
                                                                        'table'         => 'annual_quarter_plans',
                                                                        'type'          => 'inner',
                                                                        'conditions'    => ['AnnualQuarterPlans.id=AnnualQuarterPlannedProducts.annual_quarter_plan_id']
                                                                    ],
                                                                    'AnnualPlans' => [
                                                                        'table'         => 'annual_plans',
                                                                        'type'          => 'inner',
                                                                        'conditions'    => ['AnnualPlans.id=AnnualQuarterPlans.annual_plan_id']
                                                                    ],
                                                                ])
                                                                ->where([
                                                                    'AnnualPlans.id'    => $purchase_indent['annual_plan_id'],
                                                                ]);
                                                }
                                            ]
                                        ]
                                    ])
                                    ->where([ 'AnnualPlans.id' => $purchase_indent['annual_plan_id'], 'AnnualPlans.is_active' => 1 ])
                                    ->first();

    #set view
        $this->set(compact(['purchase_indent_status', 'annualPlans']));

    #before find
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) use ($purchase_indent){
            $product_id = $purchase_indent->product_id;
        #checks tag
            if ($purchase_indent->tag == 0) {
            #query
                $event->subject->query->contain([
                    'Products', 'PurchaseIndentComponents' => function ($q) {
                        return $q
                            ->select([
                                'Components.id',
                                'Components.component_name',
                                'PurchaseIndentComponents.id',
                                'PurchaseIndentComponents.purchase_indent_id',
                                'ComponentRawMaterials.component_id',
                                'ComponentRawMaterials.raw_material_id',
                                'ComponentRawMaterials.raw_material_shape',
                                'ComponentRawMaterials.min_cut_length',
                                'ComponentRawMaterials.outer_dia',
                                'ComponentRawMaterials.inner_dia',
                                'ComponentRawMaterials.raw_material_thickness',
                                'ComponentRawMaterials.raw_material_width',
                                'ComponentRawMaterials.raw_material_length',
                                'ComponentRawMaterials.unit_of_measurement',
                                'ComponentRawMaterials.quantity_per_component',
                                'RawMaterials.material_name',
                                'PurchaseIndentComponents.total_quantity',
                            ])
                            ->join([
                                'Components'    => [
                                    'table'      => 'components',
                                    'type'       => 'inner',
                                    'conditions' => [
                                        'Components.id =PurchaseIndentComponents.component_id',
                                    ]
                                ],
                                'ComponentRawMaterials' => [
                                    'table'      => 'component_raw_materials',
                                    'type'       => 'inner',
                                    'conditions' => [
                                        'ComponentRawMaterials.component_id = Components.id',
                                    ]
                                ],
                                'RawMaterials'  => [
                                    'table'      => 'raw_materials',
                                    'type'       => 'inner',
                                    'conditions' => [
                                        'RawMaterials.id = ComponentRawMaterials.raw_material_id',
                                    ]
                                ],
                            ]);
                    },
                ]);

            }
            else{
                $event->subject->query->contain([
                    'Products', 
                    'PurchaseIndentComponents' => [
                        'Components' => [
                            'ComponentProducts' => function ($q) use ($product_id) {
                                return $q->where(['ComponentProducts.product_id' => $product_id]);
                            },
                        ]
                    ]
                ]);
            }
        });

    #before save
        $this->Crud->on('beforeSave', function (\Cake\Event\Event $event) {
            $event->subject()->entity->status = 1;
        });

    #after save
        $this->Crud->on('afterSave', function (\Cake\Event\Event $event) {
            if ($event->subject()->entity->tag == 1) {
                return $this->redirect(['action' => 'outComponent']);
            }
            return $this->redirect(['action' => 'rawMaterials']);
        });

    #return
        return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Purchase Indent id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
    #return
        return $this->Crud->execute();
    }

    public function rawMaterials()
    {
    #app.php config
        $purchase_indent_status = Configure::read('purchase_indent');
    #session data
        $user_id = $this->Auth->User('id');
    #before paginate
        $this->Crud->on('beforePaginate', function (\Cake\Event\Event $event) use ($user_id) {
        #checks user role id
            if($this->Auth->User('user_role_id') == 1){
                $event->subject()
                    ->query
                    ->contain(['AnnualQuarterPlans', 'Products', 'Groups'])
                    ->where(['PurchaseIndents.tag' => 0]);
            }else{            
                $event->subject()
                    ->query
                    ->contain(['AnnualQuarterPlans', 'Products', 'Groups'])
                    ->join([
                        'GroupUsers' => [
                            'table'      => 'group_users',
                            'type'       => 'inner',
                            'conditions' => [
                                'GroupUsers.group_id = PurchaseIndents.group_id',
                            ]],
                    ])
                    ->where(['PurchaseIndents.tag' => 0, 'GroupUsers.user_id' => $user_id]);
            }
        });
    #set view
        $this->set(compact(['purchaseIndentsData', 'purchase_indent_status']));
    #pef view
        if ($this->request->params['_ext'] == 'pdf') {
            $this->viewBuilder()->layout('default');
        }
    #return
        return $this->Crud->execute();
    }

    public function outComponent()
    {
    #configure
        $purchase_indent_status = Configure::read('purchase_indent');
    #session data
        $user_id                = $this->Auth->User('id');
    #before paginate
        $this->Crud->on('beforePaginate', function (\Cake\Event\Event $event) use ($user_id) {
        #checks user role id
            if($this->Auth->User('user_role_id') == 1){
                $event->subject()
                    ->query
                    ->contain(['AnnualQuarterPlans', 'Products', 'Groups'])                
                    ->where(['PurchaseIndents.tag' => 1]);
            }else{
                $event->subject()
                    ->query
                    ->contain(['AnnualQuarterPlans', 'Products', 'Groups'])
                    ->join([
                        'GroupUsers' => [
                            'table'      => 'group_users',
                            'type'       => 'inner',
                            'conditions' => [
                                'GroupUsers.group_id = PurchaseIndents.group_id',
                            ]],
                    ])
                    ->where(['PurchaseIndents.tag' => 1, 'GroupUsers.user_id' => $user_id]);
            }
        });
    #set view
        $this->set(compact(['purchaseIndentsData', 'purchase_indent_status']));
    #return
        return $this->Crud->execute();
    }

    #add adhoc indent based on component type
    public function addAdhoc($component_type = null)
    {
    #session data
        $user_id = $this->Auth->User('id');
    #Display products
        $products           = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->toArray();
        $user_group_details = $this->GroupUsers->find()
                                ->contain(['Groups'])
                                ->where(['GroupUsers.user_id' => $user_id])->first();
    #set view
        $this->set(compact(['products', 'user_group_details']));
    #before save data
        $this->Crud->on('beforeSave', function (\Cake\Event\Event $event) {
            $entity = $event->subject()->entity;
        #loop data
            foreach ($entity->purchase_indent_components as $key => $value) {
                if (empty($value->actual_quantity)) {
                    unset($entity->purchase_indent_components[$key]);
                }
            }
            $entity->status = 2;
        });
    #after save data
        $this->Crud->on('afterSave', function (\Cake\Event\Event $event) {
            $entity = $event->subject()->entity;
        #check tag
            if ($entity->tag == 0) {
                $this->redirect(['action' => 'rawMaterials']);
            } else {
                $this->redirect(['action' => 'outComponent']);
            }
        });
    #return
        return $this->Crud->execute();
    }

    #edit adhoc
    public function editAdhoc($id = null)
    {
    #before save
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
            $event->subject->query->contain([
                'Products',
                'PurchaseIndentComments',
                'PurchaseIndentComponents'=>['Components', 'RawMaterials']
             ]);
        });
    #after save
        $this->Crud->on('beforeSave', function (\Cake\Event\Event $event) {
            $event->subject()->entity->status = 2;
        });
    #after save
        $this->Crud->on('afterSave', function (\Cake\Event\Event $event) {
            $entity = $event->subject()->entity;
        #check tag
            if ($entity->tag == 0) {
                $this->redirect(['action' => 'rawMaterials']);
            } else {
                $this->redirect(['action' => 'outComponent']);
            }
        });
    #return data
        return $this->Crud->execute();
    }

    public function adhocRawMaterials()
    {
    #session data
        $user_id = $this->Auth->User('id');
    #Display products
        $products           = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->toArray();
        $user_group_details = $this->GroupUsers->find()
                                ->contain(['Groups'])
                                ->where(['GroupUsers.user_id' => $user_id])
                                ->first();
    #set view
        $this->set(compact(['products', 'user_group_details']));
    #before save
        $this->Crud->on('beforeSave', function (\Cake\Event\Event $event) {
            $entity = $event->subject()->entity;
        #loop data
            foreach ($entity->purchase_indent_components as $key => $value) {
                if (empty($value->actual_quantity)) {
                    unset($entity->purchase_indent_components[$key]);
                }
            }
            $entity->status = 2;
        });
    #after save
        $this->Crud->on('afterSave', function (\Cake\Event\Event $event) {
            $entity = $event->subject()->entity;
            $this->redirect(['action' => 'rawMaterials']);

        });
    #return
        return $this->Crud->execute();
    }

#Get list of out components
    public function adhocRawMaterialLists($product_id)
    {
        $product_components = [];
    #checks
        if ($product_id){
            $user_id = $this->Auth->User('id');
            #Add adhoc indent based on group users
            $product_raw_materials = $this->Components->find('all')
                                        ->select([
                                            'ComponentRawMaterials.id',                    
                                            'RawMaterials.id',
                                            'RawMaterials.material_name',
                                        ])
                                        ->join([                    
                                            'ComponentProducts'     => [
                                                'table'      => 'component_products',
                                                'type'       => 'inner',
                                                'conditions' => ['ComponentProducts.component_id = Components.id',
                                                ]],

                                            'ComponentGroups'       => [
                                                'table'      => 'component_groups',
                                                'type'       => 'inner',
                                                'conditions' => ['ComponentGroups.component_id = ComponentProducts.component_id',
                                                ]],
                                            'Groups'                => [
                                                'table'      => 'groups',
                                                'type'       => 'inner',
                                                'conditions' => ['Groups.id = ComponentGroups.group_id',
                                                ]],
                                            'GroupUsers'            => [
                                                'table'      => 'group_users',
                                                'type'       => 'inner',
                                                'conditions' => ['GroupUsers.group_id = Groups.id',
                                                ]],
                                            'ComponentRawMaterials' => [
                                                'table'      => 'component_raw_materials',
                                                'type'       => 'inner',
                                                'conditions' => ['ComponentRawMaterials.component_id = ComponentProducts.component_id',
                                                ]],

                                            'RawMaterials' => [
                                                'table'      => 'raw_materials',
                                                'type'       => 'inner',
                                                'conditions' => ['RawMaterials.id = ComponentRawMaterials.raw_material_id',
                                                ]],

                                        ])
                                        ->where(['ComponentProducts.product_id' => $product_id,
                                            'GroupUsers.user_id'                    => $user_id,
                                        ])
                                        ->enableAutofields(true)
                                        ->toArray();
    #set view
            $this->set(compact('product_raw_materials'));
        }
    }

    public function adhocOutComponents()
    {
        $user_id = $this->Auth->User('id');
    #Display products
        $products           = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->toArray();
        $user_group_details = $this->GroupUsers->find()
                                ->contain(['Groups'])
                                ->where(['GroupUsers.user_id' => $user_id])->first();
    #set view
        $this->set(compact(['products', 'user_group_details']));
    #before save
        $this->Crud->on('beforeSave', function (\Cake\Event\Event $event) {
            $entity = $event->subject()->entity;
            foreach ($entity->purchase_indent_components as $key => $value) {
                if (empty($value->actual_quantity)) {
                    unset($entity->purchase_indent_components[$key]);
                }
            }
            $entity->status = 2;
        });
    #after save
        $this->Crud->on('afterSave', function (\Cake\Event\Event $event) {
            $entity = $event->subject()->entity;
            $this->redirect(['action' => 'outComponent']);
        });

    #return
        return $this->Crud->execute();
    }

#Get list of out components
    public function adhocOutComponentLists($product_id)
    {
        $product_components = [];
    #checks id
        if ($product_id) {
            $user_id = $this->Auth->User('id');
            #Add adhoc indent based on group users
            $product_components = $this->Components->find('all')
                                    ->select(['Groups.id', 'Groups.group_name'])
                                    ->join([
                                        'ComponentProducts' => [
                                            'table'      => 'component_products',
                                            'type'       => 'inner',
                                            'conditions' => ['ComponentProducts.component_id = Components.id',
                                            ]],
                                        'ComponentGroups'   => [
                                            'table'      => 'component_groups',
                                            'type'       => 'inner',
                                            'conditions' => ['ComponentGroups.component_id = Components.id',
                                            ]],
                                        'Groups'            => [
                                            'table'      => 'groups',
                                            'type'       => 'inner',
                                            'conditions' => ['Groups.id = ComponentGroups.group_id',
                                            ]],
                                        'GroupUsers'        => [
                                            'table'      => 'group_users',
                                            'type'       => 'inner',
                                            'conditions' => ['GroupUsers.group_id = Groups.id',
                                            ]],
                                    ])
                                    ->enableAutoFields(true)
                                    ->where(['ComponentProducts.product_id' => $product_id,
                                        'GroupUsers.user_id'                => $user_id,
                                    ])
                                    ->toArray();
        #set view
            $this->set(compact('product_components'));
        }
    }


    public function listAdhocRawMaterials()
    {
    #before paginate
        $this->Crud->on('beforePaginate', function (\Cake\Event\Event $event)  {
            $event->subject()->query
                            ->contain(['AnnualQuarterPlans', 'Products'])                
                            ->where(['PurchaseIndents.tag' => 0, 'PurchaseIndents.is_adhoc' => 1]);
        });
    #return
        return $this->Crud->execute();
    }

    public function listAdhocOutComponents(){
    #before paginate
         $this->Crud->on('beforePaginate', function (\Cake\Event\Event $event)  {
            $event->subject()->query
                            ->contain(['AnnualQuarterPlans', 'Products'])                
                            ->where(['PurchaseIndents.tag' => 1, 'PurchaseIndents.is_adhoc' => 1]);
        });
    #return
        return $this->Crud->execute();
    }

    public function approveAdhoc($id = null)
    {
    #set view
        $this->set(compact(['purchase_indent_status']));
    #before save
        $this->Crud->on('beforeSave', function (\Cake\Event\Event $event) {
            $event->subject()->entity->status=1;
        });
    #after save
        $this->Crud->on('afterSave', function (\Cake\Event\Event $event) {
            $entity = $event->subject()->entity;
        #check tag
            if($entity->tag == 0){
                $this->redirect(['action' => 'listAdhocRawMaterials']);
            }else{
                $this->redirect(['action' => 'listAdhocOutComponents']);
            }           
        });
    #return
        return $this->Crud->execute();
    }

    public function rejectAdhoc($id = null)
    {
    #before save
        $this->Crud->on('beforeSave', function (\Cake\Event\Event $event) {
            $event->subject()->entity->status=3;

        });
    #after save
        $this->Crud->on('afterSave', function (\Cake\Event\Event $event) {
            $entity = $event->subject()->entity;
        #check tag
            if($entity->tag == 0){
                $this->redirect(['action' => 'listAdhocRawMaterials']);
            }else{
                $this->redirect(['action' => 'listAdhocOutComponents']);
            }           
        });
    #return
        return $this->Crud->execute();
    }

    public function getPii()
    {
    #find query
        $buying_frequencies = $this->Groups->find('all')
                                ->select([
                                    'component_id'              => 'Components.id',
                                    'group_id'                  => 'Groups.id',
                                    'group_name'                => 'Groups.group_name',
                                    'buying_frequency'          => 'Components.buying_frequency',
                                    'tag'                       => 'Components.tag',
                                    'number_of_component'       => 'ComponentProducts.number_of_component',
                                    'annual_capacity_tag'       => 'AnnualPlannedCapacties.tag',
                                    'annual_capacity_in_total'  => 'AnnualPlannedCapacties.total_components',
                                    'annual_capacity_out_total' => 'AnnualPlannedCapacties.total_components',
                                ])
                                ->join(
                                    [
                                        'ComponentGroups'   => [
                                            'type'       => 'inner',
                                            'table'      => 'component_groups',
                                            'conditions' => ['ComponentGroups.group_id=Groups.id'],
                                        ],
                                        'ComponentProducts' => [
                                            'type'       => 'inner',
                                            'table'      => 'component_products',
                                            'conditions' => ['ComponentProducts.component_id=ComponentGroups.component_id'],
                                        ],
                                        'Components'    => [
                                            'type'       => 'inner',
                                            'table'      => 'components',
                                            'conditions' => ['Components.id=ComponentProducts.component_id'],
                                        ],
                                        'AnnualPlannedCapacties' => [
                                            'type'       => 'left',
                                            'table'      => 'annual_planned_capacities',
                                            'conditions' => ['AnnualPlannedCapacties.component_id=Components.id'],
                                        ],
                                    ]
                                )
                                ->enableAutofields(true)
                                ->where(['ComponentProducts.product_id' => 1])
                                ->group('Components.id')
                                ->toArray();
    #variable declaartion
        $this->result    = [];
        $purchase_idents = [];
    #loop data
        foreach ($buying_frequencies as $key => $rows) {
            $piNo = $rows['group_name'] . date("Y");

            if ($rows->tag == 2) {
                if (!empty($rows->annual_capacity_in_total)) {
                    $this->result[0][$rows->buying_frequency][$rows->group_id]['product_id']         = 1;
                    $this->result[0][$rows->buying_frequency][$rows->group_id]['group_id']           = $rows->group_id;
                    $this->result[0][$rows->buying_frequency][$rows->group_id]['purchase_intent_no'] = $piNo;
                    $this->result[0][$rows->buying_frequency][$rows->group_id]['status']             = 0;
                    $this->result[0][$rows->buying_frequency][$rows->group_id]['is_adhoc']           = 0;
                    $this->result[0][$rows->buying_frequency][$rows->group_id]['tag']                = 0;
                    $this->result[0][$rows->buying_frequency][$rows->group_id]['purchase_indent_components'][$rows->component_id]['component_id']    = $rows->component_id;
                    $this->result[0][$rows->buying_frequency][$rows->group_id]['purchase_indent_components'][$rows->component_id]['raw_material_id'] = $rows->component_id;
                    $this->result[0][$rows->buying_frequency][$rows->group_id]['purchase_indent_components'][$rows->component_id]['total_quantity']  = 10;
                    $this->result[0][$rows->buying_frequency][$rows->group_id]['purchase_indent_components'][$rows->component_id]['is_active']       = "";
                }
                if (!empty($rows->annual_capacity_out_total)) {
                    $this->result[1][$rows->buying_frequency][$rows->group_id]['product_id']         = 1;
                    $this->result[1][$rows->buying_frequency][$rows->group_id]['group_id']           = $rows->group_id;
                    $this->result[1][$rows->buying_frequency][$rows->group_id]['purchase_intent_no'] = $piNo;
                    $this->result[1][$rows->buying_frequency][$rows->group_id]['status']             = 0;
                    $this->result[1][$rows->buying_frequency][$rows->group_id]['is_adhoc']           = 0;
                    $this->result[1][$rows->buying_frequency][$rows->group_id]['tag']                = 1;
                    $this->result[1][$rows->buying_frequency][$rows->group_id]['purchase_indent_components'][$rows->component_id]['component_id']    = $rows->component_id;
                    $this->result[1][$rows->buying_frequency][$rows->group_id]['purchase_indent_components'][$rows->component_id]['raw_material_id'] = $rows->component_id;
                    $this->result[1][$rows->buying_frequency][$rows->group_id]['purchase_indent_components'][$rows->component_id]['total_quantity']  = 10;
                    $this->result[1][$rows->buying_frequency][$rows->group_id]['purchase_indent_components'][$rows->component_id]['is_active']       = "";
                }
            }
            $this->result[$rows->tag][$rows->buying_frequency][$rows->group_id]['product_id']         = 1;
            $this->result[$rows->tag][$rows->buying_frequency][$rows->group_id]['group_id']           = $rows->group_id;
            $this->result[$rows->tag][$rows->buying_frequency][$rows->group_id]['purchase_intent_no'] = $piNo;
            $this->result[$rows->tag][$rows->buying_frequency][$rows->group_id]['status']             = 0;
            $this->result[$rows->tag][$rows->buying_frequency][$rows->group_id]['is_adhoc']           = 0;
            $this->result[$rows->tag][$rows->buying_frequency][$rows->group_id]['tag']                = $rows->tag;
            $this->result[$rows->tag][$rows->buying_frequency][$rows->group_id]['purchase_indent_components'][$rows->component_id]['component_id']    = $rows->component_id;
            $this->result[$rows->tag][$rows->buying_frequency][$rows->group_id]['purchase_indent_components'][$rows->component_id]['raw_material_id'] = $rows->component_id;
            $this->result[$rows->tag][$rows->buying_frequency][$rows->group_id]['purchase_indent_components'][$rows->component_id]['total_quantity']  = 10;
            $this->result[$rows->tag][$rows->buying_frequency][$rows->group_id]['purchase_indent_components'][$rows->component_id]['is_active']       = "";

        }
        unset($this->result[2]);
    #variable declaration
        $purchase_idents = [];
    #loop data
        foreach ($this->result as $tag_key => $tag_rows) {
            foreach ($tag_rows as $buying_freq_key => $buying_freq_value) {
                for ($i = 0; $i < $buying_freq_key; $i++) {
                    foreach ($buying_freq_value as $buying_freq_value_key => $buying_freq_value_value) {
                        $buying_freq_value_value['purchase_intent_no'] = $buying_freq_value_value['purchase_intent_no'] . (mt_rand(100, 999));
                        $purchase_idents[] = $buying_freq_value_value;
                    }
                }
            }
        }
    }
}
