<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Cache\Cache;
use Cake\Core\Configure;
use Cake\I18n\Time;

/**
 * RawMaterials Controller
 *
 * @property \App\Model\Table\RawMaterialsTable $RawMaterials
 *
 * @method \App\Model\Entity\RawMaterial[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class RawMaterialsController extends AppController
{
    /**
     * initialize method
    */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);

    #load model
         $this->loadModel('Components');
         $this->loadModel('ComponentRawMaterials');
         $this->loadModel('Vendors');

    #vendor list
         $vendors = $this->Vendors->find('list', ['keyField' => 'id', 'valueField' => 'vendor_name'])->where(['is_active' => 1]);
         $this->set(compact('vendors'));
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #active status tag
        $activeStatus  = Configure::read('activeStatus');
    #find selection option
        $this->set(compact('activeStatus'));

    #before find
        $this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) {
            $event->subject()->query->order(['material_name' => 'ASC']);
        });

    #return data
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id Raw Material id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'ComponentRawMaterials', 'PurchaseIndentComponents', 'PurchaseOrderComponents', 'RawMaterialFormulas', 'VendorRawMaterials'
			]);
		});

	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Raw Material id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
    #return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Raw Material id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
	#return
		return $this->Crud->execute();
    }

    /**
     * import method
     *
     */
    public function rawMaterialImport()
    {
	#variable declaration
		$importData	= [];

	#check request method
		if($this->request->is('post'))
		{
			if(file_exists($_FILES['import_file']['tmp_name']) || is_uploaded_file($_FILES['import_file']['tmp_name']))
			{
				$importData = $this->RawMaterials->importCsv($this->request->data['import_file']['tmp_name'], [
					'drawing_no',
					'raw_material_name',
					'raw_material_shape',
					'min_cut_length',
					'outer_dia',
					'inner_dia',
					'thickness',
					'width',
					'length',
					'unit_of_measurement',
					'gst',
				]);
				$this->result = [];
			#foreach
				foreach( $importData AS $key => $rows ){
					$importData[$key]['drawing_no']             = ( empty($rows['drawing_no']) ? $importData[$key-1]['drawing_no'] : $rows['drawing_no'] );
					$importData[$key]['raw_material_name'] 		= ( empty($rows['raw_material_name']) ? $importData[$key-1]['raw_material_name'] : $rows['raw_material_name'] );
					$importData[$key]['raw_material_shape']     = ( empty($rows['raw_material_shape']) ? 0 : $rows['raw_material_shape'] );
					$importData[$key]['min_cut_length']         = ( empty($rows['min_cut_length']) ? 0 : $rows['min_cut_length'] );
					$importData[$key]['outer_dia']              = ( empty($rows['outer_dia']) ? 0 : $rows['outer_dia'] );
					$importData[$key]['inner_dia']              = ( empty($rows['inner_dia']) ? 0 : $rows['inner_dia'] );
					$importData[$key]['raw_material_thickness'] = ( empty($rows['thickness']) ? 0 : $rows['thickness'] );
					$importData[$key]['raw_material_width']     = ( empty($rows['width']) ? 0 : $rows['width'] );
					$importData[$key]['raw_material_length']    = ( empty($rows['length']) ? 0 : $rows['length'] );
					$importData[$key]['unit_of_measurement'] 	= ( empty($rows['unit_of_measurement']) ? 0 : $rows['unit_of_measurement'] );
					$importData[$key]['gst']                    = ( empty($rows['gst']) ? $importData[$key-1]['gst'] : $rows['gst'] );
				}
			}
		#delete cache
			Cache::clear(true);
		#set cache
			Cache::write('importRawMaterialData', $importData);
			return $this->redirect(['action' => 'rawMaterialConfirm']);
		}
    }

	/**
     * import confirm method
     *
     */
    public function rawMaterialConfirm()
    {
	#set value
		$importData = Cache::read('importRawMaterialData');
	#check method
		if($this->request->is('post'))
		{
		#array each
			foreach($importData AS $key => $rows){
                $componentId = NULL;
			#product name
				if(!empty($rows['drawing_no'])){
				#check data
					$drawing_no = strtolower($rows['drawing_no']);
					$cData      = $this->Components->find()->where(['drawing_reference_no' => $drawing_no, 'is_active' => 1]);
				#check duplication
					if($cData->count()>0)
                    {
					#get id
						$cData			= $cData->first();
						$componentId    = $cData->id;
					}
				}

			#group name
				if(!empty($componentId) && $componentId!=null){
				#convert string
					$raw_material_name = strtolower($rows['raw_material_name']);
				#check data
					$gData = $this->RawMaterials->find()->where(['material_name' => $raw_material_name, 'is_active' => 1]);
				#check duplication
					if($gData->count()==0){
						$data                       = [];
						$data['material_name']      = $rows['raw_material_name'];
					#prepare to insert
						$insertData                 = $this->RawMaterials->newEntity($data);
						$result                     = $this->RawMaterials->save($insertData);
					#get last insert id
						$rawMaterialId              = $result->id;
					}
					else{
					#get id
						$gData				= $gData->first();
						$rawMaterialId      = $gData->id;
					}
				}

			#sub assembly
				if(!empty($componentId) && !empty($rawMaterialId)){
				#check data
					$saData = $this->ComponentRawMaterials->find()->where(['component_id' => $componentId, 'raw_material_id' => $rawMaterialId, 'is_active' => 1]);
				#check duplication
					if($saData->count()==0){
						$data                           = [];
						$data['component_id']           = $componentId;
						$data['raw_material_id']        = $rawMaterialId;
						$data['raw_material_shape']     = $rows['raw_material_shape'];
						$data['min_cut_length']         = $rows['min_cut_length'];
						$data['outer_dia']              = $rows['outer_dia'];
						$data['inner_dia']              = $rows['inner_dia'];
						$data['raw_material_thickness'] = $rows['raw_material_thickness'];
						$data['raw_material_width']     = $rows['raw_material_width'];
						$data['raw_material_length']    = $rows['raw_material_length'];
						$data['unit_of_measurement']    = $rows['unit_of_measurement'];
						$data['gst']                    = $rows['gst'];
						$data['buy_frequency']          = 0;
						$data['quantity_per_component'] = 0;
					#prepare to insert
						$insertData 	= $this->ComponentRawMaterials->newEntity($data);
						$result 		= $this->ComponentRawMaterials->save($insertData);
					#get last insert id
						$subAssemblyId	= $result->id;
					}
					else{
					#get id
						$saData			= $saData->first();
						$subAssemblyId	= $saData->id;
					}
				}
			}
		#clear cache
			Cache::clear(true);
			return $this->redirect(['action' => 'index']);
		}
	#set time
		$this->set(compact('importData'));
    }
}
