<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\I18n\Time;

/**
 * UserRoles Controller
 *
 * @property \App\Model\Table\UserRolesTable $UserRoles
 *
 * @method \App\Model\Entity\UserRole[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class UserRolesController extends AppController
{
    /**
     * initialize method
    */
    public function initialize()
    {
	#initialization
    parent::initialize();
     $this->loadComponent('DataTables.DataTables');

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);

    #load model
        $this->loadModel('ModuleMasters');
        $this->loadModel('Modules');
        $this->loadModel('ModuleActions');
        $this->loadModel('UserRolePrivileges');
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #process tag
        $activeStatus  = Configure::read('activeStatus');
    #find selection option
        $this->set(compact('activeStatus'));
    
    #before find
        $this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) {
            $event->subject()->query->order(['user_role' => 'ASC']);
        });

    #return data
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id User Role id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Users'
			]);
		});

	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id User Role id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
    #return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id User Role id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function userPrevillege($id = null)
    {
    #if id not null
        if($id!=null){
        #find user role information
            $userRole = $this->UserRoles->get($id);
        #get all modules
            $modules = $this->ModuleMasters->find()
                                ->contain([
                                    'Modules' => function($query){
                                        return $query
                                                    ->contain([
                                                        'ModuleActions' => function($maQuery){
                                                            return $maQuery
                                                                    ->where(['ModuleActions.is_active' => 1]);
                                                        }
                                                    ])
                                                    ->where(['Modules.is_active' => 1]);
                                    }
                                ])
                                ->where(['is_active' => 1]);

        #get assigned privilleges
            $userRolePrivileges     = $this->UserRolePrivileges->find()->where(['user_role_id' => $id]);
            $userRoleModules        = $userRolePrivileges->group(['module_id'])->hydrate(false)->extract('module_id')->toArray();
            $userRoleModuleActions  = $this->UserRolePrivileges->find()
                                        ->select([
                                           'module_id'  => 'UserRolePrivileges.module_id',
                                           'action_id'  => 'GROUP_CONCAT(UserRolePrivileges.module_action_id)'
                                        ])
                                        ->group(['UserRolePrivileges.module_id'])
                                        ->where(['user_role_id' => $id])
                                        ->hydrate(false)
                                        ->toArray();
        #set values
            $this->set(compact(['id', 'userRole', 'modules', 'userRoleModules', 'userRoleModuleActions']));

        #check request
         if($this->request->is(['post', 'put'])){
            #get post data
                $userPrevillege = $this->request->data('menu_previllege');
            #delete all
                $deleteAll = $this->UserRolePrivileges->deleteAll(['user_role_id' => $id]);
            #master modules
                foreach($userPrevillege AS $key => $rows){
                #module menu list
                    if(!empty($rows['module'])){
                        foreach($rows['module'] AS $mKey => $mRows){
                        #menu actions
                            if(!empty($mRows['menu_action'])){
                            #preapre insert query
                                $this->result = [];
                            #loop action
                                foreach($mRows['menu_action'] AS $maKey => $maRows){
                                #prepare data to insert
                                    $this->result[$maRows]['user_role_id']       = $id;
                                    $this->result[$maRows]['module_master_id']   = $rows['module_master_id'];
                                    $this->result[$maRows]['module_id']          = $mRows['module_id'];
                                    $this->result[$maRows]['module_action_id']   = $maRows;
                                    $this->result[$maRows]['is_active']          = 1;
                                }
                            #prepare to insert
                                $entity  = $this->UserRolePrivileges->newEntities($this->result);
                                $this->UserRolePrivileges->saveMany($entity, array('callbacks' => false));
                            }
                        }
                    }
                }

            #return
                return $this->redirect(
                    ['action' => 'index']
                );
            }
        }
        else{
        #return
            return $this->redirect(
                ['action' => 'index']
            );
        }
    }
}
