<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Cache\Cache;
use Cake\Core\Configure;

/**
 * Vendors Controller
 *
 * @property \App\Model\Table\VendorsTable $Vendors
 *
 * @method \App\Model\Entity\Vendor[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class VendorsController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
    #initialization
        parent::initialize();

    #CRUD operations
        $this->loadComponent('Crud.Crud', [
            'actions'   => [
                'Crud.Index',
                'add'         => ['className' => 'Crud.Add'],
                'updatePrice' => ['className' => 'Crud.Edit'],
                'updateComponentPrice'      => ['className' => 'Crud.Edit'],
                'updateProcessPrice'        => ['className' => 'Crud.Edit'],
                'updateRawmaterialPrice'    => ['className' => 'Crud.Edit'],
                'updateStoreComponentPrice' => ['className' => 'Crud.Edit'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api',
            ],
        ]);

    #load model
        $this->loadModel('Components');
        $this->loadModel('Processes');
        $this->loadModel('StoreComponents');
        $this->loadModel('RawMaterials');
        $this->loadModel('VendorComponents');
        $this->loadModel('VendorProcesses');
        $this->loadModel('VendorRawMaterials');
        $this->loadModel('VendorStoreComponents');
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #active status tag
        $activeStatus = Configure::read('activeStatus');
    #find selection option
        $this->set(compact('activeStatus'));

    #before find
        $this->Crud->on('beforePaginate', function (\Cake\Event\Event $event) {
            $event->subject()->query->order(['vendor_name' => 'ASC']);
        });

    #return data
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id Vendor id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
    #before find
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
            $event->subject()->query->contain([
                'GoodsReceivedNotes', 'PurchaseOrders', 'VendorComponents', 'VendorProcesses', 'VendorRawMaterials',
            ]);
        });

    #return
        return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
    #return
        return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Vendor id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
    #return
        return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Vendor id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
    #return
        return $this->Crud->execute();
    }

    /**
     * vendor import method
     *
     */
    public function vendorImport()
    {
    #variable declaration
        $importData = [];

    #check request method
        if ($this->request->is('post')) {
        #check file exist
            if (file_exists($_FILES['import_file']['tmp_name']) || is_uploaded_file($_FILES['import_file']['tmp_name'])) {
            #import data
                $importData = $this->Vendors->importCsv($this->request->data['import_file']['tmp_name'], 
                                [
                                    'sno',
                                    'vendor_name',
                                    'vendor_code',
                                    'vendor_contact_person',
                                    'contact_number',
                                    'mobile',
                                    'email',
                                    'address',
                                    'rating',
                                ]);
            #variable declaration
                $this->result = [];
            #foreach
                foreach ($importData as $key => $rows) {
                    $importData[$key]['vendor_name']           = (empty($rows['vendor_name']) ? null : $rows['vendor_name']);
                    $importData[$key]['vendor_code']           = (empty($rows['vendor_code']) ? null : $rows['vendor_code']);
                    $importData[$key]['vendor_contact_person'] = (empty($rows['vendor_contact_person']) ? null : $rows['vendor_contact_person']);
                    $importData[$key]['contact_number']        = (empty($rows['contact_number']) ? null : $rows['contact_number']);
                    $importData[$key]['mobile']                = (empty($rows['mobile']) ? null : $rows['mobile']);
                    $importData[$key]['email']                 = (empty($rows['email']) ? null : $rows['email']);
                    $importData[$key]['address']               = (empty($rows['address']) ? null : $rows['address']);
                    $importData[$key]['rating']                = (empty($rows['rating']) ? 0 : $rows['rating']);
                }
            }
        #delete cache
            Cache::clear(true);
        #set cache
            Cache::write('importVendorData', $importData);
            return $this->redirect(['action' => 'confirmVendorImport']);
        }
    }

    /**
     * import vendor confirm method
     *
     */
    public function confirmVendorImport()
    {
    #set value
        $importData = Cache::read('importVendorData');
    #check method
        if ($this->request->is('post')) {
        #array each
            foreach ($importData as $key => $rows) {
            #product name
                if (!empty($rows['vendor_name'])) {
                #check data
                    $vendor_code = strtolower($rows['vendor_code']);
                    $vendor_name = strtolower($rows['vendor_name']);
                    $pData       = $this->Vendors->find()->where(['vendor_code' => $vendor_code, 'vendor_name' => $vendor_name, 'is_active' => 1]);
                #check duplication
                    if ($pData->count() == 0) {
                        $data['vendor_code']    = $rows['vendor_code'];
                        $data['vendor_name']    = $rows['vendor_name'];
                        $data['contact_person'] = $rows['vendor_contact_person'];
                        $data['phone']          = $rows['contact_number'];
                        $data['mobile']         = $rows['mobile'];
                        $data['email']          = $rows['email'];
                        $data['address']        = $rows['address'];
                        $data['rating']         = $rows['rating'];
                    #prepare to insert
                        $insertData = $this->Vendors->newEntity($data);
                        $result     = $this->Vendors->save($insertData);
                    #get last insert id
                        $vendorsId = $result->id;
                    }
                }
            }
        #clear cache
            Cache::clear(true);
            return $this->redirect(['action' => 'index']);
        }
    #set time
        $this->set(compact('importData'));
    }

#Price update method
    public function updatePrice($id = null)
    {
    #find data
        $components           = $this->Components->find('list', ['keyField' => 'id', 'valueField' => 'component_name'])->where(['is_active' => 1
            // , 'tag' => 1
        ]);
        $operation_components = $this->Components->find('list', ['keyField' => 'id', 'valueField' => 'component_name'])->where(['is_active' => 1, 'tag !=' => 1]);
        $processes            = $this->Processes->find('list', ['keyField' => 'id', 'valueField' => 'process_name'])->where(['is_active' => 1]);
        $store_components     = $this->StoreComponents->find('list', ['keyField' => 'id', 'valueField' => 'component_name'])->where(['is_active' => 1]);
        $raw_material         = $this->RawMaterials->find('list', ['keyField' => 'id', 'valueField' => 'material_name'])->where(['is_active' => 1]);
    #set result
        $this->set(compact('components', 'processes', 'store_components', 'raw_material', 'operation_components'));
        
    
    #crud before find
        $this->Crud->on('beforeSave', function (\Cake\Event\Event $event) {
            /* pr($event->subject->entity->raw_materials);
            exit; */
        });

    #crud before find
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
            $event->subject()->query->contain([
                'VendorComponents',
                'VendorProcesses',
                'VendorStoreComponents',
                'VendorRawMaterials',
                'Processes',
            ]);
        });

    #return result
        return $this->Crud->execute();
    }

#Price update method
    public function updateComponentPrice($id = null)
    {
    #find data
        $components = $this->Components->find('list', ['keyField' => 'id', 'valueField' => 'component_name'])->where(['is_active' => 1]);
    #set result
        $this->set(compact('components'));

    #crud before find
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
            $event->subject()->query->contain([
                'VendorComponents',
            ]);
        });

    #return result
        return $this->Crud->execute();
    }

#Price update method
    public function updateProcessPrice($id = null)
    {
    #find data
        $components = $this->Components->find('list', ['keyField' => 'id', 'valueField' => 'component_name'])->where(['is_active' => 1]);
        $processes  = $this->Processes->find('list', ['keyField' => 'id', 'valueField' => 'process_name'])->where(['is_active' => 1]);
    #set result
        $this->set(compact('components', 'processes'));

    #crud before find
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
            $event->subject()->query->contain([
                'VendorProcesses',
                'Processes',
            ]);
        });

    #return result
        return $this->Crud->execute();
    }

#Price update method
    public function updateRawmaterialPrice($id = null)
    {
    #find data
        $raw_material   = $this->RawMaterials->find('list', ['keyField' => 'id', 'valueField' => 'material_name'])->where(['is_active' => 1]);
    #set result
        $this->set(compact('raw_material'));

    #crud before find
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
            $event->subject()->query->contain([
                'VendorRawMaterials',
            ]);
        });

    #return result
        return $this->Crud->execute();
    }

#Price update method
    public function updateStoreComponentPrice($id = null)
    {
    #find data
        $store_components   = $this->StoreComponents->find('list', ['keyField' => 'id', 'valueField' => 'component_name'])->where(['is_active' => 1]);
    #set result
        $this->set(compact('store_components'));

    #crud before find
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
            $event->subject()->query->contain([
                'VendorStoreComponents',
            ]);
        });

    #return result
        return $this->Crud->execute();
    }

    #Vednor Component Import Method
    /**
     * import method
     *
     */
    public function VendorComponentImport()
    {
    #variable declaration
        $importData = [];

    #check request method
        if ($this->request->is('post')) {
            if (file_exists($_FILES['import_file']['tmp_name']) || is_uploaded_file($_FILES['import_file']['tmp_name'])) {
                $importData = $this->Vendors->importCsv($this->request
                    ->data['import_file']['tmp_name'],
                    [
                        'vendor_code',
                        'drawing_no',
                        'component_name',
                        'component_price',
                        'gst',
                    ]);
                $this->result = [];
            #foreach
                foreach ($importData as $key => $rows) {
                    $importData[$key]['vendor_code']     = (empty($rows['vendor_code']) ? '' : $rows['vendor_code']);
                    $importData[$key]['drawing_no']      = (empty($rows['drawing_no']) ? '' : $rows['drawing_no']);
                    $importData[$key]['component_name']  = (empty($rows['component_name']) ? '' : $rows['component_name']);
                    $importData[$key]['component_price'] = (empty($rows['component_price']) ? 0 : $rows['component_price']);
                    $importData[$key]['gst']             = (empty($rows['gst']) ? 0 : $rows['gst']);
                }
            }
        #delete cache
            Cache::clear(true);
        #set cache
            Cache::write('importVendorComponentsData', $importData);
            return $this->redirect(['action' => 'VendorComponentConfirm']);
        }
    }

    /**
     * import confirm method
     *
     */
    public function VendorComponentConfirm()
    {
    #set value
        $importData = Cache::read('importVendorComponentsData');
    #check method
        if ($this->request->is('post')) {
            $vendor_components_array = [];
        #array each
            foreach ($importData as $key => $rows) {
                $VendorId = null;
            #Vendor name
                if (!empty($rows['vendor_code'])) {
                #check data
                    $vendor_code = $rows['vendor_code'];
                    $vData       = $this->Vendors->find()->where(['vendor_code' => $vendor_code, 'is_active' => 1])->first();
                #check empty
                    if (!empty($vData)) {
                        $VendorId = $vData->id;
                    }
                #component name
                    if (!empty($VendorId) && $VendorId != null) {
                    #convert string
                        $component_name       = $rows['component_name'];
                        $drawing_reference_no = str_replace('/', '_', $rows['drawing_no']);
                    #check data
                        $cData = $this->Components->find()
                                    ->where([
                                        // 'component_name'     => $component_name,
                                        'drawing_reference_no'  => $drawing_reference_no,
                                        'is_active'             => 1
                                    ])
                                    ->first();
                    #check empty
                        if (!empty($cData)) {
                            $componentId                                   = $cData->id;
                            $vendor_components_array[$key]['vendor_id']    = $VendorId;
                            $vendor_components_array[$key]['component_id'] = $componentId;
                            $vendor_components_array[$key]['price']        = $rows['component_price'];
                            $vendor_components_array[$key]['gst']          = $rows['gst'];
                            $vendor_components_array[$key]['is_active']    = 1;
                        }
                    }
                }
            }
        #end of for each

        #prepare vendor component entities
            if (!empty($vendor_components_array)) {
                $entities = $this->VendorComponents->newEntities($vendor_components_array);
                $saveEntities = $this->VendorComponents->saveMany($entities);
            #check empty
                if ($saveEntities) {
                #clear cache
                    Cache::clear(true);
                    $this->Flash->success(__('The {0} has been saved.', 'Vendor Components'));
                    return $this->redirect(['action' => 'index']);
                } else {
                    $this->Flash->error(__('The {0} could not be saved. Please, try again.', 'Vendor Components'));
                }
            }
        }
    #set time
        $this->set(compact('importData'));
    }

    #Vednor Raw Material Import Method
    /**
     * import method
     *
     */
    public function VendorRawMaterialImport()
    {
    #variable declaration
        $importData = [];
    #check request method
        if ($this->request->is('post')) {
        #checks file empty
            if (file_exists($_FILES['import_file']['tmp_name']) || is_uploaded_file($_FILES['import_file']['tmp_name'])) {
            #import data
                $importData = $this->Vendors->importCsv($this->request
                                ->data['import_file']['tmp_name'],
                                [
                                    'vendor_code',
                                    'raw_material_name',
                                    'raw_material_price',
                                    'gst',
                                ]);
                $this->result = [];

            #foreach
                foreach ($importData as $key => $rows) {
                    $importData[$key]['vendor_code']        = (empty($rows['vendor_code']) ? '' : $rows['vendor_code']);
                    $importData[$key]['raw_material_name']  = (empty($rows['raw_material_name']) ? '' : $rows['raw_material_name']);
                    $importData[$key]['raw_material_price'] = (empty($rows['raw_material_price']) ? 0 : $rows['raw_material_price']);
                    $importData[$key]['gst']                = (empty($rows['gst']) ? 0 : $rows['gst']);
                }
            }
        #delete cache
            Cache::clear(true);
        #set cache
            Cache::write('importVendorRawMaterialsData', $importData);
            return $this->redirect(['action' => 'VendorRawMaterialConfirm']);
        }
    }

    /**
     * import confirm method
     *
     */
    public function VendorRawMaterialConfirm()
    {
    #set value
        $importData = Cache::read('importVendorRawMaterialsData');
    #check method
        if ($this->request->is('post')) {
            $vendor_raw_materials_array = [];
        #array each
            foreach ($importData as $key => $rows) {
                $VendorId = null;
            #Vendor name
                if (!empty($rows['vendor_code'])) {
                #check data
                    $vendor_code = $rows['vendor_code'];
                    $vData       = $this->Vendors->find()->where(['vendor_code' => $vendor_code, 'is_active' => 1])->first();
                #check empty
                    if (!empty($vData)) {
                        $VendorId = $vData->id;
                    }
                #component name
                    if (!empty($VendorId) && $VendorId != null) {
                    #convert string
                        $raw_material_name       = $rows['raw_material_name'];
                    #check data
                        $rData = $this->RawMaterials->find()
                                    ->where([
                                        'material_name' => $raw_material_name, 
                                        'is_active' => 1
                                    ])->first();
                        if (!empty($rData)) {
                            $raw_material_id                                   = $rData->id;
                            $vendor_raw_materials_array[$key]['vendor_id']    = $VendorId;
                            $vendor_raw_materials_array[$key]['raw_material_id'] = $raw_material_id;
                            $vendor_raw_materials_array[$key]['price']        = $rows['raw_material_price'];
                            $vendor_raw_materials_array[$key]['gst']          = $rows['gst'];
                            $vendor_raw_materials_array[$key]['is_active']    = 1;
                        }
                    }
                }
            }
        #end of for each

        #prepare vendor component entities
            if (!empty($vendor_raw_materials_array)) {
            #save
                $entities       = $this->VendorRawMaterials->newEntities($vendor_raw_materials_array);
                $saveEntities   = $this->VendorRawMaterials->saveMany($entities);
            #check empty
                if ($saveEntities) {
                #clear cache
                    Cache::clear(true);
                    $this->Flash->success(__('The {0} has been saved.', 'Vendor RawMaterials'));
                    return $this->redirect(['action' => 'index']);
                } else {
                    $this->Flash->error(__('The {0} could not be saved. Please, try again.', 'Vendor RawMaterials'));
                }
            }
        }
    #set time
        $this->set(compact('importData'));
    }

    /**
     * import  VendorProcess method
     *
     */
    public function VendorProcessImport()
    {
    #variable declaration
        $importData = [];
        $this->loadModel('Vendors');
    #check request method
        if ($this->request->is('post')) {
            if (file_exists($_FILES['import_file']['tmp_name']) || is_uploaded_file($_FILES['import_file']['tmp_name'])) {
                $importData = $this->Vendors->importCsv($this->request
                                ->data['import_file']['tmp_name'],
                                [
                                    'vendor_code',
                                    'drawing_no',
                                    'component_name',
                                    'process',
                                    'process_price',
                                    'gst',
                                ]);
                $this->result = [];

            #foreach
                foreach ($importData as $key => $rows) {
                    $importData[$key]['vendor_code']     = (empty($rows['vendor_code']) ? '' : $rows['vendor_code']);
                    $importData[$key]['drawing_no']      = (empty($rows['drawing_no']) ? 0 : $rows['drawing_no']);
                    $importData[$key]['component_name']  = (empty($rows['component_name']) ? '' : $rows['component_name']);
                    $importData[$key]['process'] = (empty($rows['process']) ? 0 : $rows['process']);
                    $importData[$key]['process_price'] = (empty($rows['process_price']) ? 0 : $rows['process_price']);
                    $importData[$key]['gst']             = (empty($rows['gst']) ? 0 : $rows['gst']);
                }
            }
        #delete cache
            Cache::clear(true);
        #set cache
            Cache::write('importVendorProcessesData', $importData);
            return $this->redirect(['action' => 'VendorProcessConfirm']);
        }
    }

    /**
     * import VendorProcess confirm method
     *
     */
    public function VendorProcessConfirm()
    {
    #set value
        $importData = Cache::read('importVendorProcessesData');
    #check method
        if ($this->request->is('post')) {
            $vendor_processes_array = [];
        #array each
            foreach ($importData as $key => $rows) {
                $VendorId = null;
            #Vendor name
                if (!empty($rows['vendor_code'])) {
                #check data
                    $vendor_code = $rows['vendor_code'];
                    $vData       = $this->Vendors->find()->where(['vendor_code' => $vendor_code, 'is_active' => 1])->first();
                    if (!empty($vData)) {
                        $VendorId = $vData->id;
                    }
                #component name
                    if (!empty($VendorId) && $VendorId != null) {
                    #convert string
                        $component_name       = $rows['component_name'];
                        $drawing_reference_no = str_replace('/', '_', $rows['drawing_no']);
                    #check data
                        $cData = $this->Components->find()
                                    ->where([
                                        // 'component_name' => $component_name,
                                        'drawing_reference_no' => $drawing_reference_no, 'is_active' => 1
                                    ])
                                    ->first();
                    #check data
                        $process_name   = $rows['process'];
                        $pData          = $this->Processes->find()
                                            ->where([
                                                // 'component_name' => $component_name,
                                                'process_name' => $process_name, 'is_active' => 1
                                            ])->first();
                    #check empty
                        if (!empty($cData) && !empty($pData)) {
                            $componentId                                    = $cData->id;
                            $processId                                      = $pData->id;
                            $vendor_processes_array[$key]['vendor_id']      = $VendorId;
                            $vendor_processes_array[$key]['component_id']   = $componentId;
                            $vendor_processes_array[$key]['process_id']     = $processId;
                            $vendor_processes_array[$key]['price']          = $rows['process_price'];
                            $vendor_processes_array[$key]['gst']            = $rows['gst'];
                            $vendor_processes_array[$key]['is_active']      = 1;
                        }
                    }
                }
            }
        #end of for each

        #prepare vendor component entities
            if (!empty($vendor_processes_array)) {
                $entities       = $this->VendorProcesses->newEntities($vendor_processes_array);
                $saveEntities   = $this->VendorProcesses->saveMany($entities);
            #check empty
                if ($saveEntities) {
                #clear cache
                    Cache::clear(true);
                    $this->Flash->success(__('The {0} has been saved.', 'Vendor Processes'));
                    return $this->redirect(['action' => 'index']);
                } else {
                    $this->Flash->error(__('The {0} could not be saved. Please, try again.', 'Vendor Processes'));
                }
            }
        }
    #set time
        $this->set(compact('importData'));
    }

    #Vednor store components Import Method
    /**
     * import method
     *
     */
    public function VendorStoreComponentsImport()
    {
    #variable declaration
        $importData = [];
    #check request method
        if ($this->request->is('post')) {
            if (file_exists($_FILES['import_file']['tmp_name']) || is_uploaded_file($_FILES['import_file']['tmp_name'])) {
                $importData = $this->Vendors->importCsv($this->request
                                ->data['import_file']
                                ['tmp_name'],
                                [
                                    'vendor_code',
                                    'component_name',
                                    'price',
                                    'gst',
                                ]);
                $this->result = [];

            #foreach
                foreach ($importData as $key => $rows) {

                    $importData[$key]['vendor_code']     = (empty($rows['vendor_code']) ? '' : $rows['vendor_code']);
                    $importData[$key]['component_name']      = (empty($rows['component_name']) ? '' : $rows['component_name']);
                    $importData[$key]['price']  = (empty($rows['price']) ? 0 : $rows['price']);                    
                    $importData[$key]['gst']             = (empty($rows['gst']) ? 0 : $rows['gst']);
                }
            }
        #delete cache
            Cache::clear(true);
        #set cache
            Cache::write('importStoreComponentsData', $importData);
            return $this->redirect(['action' => 'VendorStoreComponentsConfirm']);
        }
    }

    /**
     * Vendor Store Components confirm method
     *
     */
    public function VendorStoreComponentsConfirm()
    {
    #set value
        $importData = Cache::read('importStoreComponentsData');
    #check method
        if ($this->request->is('post')) {
            $vendor_store_components_array = [];
        #array each
            foreach ($importData as $key => $rows) {
                $VendorId = null;
            #Vendor name
                if (!empty($rows['vendor_code'])) {
                #check data
                    $vendor_code = $rows['vendor_code'];
                    $vData       = $this->Vendors->find()->where(['vendor_code' => $vendor_code, 'is_active' => 1])->first();
                #check empty
                    if (!empty($vData)) {
                        $VendorId = $vData->id;
                    }
                #component name
                    if (!empty($VendorId) && $VendorId != null) {
                    #convert string
                        $component_name     = $rows['component_name'];
                    #check data
                        $storeComponentData = $this->StoreComponents->find()
                                                ->where([
                                                    'component_name' => $component_name, 
                                                    'is_active' => 1
                                                ])
                                                ->first();
                    #check empty
                        if (!empty($storeComponentData)) {
                            $storComponentId                                            = $storeComponentData->id;
                            $vendor_store_components_array[$key]['vendor_id']           = $VendorId;
                            $vendor_store_components_array[$key]['store_component_id']  = $storComponentId;
                            $vendor_store_components_array[$key]['price']               = $rows['price'];
                            $vendor_store_components_array[$key]['gst']                 = $rows['gst'];
                            $vendor_store_components_array[$key]['is_active']           = 1;
                        }
                    }
                }
            }
        #end of for each

        #prepare vendor component entities
            if (!empty($vendor_store_components_array)) {
                $entities       = $this->VendorStoreComponents->newEntities($vendor_store_components_array);
                $saveEntities   = $this->VendorStoreComponents->saveMany($entities);
            #check empty
                if ($saveEntities) {
                #clear cache
                    Cache::clear(true);
                    $this->Flash->success(__('The {0} has been saved.', 'Vendor Store Components'));
                    return $this->redirect(['action' => 'index']);
                } else {
                    $this->Flash->error(__('The {0} could not be saved. Please, try again.', 'Vendor Store Components'));
                }
            }
        }
    #set time
        $this->set(compact('importData'));
    }
}
