<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;

/**
 * AnnualComponentStatusTrackings Model
 *
 * @property \App\Model\Table\AnnualComponentTrackingsTable|\Cake\ORM\Association\BelongsTo $AnnualComponentTrackings
 *
 * @method \App\Model\Entity\AnnualComponentStatusTracking get($primaryKey, $options = [])
 * @method \App\Model\Entity\AnnualComponentStatusTracking newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\AnnualComponentStatusTracking[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\AnnualComponentStatusTracking|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualComponentStatusTracking|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualComponentStatusTracking patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualComponentStatusTracking[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualComponentStatusTracking findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class AnnualComponentStatusTrackingsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('annual_component_status_trackings');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('AnnualComponentTrackings', [
            'foreignKey' => 'annual_component_tracking_id',
            'joinType' => 'INNER'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    
    #Add search behavior
        $this->addBehavior('Search.Search');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->integer('total_components')
            ->requirePresence('total_components', 'create')
            ->allowEmptyString('total_components', false);

        $validator
            ->integer('total_completed_components')
            ->requirePresence('total_completed_components', 'create')
            ->allowEmptyString('total_completed_components', false);

        $validator
            ->integer('total_rejected_components')
            ->requirePresence('total_rejected_components', 'create')
            ->allowEmptyString('total_rejected_components', false);

        $validator
            ->requirePresence('status', 'create')
            ->allowEmptyString('status', false);

        $validator
            ->dateTime('status_created_on')
            ->allowEmptyDateTime('status_created_on');

        $validator
            ->scalar('description')
            ->allowEmptyString('description');

        $validator
            ->integer('is_last_updated')
            ->requirePresence('is_last_updated', 'create')
            ->allowEmptyString('is_last_updated', false);

        $validator
            ->requirePresence('is_undo', 'create')
            ->allowEmptyString('is_undo', false);

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['annual_component_tracking_id'], 'AnnualComponentTrackings'));

        return $rules;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('product_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){
                if(!empty($args['product_id'])){
                    return $query->join([
                        'AnnualComponentTrackings' => [
                            'table'         => 'products',
                            'type'          => 'inner',
                            'conditions'    => ['AnnualComponentTrackings.id=AnnualComponentStatusTrackings.annual_component_tracking_id']
                        ],
                        'ComponentProducts' => [
                            'table'         => 'component_products',
                            'type'          => 'inner',
                            'conditions'    => ['ComponentProducts.id=AnnualComponentTrackings.component_product_id']
                        ],
                    ])
                    ->where(['ComponentProducts.product_id' => $args['product_id']]);
                }
                return $query;
            }
        ])
        ->add('group_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){
                if(!empty($args['group_id'])){
                    return $query->join([
                        'AnnualComponentTrackings' => [
                            'table'         => 'annual_component_trackings',
                            'type'          => 'inner',
                            'conditions'    => ['AnnualComponentTrackings.id=AnnualComponentStatusTrackings.annual_component_tracking_id']
                        ],
                        'ComponentProducts' => [
                            'table'         => 'component_products',
                            'type'          => 'inner',
                            'conditions'    => ['ComponentProducts.id=AnnualComponentTrackings.component_product_id']
                        ],
                        'ComponentGroups' => [
                            'table'         => 'component_groups',
                            'type'          => 'inner',
                            'conditions'    => ['ComponentGroups.component_id=ComponentProducts.component_id']
                        ],
                    ])
                    ->where(['ComponentGroups.group_id' => $args['group_id']]);
                }
                return $query;
            }
        ])
        ->add('product_number', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){
                if(!empty($args['product_number'])){
                    return $query->join([
                        'AnnualComponentTrackings' => [
                            'table'         => 'annual_component_trackings',
                            'type'          => 'inner',
                            'conditions'    => ['AnnualComponentTrackings.id=AnnualComponentStatusTrackings.annual_component_tracking_id']
                        ],
                    ])
                    ->where(['AnnualComponentTrackings.product_number' => $args['product_number']]);
                }
                return $query;
            }
        ])
        ->add('annual_plan_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){
                if(!empty($args['annual_plan_id'])){
                    return $query->join([
                        'AnnualComponentTrackings' => [
                            'table'         => 'annual_component_trackings',
                            'type'          => 'inner',
                            'conditions'    => ['AnnualComponentTrackings.id=AnnualComponentStatusTrackings.annual_component_tracking_id']
                        ],
                        'AnnualQuarterPlans' => [
                            'table'     => 'annual_quarter_plans',
                            'type'		=> 'INNER',
                            'conditions'=> ['AnnualQuarterPlans.id=AnnualComponentTrackings.annual_quarter_plan_id']
                        ],
                    ])
                    ->where(['AnnualQuarterPlans.annual_plan_id' => $args['annual_plan_id']]);
                }
                return $query;
            }
        ])
        ->add('status', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){
                if(!empty($args['status'])){
                    return $query->where(['AnnualComponentStatusTrackings.status' => $args['status']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
