<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;
/**
 * AnnualPlans Model
 *
 * @property \App\Model\Table\AnnualPlannedCapacitiesTable|\Cake\ORM\Association\HasMany $AnnualPlannedCapacities
 * @property \App\Model\Table\AnnualPlannedMachinesTable|\Cake\ORM\Association\HasMany $AnnualPlannedMachines
 * @property \App\Model\Table\AnnualPlannedProductsTable|\Cake\ORM\Association\HasMany $AnnualPlannedProducts
 * @property \App\Model\Table\AnnualQuarterPlansTable|\Cake\ORM\Association\HasMany $AnnualQuarterPlans
 *
 * @method \App\Model\Entity\AnnualPlan get($primaryKey, $options = [])
 * @method \App\Model\Entity\AnnualPlan newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\AnnualPlan[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\AnnualPlan|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualPlan|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualPlan patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualPlan[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualPlan findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class AnnualPlansTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('annual_plans');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->hasMany('AnnualPlannedCapacities', [
            'foreignKey' => 'annual_plan_id'
        ]);
        $this->hasMany('AnnualPlannedMachines', [
            'foreignKey' => 'annual_plan_id'
        ]);
        $this->hasMany('AnnualPlannedProducts', [
            'foreignKey' => 'annual_plan_id'
        ]);
        $this->hasMany('AnnualQuarterPlans', [
            'foreignKey' => 'annual_plan_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');

    #Add search behavior
        $this->addBehavior('Search.Search');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('financial_year')
            ->maxLength('financial_year', 50)
            ->requirePresence('financial_year', 'create')
            ->allowEmptyString('financial_year', false);

        $validator
            ->scalar('month_from')
            ->maxLength('month_from', 25)
            ->requirePresence('month_from', 'create')
            ->allowEmptyString('month_from', false);

        $validator
            ->scalar('month_to')
            ->maxLength('month_to', 25)
            ->requirePresence('month_to', 'create')
            ->allowEmptyString('month_to', false);

        $validator
            ->allowEmptyString('status');

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /*
    *build rules
    */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->isUnique(['financial_year'], 'Year plan already exist'));
        return $rules;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('annual_plan_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if(!empty($args['annual_plan_id'])){
                    return $query
                        ->join([
                            'AnnualPlannedProducts' => [
                                'table'     => 'annual_planned_products',
                                'type'		=> 'INNER',
                                'conditions'=> ['AnnualPlannedProducts.annual_plan_id=AnnualPlans.id']
                            ],
                        ])
                        ->where([
                            'AnnualPlannedProducts.annual_plan_id' => $args['annual_plan_id']
                        ]);
                }
                return $query;
            }
        ])
        ->add('component_name', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if(!empty($args['component_name'])){
                    return $query
                        ->join([
                            'AnnualPlannedProducts' => [
                                'table'     => 'annual_planned_products',
                                'type'		=> 'INNER',
                                'conditions'=> ['AnnualPlannedProducts.annual_plan_id=AnnualPlans.id']
                            ],
                            'ComponentProducts' => [
                                'table'     => 'component_products',
                                'type'		=> 'INNER',
                                'conditions'=> ['ComponentProducts.product_id=AnnualPlannedProducts.product_id']
                            ],
                            'Components' => [
                                'table'     => 'components',
                                'type'		=> 'INNER',
                                'conditions'=> ['Components.id=ComponentProducts.component_id']
                            ],
                        ])
                        ->where([
                            'Components.component_name' => $args['component_name']
                        ]);
                }
                return $query;
            }
        ])
        ->add('raw_material_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if(!empty($args['raw_material_id'])){
                    return $query
                        ->join([
                            'AnnualPlannedProducts' => [
                                'table'     => 'annual_planned_products',
                                'type'		=> 'INNER',
                                'conditions'=> ['AnnualPlannedProducts.annual_plan_id=AnnualPlans.id']
                            ],
                            'ComponentProducts' => [
                                'table'     => 'component_products',
                                'type'		=> 'INNER',
                                'conditions'=> ['ComponentProducts.product_id=AnnualPlannedProducts.product_id']
                            ],
                            'Components' => [
                                'table'     => 'components',
                                'type'		=> 'INNER',
                                'conditions'=> ['Components.id=ComponentProducts.component_id']
                            ],
                            'ComponentRawMaterials' => [
                                'table'     => 'component_raw_materials',
                                'type'		=> 'INNER',
                                'conditions'=> ['ComponentRawMaterials.component_id=Components.id']
                            ],
                        ])->where([
                            'ComponentRawMaterials.raw_material_id' => $args['raw_material_id']
                        ]);
                }
                return $query;
            }
        ])
        ->add('group_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if(!empty($args['group_id'])){
                    return $query
                        ->join([
                            'AnnualPlannedProducts' => [
                                'table'     => 'annual_planned_products',
                                'type'		=> 'INNER',
                                'conditions'=> ['AnnualPlannedProducts.annual_plan_id=AnnualPlans.id']
                            ],
                            'ComponentProducts' => [
                                'table'     => 'component_products',
                                'type'		=> 'INNER',
                                'conditions'=> ['ComponentProducts.product_id=AnnualPlannedProducts.product_id']
                            ],
                            'ComponentGroups' => [
                                'table'     => 'component_groups',
                                'type'		=> 'INNER',
                                'conditions'=> ['ComponentGroups.component_id=ComponentProducts.component_id']
                            ],
                        ])->where([
                            'ComponentGroups.group_id' => $args['group_id']
                        ]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
