<?php
namespace App\Model\Table;

use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\ORM\TableRegistry;
use Cake\Validation\Validator;

/**
 * AnnualQuarterPlans Model
 *
 * @property \App\Model\Table\AnnualPlansTable|\Cake\ORM\Association\BelongsTo $AnnualPlans
 * @property \App\Model\Table\AnnualAssemblyTrackingsTable|\Cake\ORM\Association\HasMany $AnnualAssemblyTrackings
 * @property \App\Model\Table\AnnualComponentTrackingsTable|\Cake\ORM\Association\HasMany $AnnualComponentTrackings
 * @property \App\Model\Table\AnnualProductTrackingsTable|\Cake\ORM\Association\HasMany $AnnualProductTrackings
 * @property \App\Model\Table\AnnualQuarterPlannedProductsTable|\Cake\ORM\Association\HasMany $AnnualQuarterPlannedProducts
 * @property \App\Model\Table\PurchaseIndentsTable|\Cake\ORM\Association\HasMany $PurchaseIndents
 *
 * @method \App\Model\Entity\AnnualQuarterPlan get($primaryKey, $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlan newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlan[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlan|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlan|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlan patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlan[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlan findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class AnnualQuarterPlansTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('annual_quarter_plans');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('AnnualPlans', [
            'foreignKey' => 'annual_plan_id',
        ]);
        $this->hasMany('AnnualAssemblyTrackings', [
            'foreignKey' => 'annual_quarter_plan_id',
        ]);
        $this->hasMany('AnnualComponentTrackings', [
            'foreignKey' => 'annual_quarter_plan_id',
        ]);
        $this->hasMany('AnnualProductTrackings', [
            'foreignKey' => 'annual_quarter_plan_id',
        ]);
        $this->hasMany('AnnualQuarterPlannedProducts', [
            'foreignKey' => 'annual_quarter_plan_id',
        ]);
        $this->hasMany('PurchaseIndents', [
            'foreignKey' => 'annual_quarter_plan_id',
        ]);

    #foot print
        $this->addBehavior('Muffin/Footprint.Footprint');

    #audit log
        $this->addBehavior('AuditStash.AuditLog');
        
    #audit log
        $this->addBehavior('AuditStash.AuditLog');

    #Add search behavior
        $this->addBehavior('Search.Search');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('duration')
            ->maxLength('duration', 50)
            ->allowEmptyString('duration');

        $validator
            ->requirePresence('status', 'create')
            ->allowEmptyString('status', false);

        $validator
            ->requirePresence('is_active', 'create')
            ->allowEmptyString('is_active', false);

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['annual_plan_id'], 'AnnualPlans'));

        return $rules;
    }

    public function afterSave1($event, $entity, $options)
    {
        $annual_quarter_planned_products = TableRegistry::get('AnnualQuarterPlannedProducts')->find('all')->where(['AnnualQuarterPlannedProducts.annual_quarter_plan_id' => $entity->id])->toArray();

        foreach ($annual_quarter_planned_products as $annual_quarter_planned_products_key => $annual_quarter_planned_products_value) {

            $product_id = $annual_quarter_planned_products_value->product_id;
            $groups     = TableRegistry::get('Groups');
            
            $grups_data = $groups->find()->contain(['Components' => function ($q) use ($product_id) {
                return $q->contain(['AnnualPlannedCapacities' => ['joinType' => 'left'], 'Products' => function ($q) use ($product_id) {
                    return $q->where(['Products.id' => $product_id]);
                },
                ]);
            },
            ])
            ->toArray();

            foreach ($grups_data as $grups_data_key => $grups_data_value) {
                if (!empty($grups_data_value->components)) {
                    $purchase_idents_components = [];

                    $purchase_idents['purchase_idents']['annual_quarter_plan_id'] = $entity->id;
                    $purchase_idents['purchase_idents']['product_id']             = $annual_quarter_planned_products_value->product_id;

                    $purchase_idents['purchase_idents']['group_id']           = $grups_data_value->id;
                    $purchase_idents['purchase_idents']['purchase_intent_no'] = $grups_data_value->group_name . date("Y") . (mt_rand(100, 999));

                    $purchase_idents['purchase_idents']['status']      = 0;
                    $purchase_idents['purchase_idents']['is_active']   = 1;
                    $purchase_idents['purchase_idents']['`is_adhoc` '] = 0;

                #Add purchase Indent components
                    foreach ($grups_data_value->components as $components_key => $components_value) {

                        if (isset($components_value->products[0])) {

                            if (!empty($components_value->annual_planned_capacities)) {
                                $purchase_idents['purchase_idents']['tag'] = $components_value->annual_planned_capacities[0]->tag;
                            } else {
                                $purchase_idents['purchase_idents']['tag'] = $components_value->tag;
                            }
                            $purchase_idents_components[$components_key]['component_id']    = $components_value->id;
                            $purchase_idents_components[$components_key]['raw_material_id'] = 1;

                            $purchase_idents_components[$components_key]['total_quantity'] =
                            $components_value->products[0]['_joinData']->number_of_component * (
                                $annual_quarter_planned_products_value->product_quantity + $annual_quarter_planned_products_value->product_buffer_quantity
                            );
                            $purchase_idents_components[$components_key]['is_active'] = 1;

                        }
                    }
                    $purchase_idents['purchase_idents']['purchase_indent_components'] = $purchase_idents_components;

                   // $entities = TableRegistry::get('PurchaseIndents')->newEntities($purchase_idents);
                    //$result   = TableRegistry::get('PurchaseIndents')->saveMany($entities);

                }
            }
            #Save Purchase Indent
        }
    }
}
