<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * ComponentProcesses Model
 *
 * @property \App\Model\Table\ComponentsTable|\Cake\ORM\Association\BelongsTo $Components
 * @property \App\Model\Table\MachinesTable|\Cake\ORM\Association\BelongsTo $Machines
 * @property \App\Model\Table\ProcessesTable|\Cake\ORM\Association\BelongsTo $Processes
 *
 * @method \App\Model\Entity\ComponentProcess get($primaryKey, $options = [])
 * @method \App\Model\Entity\ComponentProcess newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\ComponentProcess[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\ComponentProcess|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\ComponentProcess|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\ComponentProcess patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\ComponentProcess[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\ComponentProcess findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class ComponentProcessesTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('component_processes');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('Components', [
            'foreignKey' => 'component_id'
        ]);
        $this->belongsTo('Machines', [
            'foreignKey' => 'machine_id'
        ]);
        $this->belongsTo('Processes', [
            'foreignKey' => 'process_id'
        ]);

	//foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	//audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

       /*  $validator
            ->numeric('average_time_taken')
            ->requirePresence('average_time_taken', 'create')
            ->allowEmptyString('average_time_taken', false);

        $validator
            ->numeric('set_time_for_batch')
            ->requirePresence('set_time_for_batch', 'create')
            ->allowEmptyString('set_time_for_batch', false); */

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['component_id'], 'Components'));
        // $rules->add($rules->existsIn(['machine_id'], 'Machines'));
        $rules->add($rules->existsIn(['process_id'], 'Processes'));

        return $rules;
    }
}
