<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;

/**
 * Components Model
 *
 * @property \App\Model\Table\AnnualPlannedCapacitiesTable|\Cake\ORM\Association\HasMany $AnnualPlannedCapacities
 * @property \App\Model\Table\ComponentAssembliesTable|\Cake\ORM\Association\HasMany $ComponentAssemblies
 * @property \App\Model\Table\ComponentGroupsTable|\Cake\ORM\Association\HasMany $ComponentGroups
 * @property \App\Model\Table\ComponentProcessesTable|\Cake\ORM\Association\HasMany $ComponentProcesses
 * @property \App\Model\Table\ComponentProductsTable|\Cake\ORM\Association\HasMany $ComponentProducts
 * @property \App\Model\Table\ComponentRawMaterialsTable|\Cake\ORM\Association\HasMany $ComponentRawMaterials
 * @property |\Cake\ORM\Association\HasMany $JobOrderComponents
 * @property \App\Model\Table\PurchaseIndentComponentsTable|\Cake\ORM\Association\HasMany $PurchaseIndentComponents
 * @property \App\Model\Table\PurchaseOrderComponentsTable|\Cake\ORM\Association\HasMany $PurchaseOrderComponents
 * @property |\Cake\ORM\Association\HasMany $PurchaseOrdersTemp
 * @property |\Cake\ORM\Association\HasMany $SpareInventoryComponents
 * @property \App\Model\Table\VendorComponentsTable|\Cake\ORM\Association\HasMany $VendorComponents
 *
 * @method \App\Model\Entity\Component get($primaryKey, $options = [])
 * @method \App\Model\Entity\Component newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\Component[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\Component|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Component|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Component patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\Component[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\Component findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class ComponentsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('components');
        $this->setDisplayField('component_name');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->hasMany('AnnualPlannedCapacities', [
            'foreignKey' => 'component_id'
        ]);
        $this->hasMany('ComponentAssemblies', [
            'foreignKey' => 'component_id'
        ]);
        $this->hasMany('ComponentProcesses', [
            'foreignKey' => 'component_id'
        ]);
        $this->hasMany('ComponentGroups', [
            'foreignKey' => 'component_id'
        ]);
        $this->belongsToMany('SubAssemblies', [
            'foreignKey'        => 'component_id',
            'targetForiegnKey'  => 'sub_assembly_id',
            'joinTable'         => 'component_assemblies'
        ]);
        $this->belongsToMany('Groups', [
            'foreignKey'        => 'component_id',
            'targetForiegnKey'  => 'group_id',
            'joinTable'         => 'component_groups'
        ]);
        
        $this->belongsToMany('Processes', [
            'foreignKey' => 'component_id',
            'targetForiegnKey' => 'process_id',
            'joinTable' => 'component_processes'
        ]);

        $this->hasMany('ComponentProducts', [
            'foreignKey' => 'component_id'
        ]);
        
        $this->belongsToMany('Products', [
            'foreignKey' => 'component_id',
            'targetForiegnKey' => 'product_id',
            'joinTable' => 'component_products'
        ]);
        
        $this->hasMany('ComponentRawMaterials', [
            'foreignKey' => 'component_id'
        ]);
        $this->hasMany('JobOrderComponents', [
            'foreignKey' => 'component_id'
        ]);
        $this->hasMany('PurchaseIndentComponents', [
            'foreignKey' => 'component_id'
        ]);
        $this->hasMany('PurchaseOrderComponents', [
            'foreignKey' => 'component_id'
        ]);
        $this->hasMany('PurchaseOrdersTemp', [
            'foreignKey' => 'component_id'
        ]);
        $this->hasMany('SpareInventoryComponents', [
            'foreignKey' => 'component_id'
        ]);
        $this->hasMany('VendorComponents', [
            'foreignKey' => 'component_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    
    #Add search behavior
        $this->addBehavior('Search.Search');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('component_name')
            ->maxLength('component_name', 100)
            ->requirePresence('component_name', 'create')
            ->allowEmptyString('component_name', false);

        $validator
            ->scalar('component_code')
            ->maxLength('component_code', 100)
            ->allowEmptyString('component_code');

        $validator
            ->scalar('drawing_reference_no')
            ->maxLength('drawing_reference_no', 100)
            ->requirePresence('drawing_reference_no', 'create')
            ->allowEmptyString('drawing_reference_no', false);

        $validator
            ->numeric('component_cost')
            ->allowEmptyString('component_cost');

        $validator
            ->numeric('component_gst')
            ->requirePresence('component_gst', 'create')
            ->allowEmptyString('component_gst', false);

        $validator
            ->integer('batch_quantity')
            ->requirePresence('batch_quantity', 'create')
            ->allowEmptyString('batch_quantity', false);

        $validator
            ->integer('po_turnaround_time')
            ->requirePresence('po_turnaround_time', 'create')
            ->allowEmptyString('po_turnaround_time', false);

        $validator
            ->integer('buying_frequency')
            ->requirePresence('buying_frequency', 'create')
            ->allowEmptyString('buying_frequency', false);

        $validator
            ->scalar('drawing_file')
            ->maxLength('drawing_file', 255)
            ->allowEmptyFile('drawing_file');

        $validator
            ->scalar('drawing_path')
            ->maxLength('drawing_path', 255)
            ->allowEmptyString('drawing_path');

        $validator
            ->allowEmptyString('tag');

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    public function findByGroup(\Cake\ORM\Query $query, array $options)
    {
        $query
            ->contain([
                'Groups'
            ])
            ->first();
        return $query;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('component_name', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['Components.component_name']
        ])
        ->add('drawing_reference_no', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['Components.drawing_reference_no']
        ])
        ->add('group_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if(!empty($args['group_id'])){
                    return $query
                        ->matching('ComponentGroups', function (Query $query) use ($args) {
                            return $query
                            ->where(function (QueryExpression $exp, Query $q) use($args) {
                                return $exp->in('ComponentGroups.group_id', $args['group_id']);
                            });
                        });
                }
                return $query;
            }
        ])
        ->add('product_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if(!empty($args['product_id'])){
                    return $query
                        ->matching('ComponentProducts', function (Query $query) use ($args) {
                            return $query
                            ->where(function (QueryExpression $exp, Query $q) use($args) {
                                return $exp->in('ComponentProducts.product_id', $args['product_id']);
                            });
                        });
                }
                return $query;
            }
        ])
        ->add('sub_assembly_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if(!empty($args['sub_assembly_id'])){
                    return $query
                        ->matching('ComponentAssemblies', function (Query $query) use ($args) {
                            return $query
                            ->where(function (QueryExpression $exp, Query $q) use($args) {
                                return $exp->in('ComponentAssemblies.sub_assembly_id', $args['sub_assembly_id']);
                            });
                        });
                }
                return $query;
            }
        ])
        ->add('tag', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['tag']!=null){
                    return $query->where(['Components.tag' => $args['tag']]);
                }
                return $query;
            }
        ])
        ->add('is_active', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['is_active']!=null){
                    return $query->where(['Components.is_active' => $args['is_active']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }

    /*
        *after save method
        *after component status updated change status in assembly tracking and product tracking status and count
    */
    public function afterSave($event, $entity, $options)
    {
    #not new update assembly component count
        if( !$entity->isNew() ) {
        #find assembly product tracking
            $annualAssemblyTrackings = TableRegistry::get('AnnualAssemblyTrackings')
                                        ->find()
                                        ->join([
                                            'AnnualProductTrackings' => [
                                                'table'     => 'annual_product_trackings',
                                                'type'		=> 'INNER',
                                                'conditions'=> ['AnnualProductTrackings.id=AnnualAssemblyTrackings.annual_product_tracking_id']
                                            ],
                                        ])
                                        ->where([
                                            'AnnualProductTrackings.status !='  => 4,
                                        ]);
        #prepare data
            $annualAssemblyTrackingsData = $annualAssemblyTrackings
                                            ->select([
                                                'id'                => 'AnnualAssemblyTrackings.id',
                                                'status'            => '(IF(AnnualAssemblyTrackings.total_component=AnnualAssemblyTrackings.completed_component, 0, AnnualAssemblyTrackings.status))',
                                                'total_component'   => '(SELECT COUNT(*) FROM `component_assemblies` AS CA INNER JOIN `component_products` AS CP ON CP.`component_id`=CA.`component_id` INNER JOIN `components` AS C ON C.`id`=CP.`component_id` AND C.`is_active`=1 WHERE CA.`sub_assembly_id`=AnnualAssemblyTrackings.`sub_assembly_id` AND CP.`product_id`=AnnualAssemblyTrackings.`product_id`  AND ( (CP.`start_product_no`=0 AND CP.`end_product_no`=0) OR (CP.`start_product_no`<=AnnualProductTrackings.product_number AND CP.`end_product_no`>=AnnualProductTrackings.product_number) ) GROUP BY CA.`sub_assembly_id`)',
                                            ])
                                            ->hydrate(false)
                                            ->toArray();
        #patch entity update data
            $entity = TableRegistry::get('AnnualAssemblyTrackings')->patchEntities($annualAssemblyTrackings, $annualAssemblyTrackingsData);
        #checks saved or not
            if(TableRegistry::get('AnnualAssemblyTrackings')->saveMany($entity)){
            }
        }
    }
}
