<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use Cake\ORM\TableRegistry;
/**
 * GoodsReceivedComponents Model
 *
 * @property \App\Model\Table\GoodsReceivedNotesTable|\Cake\ORM\Association\BelongsTo $GoodsReceivedNotes
 * @property \App\Model\Table\PurchaseOrderComponentsTable|\Cake\ORM\Association\BelongsTo $PurchaseOrderComponents
 *
 * @method \App\Model\Entity\GoodsReceivedComponent get($primaryKey, $options = [])
 * @method \App\Model\Entity\GoodsReceivedComponent newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\GoodsReceivedComponent[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\GoodsReceivedComponent|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\GoodsReceivedComponent|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\GoodsReceivedComponent patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\GoodsReceivedComponent[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\GoodsReceivedComponent findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class GoodsReceivedComponentsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('goods_received_components');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('GoodsReceivedNotes', [
            'foreignKey' => 'goods_received_note_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('PurchaseOrderComponents', [
            'foreignKey' => 'purchase_order_component_id',
            'joinType' => 'INNER'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->numeric('received_quantity')
            ->requirePresence('received_quantity', 'create')
            ->allowEmptyString('received_quantity', false);

        // $validator
        //     ->requirePresence('is_active', 'create')
        //     ->allowEmptyString('is_active', false);

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['goods_received_note_id'], 'GoodsReceivedNotes'));
        $rules->add($rules->existsIn(['purchase_order_component_id'], 'PurchaseOrderComponents'));

        return $rules;
    }


    public function afterSave($event, $entity, $options)
    {

        $PurchaseOrderComponentsTable = TableRegistry::get('PurchaseOrderComponents');
        $purchase_order_component      = $PurchaseOrderComponentsTable->get($entity->purchase_order_component_id);

        #check grn component with purchase order components

        $purchase_order_component->grn_quantity=$purchase_order_component->grn_quantity+ $entity->received_quantity;
        $PurchaseOrderComponentsTable->save($purchase_order_component);
        
    }
}