<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;

/**
 * Groups Model
 *
 * @property \App\Model\Table\ComponentGroupsTable|\Cake\ORM\Association\HasMany $ComponentGroups
 * @property \App\Model\Table\GroupUsersTable|\Cake\ORM\Association\HasMany $GroupUsers
 * @property \App\Model\Table\MachinesTable|\Cake\ORM\Association\HasMany $Machines
 * @property \App\Model\Table\PurchaseIndentsTable|\Cake\ORM\Association\HasMany $PurchaseIndents
 *
 * @method \App\Model\Entity\Group get($primaryKey, $options = [])
 * @method \App\Model\Entity\Group newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\Group[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\Group|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Group|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Group patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\Group[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\Group findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class GroupsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('groups');
        $this->setDisplayField('group_name');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->hasMany('ComponentGroups', [
            'foreignKey' => 'group_id'
        ]);
        $this->hasMany('Machines', [
            'foreignKey' => 'group_id'
        ]);
        $this->hasMany('PurchaseIndents', [
            'foreignKey' => 'group_id'
        ]);


        $this->belongsToMany('Components', [
            'foreignKey' => 'group_id',
            'targetForeignKey' => 'component_id',
            'joinTable' => 'component_groups'
        ]);
        
        $this->belongsToMany('Users', [
            'foreignKey' => 'group_id',
            'targetForiegnKey' => 'user_id',
            'joinTable' => 'group_users'
        ]);


	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');

    #Add search behavior
        $this->addBehavior('Search.Search');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('group_name')
            ->maxLength('group_name', 50)
            ->requirePresence('group_name', 'create')
            ->allowEmptyString('group_name', false)
            ->add('group_name', 'unique', ['rule' => 'validateUnique', 'provider' => 'table', 'message' => 'Group name already exists.']);

        $validator
            ->scalar('description')
            ->allowEmptyString('description');

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;

    }

    // In a table class
    public function buildRules(RulesChecker $rules) {
        $rules->add($rules->isUnique(
            ['group_name'],
            'This Group name already exist.'
        ));
        return $rules;
    }

    public function findByUser(\Cake\ORM\Query $query, array $options)
    {
        $query
            ->contain([
                'Users'
            ])
            ->first();

        return $query;
    }
    
    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('group_name', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['Groups.group_name']
        ])
        ->add('is_active', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['is_active']!=null){
                    return $query->where(['Groups.is_active' => $args['is_active']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
