<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;

/**
 * MachineTypes Model
 *
 * @property \App\Model\Table\MachinesTable|\Cake\ORM\Association\HasMany $Machines
 *
 * @method \App\Model\Entity\MachineType get($primaryKey, $options = [])
 * @method \App\Model\Entity\MachineType newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\MachineType[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\MachineType|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\MachineType|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\MachineType patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\MachineType[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\MachineType findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class MachineTypesTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('machine_types');
        $this->setDisplayField('machine_type_name');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->hasMany('Machines', [
            'foreignKey' => 'machine_type_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');

    #Add search behavior
        $this->addBehavior('Search.Search');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('machine_type_name')
            ->maxLength('machine_type_name', 100)
            ->requirePresence('machine_type_name', 'create')
            ->allowEmptyString('machine_type_name', false)
            ->add('machine_type_name', 'unique', ['rule' => 'validateUnique', 'provider' => 'table', 'message' => 'Machine type already exists.']);

        $validator
            ->scalar('machine_type_code')
            ->maxLength('machine_type_code', 10)
            ->requirePresence('machine_type_code', 'create')
            ->allowEmptyString('machine_type_code', false)
            ->add('machine_type_code', 'unique', ['rule' => 'validateUnique', 'provider' => 'table', 'message' => 'Machine type code already exists.']);

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

       public function buildRules(RulesChecker $rules) {
        $rules->add($rules->isUnique(
            ['machine_type_name', 'machine_type_code'],
            'This Machine name and Machine Code already exist.'
        ));
        return $rules;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('machine_type_name', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['MachineTypes.machine_type_name']
        ])
        ->add('machine_type_code', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['MachineTypes.machine_type_code']
        ])
        ->add('is_active', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['is_active']!=null){
                    return $query->where(['MachineTypes.is_active' => $args['is_active']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
