<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * ModuleMasters Model
 *
 * @property \App\Model\Table\ModulesTable|\Cake\ORM\Association\HasMany $Modules
 * @property \App\Model\Table\UserRolePrivilegesTable|\Cake\ORM\Association\HasMany $UserRolePrivileges
 *
 * @method \App\Model\Entity\ModuleMaster get($primaryKey, $options = [])
 * @method \App\Model\Entity\ModuleMaster newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\ModuleMaster[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\ModuleMaster|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\ModuleMaster|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\ModuleMaster patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\ModuleMaster[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\ModuleMaster findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class ModuleMastersTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('module_masters');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->hasMany('Modules', [
            'foreignKey' => 'module_master_id'
        ]);
        $this->hasMany('UserRolePrivileges', [
            'foreignKey' => 'module_master_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('menu_name')
            ->maxLength('menu_name', 255)
            ->requirePresence('menu_name', 'create')
            ->allowEmptyString('menu_name', false);

        $validator
            ->scalar('icon')
            ->maxLength('icon', 100)
            ->allowEmptyString('icon');

        $validator
            ->integer('module_order')
            ->allowEmptyString('module_order');

        $validator
            ->requirePresence('is_main_menu', 'create')
            ->allowEmptyString('is_main_menu', false);

        $validator
            ->requirePresence('is_active', 'create')
            ->allowEmptyString('is_active', false);

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }
}