<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;

/**
 * Processes Model
 *
 * @property \App\Model\Table\AnnualPlannedCapacitiesTable|\Cake\ORM\Association\HasMany $AnnualPlannedCapacities
 * @property \App\Model\Table\ComponentProcessesTable|\Cake\ORM\Association\HasMany $ComponentProcesses
 * @property \App\Model\Table\MachineProcessesTable|\Cake\ORM\Association\HasMany $MachineProcesses
 * @property \App\Model\Table\VendorProcessesTable|\Cake\ORM\Association\HasMany $VendorProcesses
 *
 * @method \App\Model\Entity\Process get($primaryKey, $options = [])
 * @method \App\Model\Entity\Process newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\Process[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\Process|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Process|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Process patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\Process[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\Process findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class ProcessesTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('processes');
        $this->setDisplayField('process_name');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->hasMany('AnnualPlannedCapacities', [
            'foreignKey' => 'process_id'
        ]);
        $this->hasMany('ComponentProcesses', [
            'foreignKey' => 'process_id'
        ]);
        $this->hasMany('MachineProcesses', [
            'foreignKey' => 'process_id'
        ]);
        $this->hasMany('VendorProcesses', [
            'foreignKey' => 'process_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');

    #Add search behavior
        $this->addBehavior('Search.Search');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        /* $validator
            ->scalar('process_name')
            ->maxLength('process_name', 100)
            ->requirePresence('process_name', 'create')
            ->allowEmptyString('process_name', true); */
            
        $validator
            ->allowEmptyString('process_name', true)
            ->add('process_name', 'unique', ['rule' => 'validateUnique', 'provider' => 'table', 'message' => 'Process already exists.']);

        $validator
            ->scalar('description')
            ->allowEmptyString('description');

        // $validator
            // ->requirePresence('process_tag', 'create')
            // ->allowEmptyString('process_tag', false);

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    public function buildRules(RulesChecker $rules) {
        $rules->add($rules->isUnique(
            ['process_name'],
            'This Operation name already exist.'
        ));
        return $rules;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('process_name', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['Processes.process_name']
        ])
        ->add('process_tag', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['process_tag']!=null){
                    return $query->where(['Processes.process_tag' => $args['process_tag']]);
                }
                return $query;
            }
        ])
        ->add('is_active', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['is_active']!=null){
                    return $query->where(['Processes.is_active' => $args['is_active']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
