<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;

/**
 * Products Model
 *
 * @property \App\Model\Table\ProductsTable|\Cake\ORM\Association\BelongsTo $ParentProducts
 * @property \App\Model\Table\AnnualAssemblyTrackingsTable|\Cake\ORM\Association\HasMany $AnnualAssemblyTrackings
 * @property \App\Model\Table\AnnualPlannedCapacitiesTable|\Cake\ORM\Association\HasMany $AnnualPlannedCapacities
 * @property \App\Model\Table\AnnualPlannedProductsTable|\Cake\ORM\Association\HasMany $AnnualPlannedProducts
 * @property \App\Model\Table\AnnualProductTrackingsTable|\Cake\ORM\Association\HasMany $AnnualProductTrackings
 * @property \App\Model\Table\AnnualQuarterPlannedProductsTable|\Cake\ORM\Association\HasMany $AnnualQuarterPlannedProducts
 * @property \App\Model\Table\ComponentProductsTable|\Cake\ORM\Association\HasMany $ComponentProducts
 * @property \App\Model\Table\ProductsTable|\Cake\ORM\Association\HasMany $ChildProducts
 * @property \App\Model\Table\PurchaseIndentsTable|\Cake\ORM\Association\HasMany $PurchaseIndents
 * @property \App\Model\Table\SubAssemblyProductsTable|\Cake\ORM\Association\HasMany $SubAssemblyProducts
 *
 * @method \App\Model\Entity\Product get($primaryKey, $options = [])
 * @method \App\Model\Entity\Product newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\Product[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\Product|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Product|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Product patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\Product[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\Product findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class ProductsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('products');
        $this->setDisplayField('product_name');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('ParentProducts', [
            'className' => 'Products',
            'foreignKey' => 'parent_id'
        ]);
        $this->hasMany('AnnualAssemblyTrackings', [
            'foreignKey' => 'product_id'
        ]);
        $this->hasMany('AnnualPlannedCapacities', [
            'foreignKey' => 'product_id'
        ]);
        $this->hasMany('AnnualPlannedProducts', [
            'foreignKey' => 'product_id'
        ]);
        $this->hasMany('AnnualProductTrackings', [
            'foreignKey' => 'product_id'
        ]);
        $this->hasMany('AnnualQuarterPlannedProducts', [
            'foreignKey' => 'product_id'
        ]);
        $this->hasMany('ComponentProducts', [
            'foreignKey' => 'product_id'
        ]);
        $this->hasMany('ChildProducts', [
            'className' => 'Products',
            'foreignKey' => 'parent_id'
        ]);
        $this->hasMany('PurchaseIndents', [
            'foreignKey' => 'product_id'
        ]);
        $this->hasMany('SubAssemblyProducts', [
            'foreignKey' => 'product_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');

    #Add search behavior
        $this->addBehavior('Search.Search');

    #Import CSV Plugin
        $options = array(
        #Refer to php.net fgetcsv for more information
            'length'    => 0,
            'delimiter' => ',',
            'enclosure' => '"',
            'escape'    => '\\',
        #Generates a Model.field headings row from the csv file
            'headers'   => true,
        #If true, String $content is the data, not a path to the file
            'text'      => false,
        );
        $this->addBehavior('CakePHPCSV.Csv', $options);
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('product_name')
            ->maxLength('product_name', 100)
            ->requirePresence('product_name', 'create')
            ->allowEmptyString('product_name', false)
            ->add('product_name', 'unique', ['rule' => 'validateUnique', 'provider' => 'table', 'message' => 'Product name already exists.']);

        /* $validator
            ->scalar('description')
            ->allowEmptyString('description');

        $validator
            ->scalar('photo')
            ->maxLength('photo', 255)
            ->allowEmptyString('photo');

        $validator
            ->scalar('dir')
            ->maxLength('dir', 255)
            ->allowEmptyString('dir'); */

        $validator
            ->allowEmptyString('is_eventual_product');

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['parent_id'], 'ParentProducts'));
        $rules->add($rules->isUnique(
            ['product_name'],
            'This Product name already exist.'
        ));

        return $rules;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('product_name', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['Products.product_name']
        ])
        ->add('parent_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['parent_id']!=null){
                    return $query->where(['Products.parent_id' => $args['parent_id']]);
                }
                return $query;
            }
        ])
        ->add('is_active', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['is_active']!=null){
                    return $query->where(['Products.is_active' => $args['is_active']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
