<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * PurchaseIndentComponents Model
 *
 * @property \App\Model\Table\PurchaseIndentsTable|\Cake\ORM\Association\BelongsTo $PurchaseIndents
 * @property \App\Model\Table\ComponentsTable|\Cake\ORM\Association\BelongsTo $Components
 * @property \App\Model\Table\RawMaterialsTable|\Cake\ORM\Association\BelongsTo $RawMaterials
 *
 * @method \App\Model\Entity\PurchaseIndentComponent get($primaryKey, $options = [])
 * @method \App\Model\Entity\PurchaseIndentComponent newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\PurchaseIndentComponent[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseIndentComponent|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\PurchaseIndentComponent|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\PurchaseIndentComponent patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseIndentComponent[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseIndentComponent findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class PurchaseIndentComponentsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('purchase_indent_components');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('PurchaseIndents', [
            'foreignKey' => 'purchase_indent_id'
        ]);
        $this->belongsTo('Components', [
            'foreignKey' => 'component_id'
        ]);
        $this->belongsTo('RawMaterials', [
            'foreignKey' => 'raw_material_id'
        ]);

        $this->belongsTo('VendorRawMaterials', [
            'foreignKey' => 'raw_material_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        // $validator
        //     ->integer('total_quantity')
        //     ->allowEmptyString('total_quantity');

        $validator
            ->integer('is_active')
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        // $validator
        //     ->integer('modified_by')
        //     ->requirePresence('modified_by', 'create')
        //     ->allowEmptyString('modified_by', false);

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        // $rules->add($rules->existsIn(['purchase_indent_id'], 'PurchaseIndents'));
        // $rules->add($rules->existsIn(['component_id'], 'Components'));
        // $rules->add($rules->existsIn(['raw_material_id'], 'RawMaterials'));

        return $rules;
    }



     public function afterSave($event, $entity, $options)
    {
        
    }
}