<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use Cake\ORM\TableRegistry;

/**
 * PurchaseOrderComponents Model
 *
 * @property \App\Model\Table\PurchaseOrdersTable|\Cake\ORM\Association\BelongsTo $PurchaseOrders
 * @property \App\Model\Table\ComponentsTable|\Cake\ORM\Association\BelongsTo $Components
 * @property \App\Model\Table\RawMaterialsTable|\Cake\ORM\Association\BelongsTo $RawMaterials
 * @property |\Cake\ORM\Association\HasMany $GoodsReceivedComponents
 *
 * @method \App\Model\Entity\PurchaseOrderComponent get($primaryKey, $options = [])
 * @method \App\Model\Entity\PurchaseOrderComponent newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\PurchaseOrderComponent[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseOrderComponent|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\PurchaseOrderComponent|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\PurchaseOrderComponent patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseOrderComponent[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseOrderComponent findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class PurchaseOrderComponentsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('purchase_order_components');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('PurchaseOrders', [
            'foreignKey' => 'purchase_order_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Components', [
            'foreignKey' => 'component_id',
            'joinType' => 'LEFT'
        ]);
        $this->belongsTo('RawMaterials', [
            'foreignKey' => 'raw_material_id'
        ]);


        $this->belongsTo('StoreComponents', [
            'foreignKey' => 'store_component_id'
        ]);


        $this->hasMany('GoodsReceivedComponents', [
            'foreignKey' => 'purchase_order_component_id'
        ]);

        $this->belongsTo('Processes', [
            'foreignKey' => 'process_id',
            'joinType' => 'LEFT'
        ]);
	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->numeric('order_quantity')
            ->requirePresence('order_quantity', 'create')
            ->allowEmptyString('order_quantity', false);


        $validator
            ->numeric('gst')
            ->requirePresence('gst', 'create')
            ->allowEmptyString('gst', false);

        $validator
            ->numeric('amount')
            ->requirePresence('amount', 'create')
            ->allowEmptyString('amount', false);

       

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['purchase_order_id'], 'PurchaseOrders'));
        $rules->add($rules->existsIn(['component_id'], 'Components'));
        $rules->add($rules->existsIn(['raw_material_id'], 'RawMaterials'));

        return $rules;
    }

    public function afterSave($event, $entity, $options)
    {

        $PurchaseOrdersTable          = TableRegistry::get('PurchaseOrders');
        $PurchaseOrderComponentsTable = TableRegistry::get('PurchaseOrderComponents');
        $purchase_order_component     = $PurchaseOrderComponentsTable
            ->find()
            ->select([
                'poc_count'     => ' count(PurchaseOrderComponents.id)',
                'poc_grn_count' => ' SUM(if(PurchaseOrderComponents.order_quantity = PurchaseOrderComponents.grn_quantity, 1, 0))',
            ])
            ->where(['PurchaseOrderComponents.purchase_order_id' => $entity->purchase_order_id])
            ->enableAutoFields(true)
            ->first()
            ->toArray();
        if (!empty($purchase_order_component['poc_count'] <= $purchase_order_component['poc_grn_count'])) {

            $purchase_order = $PurchaseOrdersTable->get($purchase_order_component['purchase_order_id']);

            $purchase_order->status = 3;
            $PurchaseOrdersTable->save($purchase_order);

        }

    }
}