<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * PurchaseOrdersTemp Model
 *
 * @property \App\Model\Table\PurchaseIndentsTable|\Cake\ORM\Association\BelongsTo $PurchaseIndents
 * @property \App\Model\Table\VendorsTable|\Cake\ORM\Association\BelongsTo $Vendors
 * @property \App\Model\Table\ComponentsTable|\Cake\ORM\Association\BelongsTo $Components
 * @property \App\Model\Table\RawMaterialsTable|\Cake\ORM\Association\BelongsTo $RawMaterials
 *
 * @method \App\Model\Entity\PurchaseOrdersTemp get($primaryKey, $options = [])
 * @method \App\Model\Entity\PurchaseOrdersTemp newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\PurchaseOrdersTemp[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseOrdersTemp|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\PurchaseOrdersTemp|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\PurchaseOrdersTemp patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseOrdersTemp[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseOrdersTemp findOrCreate($search, callable $callback = null, $options = [])
 */
class PurchaseOrdersTempTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('purchase_orders_temp');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->belongsTo('PurchaseIndents', [
            'foreignKey' => 'purchase_indent_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Vendors', [
            'foreignKey' => 'vendor_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Components', [
            'foreignKey' => 'component_id'
        ]);
        $this->belongsTo('RawMaterials', [
            'foreignKey' => 'raw_material_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

      
        $validator
            ->requirePresence('tag', 'create')
            ->allowEmptyString('tag', true);

        $validator
            ->numeric('order_quantity')
            ->requirePresence('order_quantity', 'create')
            ->allowEmptyString('order_quantity', false);


        $validator
            ->numeric('amount')
            ->requirePresence('amount', 'create')
            ->allowEmptyString('amount', false);


        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['purchase_indent_id'], 'PurchaseIndents'));
        $rules->add($rules->existsIn(['vendor_id'], 'Vendors'));
        $rules->add($rules->existsIn(['component_id'], 'Components'));
        $rules->add($rules->existsIn(['raw_material_id'], 'RawMaterials'));

        return $rules;
    }
}