<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * SpareInventoryComponents Model
 *
 * @property \App\Model\Table\ComponentSpareInventoriesTable|\Cake\ORM\Association\BelongsTo $ComponentSpareInventories
 * @property \App\Model\Table\ComponentsTable|\Cake\ORM\Association\BelongsTo $Components
 *
 * @method \App\Model\Entity\SpareInventoryComponent get($primaryKey, $options = [])
 * @method \App\Model\Entity\SpareInventoryComponent newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\SpareInventoryComponent[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\SpareInventoryComponent|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\SpareInventoryComponent|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\SpareInventoryComponent patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\SpareInventoryComponent[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\SpareInventoryComponent findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class SpareInventoryComponentsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('spare_inventory_components');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('ComponentSpareInventories', [
            'foreignKey' => 'component_spare_inventory_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Components', [
            'foreignKey' => 'component_id',
            'joinType' => 'INNER'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->integer('quantity')
            ->requirePresence('quantity', 'create')
            ->allowEmptyString('quantity', false);

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['component_spare_inventory_id'], 'ComponentSpareInventories'));
        $rules->add($rules->existsIn(['component_id'], 'Components'));

        return $rules;
    }
}
