<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * StoreComponents Model
 *
 * @property \App\Model\Table\ProductsTable|\Cake\ORM\Association\BelongsTo $Products
 * @property \App\Model\Table\VendorStoreComponentsTable|\Cake\ORM\Association\HasMany $VendorStoreComponents
 *
 * @method \App\Model\Entity\StoreComponent get($primaryKey, $options = [])
 * @method \App\Model\Entity\StoreComponent newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\StoreComponent[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\StoreComponent|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\StoreComponent|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\StoreComponent patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\StoreComponent[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\StoreComponent findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class StoreComponentsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('store_components');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('Products', [
            'foreignKey' => 'product_id',
            'joinType' => 'INNER'
        ]);
        $this->hasMany('VendorStoreComponents', [
            'foreignKey' => 'store_component_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');

    #Import CSV Plugin
        $options = array(
        #Refer to php.net fgetcsv for more information
            'length'    => 0,
            'delimiter' => ',',
            'enclosure' => '"',
            'escape'    => '\\',
        #Generates a Model.field headings row from the csv file
            'headers'   => true,
        #If true, String $content is the data, not a path to the file
            'text'      => false,
        );
        $this->addBehavior('CakePHPCSV.Csv', $options);
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('component_name')
            ->maxLength('component_name', 255)
            ->allowEmptyString('component_name',false);

        $validator
            ->scalar('component_part_no')
            ->allowEmptyString('component_part_no',true);

        $validator
            ->scalar('component_type')
            ->maxLength('component_type', 50)
            ->allowEmptyString('component_type',false);

        $validator
            ->scalar('component_make')
            ->maxLength('component_make', 50)
            ->allowEmptyString('component_make',true);

        $validator
            ->requirePresence('npm', 'create')
            ->allowEmptyString('npm',true);

        $validator
            ->integer('buying_frequency')
            ->requirePresence('buying_frequency', 'create')
            ->allowEmptyString('buying_frequency', true);

        $validator
            //->requirePresence('is_active', 'create')
            ->allowEmptyString('is_active', true);


        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['product_id'], 'Products'));

        return $rules;
    }
}
