<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * VendorStoreComponents Model
 *
 * @property \App\Model\Table\VendorsTable|\Cake\ORM\Association\BelongsTo $Vendors
 * @property \App\Model\Table\StoreComponentsTable|\Cake\ORM\Association\BelongsTo $StoreComponents
 *
 * @method \App\Model\Entity\VendorStoreComponent get($primaryKey, $options = [])
 * @method \App\Model\Entity\VendorStoreComponent newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\VendorStoreComponent[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\VendorStoreComponent|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\VendorStoreComponent|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\VendorStoreComponent patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\VendorStoreComponent[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\VendorStoreComponent findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class VendorStoreComponentsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('vendor_store_components');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('Vendors', [
            'foreignKey' => 'vendor_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('StoreComponents', [
            'foreignKey' => 'store_component_id',
            'joinType' => 'INNER'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->integer('component_part_no')
            ->allowEmptyString('component_part_no');

        $validator
            ->numeric('price')
            ->requirePresence('price', 'create')
            ->allowEmptyString('price', false);

        // $validator
        //     ->numeric('gst')
        //     ->requirePresence('gst', 'create')
        //     ->allowEmptyString('gst', false);

        // $validator
        //     ->requirePresence('is_active', 'create')
        //     ->allowEmptyString('is_active', false);

        // $validator
        //     ->integer('created_by')
        //     ->requirePresence('created_by', 'create')
        //     ->allowEmptyString('created_by', false);

        // $validator
        //     ->integer('modified_by')
        //     ->requirePresence('modified_by', 'create')
        //     ->allowEmptyString('modified_by', false);

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['vendor_id'], 'Vendors'));
        $rules->add($rules->existsIn(['store_component_id'], 'StoreComponents'));

        return $rules;
    }
}