<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;

/**
 * Vendors Model
 *
 * @property \App\Model\Table\GoodsReceivedNotesTable|\Cake\ORM\Association\HasMany $GoodsReceivedNotes
 * @property \App\Model\Table\PurchaseOrdersTable|\Cake\ORM\Association\HasMany $PurchaseOrders
 * @property \App\Model\Table\VendorComponentsTable|\Cake\ORM\Association\HasMany $VendorComponents
 * @property \App\Model\Table\VendorProcessesTable|\Cake\ORM\Association\HasMany $VendorProcesses
 * @property \App\Model\Table\VendorRawMaterialsTable|\Cake\ORM\Association\HasMany $VendorRawMaterials
 *
 * @method \App\Model\Entity\Vendor get($primaryKey, $options = [])
 * @method \App\Model\Entity\Vendor newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\Vendor[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\Vendor|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Vendor|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Vendor patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\Vendor[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\Vendor findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class VendorsTable extends Table
{
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('vendors');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->hasMany('GoodsReceivedNotes', [
            'foreignKey' => 'vendor_id'
        ]);
        $this->hasMany('PurchaseOrders', [
            'foreignKey' => 'vendor_id'
        ]);
        $this->hasMany('VendorComponents', [
            'foreignKey' => 'vendor_id'
        ]);
        $this->hasMany('VendorProcesses', [
            'foreignKey' => 'vendor_id'
        ]);
        $this->hasMany('VendorRawMaterials', [
            'foreignKey' => 'vendor_id'
        ]);

        $this->hasMany('VendorStoreComponents', [
            'foreignKey' => 'vendor_id'
        ]);

        $this->hasMany('PurchaseOrdersTemp', [
            'foreignKey' => 'vendor_id'
        ]);

        $this->belongsToMany('Components', [
            'foreignKey'        => 'vendor_id',
            'targetForeignKey'  => 'component_id',
            'joinTable'         => 'vendor_components'
        ]);

        $this->belongsToMany('Processes', [
            'foreignKey'        => 'vendor_id',
            'targetForeignKey'  => 'process_id',
            'joinTable'         => 'vendor_processes'
        ]);

        $this->belongsToMany('RawMaterials', [
            'foreignKey'        => 'vendor_id',
            'targetForeignKey'  => 'raw_material_id',
            'joinTable'         => 'vendor_raw_materials'
        ]);

        #Vendor Store Components
         $this->belongsToMany('StoreComponents', [
            'foreignKey'        => 'vendor_id',
            'targetForeignKey'  => 'store_component_id',
            'joinTable'         => 'vendor_store_components'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');

    #Add search behavior
        $this->addBehavior('Search.Search');

    #Import CSV Plugin
        $options = array(
        #Refer to php.net fgetcsv for more information
            'length'    => 0,
            'delimiter' => ',',
            'enclosure' => '"',
            'escape'    => '\\',
        #Generates a Model.field headings row from the csv file
            'headers'   => true,
        #If true, String $content is the data, not a path to the file
            'text'      => false,
        );
        $this->addBehavior('CakePHPCSV.Csv', $options);
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('vendor_code')
            ->maxLength('vendor_code', 100)
            ->requirePresence('vendor_code', 'create')
            ->allowEmptyString('vendor_code', false)
            ->add('vendor_code', 'unique', ['rule' => 'validateUnique', 'provider' => 'table', 'message' => 'Vendor code already exists.']);

        $validator
            ->scalar('vendor_name')
            ->maxLength('vendor_name', 50)
            ->requirePresence('vendor_name', 'create')
            ->allowEmptyString('vendor_name', false)
            ->add('vendor_name', 'unique', ['rule' => 'validateUnique', 'provider' => 'table', 'message' => 'Vendor name already exists.']);

        $validator
            ->scalar('contact_person')
            ->maxLength('contact_person', 255)
            ->allowEmptyString('contact_person');

        $validator
            ->scalar('gst_no')
            ->maxLength('gst_no', 50)
            ->allowEmptyString('gst_no',true);

        $validator
            ->scalar('phone')
            ->maxLength('phone', 20)
            ->allowEmptyString('phone');

        $validator
            ->scalar('mobile')
            ->maxLength('mobile', 20)
            ->allowEmptyString('mobile');

        $validator
            ->email('email')
            ->allowEmptyString('email');

        $validator
            ->scalar('address')
            ->allowEmptyString('address');

        $validator
            ->requirePresence('rating', 'create')
            ->allowEmptyString('rating', false);

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->isUnique(['vendor_code']));
        $rules->add($rules->isUnique(['vendor_name']));
        $rules->add($rules->isUnique(['email']));
    #return rules
        return $rules;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('vendor_name', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['Vendors.vendor_name']
        ])
        ->add('vendor_code', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['Vendors.vendor_code']
        ])
        ->add('is_active', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['is_active']!=null){
                    return $query->where(['Vendors.is_active' => $args['is_active']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
