<?php
namespace App\Shell;

use Cake\Console\Shell;
use Cake\ORM\TableRegistry;
use Cake\Core\Configure;
use Cake\Routing\Router;
use Cake\I18n\Time;

/**
 * Cron shell command.
 */
class CronShell extends Shell
{
    /**
     * Manage the available sub-commands along with their arguments and help
     *
     * @see http://book.cakephp.org/3.0/en/console-and-shells.html#configuring-options-and-generating-help
     *
     * @return \Cake\Console\ConsoleOptionParser
     */
    public function getOptionParser()
    {
        $parser = parent::getOptionParser();

        return $parser;
    }

    /**
     * main() method.
     *
     * @return bool|int|null Success or error code.
     */
    public function main()
    {
        $this->out($this->OptionParser->help());
    }

    /**
     * update component status based on number of days
     *
     * @return bool|int|null Success or error code.
     */
    public function updateTrackSheetStatus()
    {
    #load model
        $this->loadModel('AnnualProductTrackings');
        $this->loadModel('AnnualComponentTrackings');
        $this->loadModel('AnnualAssemblyTrackings');
    #delay days
        $qcDelay  = [   
                        ['findStatus' => [2]        , 'delayDays' => 3, 'changeStatus' => 3], 
                        ['findStatus' => [2,3]      , 'delayDays' => 5, 'changeStatus' => 4],
                        ['findStatus' => [2,3,4]    , 'delayDays' => 8, 'changeStatus' => 17],
                        ['findStatus' => [7,8]      , 'delayDays' => 3, 'changeStatus' => 9], 
                        ['findStatus' => [7,8,9]    , 'delayDays' => 5, 'changeStatus' => 10],
                        ['findStatus' => [7,8,9,10] , 'delayDays' => 8, 'changeStatus' => 18],
                    ];
    #variable declaration
        $assblyTrackingId   = [];

    #loop delay days data
        foreach($qcDelay AS $qKey => $qRows){
        #date time
            $now    = Time::now();
            $days   = new Time($qRows["delayDays"].' days ago');
        #get all tacksheet data
            $annualComponentTrackings = $this->AnnualComponentTrackings->find('all')
                                            ->where([
                                                'is_active'             => 1,
                                                'status IN'             => $qRows["findStatus"],
                                                "status_updated_on <"   => $days
                                            ])
                                            ->toArray();
        #checks result empty
            if(!empty($annualComponentTrackings)){
            #prepare variable
                $data               = [];
            #loop data
                foreach($annualComponentTrackings AS $key=>$rows){
                    $data[$key]['id']       = $rows->id;
                    $data[$key]['status']   = $qRows["changeStatus"];
                #check rejected count
                    $rejectedProductTracking= $this->AnnualComponentTrackings
                                                ->find('all')
                                                ->where([
                                                    'is_active'                     => 1, 
                                                    'status IN'                     => [5,6,11,12], 
                                                    'annual_assembly_tracking_id'   => $rows->annual_assembly_tracking_id 
                                                ])
                                                ->toArray();
                #checks rejection status
                    if(empty($rejectedProductTracking) && ($qRows["changeStatus"]==2 || $qRows["changeStatus"]==3 || $qRows["changeStatus"]==6 || $qRows["changeStatus"]==7))
                    {
                        $assblyTrackingId[] = $rows->annual_assembly_tracking_id;
                    }
                }
            #update multiple records
                $entity = $this->AnnualComponentTrackings->patchEntities($annualComponentTrackings, $data);
                $this->AnnualComponentTrackings->saveMany($entity);
            }
        }

    #check array empty to update status in sub assembly
        if(!empty($assblyTrackingId)){
        #array unique
            $assblyTrackingId           = array_unique($assblyTrackingId);
        #get all tacksheet data
            $annualAssemblyTrackings    = $this->AnnualAssemblyTrackings->find('all')->where(['id IN' => $assblyTrackingId])->toArray();
        #prepare variable
            $aData                      = [];
        #loop data
            foreach( $annualAssemblyTrackings AS $key=>$rows ){
                $aData[$key]['id']       = $rows->id;
                $aData[$key]['status']   = 2;
            }
        #update multiple records
            $entity = $this->AnnualComponentTrackings->patchEntities($annualAssemblyTrackings, $aData);
            $this->AnnualAssemblyTrackings->saveMany($entity);
        }

    #group delay
        $currentDate = date('Y-m-d');
        $annualProductTrackings = $this->AnnualProductTrackings->find()
                                    ->select([
                                       'id'         => 'AnnualComponentTrackings.id',
                                       'status'     => '(IF( datediff( now(), expected_delivery_date) < 6, ( if( (now()>expected_delivery_date), 15, 0) ), 16 ))',
                                    ])
                                    ->join([
                                        'AnnualAssemblyTrackings' => [
                                            'table'     => 'annual_assembly_trackings',
                                            'type'		=> 'INNER',
                                            'conditions'=> ['AnnualAssemblyTrackings.annual_product_tracking_id=AnnualProductTrackings.id']
                                        ],
                                        'AnnualComponentTrackings' => [
                                            'table'     => 'annual_component_trackings',
                                            'type'		=> 'INNER',
                                            'conditions'=> ['AnnualComponentTrackings.annual_assembly_tracking_id=AnnualAssemblyTrackings.id']
                                        ],
                                    ])
                                    ->where([
                                        'AnnualProductTrackings.expected_delivery_date < ' => $currentDate, 
                                        'AnnualComponentTrackings.status' => 0
                                    ]);
    #prepare data
        $annualComponentTrackingData    = $annualProductTrackings->hydrate(false)->extract('id')->toArray();
        $annualComponentTrackingUpdate  = $annualProductTrackings->hydrate(false)->toArray();

    #checks not empty
        if(!empty($annualComponentTrackingUpdate)){
        #find records
            $annualComponentTrackings   = $this->AnnualComponentTrackings->find()->where(['id IN ' => $annualComponentTrackingData]);
        #update multiple records
            $entity = $this->AnnualComponentTrackings->patchEntities($annualComponentTrackings, $annualComponentTrackingUpdate);
            $this->AnnualComponentTrackings->saveMany($entity);
        }
    }
}
