<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\I18n\Date;
use Cake\I18n\Time;

/**
 * AnnualComponentTrackings Controller
 *
 * @property \App\Model\Table\AnnualComponentTrackingsTable $AnnualComponentTrackings
 *
 * @method \App\Model\Entity\AnnualComponentTracking[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class AnnualComponentStatusTrackingsController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

    #load component
        $this->loadComponent('RequestHandler');

    #CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'completedProducts' => ['className' => 'Crud.Index'],
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);

	#load model
		$this->loadModel("Groups");
		$this->loadModel("GroupUsers");
		$this->loadModel("SubAssemblies");
		$this->loadModel("SubAssemblyProducts");
		$this->loadModel("Products");
		$this->loadModel("AnnualPlans");
        $this->loadModel("AnnualQuarterPlans");
        $this->loadModel("AnnualAssemblyTrackings");
        $this->loadModel("AnnualPlannedProducts");
        $this->loadModel("AnnualProductTrackings");
        $this->loadModel("AnnualComponentTrackings");
        $this->loadModel("AnnualComponentStatusTrackings");
        $this->loadModel("ComponentRejectionReasons");
        $this->loadModel("ComponentProducts");
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #get global configuration from config/app.php
        $trackSheetComponentStatus  = Configure::read('TrackSheetComponentStatus');
        $trackingSheetStatusArray   = array(5 => "QC Partial Rework", 6 => "QC Rework", 7 => "QC Conditional Acceptance", 8 => "QC Completed", 11 => "Assembly Partial Rework", 12 => "Assembly Rework", 13 => "Assembly Conditional Acceptance", 14 => "Assembly Completed" );

    #get product list
		$products = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->where(['is_active' => 1, 'is_eventual_product' => 0]);

    #get annual plans
        $annualPlanOptions  = $this->AnnualPlans->find('list', ['keyField' => 'id', 'valueField' => 'financial_year'])->where(['is_active' => 1])->order(['id' => 'DESC']);

    #get groups list
		$groups = $this->Groups->find('list', ['keyField' => 'id', 'valueField' => 'group_name'])->where(['is_active' => 1]);

    #find year plan
        $annualPlanFilter = $this->AnnualPlans->find()->where(['is_active' => 1])->order(['id' => 'DESC'])->first();

    #find year plan
        $annualComponentStatusTrackingsStatus = $this->AnnualComponentStatusTrackings->find()
                                                    ->select([
                                                    #total components completed
                                                        'total_components_completed'        => '(SUM( IF(AnnualComponentTrackings.status=14, AnnualComponentTrackings.total_completed_components, 0) ))',
                                                    #qc partial rejection
                                                        'qc_partial_rejection'              => '(SUM( IF(AnnualComponentStatusTrackings.status=5, AnnualComponentStatusTrackings.total_completed_components, 0) ))',
                                                    #qc rejection
                                                        'qc_rejection'                      => '(SUM( IF(AnnualComponentStatusTrackings.status=6, AnnualComponentStatusTrackings.total_completed_components, 0) ))',
                                                    #qc conditinal acceptance
                                                        'qc_conditional_acceptance'         => '(SUM( IF(AnnualComponentStatusTrackings.status=7, AnnualComponentStatusTrackings.total_completed_components, 0) ))',
                                                    #assembly partial rejection
                                                        'assembly_partial_rejection'        => '(SUM( IF(AnnualComponentStatusTrackings.status=11, AnnualComponentStatusTrackings.total_completed_components, 0) ))',
                                                    #assembly rejection
                                                        'assembly_rejection'                => '(SUM( IF(AnnualComponentStatusTrackings.status=12, AnnualComponentStatusTrackings.total_completed_components, 0) ))',
                                                    #assembly conditinal acceptance
                                                        'assembly_conditional_acceptance'   => '(SUM( IF(AnnualComponentStatusTrackings.status=13, AnnualComponentStatusTrackings.total_completed_components, 0) ))',
                                                    ])
                                                    ->join([
                                                        'AnnualComponentTrackings' => [
                                                            'table'     => 'annual_component_trackings',
                                                            'type'		=> 'INNER',
                                                            'conditions'=> ['AnnualComponentTrackings.id=AnnualComponentStatusTrackings.annual_component_tracking_id']
                                                        ],
                                                        'AnnualQuarterPlans' => [
                                                            'table'     => 'annual_quarter_plans',
                                                            'type'		=> 'INNER',
                                                            'conditions'=> ['AnnualQuarterPlans.id=AnnualComponentTrackings.annual_quarter_plan_id']
                                                        ],
                                                    ])
                                                    ->where([
                                                        'AnnualQuarterPlans.annual_plan_id' => $annualPlanFilter['id'],
                                                    ])
                                                    ->hydrate(false)
                                                    ->first();
    #find total number of components in current year plan
        $annualAssemblyComponentsStatus   = $this->AnnualQuarterPlans->find()
                                                ->select([
                                                    'total_components' => 'SUM(AnnualAssemblyTrackings.total_component)'
                                                ])
                                                ->join([
                                                    'AnnualAssemblyTrackings' => [
                                                        'table'     => 'annual_assembly_trackings',
                                                        'type'		=> 'INNER',
                                                        'conditions'=> ['AnnualAssemblyTrackings.annual_quarter_plan_id=AnnualQuarterPlans.id']
                                                    ],
                                                ])
                                                ->where([
                                                    'AnnualQuarterPlans.annual_plan_id' => $annualPlanFilter['id'],
                                                ])
                                                ->group(['AnnualAssemblyTrackings.sub_assembly_id'])
                                                ->hydrate(false)
                                                ->first();
    #set result for view
        $this->set(compact(['trackSheetComponentStatus', 'trackingSheetStatusArray', 'products', 'groups', 'annualPlanOptions', 'annualComponentStatusTrackingsStatus', 'annualAssemblyComponentsStatus']));

    #before find
		$this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) use($annualPlanFilter){
        #find query
			$event->subject()->query
                    ->contain([
                        'AnnualComponentTrackings' => ['ComponentProducts' => ['Products', 'Components']]
                    ])
                    ->join([
                        'AnnualComponentTrackings' => [
                            'table'     => 'annual_component_trackings',
                            'type'		=> 'INNER',
                            'conditions'=> ['AnnualComponentTrackings.id=AnnualComponentStatusTrackings.annual_component_tracking_id']
                        ],
                        'AnnualQuarterPlans' => [
                            'table'     => 'annual_quarter_plans',
                            'type'		=> 'INNER',
                            'conditions'=> ['AnnualQuarterPlans.id=AnnualComponentTrackings.annual_quarter_plan_id']
                        ],
                    ])
                    ->where([
                        'AnnualComponentStatusTrackings.status IN'  => [5,6,7,8,11,12,13,14],
                        'AnnualQuarterPlans.annual_plan_id'         => $annualPlanFilter['id']
                    ])
                    ->order(['AnnualComponentStatusTrackings.created' => 'DESC']);
		});

    #return data
        return $this->Crud->execute();
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function completedProducts()
    {
    #load class
        $this->modelClass = "annual_product_trackings";

    #get global configuration from config/app.php
        $trackSheetComponentStatus  = Configure::read('TrackSheetComponentStatus');
        $trackSheetProductStatus    = Configure::read('TrackSheetProductStatus');

        $status = [];
    #loop tracking sheet status for select option fields
        foreach($trackSheetProductStatus AS $key => $rows){
            $status[] = $rows['status'];
        }

    #get product list
		$products = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->where(['is_active' => 1, 'is_eventual_product' => 0]);

    #get annual plans
        $annualPlanOptions  = $this->AnnualPlans
                                    ->find('list', ['keyField' => 'id', 'valueField' => 'financial_year'])
                                    ->where(['is_active' => 1])
                                    ->order(['id' => 'DESC']);

    #set result for view
        $this->set(compact(['trackSheetComponentStatus', 'trackSheetProductStatus', 'annualPlanOptions', 'products', 'status']));

    #before find
		$this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) {
        #find year plan
            $annualPlanFilter = $this->AnnualPlans->find()->where(['is_active' => 1])->order(['id' => 'DESC'])->first();
        #query string
			$event->subject()->query
                ->contain([
                    'Products' , 'AnnualQuarterPlans'
                ])
                ->where([
                    'annual_product_trackings.is_eventual_product'  => 0,
                ]);
		});

    #return data
        return $this->Crud->execute();
    }
}
