<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\I18n\Time;

/**
 * AnnualProductTrackings Controller
 *
 * @property \App\Model\Table\AnnualProductTrackingsTable $AnnualProductTrackings
 *
 * @method \App\Model\Entity\AnnualProductTracking[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class AnnualProductTrackingsController extends AppController
{

    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

        $this->loadComponent('RequestHandler');

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
            ],
            'listeners' => [
                'Crud.Api'
            ]
        ]);

	#load model
		$this->loadModel("Groups");
		$this->loadModel("GroupUsers");
		$this->loadModel("SubAssemblies");
		$this->loadModel("SubAssemblyProducts");
		$this->loadModel("Products");
		$this->loadModel("AnnualPlans");
        $this->loadModel("AnnualQuarterPlans");
        $this->loadModel("AnnualAssemblyTrackings");
        $this->loadModel("AnnualPlannedProducts");
        $this->loadModel("AnnualProductTrackings");
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #get current login user id
        $id                         = $this->Auth->user('id');
        $user_role_id               = $this->Auth->user('user_role_id');
    #get global configuration from config/app.php
        $assemblyStatus             = Configure::read('TrackSheetAssemblyStatus');
        $componentStatus            = Configure::read('TrackSheetComponentStatus');
        $trackSheetProductStatus    = Configure::read('TrackSheetProductStatus');
    #variable declaration
        $productId                  = $this->request->getQuery('productId');
        $subAssemblyId              = $this->request->getQuery('subAssembly');
        $groupId                    = $this->request->getQuery('groupId');
        $status                     = $this->request->getQuery('status');
        $productNo                  = $this->request->getQuery('productNo');
        $subAssemblies              = [];
    #check data null
        $productId                  = ( (isset($productId) && !empty($productId)) ? $productId : null );
        $subAssemblyId              = ( (isset($subAssemblyId) && !empty($subAssemblyId)) ? $subAssemblyId : null );
        $groupId                    = ( (isset($groupId) && !empty($groupId)) ? $groupId : null );
        $status                     = ( (isset($status) && $status!="") ? $status : null );
        $productNo                  = ( (isset($productNo) && !empty($productNo)) ? $productNo : null );

    #get product details if product id is empty
        if($productId==null){
            $productDetails = $this->Products->find('all')->where(['is_active' => 1])->first();
            $productId      = $productDetails->id;
        }
        else{
        #product details
            $productDetails = $this->Products->get($productId);
        }

    #get group users
        $groupUsers = $this->GroupUsers->find()->where(['user_id'=>$id, 'is_active' => 1])->first();

    #get all group components if group id paramter is empty or null
        if($user_role_id==4){
        #prepare group filter where condition
            $assemblyTrackGroupWhere    = ['AnnualAssemblyTrackings.product_id'  => $productId, 'ComponentGroups.group_id' => $groupUsers->group_id];
            $componentTrackGroupWhere   = ['ComponentProducts.product_id'  => $productId, 'ComponentGroups.group_id' => $groupUsers->group_id];
        }
        elseif($groupId==null){
        #prepare group filter where condition
            $assemblyTrackGroupWhere    = ['AnnualAssemblyTrackings.product_id'  => $productId];
            $componentTrackGroupWhere   = ['ComponentProducts.product_id'  => $productId];
        }
        else{
        #prepare group filter where condition
            $assemblyTrackGroupWhere    = ['AnnualAssemblyTrackings.product_id'  => $productId, 'ComponentGroups.group_id' => $groupId];
            $componentTrackGroupWhere   = ['ComponentProducts.product_id'  => $productId, 'ComponentGroups.group_id' => $groupId];
        }

    #product assembly
        $productAssmbly = $this->SubAssemblyProducts->find('list')->select(['id'])->where([ 'product_id' => $productId ])->toArray();

    #sub assembly
        if($subAssemblyId!=null){
           $productAssmbly      = [];
           $productAssmbly[]    = $subAssemblyId;
        }

    #check sub assembly empty    
        if(!empty($productAssmbly)){
        #get sub assembly data based on the component product and component assembly
            $subAssembliesData = $this->SubAssemblies->find("all")->contain([ 
            #annual assembly tracking
                'AnnualAssemblyTrackings' => function($query) use($assemblyTrackGroupWhere){
                    return $query
                                ->select([
                                    'id'                    => 'AnnualAssemblyTrackings.id',
                                    'annual_quarter_plan_id'=> 'AnnualAssemblyTrackings.annual_quarter_plan_id',
                                    'sub_assembly_id'       => 'AnnualAssemblyTrackings.sub_assembly_id',
                                    'product_id'            => 'AnnualAssemblyTrackings.product_id',
                                    'product_number'        => 'AnnualAssemblyTrackings.product_number',
                                    'total_component'       => 'AnnualAssemblyTrackings.total_component',
                                    'completed_component'   => 'AnnualAssemblyTrackings.completed_component',
                                    'status'                => 'AnnualAssemblyTrackings.status',
                                    'status_updated_on'     => 'AnnualAssemblyTrackings.status_updated_on',
                                    'description'           => 'AnnualAssemblyTrackings.description',
                                    'total'                 => 'COUNT(AnnualProductTrackings.status)',
                                    'completed'             => 'SUM(if(AnnualProductTrackings.status=9, 1, 0))',
                                    'group_id'              => 'ComponentGroups.group_id',
                                ])
                                ->join([
                                    'AnnualProductTrackings' => [
                                        'table'         => 'annual_product_trackings',
                                        'type'          => 'inner',
                                        'conditions'    => ['AnnualProductTrackings.annual_assembly_tracking_id=AnnualAssemblyTrackings.id']
                                    ],
                                    'ComponentProducts' => [
                                        'table'         => 'component_products',
                                        'type'          => 'inner',
                                        'conditions'    => ['ComponentProducts.id=AnnualProductTrackings.component_product_id']
                                    ],
                                    'ComponentGroups' => [
                                        'table'         => 'component_groups',
                                        'type'          => 'inner',
                                        'conditions'    => ['ComponentGroups.component_id=ComponentProducts.component_id']
                                    ],
                                ])
                                ->where($assemblyTrackGroupWhere)
                                ->group(['AnnualProductTrackings.annual_assembly_tracking_id']);
                }, 
            #annual component tracking
                'ComponentAssemblies' => function($query) use($productId, $componentTrackGroupWhere, $status){
                    return $query
                                ->select([
                                    'group_id'                  => 'ComponentGroups.id',
                                    'component_id'              => 'Components.id',
                                    'component_name'            => 'Components.component_name',
                                    'component_code'            => 'Components.component_code',
                                    'drawing_reference_no'      => 'Components.drawing_reference_no',
                                    'component_cost'            => 'Components.component_cost',
                                    'batch_quantity'            => 'Components.batch_quantity',
                                    'tag'                       => 'Components.tag',
                                    'sub_assembly_id'           => 'ComponentAssemblies.sub_assembly_id',
                                    'product_id'                => 'ComponentProducts.product_id',
                                    'number_of_component'       => 'ComponentProducts.number_of_component',
                                    'start_product_no'          => 'ComponentProducts.start_product_no',
                                    'end_product_no'            => 'ComponentProducts.end_product_no',
                                    'annual_product_tracking_id'=> 'AnnualProductTrackings.id',
                                    'product_number'            => 'AnnualProductTrackings.product_number',
                                    'status'                    => 'AnnualProductTrackings.status',
                                    'status_updated_on'         => 'AnnualProductTrackings.status_updated_on',
                                    'description'               => 'AnnualProductTrackings.description',
                                    'delays'                    => 'DATEDIFF(NOW(),AnnualProductTrackings.status_updated_on)'
                                ])
                                ->contain(['Components'])
                                ->join([
                                    'ComponentProducts' => [
                                        'table'			=> 'component_products',
                                        'type'			=> 'inner',
                                        'conditions'	=> ['ComponentProducts.component_id=ComponentAssemblies.component_id']
                                    ],
                                    'ComponentGroups'   => [
                                        'table'         => 'component_groups',
                                        'type'          => 'inner',
                                        'conditions'    => ['ComponentGroups.component_id=ComponentProducts.component_id']
                                    ],
                                    'AnnualProductTrackings' => [
                                        'table'         => 'annual_product_trackings',
                                        'type'          => 'inner',
                                        'conditions'    => ['AnnualProductTrackings.component_product_id=ComponentProducts.id']
                                    ],
                                ])
                                ->where($componentTrackGroupWhere);
                }
            ])
            ->where(['SubAssemblies.id in '=>$productAssmbly])
            ->toArray();

        #eventual products mapping
            $eventualProducts = $this->Products->find('all')->where([ 'parent_id' => $productId ])->toArray();
            $eventualData     = [];

        #each loop
            foreach($eventualProducts AS $key => $rows){
            #check group id null
                if($user_role_id==4){
                #prepare group filter where condition
                    $assemblyTrackGroupWhere    = ['AnnualAssemblyTrackings.product_id' => $rows->id, 'ComponentGroups.group_id' => $groupUsers->group_id];
                    $componentTrackGroupWhere = ['ComponentProducts.product_id'  => $rows->id, 'ComponentGroups.group_id' => $groupUsers->group_id];
                }
                elseif($groupId==null){
                    $assemblyTrackGroupWhere    = ['AnnualAssemblyTrackings.product_id' => $rows->id];
                    $componentTrackGroupWhere = ['ComponentProducts.product_id'  => $rows->id];
                }
                else{
                    $assemblyTrackGroupWhere    = ['AnnualAssemblyTrackings.product_id' => $rows->id, 'ComponentGroups.group_id' => $groupId];
                    $componentTrackGroupWhere = ['ComponentProducts.product_id'  => $rows->id, 'ComponentGroups.group_id' => $groupId];
                }

            #table header
                $eventualData[$key]['header'] = $this->AnnualAssemblyTrackings->find('all')->select(['product_number', 'status'])->where(['is_active' => 1])->group('product_number')->where(['product_id' => $rows->id])->order(['product_number' => 'ASC', 'status' => 'DESC'])->toArray();

            #product name
                $eventualData[$key]['product_name']   = $rows->product_name;

            #assembly tracking
                $eventualData[$key]['assembly']       = $this->SubAssemblyProducts->find('all')
                ->contain([
                    'SubAssemblies' => [
                        'AnnualAssemblyTrackings' => function($query) use($assemblyTrackGroupWhere){
                            return $query
                                        ->select([
                                            'id'                    => 'AnnualAssemblyTrackings.id',
                                            'annual_quarter_plan_id'=> 'AnnualAssemblyTrackings.annual_quarter_plan_id',
                                            'sub_assembly_id'       => 'AnnualAssemblyTrackings.sub_assembly_id',
                                            'product_id'            => 'AnnualAssemblyTrackings.product_id',
                                            'product_number'        => 'AnnualAssemblyTrackings.product_number',
                                            'total_component'       => 'AnnualAssemblyTrackings.total_component',
                                            'completed_component'   => 'AnnualAssemblyTrackings.completed_component',
                                            'status'                => 'AnnualAssemblyTrackings.status',
                                            'status_updated_on'     => 'AnnualAssemblyTrackings.status_updated_on',
                                            'description'           => 'AnnualAssemblyTrackings.description',
                                            'total'                 => 'COUNT(AnnualProductTrackings.status)',
                                            'completed'             => 'SUM(if(AnnualProductTrackings.status=9, 1, 0))',
                                            'group_id'              => 'ComponentGroups.group_id',
                                        ])
                                        ->join([
                                            'AnnualProductTrackings' => [
                                                'table'         => 'annual_product_trackings',
                                                'type'          => 'inner',
                                                'conditions'    => ['AnnualProductTrackings.annual_assembly_tracking_id=AnnualAssemblyTrackings.id']
                                            ],
                                            'ComponentProducts' => [
                                                'table'         => 'component_products',
                                                'type'          => 'inner',
                                                'conditions'    => ['ComponentProducts.id=AnnualProductTrackings.component_product_id']
                                            ],
                                            'ComponentGroups' => [
                                                'table'         => 'component_groups',
                                                'type'          => 'inner',
                                                'conditions'    => ['ComponentGroups.component_id=ComponentProducts.component_id']
                                            ],
                                        ])
                                        ->where($assemblyTrackGroupWhere)
                                        ->group(['AnnualProductTrackings.annual_assembly_tracking_id']);
                        },
                        'ComponentAssemblies' => function($query) use($rows, $componentTrackGroupWhere){
                            return $query
                                        ->select([
                                            'component_id'              => 'Components.id',
                                            'component_name'            => 'Components.component_name',
                                            'component_code'            => 'Components.component_code',
                                            'drawing_reference_no'      => 'Components.drawing_reference_no',
                                            'component_cost'            => 'Components.component_cost',
                                            'batch_quantity'            => 'Components.batch_quantity',
                                            'tag'                       => 'Components.tag',
                                            'sub_assembly_id'           => 'ComponentAssemblies.sub_assembly_id',
                                            'product_id'                => 'ComponentProducts.product_id',
                                            'number_of_component'       => 'ComponentProducts.number_of_component',
                                            'start_product_no'          => 'ComponentProducts.start_product_no',
                                            'end_product_no'            => 'ComponentProducts.end_product_no',
                                            'annual_product_tracking_id'=> 'AnnualProductTrackings.id',
                                            'product_number'            => 'AnnualProductTrackings.product_number',
                                            'status'                    => 'AnnualProductTrackings.status',
                                            'status_updated_on'         => 'AnnualProductTrackings.status_updated_on',
                                            'description'               => 'AnnualProductTrackings.description',
                                            'delays'                    => 'DATEDIFF(NOW(),AnnualProductTrackings.status_updated_on)'
                                        ])
                                        ->contain(['Components'])
                                        ->join([
                                            'ComponentProducts' => [
                                                'table'         => 'component_products',
                                                'type'          => 'inner',
                                                'conditions'    => ['ComponentProducts.component_id=ComponentAssemblies.component_id']
                                            ],
                                            'ComponentGroups'   => [
                                                'table'         => 'component_groups',
                                                'type'          => 'inner',
                                                'conditions'    => ['ComponentGroups.component_id=ComponentProducts.component_id']
                                            ],
                                            'AnnualProductTrackings' => [
                                                'table'         => 'annual_product_trackings',
                                                'type'          => 'inner',
                                                'conditions'    => ['AnnualProductTrackings.component_product_id=ComponentProducts.id']
                                            ],
                                        ])->where($componentTrackGroupWhere);
                        }
                    ],
                ])->where([ 'SubAssemblyProducts.sub_assembly_id IN' => $productAssmbly, 'SubAssemblyProducts.product_id' => $rows->id,  'SubAssemblyProducts.is_active' => 1])->toArray();
            }
        }

    #get product list
		$products = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->where(['is_active' => 1, 'is_eventual_product' => 0]);

    #sub assembly list
		$subAssemblyList = $this->SubAssemblies->find('list', ['keyField' => 'id', 'valueField' => 'sub_assembly_name'])->where(['is_active' => 1])->order(['sub_assembly_name' => 'ASC']);

	#get groups list
		$groups	  = $this->Groups->find('list', ['keyField' => 'id', 'valueField' => 'group_name'])->where(['is_active' => 1]);

    #tracking sheet header
        $trackSheetHeader	= $this->AnnualAssemblyTrackings->find('all')->select(['product_number', 'status'])->where(['is_active' => 1])->group('product_number')->where(['product_id' => $productId])->order(['product_number' => 'ASC', 'status' => 'DESC'])->toArray();

    # set result for view
        $this->set(compact(['trackSheetProductStatus','assemblyStatus', 'componentStatus', 'productDetails', 'subAssemblyId', 'groupId', 'productNo', 'status', 'subAssembliesData', 'eventualData', 'products', 'subAssemblyList', 'groups', 'trackSheetHeader']));
    }

    /**
     * update product status
     *
     */
    public function updateStatus()
    {
    #get current login user id
        $uid = $this->Auth->user('id');
    #variable declaration
        #validation for status updation
        $statusChange   = [ 
            0 => [1,2,3], 
            1 => [0,2,3,4,5], 
            2 => [1], 
            3 => [1,2], 
            4 => [1,2,3,8], 
            5 => [1,2,3,6,7,8,9], 
            6 => [5],
            7 => [5,6],
            8 => [5,6,7],
            9 => [5,6,7] 
        ];
        $this->result   = [];
        $data           = true;
        $statusUpdated  = [];
        $data           = [];
        $status         = true;
        $title          = "";
        $message        = "";
    #check method
		if($this->request->is(['post', 'put'])){
        #find all data by array of id's
            $annualProductTrackings = $this->AnnualProductTrackings->find('all',[
                'conditions'=>[
                    'id IN'     => $this->request->data('ids'),
                    'status IN' => $statusChange[$this->request->data('status')]
                ]
            ]);
        #check selected data valid for update given status
            if($annualProductTrackings->count()>0){
            #update current datetime
                $status_updated_on = Time::now();
            #prepare data for multiple insert
                foreach($annualProductTrackings AS $key => $row):
                #prepare array difference
                    $statusUpdated[] = $row->id;
                #prepare data for update/savemany
                    $this->result[$key]['id']                   = $row->id;
                    $this->result[$key]['status']               = $this->request->data('status');
                    $this->result[$key]['description']          = $this->request->data('description');
                    $this->result[$key]['status_updated_on']    = $status_updated_on;
                endforeach;
            #patch entity
                $entity = $this->AnnualProductTrackings->patchEntities($annualProductTrackings, $this->result);
            #save many check result true/false
                if($this->AnnualProductTrackings->saveMany($entity)){
                    $status     = true;
                    $title      = "Alert";
                    $message    = "Status Updated as Completed";
                }
                else{
                    $status     = false;
                    $title      = "Alert";
                    $message    = "Status Updation Failed";
                }
            }
            else{
                $status     = false;
                $title      = "Alert";
                $message    = "Status Updation Failed";
            }
        }
        else{
            $status     = false;
            $title      = "Alert";
            $message    = "Status Updation Failed";
        }
    #array difference
        $invalidStatus      = array_diff( $this->request->data('ids'), $statusUpdated );
    #check array empty
        if( count($invalidStatus) > 0 ){
        #find status cannot update products list
            $invalidStatusData  = $this->AnnualProductTrackings->find('all',[
                'conditions'=>[
                    'id IN'     => $invalidStatus
                ]
            ])->group(['product_number'])->toArray();
        #product number
            $productNumber = [];
        #array loop
            foreach($invalidStatusData AS $key => $rows){
                $productNumber[] = $rows->product_number;
            }
        #message
            $title   = "Alert";
            $message = 'The below produccts status not updated <br> (Product Number:'.implode(',', $productNumber).')';
        }
    #prepare response data
        $returnData = array( "status" => $status, 'title' => $title, "message" => $message, "data" => $data );
    #return data
        $this->set([
            'data'          => $returnData,
            '_serialize'    => 'data',
        ]);
        $this->RequestHandler->renderAs($this, 'json');
    }
}
