<?php
/**
 * CakePHP(tm) : Rapid Development Framework (https:#cakephp.org)
 * Copyright (c) Cake Software Foundation, Inc. (https:#cakefoundation.org)
 *
 * Licensed under The MIT License
 * For full copyright and license information, please see the LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright Copyright (c) Cake Software Foundation, Inc. (https:#cakefoundation.org)
 * @link      https:#cakephp.org CakePHP(tm) Project
 * @since     0.2.9
 * @license   https:#opensource.org/licenses/mit-license.php MIT License
 */
namespace App\Controller;

use Cake\Controller\Controller;
use Cake\Event\Event;
use Muffin\Footprint\Auth\FootprintAwareTrait;
use AuditStash\Meta\RequestMetadata;
use AuditStash\Meta\ApplicationMetadata;
use DataTables\Controller\DataTablesAjaxRequestTrait;
use Cake\Core\Configure;

/**
 * Application Controller
 *
 * Add your application-wide methods in the class below, your controllers
 * will inherit them.
 *
 * @link https:#book.cakephp.org/3.0/en/controllers.html#the-app-controller
 */
class AppController extends Controller
{
    public $myGlobalVar;
    /**
	 * plugin usage
	**/
	use FootprintAwareTrait {
        _setCurrentUser as _footprintSetCurrentUser;
    }
    use DataTablesAjaxRequestTrait;
	use \Crud\Controller\ControllerTrait;

    /**
     * Initialization hook method.
     *
     * Use this method to add common initialization code like loading components.
     *
     * e.g. `$this->loadComponent('Security');`
     *
     * @return void
     */
    public function initialize()
    {
        parent::initialize();

        $this->loadComponent('RequestHandler', [
            'enableBeforeRedirect' => false,
        ]);
        $this->loadComponent('Flash');

	#Authentication Initialization
		$this->loadComponent('Auth', [
            'loginRedirect' => [
                'controller' => 'AnnualComponentTrackings',
                'action' => 'index'
            ],
            'loginAction' => [
                'controller' => 'Users',
                'action' => 'login',
            ],
			'authenticate' => [
                'Form' => [
                    'fields' => [
                        'username' => 'username',
                        'password' => 'password',
                    ],
                    'userModel' => 'Users',
                ],
			],
            'unauthorizedRedirect' => $this->referer(),
            'authError' => false,
        ]);

	#Load Datatable
		$this->loadComponent('DataTables.DataTables');

        /*
         * Enable the following component for recommended CakePHP security settings.
         * see https:#book.cakephp.org/3.0/en/controllers/components/security.html
         */
        #$this->loadComponent('Security');
    }

	/**
     * Initialization hook method.
    */
	public function beforeRender(Event $event)
	{
	#adminLTE theme
		$this->viewBuilder()->setTheme('AdminLTE');
		$this->viewBuilder()->setLayout('top');
    #load model
        $this->loadModel('ModuleMasters');
        $this->loadModel('UserRolePrivileges');

    #get dynmic menu based on user role
        /* if($this->Auth->user()){
        #get login user
            $user       = $this->Auth->user();
            $userRoleId = $user['user_role_id'];
        #get menu assignment
            $moduleMasters = $this->ModuleMasters->find()
                                ->contain([
                                #get modules
                                    'Modules' => function($query) use($userRoleId){
                                    #join query
                                        return $query
                                                    ->join([
                                                        'UserRolePrivileges' => [
                                                            'table'         => 'user_role_privileges',
                                                            'type'          => 'inner',
                                                            'conditions'    => ['UserRolePrivileges.module_id=Modules.id']
                                                        ]
                                                    ])
                                                    ->where(['UserRolePrivileges.user_role_id' => $userRoleId])
                                                    ->group(['Modules.id', 'Modules.module_master_id']);
                                    },
                                ])
                            #matching user pre
                                ->matching(
                                    'UserRolePrivileges', function($query) use($userRoleId){
                                         return $query->where(['UserRolePrivileges.user_role_id' => $userRoleId]);
                                    }
                                )
                                ->order(['ModuleMasters.module_order' => 'ASC'])
                                ->group(['ModuleMasters.id'])
                                ->toArray();

        #set menus
             $this->set(compact(['moduleMasters']));
        } */
	}

	/*
	* before filter hook method
	*/
	public function beforeFilter(Event $event)
    {
        parent::beforeFilter($event);
    #load model
        $this->loadModel('ModuleMasters');
        $this->loadModel('UserRolePrivileges');
    #check
        if($this->Auth->user())
        {
        #active status tag
            $userDefaultPrivilleges = Configure::read('userDefaultPrivilleges');
            $userDefaultPrivilleges = array_flip($userDefaultPrivilleges);
        #get auth users
            $users                  = $this->Auth->user();
            $userRoleId             = $users['user_role_id'];
        #prepare ACL
            $userRolePrevillege = $this->UserRolePrivileges->find()
                                    ->select([
                                        'controller' => 'Modules.controller',
                                        'actions'    => 'GROUP_CONCAT(ModuleActions.action)'
                                    ])
                                    ->contain(['Modules', 'ModuleActions'])
                                    ->group(['UserRolePrivileges.module_id '])
                                    ->where(['UserRolePrivileges.user_role_id' => $userRoleId])
                                    ->hydrate(false)
                                    ->toArray();
        #clear
            $aclActions = [];
        #prepare for array
            foreach($userRolePrevillege AS $key => $rows){
                $actions = explode(",", $rows['actions']);
                $aclActions[$rows['controller']] = array_flip($actions);
            }

        #prepare actions
            $controller = $this->request->params['controller'];
            $action     = $this->request->params['action'];

        #authorized actions
            #check request type json
            /* if(!$this->request->is('json')){
                if(!isset($userDefaultPrivilleges[$action])){
                #check action exist in controller
                    if(!isset($aclActions[$controller][$action]) && $action!='logout') {
                    #flash message
                        $this->Flash->error(__('Sorry! you area not allowed to access this page.'));
                    # Not Authorized so redirect user to Dashboard
                        return $this->redirect(['controller' => $userRolePrevillege[0]['controller'], 'action' => 'index']);
                    }
                }
            } */

        #check login
            if($action=="login"){
                return $this->redirect(['controller' => 'Users', 'action' => 'index']);
            }
        }
    #allow public to login
        $this->Auth->allow(['login']);
        $this->Auth->allow('forgotPassword');
        $this->Auth->allow('resetPassword');

    #load model
        $eventManager = $this->loadModel()->eventManager();
        $eventManager->on(new RequestMetadata($this->request, $this->Auth->user('id')));
    }

	/*
	* set current session user in foot print for tackt added/modified user information
	*/
	protected function _setCurrentUser($user = null)
    {
	#check current user
        if (!$user) {
            $user = $this->request->getAttribute('identity');
        }
    #return
        return $this->_footprintSetCurrentUser($user);
    }

	/**
     * split given number into given number of parts
    */
	public function splitEqualParts($number, $parts)
	{
		$zp 	= $parts - ($number % $parts);
		$pp 	= $number / $parts;
		$result	= [];
    #loop data
		for ($i = 0; $i < $parts; $i++)
		{
			if($i >= $zp){
				$result[] = (int)$pp + 1;
			}
			else{
				$result[] = (int)$pp;
			}
		}
	#decending order sort
		rsort($result);
	#return data
		return $result;
	}
}
