<?php
namespace App\Controller;

use App\Controller\AppController;

/**
 * ComponentSpareInventories Controller
 *
 * @property \App\Model\Table\ComponentSpareInventoriesTable $ComponentSpareInventories
 *
 * @method \App\Model\Entity\ComponentSpareInventory[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class ComponentSpareInventoriesController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);

    #load model
        $this->loadModel('Users');
        $this->loadModel('Products');
        $this->loadModel('Components');
        $this->loadModel('ComponentProducts');
        $this->loadModel('SpareInventoryComponents');
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #find selection option
        $products = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->where(['is_active' => 1]);
    #set deatils for view
        $this->set(compact('products'));

    #before find
		$this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) {
			$event->subject()->query
                ->contain([
                    'Products'
                ]);
		});

    #return data
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id Component Spare Inventory id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query
                ->contain([
                    'Products', 'SpareInventoryComponents.Components'
                ]);
		});

	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        
    #get user roles
        $products   = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->where(['is_active' => 1]);
    #set result for view
        $this->set(compact('products'));

    #check method
		$this->Crud->on('afterSave', function(\Cake\Event\Event $event){
        #loop data
            foreach($event->subject()->entity->spare_inventory_components AS $key => $rows){
            #get last insert id
                $product_id     = $event->subject()->entity->product_id;
                $component_id   = $rows->component_id;
                $quantity       = $rows->quantity;
            #update query
                $componentProducts = $this->ComponentProducts->find()
                                        ->select([
                                            'id'                => 'id',
                                            'spare_remaning'    => '(spare_remaning-'.$quantity.')',
                                            'spares_taken'      => '(spares_taken+'.$quantity.')',
                                            'spares_taken'      => '(spares_taken+'.$quantity.')',
                                            'status'            => '(if((((spare_remaning-'.$rows->quantity.')/spare_count*100)<50), 0, status))',
                                            'spare_required'    => '(if((((spare_remaning-'.$rows->quantity.')/spare_count*100)<50), (spare_count-(spare_remaning-'.$rows->quantity.')), spare_required))',
                                        ])
                                        ->where(['component_id' => $component_id, 'product_id' => $product_id])
                                        ->hydrate(false)
                                        ->toArray();
            #insert data
                $entity = $this->ComponentProducts->patchEntities($componentProducts, $componentProducts);
                $this->ComponentProducts->saveMany($entity);
            }
        #return
            return $this->redirect(
                ['action' => 'index']
            );
        });

    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Component Spare Inventory id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
    #get products list
        $products                   = $this->Products->find('list', ['keyField' => 'id', 'valueField' => 'product_name'])->where(['is_active' => 1]);
        $spareInventoryComponents   = $this->SpareInventoryComponents->find()->where(['component_spare_inventory_id' => $id, 'is_active' => 1]);
    #set result for view
        $this->set(compact('products', 'spareInventoryComponents'));

    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Component Spare Inventory id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function cancelInventory($id = null)
    {
    #set inventory cancel
        $annualProductTracking = $this->ComponentSpareInventories->updateAll(
            ["is_active" => 0],
            ['id' => $id]
        );
    #set inventory component cancel
        $annualProductTracking = $this->SpareInventoryComponents->updateAll(
            ["is_active" => 0],
            ['component_spare_inventory_id' => $id]
        );
    #find inventory components
        $componentSpareInventories = $this->ComponentSpareInventories->find()->contain(['SpareInventoryComponents'])->where(['id' => $id]);
    #loop product details
        foreach($componentSpareInventories AS $key => $rows){
        #loop components
            foreach($rows->spare_inventory_components AS $cKey => $cRows){
            #get last insert id
                $product_id     = $rows->product_id;
                $component_id   = $cRows->component_id;
                $quantity       = $cRows->quantity;
            #update query
                $componentProducts = $this->ComponentProducts->find()
                                        ->select([
                                            'id'                => 'id',
                                            'spare_remaning'    => '(spare_remaning+'.$quantity.')',
                                            'spares_taken'      => '(spares_taken-'.$quantity.')',
                                            'status'            => '(if((((spare_remaning+'.$quantity.')/spare_count*100)>50), 8, status))',
                                            'spares_taken'      => '(if((((spare_remaning+'.$quantity.')/spare_count*100)>50), (spares_taken-'.$quantity.'), spares_taken))',
                                            'spare_required'    => '(if((((spare_remaning+'.$quantity.')/spare_count*100)>50), (spare_remaning+'.$quantity.'), spare_required))',
                                        ])->where([
                                            'component_id' => $component_id,
                                            'product_id' => $product_id
                                        ])
                                        ->hydrate(false)
                                        ->toArray();
            #insert data
                $entity = $this->ComponentProducts->patchEntities($componentProducts, $componentProducts);
                $this->ComponentProducts->saveMany($entity);
            }
        }
    #return
        return $this->redirect(
            ['action' => 'view', $id]
        );
    }

    /**
     * Delete method
     *
     * @param string|null $id Component Spare Inventory id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
	#return
		return $this->Crud->execute();
    }

    /**
     * get component tracking information
     *
    */
    public function getProductComponents()
    {
    #check method
        if($this->request->is(['post', 'put']))
        {
        #request data
            $product_id         = $this->request->data('product_id');
            $componentProducts  = $this->ComponentProducts
                                    ->find()
                                    ->select([
                                        'id'                => 'Components.id',
                                        'component_name'    => 'Components.component_name',
                                        'spare_count'       => 'ComponentProducts.spare_count',
                                        'spare_remaning'    => 'ComponentProducts.spare_remaning',
                                    ])
                                    ->contain([
                                        'Components'
                                    ])
                                    ->where([
                                        'ComponentProducts.product_id'                  => $product_id,
                                        'ComponentProducts.is_active'                   => 1,
                                        'ComponentProducts.spare_remaning >'            => 0,
                                        'NOT' => array('ComponentProducts.spare_count'  => 0)
                                    ]);
        #prepare response data
            $returnData = array( "status" => null, 'title' => null, "message" => null, "data" => $componentProducts );
        #return data
            $this->set([
                'data'          => $returnData,
                '_serialize'    => 'data',
            ]);
            $this->RequestHandler->renderAs($this, 'json');
        }
    }
}
