<?php
namespace App\Controller;

use App\Controller\AppController;

/**
 * JobOrderComponents Controller
 *
 * @property \App\Model\Table\JobOrderComponentsTable $JobOrderComponents
 *
 * @method \App\Model\Entity\JobOrderComponent[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class JobOrderComponentsController extends AppController
{

    /**
     * initialize method
     */
    public function initialize()
    {
	//initialization
		parent::initialize();

	//CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
        ]);
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
        $this->paginate = [
            'contain' => ['JobOrders', 'Components', 'Processes']
        ];
        $jobOrderComponents = $this->paginate($this->JobOrderComponents);

        $this->set(compact('jobOrderComponents'));
    }

    /**
     * View method
     *
     * @param string|null $id Job Order Component id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'JobOrders', 'Components', 'Processes'
			]);
		});

	#return
		return $this->Crud->execute();
    }
    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $jobOrders = $this->JobOrderComponents->JobOrders->find('list', ['limit' => 200]);
        $components = $this->JobOrderComponents->Components->find('list', ['limit' => 200]);
        $processes = $this->JobOrderComponents->Processes->find('list', ['limit' => 200]);
	//set result values
		$this->set(compact('jobOrders', 'components', 'processes'));
    //return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Job Order Component id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $jobOrders = $this->JobOrderComponents->JobOrders->find('list', ['limit' => 200]);
        $components = $this->JobOrderComponents->Components->find('list', ['limit' => 200]);
        $processes = $this->JobOrderComponents->Processes->find('list', ['limit' => 200]);
	//set result values
		$this->set(compact('jobOrders', 'components', 'processes'));
    //return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Job Order Component id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
    //return
		return $this->Crud->execute();
    }
}
