<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;

/**
 * JobOrders Controller
 *
 * @property \App\Model\Table\JobOrdersTable $JobOrders
 *
 * @method \App\Model\Entity\JobOrder[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class JobOrdersController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
        ]);

    #Load Model
        $this->loadModel('Vendor');
        $this->loadModel('Processes');
        $this->loadModel('VendorComponents');
        $this->loadModel('ComponentProcesses');
        $this->loadModel('GroupUsers');

    #app.php
        $job_order_status = Configure::read('job_order');
        $this->set(compact(['job_order_status']));
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #before paginate
        $this->Crud->on('beforePaginate', function(\Cake\Event\Event $event){
        #Get User Group Id
            $user_id= $this->Auth->User('id');
            $group  = $this->GroupUsers->find()->where(['GroupUsers.user_id'=>$user_id])->first();
        #To show all job orders for md users
            if($this->Auth->User('user_role_id') == 1){
                $event->subject()->query->contain(['Groups', 'Products','Vendors']);
            }else{
            #TO show corresponding group job orders list
                $event->subject()->query->contain(['Groups', 'Products','Vendors'])->where(['JobOrders.group_id'=>$group->group_id]);
            }
        });
    #return
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id Job Order id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Products', 'Vendors','Groups','JobOrderComponents'=>['Components','Processes']
			]);
		});
	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $products = $this->JobOrders->Products
                        ->find(
                            'list', 
                            [
                                'keyField'   => 'id', 'valueField' => 'product_name',
                            ]);
        $vendors = $this->JobOrders->Vendors
                        ->find(
                            'list', 
                            [
                                'keyField'   => 'id', 'valueField' => 'vendor_name',
                        ]);
	#set result values
		$this->set(compact('products', 'vendors'));
    #before save
        $this->Crud->on('beforeSave', function(\Cake\Event\Event $event) {
           $entity= $event->subject()->entity;
        #session
           $user_id=$this->Auth->User('id');
        #find query
           $group=$this->GroupUsers->find()->where(['GroupUsers.user_id'=>$user_id])->first();
           $entity->job_order_no    = 'JOB'.date("Yds").mt_rand(100, 999);
           $entity->group_id        = $group->group_id;
           $entity->status          = 1;
           $entity->is_active       = 1;
        #checks components empty
           if(!empty($entity->job_order_components)){
            #loop
                foreach ($entity->job_order_components as $key => $value) {
                    if(!empty($value->component_qty) && !empty($value->expected_date_delivery))
                    {
                    }
                    else{
                        unset($entity->job_order_components[$key]);
                    }
                }
           }else{
                $this->Flash->error(__('Could not create Job Orders .Please Select atleast one component'));
                return $this->redirect(['action'=>'add']);
           }
        });
    #after save
        $this->Crud->on('afterSave', function(\Cake\Event\Event $event) {
            $this->Flash->success(__('Job Order Created Successfully.'));
            return $this->redirect(['action' => 'index']);
        });
    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Job Order id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $products   = $this->JobOrders->Products->find('list', ['limit' => 200]);
        $vendors    = $this->JobOrders->Vendors->find('list', ['limit' => 200]);
	#set result values
		$this->set(compact('products', 'vendors'));
    #return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Job Order id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
    #return
		return $this->Crud->execute();
    }

    public function getVendorComponentsOld($vendor_id,$product_id){
        $this->autoRender   = false;
        $vendor_components  = $this->ComponentProcesses->find()
                                ->contain(['Components','Processes'])
                                ->join([
                                    'VendorComponents' => [
                                        'table'      => 'vendor_components',
                                        'type'       => 'inner',
                                        'conditions' => ['VendorComponents.component_id = ComponentProcesses.component_id',]
                                    ],
                                    'ComponentProducts' => [
                                        'table'      => 'component_products',
                                        'type'       => 'inner',
                                        'conditions' => ['ComponentProducts.component_id = VendorComponents.component_id',]
                                    ]
                                ])
                                ->enableAutoFields(true)
                                ->where([
                                    'VendorComponents.vendor_id'=>$vendor_id,
                                    'ComponentProducts.product_id'=>$product_id,
                                ])
                                ->toArray();
    #checks data
        if(!empty($vendor_components)){
            $status = 'success';
            $data   = $vendor_components;
        }else{
            $status = 'fail';
            $data   = [];
        }
    #prepare response data
        $returnData = array("status" => $status, "data" => $data);
    #return data
        echo json_encode($returnData);
    }

    public function getVendorComponents($vendor_id,$product_id){
        $this->autoRender  = false;
        $vendor_components = $this->ComponentProcesses->find()
                                ->contain(['Components','Processes'])
                                ->join([
                                    'ComponentProducts' => [
                                        'table'      => 'component_products',
                                        'type'       => 'inner',
                                        'conditions' => ['ComponentProducts.component_id = ComponentProcesses.component_id',]
                                    ],
                                    'VendorProcesses' => [
                                        'table'      => 'vendor_processes',
                                        'type'       => 'inner',
                                        'conditions' => ['VendorProcesses.component_id = ComponentProcesses.component_id','VendorProcesses.process_id = ComponentProcesses.process_id',]
                                    ],
                                ])
                                ->enableAutoFields(true)
                                ->where([
                                    'VendorProcesses.vendor_id'     => $vendor_id,
                                    'ComponentProducts.product_id'  => $product_id,
                                ])->toArray();
    #checks vendor components
        if(!empty($vendor_components)){
            $status = 'success';
            $data   = $vendor_components;
        }else{
            $status = 'fail';
            $data   = [];
        }
    #prepare response data
        $returnData = array("status" => $status, "data" => $data);
    #return data
        echo json_encode($returnData);
    }
}
