<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\I18n\Time;

/**
 * MachineTypes Controller
 *
 * @property \App\Model\Table\MachineTypesTable $MachineTypes
 *
 * @method \App\Model\Entity\MachineType[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class MachineTypesController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #active status tag
        $activeStatus  = Configure::read('activeStatus');
    #find selection option
        $this->set(compact('activeStatus'));
    
    #before find
        $this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) {
            $event->subject()->query->order(['machine_type_name' => 'ASC']);
        });

    #return data
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id Machine Type id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Machines'
			]);
		});

	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Machine Type id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
    #return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Machine Type id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
	#return
		return $this->Crud->execute();
    }
}
