<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;

/**
 * PurchaseOrders Controller
 *
 * @property \App\Model\Table\PurchaseOrdersTable $PurchaseOrders
 *
 * @method \App\Model\Entity\PurchaseOrder[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class PurchaseOrdersController extends AppController
{
    /**
     * initialize method
     */
    public function initialize()
    {
    #initialization
        parent::initialize();

    #CRUD operations
        $this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
        ]);

        $this->loadComponent('Cookie');

        $this->Crud->mapAction('savePurchaseOrder', [
            'className'     => 'Crud.Index',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('createJobOrderPurchaseOrders', [
            'className'     => 'Crud.Edit',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('viewJobOrders', [
            'className'     => 'Crud.View',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('viewStoreComponents', [
            'className'     => 'Crud.View',
            'relatedModels' => true,
        ]);
        

        $this->Crud->mapAction('editJobOrders', [
            'className'     => 'Crud.Edit',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('editStoreComponents', [
            'className'     => 'Crud.Edit',
            'relatedModels' => true,
        ]);
        

        $this->Crud->mapAction('listGroupWiseRawMaterialPurchaseOrders', [
            'className'     => 'Crud.Index',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('listGroupWiseJobOrderPurchaseOrders', [
            'className'     => 'Crud.Index',
            'relatedModels' => true,
        ]);

        $this->Crud->mapAction('listGroupWiseOutComponentPurchaseOrders', [
            'className'     => 'Crud.Index',
            'relatedModels' => true,
        ]);

    #List PO store Components
        $this->Crud->mapAction('listStoreComponents', [
            'className'     => 'Crud.Index',
            'relatedModels' => true,
        ]);

    #create Store Component PO
        $this->Crud->mapAction('createStoreComponentPurchaseOrders', [
            'className'     => 'Crud.Add',
            'relatedModels' => true,
        ]);

    #load model
        $this->loadModel('PurchaseIndents');
        $this->loadModel('PurchaseIndentComponents');
        $this->loadModel('Vendors');
        $this->loadModel('VendorComponents');
        $this->loadModel('VendorRawMaterials');
        $this->loadModel('JobOrders');
        $this->loadModel('VendorStoreComponents');
        $this->loadModel('StoreComponents');
        $this->loadModel('PurchaseOrdersTemp');
        $this->loadModel('AnnualPlans');

    #Purchase Order Terms of delivery
        $purchase_order_terms   = Configure::read('purchase_order_terms');
        $job_order_status       = Configure::read('job_order');
        $purchase_order_status  = Configure::read('purchase_order');
        $purchase_indent_status = Configure::read('purchase_indent');
    #set view
        $this->set(compact(['purchase_order_terms', 'purchase_order_status', 'job_order_status', 'purchase_indent_status']));

    }

    #Purchase Indent for raw material purchase order
    public function rawMaterials()
    {
    #sesiion
        $user_id = $this->Auth->User('id');
    #paginate
        $purchaseIndentsData = $this->paginate($this->PurchaseIndents->find('all')
                                    ->contain(['Products', 'Groups'])
                                    ->where(['PurchaseIndents.status IN' => [1], 'PurchaseIndents.tag =' => 0]));
    #set view
        $this->set(compact(['purchaseIndentsData']));
    }

    #Purchase Indent for Out Components
    public function outComponents()
    {
    #session
        $user_id = $this->Auth->User('id');
    #paginate
        $purchaseIndentsData = $this->paginate($this->PurchaseIndents->find('all')
                                    ->contain(['Products', 'Groups'])
                                    ->where(['PurchaseIndents.status IN' => [1], 'PurchaseIndents.tag =' => 1]));
    #set view
        $this->set(compact(['purchaseIndentsData']));
    }

    #Purchase Order for Job Orders
    public function jobOrders()
    {
    #paginate
        $jobOrders = $this->paginate($this->JobOrders
                        ->find('all')
                        ->contain(['Products', 'Vendors']));
    #set view
        $this->set(compact(['jobOrders']));
    }

    #Create Job Order Purchase Order
    public function createJobOrderPurchaseOrders($job_order_id = null)
    {
        $jobOrder  = $this->JobOrders->get($job_order_id);
        $jobOrders = $this->JobOrders->find('all')
                        ->contain(['Vendors', 'JobOrderComponents'=>['VendorProcesses'=>['Processes','Components']]])
                        ->where(['JobOrders.id' => $job_order_id])->first();
    #set view
        $this->set(compact(['jobOrders', 'job_order_id']));
    #check method
        if ($this->request->is('post') || $this->request->is('put')) {
            $entity = $this->PurchaseOrders->newEntity($this->request->data);
            $result = $this->PurchaseOrders->Save($entity);
        #checks result
            if ($result) {
                $this->Flash->success(__('Purchase Order Created Successfully.'));
                return $this->redirect(['action' => 'jobOrders']);
            }
            else{
                $this->Flash->success(__('Could Not Create Purchase Order'));
                return $this->redirect($this->referrer());
            }
        }
    }

    #Purchase Indent for raw material purchase order
    public function rawMaterialPurchaseOrder()
    {
    #session
        $user_id = $this->Auth->User('id');
    #paginate
        $purchaseIndentsData = $this->paginate($this->PurchaseIndents
                                ->find('all')
                                ->contain(['Products', 'Groups'])
                                ->where(['PurchaseIndents.status !=' => 0]));
    #set view
        $this->set(compact(['purchaseIndentsData']));
    }

    /**
     * List for Raw Materials method
     *
     * @return \Cake\Http\Response|void
     */
    public function listRawMaterials()
    {
        $user_id = $this->Auth->User('id');
    #paginate
        $purchase_orders = $this->paginate($this->PurchaseOrders->find('all')
                                ->contain(['PurchaseIndents' => ['Products', 'Groups'], 
                                    'Vendors', 
                                    'PurchaseOrderComponents' => function ($q) {
                                        return $q
                                                ->select(['PurchaseOrderComponents.purchase_order_id',
                                                    'total_po_amount'  => 'SUM(PurchaseOrderComponents.amount)',
                                                    'total_gst_amount' => 'SUM(PurchaseOrderComponents.gst_amount)',
                                                    'PurchaseOrderComponents.id'])
                                                ->group('PurchaseOrderComponents.purchase_order_id');
                                    },
                                ])
                                ->where(['PurchaseOrders.tag' => 0, 'PurchaseOrders.job_order_id IS NULL']));
        $this->set(compact(['purchase_orders']));
    }

    /**
     * List for Out Components method
     *
     * @return \Cake\Http\Response|void
     */
    public function listOutComponents()
    {
    #session
        $user_id = $this->Auth->User('id');
    #paginate
        $purchase_orders = $this->paginate($this->PurchaseOrders->find('all')
                                ->contain(['PurchaseIndents' => ['Products', 'Groups'], 
                                    'Vendors', 
                                    'PurchaseOrderComponents' => function ($q) {
                                        return $q
                                                ->select(['PurchaseOrderComponents.purchase_order_id',
                                                    'total_po_amount'  => 'SUM(PurchaseOrderComponents.amount)',
                                                    'total_gst_amount' => 'SUM(PurchaseOrderComponents.gst_amount)',
                                                    'PurchaseOrderComponents.id'])
                                                ->group('PurchaseOrderComponents.purchase_order_id');
                                    },
                                ])
                                ->where(['PurchaseOrders.tag' => 1, 'PurchaseOrders.job_order_id IS NULL']));
    #set view
        $this->set(compact(['purchase_orders']));
    }

    /**
     * List for Job Orders method
     *
     * @return \Cake\Http\Response|void
     */
    public function listJobOrders()
    {
        $user_id = $this->Auth->User('id');

        $purchase_orders = $this->paginate($this->PurchaseOrders->find('all')
                                ->contain(['JobOrders' => ['Products', 'Groups'], 
                                    'Vendors', 
                                    'PurchaseOrderComponents' => function ($q) {
                                        return $q
                                                ->select(['PurchaseOrderComponents.purchase_order_id',
                                                    'total_po_amount'  => 'SUM(PurchaseOrderComponents.amount)',
                                                    'total_gst_amount' => 'SUM(PurchaseOrderComponents.gst_amount)',
                                                    'PurchaseOrderComponents.id'])
                                                ->group('PurchaseOrderComponents.purchase_order_id');
                                    },
                                ])
                                ->where(['PurchaseOrders.purchase_indent_id' => 0]));
    #set view
        $this->set(compact(['purchase_orders']));
    }

    #List Store Components
    public function listStoreComponents()
    {
    #paginate
        $this->Crud->on('beforePaginate', function (\Cake\Event\Event $event) {
            $event->subject->query
                            ->contain([
                                'Vendors',
                                 'PurchaseOrderComponents' => function ($q) {
                                        return $q
                                                ->contain(['StoreComponents'])
                                                ->select(['PurchaseOrderComponents.purchase_order_id',
                                                        'total_po_amount'  => 'SUM(PurchaseOrderComponents.amount)',
                                                        'total_gst_amount' => 'SUM(PurchaseOrderComponents.gst_amount)',
                                                        'PurchaseOrderComponents.id'])
                                                ->group('PurchaseOrderComponents.purchase_order_id');
                                    },

                                ])
                            ->where(['PurchaseOrders.is_store_order' =>1]);
        });
    #return
        return $this->Crud->execute();
    }

    #create Store Component PO

    public function createStoreComponentPurchaseOrders()
    {
    #find
        $storeComponents = $this->StoreComponents->find('all')
                                # ->contain(['VendorStoreComponents'])
                                ->where(['StoreComponents.is_active' => 1])
                                ->toArray();
    #find
        $vendors = $this->Vendors
                            ->find('list', [
                                'keyField'   => 'id',
                                'valueField' => 'vendor_name',
                            ])
                            ->join([
                                'VendorStoreComponents' => [
                                    'table'      => 'vendor_store_components',
                                    'type'       => 'INNER',
                                    'conditions' => 'VendorStoreComponents.vendor_id = Vendors.id',
                                ],
                            ])
                            ->toArray();
    #set view
        $this->set(compact(['storeComponents', 'vendors']));
    #check data
        if (!empty($this->request->getData())) {
            $ref_no              = rand(10, 10000);
            $purchase_order_temp = [];
        #loop data
            foreach ($this->request->getData('purchase_order_components') as $key => $value) {
                if (!empty($value['vendor_id']) && !empty($value['amount'])) {
                    $value['ref_no']       = $ref_no;
                    $value['tag']          = 0;
                    $value['gst']          = $value['vendor_component_gst'];
                    $value['price']        = $value['vendor_component_price'];
                    $purchase_order_temp[] = $value;
                }
            }
            $entities = $this->PurchaseOrdersTemp->newEntities($purchase_order_temp);
            $result   = $this->PurchaseOrdersTemp->saveMany($entities);
        #return
            return $this->redirect(['action' => 'confirmStoreComponentPurchaseOrders', $ref_no]);
        }
    }



    #Temp Save Of Store Purchase Order
    public function confirmStoreComponentPurchaseOrders($ref_no)
    {
        $vendor_purchase_order_data = $this->Vendors->find('all')
                                        ->enableAutoFields(true)
                                        ->contain(['PurchaseOrdersTemp' => function ($q) use ($ref_no) {
                                            return $q
                                                    ->select([
                                                        'PurchaseOrdersTemp.vendor_id',
                                                        'component_name' => 'StoreComponents.component_name',
                                                        'order_quantity' => 'PurchaseOrdersTemp.order_quantity',
                                                        'gst_value'      => 'VendorStoreComponents.gst',
                                                        'price_value'    => 'VendorStoreComponents.price',
                                                    ])
                                                    ->enableAutoFields(true)
                                                    ->join([
                                                        'VendorStoreComponents' => [
                                                            'table'      => 'vendor_store_components',
                                                            'type'       => 'inner',
                                                            'conditions' => [
                                                                'VendorStoreComponents.store_component_id = PurchaseOrdersTemp.store_component_id',
                                                            ],
                                                        ],
                                                        'StoreComponents'       => [
                                                            'table'      => 'store_components',
                                                            'type'       => 'inner',
                                                            'conditions' => [
                                                                'StoreComponents.id = PurchaseOrdersTemp.store_component_id',
                                                            ],
                                                        ],
                                                    ])
                                                    ->distinct(['PurchaseOrdersTemp.store_component_id'])
                                                    ->where(['PurchaseOrdersTemp.ref_no' => $ref_no]);
                                            },
                                        ])
                                        ->toArray();
    #set view
        $this->set(compact(['vendor_purchase_order_data']));
    }

    /**
     * Edit Job Orders method
     *
     * @param string|null $id Purchase Order id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function editJobOrders($id = null)
    {
    #before find
        $purchaseOrder = $this->PurchaseOrders->get($id);
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
            $event->subject->query->contain(['JobOrders', 'Vendors', 'PurchaseOrderComponents' => ['Components', 'Processes']]);
        });
    #after save
        $this->Crud->on('afterSave', function (\Cake\Event\Event $event) {
            $entity = $event->subject->entity;
            $this->Flash->success(__('Purchase Order Updated Successfully.'));
            return $this->redirect(['action' => 'listJobOrders']);
        });
    #return
        return $this->Crud->execute();
    }

    /**
     * Edit Store Components method
     *
     * @param string|null $id Purchase Order id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function editStoreComponents($id = null)
    {
    #before find
        $purchaseOrder = $this->PurchaseOrders->get($id);
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
            $event->subject->query->contain(['Vendors', 'PurchaseOrderComponents' => ['StoreComponents']]);
        });
    #after save
        $this->Crud->on('afterSave', function (\Cake\Event\Event $event) {
            $entity = $event->subject->entity;
            $this->Flash->success(__('Purchase Order Updated Successfully.'));
            return $this->redirect(['action' => 'listStoreComponents']);
        });
    #return
        return $this->Crud->execute();
    }

    /**
     * View Job Orders method
     *
     * @param string|null $id Purchase Order id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function viewJobOrders($id = null)
    {
    #before find
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
        #query
            $event->subject()->query
                            ->select(['Users.username', 'Users.user_full_name'])
                            ->contain([
                                'JobOrders'               => ['Products', 'Groups'], 'Vendors',
                                'PurchaseOrderComponents' => function ($q) {
                                    return $q
                                            ->contain([
                                                'Components', 'Processes',
                                            ])
                                            #->distinct(['PurchaseOrderComponents.component_id'])
                                            ->enableAutoFields(true);
                                },
                                'GoodsReceivedNotes' => function ($q) {
                                    return $q
                                            ->select([
                                                'GoodsReceivedNotes.purchase_order_id',
                                                'GoodsReceivedNotes.id',
                                                'Users.id',
                                                'Users.user_full_name',
                                            ])
                                            ->join([
                                                'Users' => [
                                                    'table'      => 'users',
                                                    'type'       => 'left',
                                                    'conditions' => ['Users.id = GoodsReceivedNotes.created_by',]
                                                ],
                                            ])
                                            ->enableAutoFields(true);
                                },
                            ])
                            ->join([
                                'Users' => [
                                    'table'      => 'users',
                                    'type'       => 'left',
                                    'conditions' => ['Users.id = PurchaseOrders.created_by',]
                                ],
                            ])
                            ->enableAutoFields(true);
        });
    #return
        return $this->Crud->execute();

    }

     /**
     * View Store Components method
     *
     * @param string|null $id Purchase Order id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function viewStoreComponents($id = null)
    {

        #before find
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
            $event->subject()->query
                                ->select(['Users.username', 'Users.user_full_name'])
                                ->contain([
                                    'Vendors',
                                    'PurchaseOrderComponents' => function ($q) {
                                        return $q
                                            ->contain([
                                                'StoreComponents',
                                            ])
                                            #->distinct(['PurchaseOrderComponents.component_id'])
                                            ->enableAutoFields(true);
                                    },
                                    'GoodsReceivedNotes' => function ($q) {
                                    return $q
                                            ->select([
                                                'GoodsReceivedNotes.purchase_order_id',
                                                'GoodsReceivedNotes.id',
                                                'Users.id',
                                                'Users.user_full_name',
                                            ])
                                            ->join([
                                                'Users' => [
                                                    'table'      => 'users',
                                                    'type'       => 'left',
                                                    'conditions' => ['Users.id = GoodsReceivedNotes.created_by',]
                                                ],
                                            ])
                                            ->enableAutoFields(true);
                                    },
                                ])->join([
                                    'Users' => [
                                        'table'      => 'users',
                                        'type'       => 'left',
                                        'conditions' => ['Users.id = PurchaseOrders.created_by',]
                                    ],
                                ])
                                ->enableAutoFields(true);
        });
    #return
        return $this->Crud->execute();
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #session
        $user_id = $this->Auth->User('id');
    #paginate
        $purchase_orders = $this->paginate($this->PurchaseOrders->find('all')
                                ->contain(['PurchaseIndents' => ['Products', 'Groups'], 'Vendors', 
                                            'PurchaseOrderComponents' => function ($q) {
                                                return $q
                                                        ->select([
                                                            'PurchaseOrderComponents.purchase_order_id',
                                                            'total_po_amount'  => 'SUM(PurchaseOrderComponents.amount)',
                                                            'total_gst_amount' => 'SUM(PurchaseOrderComponents.gst_amount)',
                                                            'PurchaseOrderComponents.id'
                                                        ])
                                                        ->group('PurchaseOrderComponents.purchase_order_id');
                                    },
                                ])
                                ->where(['PurchaseIndents.status !=' => 0]));
    #paginate
        $this->set(compact(['purchase_orders']));
    }

    /**
     * View method
     *
     * @param string|null $id Purchase Order id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
    #before find
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
        #query
            $event->subject()->query
                            ->select(['Users.username', 'Users.user_full_name'])
                            ->contain([
                                'PurchaseIndents'         => ['Products', 'Groups'], 'Vendors',
                                'PurchaseOrderComponents' => function ($q) {
                                    return $q
                                            ->select([
                                                'PurchaseOrderComponents.purchase_order_id',
                                                'PurchaseOrderComponents.id',
                                                'Components.id',
                                                'RawMaterials.id',
                                                'Components.component_name',
                                                'Components.component_cost',
                                                'Components.component_gst',
                                                'RawMaterials.material_name',
                                                # 'VendorComponents.id',
                                                # 'VendorComponents.vendor_id',
                                                # 'VendorComponents.component_id',
                                                # 'VendorComponents.price',
                                                # 'VendorComponents.gst',
                                            ])
                                            ->join([
                                                # 'VendorComponents' => [
                                                #     'table'      => 'vendor_components',
                                                #     'type'       => 'inner',
                                                #     'conditions' => [
                                                #         'VendorComponents.component_id = PurchaseOrderComponents.component_id',
                                                #     ]],
                                                'Components'   => [
                                                    'table'      => 'components',
                                                    'type'       => 'inner',
                                                    'conditions' => ['Components.id = PurchaseOrderComponents.component_id',]
                                                ],
                                                'RawMaterials' => [
                                                    'table'      => 'raw_materials',
                                                    'type'       => 'left',
                                                    'conditions' => ['RawMaterials.id = PurchaseOrderComponents.raw_material_id',]
                                                ],
                                            ])
                                            ->distinct(['PurchaseOrderComponents.component_id'])
                                            ->enableAutoFields(true);
                                },
                                'GoodsReceivedNotes' => function ($q) {
                                    return $q
                                            ->select([
                                                'GoodsReceivedNotes.purchase_order_id',
                                                'GoodsReceivedNotes.id',
                                                'Users.id',
                                                'Users.user_full_name',
                                            ])
                                            ->join([
                                                'Users' => [
                                                    'table'      => 'users',
                                                    'type'       => 'left',
                                                    'conditions' => ['Users.id = GoodsReceivedNotes.created_by',]
                                                ],
                                            ])
                                            ->enableAutoFields(true);
                                },
                            ])
                            ->join([
                                'Users' => [
                                    'table'      => 'users',
                                    'type'       => 'left',
                                    'conditions' => ['Users.id = PurchaseOrders.created_by',]
                                ],
                            ])->enableAutoFields(true);
        });
    #return
        return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add($id = null)
    {
    #Get Purchase Indent Data
        $purchaseIndentsData = $this->PurchaseIndents->find('all')
            ->contain(['Products', 'AnnualQuarterPlans', 'Groups'])
            ->where(['PurchaseIndents.id' => $id])
            ->first();
    #get quarter plan based on annual plan
        $annualPlans    = $this->AnnualPlans->find()
                            ->contain([
                                'AnnualPlannedProducts' => [
                                    'Products' => [
                                        'AnnualQuarterPlannedProducts' => function($query) use($purchaseIndentsData){
                                            return $query
                                                        ->select([
                                                            'id'                                => 'AnnualQuarterPlans.id',
                                                            'status'                            => 'AnnualQuarterPlans.status',
                                                            'duration'                          => 'AnnualQuarterPlans.duration',
                                                            'annual_quarter_planned_product_id' => 'AnnualQuarterPlannedProducts.id',
                                                            'product_id'                        => 'AnnualQuarterPlannedProducts.product_id',
                                                            'product_quantity'                  => 'AnnualQuarterPlannedProducts.product_quantity',
                                                            'product_buffer_quantity'           => 'AnnualQuarterPlannedProducts.product_buffer_quantity',
                                                            'product_requested_quantity'        => 'AnnualQuarterPlannedProducts.product_requested_quantity',
                                                            'product_total_quantity'            => 'AnnualQuarterPlannedProducts.product_total_quantity',
                                                            'last_product_no'                   => 'AnnualQuarterPlannedProducts.last_product_no',
                                                        ])
                                                        ->join([
                                                            'AnnualQuarterPlans' => [
                                                                'table'         => 'annual_quarter_plans',
                                                                'type'          => 'inner',
                                                                'conditions'    => ['AnnualQuarterPlans.id=AnnualQuarterPlannedProducts.annual_quarter_plan_id']
                                                            ],
                                                            'AnnualPlans' => [
                                                                'table'         => 'annual_plans',
                                                                'type'          => 'inner',
                                                                'conditions'    => ['AnnualPlans.id=AnnualQuarterPlans.annual_plan_id']
                                                            ],
                                                        ])
                                                        ->where([
                                                            'AnnualPlans.id'    => $purchaseIndentsData['annual_plan_id'],
                                                        ]);
                                        }
                                    ]
                                ]
                            ])
                            ->where([ 'AnnualPlans.id' => $purchaseIndentsData['annual_plan_id'], 'AnnualPlans.is_active' => 1 ])
                            ->first();
    #set data
        $this->set(compact('purchaseIndentsData', 'annualPlans'));

    #find vendor details
        $vendors = $this->Vendors
                        ->find('list', [
                            'keyField'   => 'id',
                            'valueField' => 'vendor_name',
                        ])->toArray();

    #check purchase indent tag
        $purchaseIndentVendorsData = [];

        if (!empty($purchaseIndentsData) && $purchaseIndentsData->tag == 0) {

            $this->loadModel('ComponentRawMaterials');

            $purchaseIndents = $this->PurchaseIndents->find('all')
                ->contain(['PurchaseIndentComponents' => ['Components' => ['ComponentRawMaterials' => ['RawMaterials' => ['VendorRawMaterials' => 'Vendors']]]]])
                ->where(['PurchaseIndents.id' => $id])->first();

        } else {
            #For out components
            $purchaseIndents = $this->PurchaseIndents->find('all')
                ->contain(['PurchaseIndentComponents' => ['Components' => ['VendorComponents' => 'Vendors']]])
                ->where(['PurchaseIndents.id' => $id])->first();

        }

        $this->set(compact(['purchaseIndentVendorsData', 'purchaseIndents', 'vendors']));
        return $this->Crud->execute();
    }
    /**
     * Edit method
     *
     * @param string|null $id Purchase Order id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {

        #before find
        $purchaseOrder = $this->PurchaseOrders->get($id);

        if ($purchaseOrder->tag == 0) {
            $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
                $event->subject->query->contain(['PurchaseIndents', 'Vendors', 'PurchaseOrderComponents' => ['Components', 'RawMaterials']]);
            });
        } else {
            $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
                $event->subject->query->contain(['PurchaseIndents', 'Vendors', 'PurchaseOrderComponents' => ['Components']]);
            });
        }

        $this->Crud->on('beforeSave', function (\Cake\Event\Event $event) {
            $entity         = $event->subject->entity;
            $entity->status = 1;
        });

        $this->Crud->on('afterSave', function (\Cake\Event\Event $event) {
            $entity = $event->subject->entity;

            if ($entity->tag == 0) {
                return $this->redirect(['action' => 'listRawMaterials']);
            } else {
                return $this->redirect(['action' => 'listOutComponents']);
            }

            $this->Flash->success(__('Purchase Order Updated Successfully.'));

        });
        return $this->Crud->execute();
    }
    /**
     * Delete method
     *
     * @param string|null $id Purchase Order id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
        #return
        return $this->Crud->execute();
    }

    #Create Purchase Order
    public function savePurchaseOrder()
    {
        $this->viewBuilder()->setLayout('print');
        if ($this->request->is(['ajax'])) {
            $entity    = $this->PurchaseOrders->newEntity($this->request->getData());
            
            $dataSaved = $this->PurchaseOrders->save($entity);
            if ($dataSaved) {
                $response = array('code' => 1);
                echo json_encode($response);
                exit;
            } else {
                $response = array('errors' => $entity->errors(), 'code' => 0);
                echo json_encode($response);
                exit;
            }
        }

    }

    public function confirmPurchaseOrder()
    {
        #return
        $this->autoRender = false;
        if ($this->request->is(['post'])) {
            $entity    = $this->PurchaseOrders->newEntity($this->request->getData());

            $dataSaved = $this->PurchaseOrders->save($entity);
            if ($dataSaved) {

                $response = array('code' => 1);
                echo json_encode($response);
                exit;
            } else {
                $response = array('errors' => $entity->errors(), 'code' => 0);
                echo json_encode($response);
                exit;
            }
        }
    }

    public function approve($id)
    {

        $this->autoRender      = false;
        $purchaseOrder         = $this->PurchaseOrders->get($id);
        $purchaseOrder->status = 2;
        $this->PurchaseOrders->save($purchaseOrder);
        #return
        return $this->redirect($this->request->referer());
    }

    public function hold($id)
    {

        $this->autoRender      = false;
        $purchaseOrder         = $this->PurchaseOrders->get($id);
        $purchaseOrder->status = 4;
        $this->PurchaseOrders->save($purchaseOrder);
        #return
        return $this->redirect($this->request->referer());
    }

    public function reject($id)
    {

        $this->autoRender      = false;
        $purchaseOrder         = $this->PurchaseOrders->get($id);
        $purchaseOrder->status = 5;
        $this->PurchaseOrders->save($purchaseOrder);
        #return
        return $this->redirect($this->request->referer());
    }

    public function getVendorComponentPrice($vendor_id, $component_id)
    {

        $this->autoRender = false;
        $component_price  = $this->VendorComponents->find('all')
            ->select(['VendorComponents.price', 'VendorComponents.gst'])
            ->where(['VendorComponents.vendor_id' => $vendor_id,
                'VendorComponents.component_id'       => $component_id,
            ])
            ->first();

        #prepare response data
        $returnData = array("status" => 'success', "data" => $component_price);
        #return data
        echo json_encode($returnData);

    }

    public function getVendorRawMaterialPrice($vendor_id, $component_id, $raw_material_id)
    {
        $this->autoRender  = false;
        $raw_materil_price = $this->VendorRawMaterials->find('all')
            ->select(['VendorRawMaterials.price',
                'VendorRawMaterials.gst',
                'ComponentRawMaterials.id',
                'ComponentRawMaterials.raw_material_shape',
                'ComponentRawMaterials.raw_material_id',
                'ComponentRawMaterials.min_cut_length',
                'ComponentRawMaterials.outer_dia',
                'ComponentRawMaterials.inner_dia',
                'ComponentRawMaterials.raw_material_thickness',
                'ComponentRawMaterials.raw_material_width',
                'ComponentRawMaterials.raw_material_length',
                'ComponentRawMaterials.unit_of_measurement',
                'ComponentRawMaterials.quantity_per_component',
            ])
            ->join([
                'ComponentRawMaterials' => [
                    'table'      => 'component_raw_materials',
                    'type'       => 'inner',
                    'conditions' => [
                        'ComponentRawMaterials.raw_material_id = VendorRawMaterials.raw_material_id',
                    ],
                ],
            ])
            ->where(['VendorRawMaterials.vendor_id' => $vendor_id,
                'VendorRawMaterials.raw_material_id'    => $raw_material_id,
            ])
            ->first();

        #prepare response data
        $returnData = array("status" => 'success', "data" => $raw_materil_price);
        #return data
        echo json_encode($returnData);

    }

    #Get Store Components Price
    public function getVendorStoreComponentPrice($vendor_id, $store_component_id)
    {
        $this->autoRender      = false;
        $store_component_price = $this->VendorStoreComponents->find('all')
            ->where(['VendorStoreComponents.vendor_id' => $vendor_id,
                'VendorStoreComponents.store_component_id' => $store_component_id,
            ])
            ->first();

        #prepare response data
        $returnData = array("status" => 'success', "data" => $store_component_price);
        #return data
        echo json_encode($returnData);
    }

    #Create Purchase Order

    public function savePurchaseOrders()
    {
        if (!empty($this->request->getData())) {

            $this->loadModel('PurchaseOrdersTemp');
            $ref_no              = rand(10, 10000);
            $purchase_order_temp = [];

            $purchase_indent_id = $this->request->getData('purchase_indent_id');

            foreach ($this->request->getData('purchase_order_components') as $key => $value) {
                if (!empty($value['vendor_id']) && !empty($value['amount'])) {
                    $value['ref_no']       = $ref_no;
                    $value['gst']          = $value['vendor_component_gst'];
                    $value['price']        = $value['vendor_component_price'];
                    $purchase_order_temp[] = $value;
                }
            }
            $entities = $this->PurchaseOrdersTemp->newEntities($purchase_order_temp);

            $result = $this->PurchaseOrdersTemp->saveMany($entities);

            return $this->redirect(['action' => 'confirmPurchaseOrders', $purchase_indent_id, $ref_no]);

        }

    }

    #Confirm Purchase Orders

    public function confirmPurchaseOrders($purchase_indent_id, $ref_no)
    {

        $this->loadModel('Vendors');
        $this->loadModel('PurchaseOrdersTemp');

        $purchaseIndents = $this->PurchaseIndents->find('all')
            ->contain(['Products', 'Groups'])
            ->where(['PurchaseIndents.id' => $purchase_indent_id])->first();

        if ($purchaseIndents->tag == 0 && $ref_no) {
            $vendor_purchase_order_data = $this->Vendors->find('all')
                ->enableAutoFields(true)
                ->contain(['PurchaseOrdersTemp' => function ($q) use ($ref_no) {
                    return $q
                        ->select(['PurchaseOrdersTemp.vendor_id',
                            'raw_material_id'   => 'RawMaterials.id',
                            'raw_material_name' => 'RawMaterials.material_name',
                            #'raw_material_shape'     => 'ComponentRawMaterials.raw_material_shape',
                            # 'raw_material_thickness' => 'ComponentRawMaterials.raw_material_thickness',
                            # 'raw_material_width'     => 'ComponentRawMaterials.raw_material_width',
                            # 'raw_material_length'    => 'ComponentRawMaterials.raw_material_length',
                            # 'unit_of_measurement'    => 'ComponentRawMaterials.unit_of_measurement',
                            'order_quantity'    => 'PurchaseOrdersTemp.order_quantity',
                            'po_quantity'       => 'PurchaseIndentComponents.po_quantity',
                            'actual_quantity'   => 'PurchaseIndentComponents.actual_quantity',
                            'gst'               => 'PurchaseOrdersTemp.gst',
                            'price'             => 'PurchaseOrdersTemp.price',

                        ])
                        ->enableAutoFields(true)
                        ->contain(['Components'])
                        ->join([
                            # 'VendorRawMaterials'       => [
                            #     'table'      => 'vendor_raw_materials',
                            #     'type'       => 'inner',
                            #     'conditions' => [
                            #         'VendorRawMaterials.vendor_id = PurchaseOrdersTemp.vendor_id',
                            #         'VendorRawMaterials.raw_material_id = PurchaseOrdersTemp.raw_material_id',
                            #     ],
                            # ],
                            # 'ComponentRawMaterials'    => [
                            #     'table'      => 'component_raw_materials',
                            #     'type'       => 'inner',
                            #     'conditions' => [
                            #         'ComponentRawMaterials.raw_material_id = PurchaseOrdersTemp.raw_material_id',
                            #         'ComponentRawMaterials.component_id = PurchaseOrdersTemp.component_id',
                            #     ],
                            # ],
                            'RawMaterials'             => [
                                'table'      => 'raw_materials',
                                'type'       => 'inner',
                                'conditions' => [
                                    'RawMaterials.id = PurchaseOrdersTemp.raw_material_id',
                                ],
                            ],
                            # 'VendorComponents'         => [
                            #     'table'      => 'vendor_components',
                            #     'type'       => 'inner',
                            #     'conditions' => [
                            #         'VendorComponents.component_id = PurchaseOrdersTemp.component_id',
                            #     ],
                            # ],
                            'PurchaseIndentComponents' => [
                                'table'      => 'purchase_indent_components',
                                'type'       => 'inner',
                                'conditions' => ['PurchaseIndentComponents.component_id = PurchaseOrdersTemp.component_id',
                                    'PurchaseIndentComponents.purchase_indent_id = PurchaseOrdersTemp.purchase_indent_id'],

                            ],
                        ])
                        ->distinct('PurchaseOrdersTemp.id')
                        ->where(['PurchaseOrdersTemp.ref_no' => $ref_no]);
                },
                ])
                ->toArray();

        } elseif ($purchaseIndents->tag == 1 && $ref_no) {
            $vendor_purchase_order_data = $this->Vendors->find('all')
                ->enableAutoFields(true)
                ->contain(['PurchaseOrdersTemp' => function ($q) use ($ref_no) {
                    return $q
                        ->select(['PurchaseOrdersTemp.vendor_id',
                            'tag'             => 'PurchaseOrdersTemp.tag',
                            'component_name'  => 'Components.component_name',
                            'order_quantity'  => 'PurchaseOrdersTemp.order_quantity',
                            'po_quantity'     => 'PurchaseIndentComponents.po_quantity',
                            'actual_quantity' => 'PurchaseIndentComponents.actual_quantity',
                            'gst_value'       => 'VendorComponents.gst',
                            'price_value'     => 'VendorComponents.price',

                        ])
                        ->enableAutoFields(true)
                        ->join([
                            'VendorComponents'         => [
                                'table'      => 'vendor_components',
                                'type'       => 'inner',
                                'conditions' => [
                                    'VendorComponents.component_id = PurchaseOrdersTemp.component_id',
                                ],
                            ],
                            'Components'               => [
                                'table'      => 'components',
                                'type'       => 'inner',
                                'conditions' => [
                                    'Components.id = PurchaseOrdersTemp.component_id',
                                ],
                            ],
                            'PurchaseIndentComponents' => [
                                'table'      => 'purchase_indent_components',
                                'type'       => 'inner',
                                'conditions' => ['PurchaseIndentComponents.component_id =   PurchaseOrdersTemp.component_id',
                                    'PurchaseIndentComponents.purchase_indent_id = PurchaseOrdersTemp.purchase_indent_id',
                                ],

                            ],
                        ])
                        ->distinct(['PurchaseOrdersTemp.component_id'])
                        ->where(['PurchaseOrdersTemp.ref_no' => $ref_no]);
                },
                ])
                ->toArray();

            #pr($vendor_purchase_order_data);exit;

        }

        $this->set(compact(['purchaseIndents', 'vendor_purchase_order_data']));
    }

    public function listGroupWiseRawMaterialPurchaseOrders($id = null)
    {

        $user_details = $this->Auth->User();
        $this->Crud->on('beforePaginate', function (\Cake\Event\Event $event) use ($user_details) {
            $event->subject->query
                ->contain(['Vendors', 'PurchaseOrderComponents' => function ($q) {
                    return $q
                        ->select(['PurchaseOrderComponents.purchase_order_id',
                            'total_po_amount'  => 'SUM(PurchaseOrderComponents.amount)',
                            'total_gst_amount' => 'SUM(PurchaseOrderComponents.gst_amount)',
                            'PurchaseOrderComponents.id'])
                        ->group('PurchaseOrderComponents.purchase_order_id');
                },
                ])
                ->select([
                    'Groups.id',
                    'Groups.group_name',
                    'PurchaseIndents.id',
                    'PurchaseIndents.purchase_intent_no',
                    'Products.id',
                    'Products.product_name',
                ])
                ->join([
                    'PurchaseIndents' => [
                        'table'      => 'purchase_indents',
                        'type'       => 'inner',
                        'conditions' => [
                            'PurchaseIndents.id = PurchaseOrders.purchase_indent_id',
                        ],
                    ],
                    'Products'        => [
                        'table'      => 'products',
                        'type'       => 'inner',
                        'conditions' => [
                            'Products.id = PurchaseIndents.product_id',
                        ],
                    ],
                    'Groups'          => [
                        'table'      => 'groups',
                        'type'       => 'inner',
                        'conditions' => [
                            'Groups.id = PurchaseIndents.group_id',
                        ],
                    ],
                    'GroupUsers'      => [
                        'table'      => 'group_users',
                        'type'       => 'inner',
                        'conditions' => [
                            'GroupUsers.group_id = Groups.id',
                        ],
                    ],
                ]
                )
                ->enableAutoFields(true)
                ->where(['GroupUsers.user_id' => $user_details['id'],
                    'PurchaseOrders.job_order_id IS NULL',
                    'PurchaseOrders.tag'          => 0,
                ]);
        });

        return $this->Crud->execute();
    }

    public function testSms()
    {
        $this->autoRender = false;
        # $Textlocal = new \Textlocal(false, false, 'mRko+RHZ2xo-8vGpQ7yPpJ5zwbnEED57MJMYJYnt54');

        # $names = $Textlocal->getSenderNames();

        # $numbers = array(918489855454,918072617697);
        # $sender = 'TXTLCL ';
        # $message = 'New PO created';

        # $response = $Textlocal->sendSms($numbers, $message, $sender);

        # Account details
        $apiKey = urlencode('mRko+RHZ2xo-8vGpQ7yPpJ5zwbnEED57MJMYJYnt54');

        # Message details
        $numbers = array(918489855454, 918072617697);
        $sender  = urlencode('TXTLCL');
        $message = rawurlencode('New PO created from super machine works');

        $numbers = implode(',', $numbers);

        # Prepare data for POST request
        $data = array('apikey' => $apiKey, 'numbers' => $numbers, "sender" => $sender, "message" => $message);

        # Send the POST request with cURL
        $ch = curl_init('https:#api.textlocal.in/send/');
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);
        curl_close($ch);

        # Process your response here
        echo $response;
        print_r($response);exit;
    }

}
