<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Cache\Cache;

/**
 * StoreComponents Controller
 *
 * @property \App\Model\Table\StoreComponentsTable $StoreComponents
 *
 * @method \App\Model\Entity\StoreComponent[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class StoreComponentsController extends AppController
{

    /**
     * initialize method
     */
    public function initialize()
    {
    #initialization
        parent::initialize();

    #CRUD operations
        $this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
        ]);
    
    #load model
        $this->loadModel('Vendors');
        $this->loadModel('VendorStoreComponents');

    #common select options
        $buyer_frequency = [1 => 'Monthly', 2 => 'Bimonthly', 3 => 'Quarterly', 4 => 'Half Quarterly', 5 => 'Yearly'];
        $this->set(compact('buyer_frequency'));
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
        $this->paginate = [
            'contain' => ''
        ];
        $storeComponents = $this->paginate($this->StoreComponents);

        $this->set(compact('storeComponents'));
    }

    /**
     * View method
     *
     * @param string|null $id Store Component id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
    #before find
        $this->Crud->on('beforeFind', function (\Cake\Event\Event $event) {
            $event->subject()->query->contain([
                'Products'
            ]);
        });

    #return
        return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
    #find query
        $products   = $this->StoreComponents->Products->find('list', ['limit' => 200]);
        $vendors    = $this->Vendors->find('list', ['keyField' => 'id', 'valueField' => 'vendor_name'])->where(['is_active' => 1]);

    #set result values
        $this->set(compact('products', 'vendors'));

    #Before Save
        $this->Crud->on('beforeSave', function (\Cake\Event\Event $event) {
            $entity            = $event->subject->entity;
            $entity->is_active = 1;
        });

    #return
        return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Store Component id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
    #find query
        $products   = $this->StoreComponents->find('list', ['limit' => 200]);
        $vendors    = $this->Vendors->find('list', ['keyField' => 'id', 'valueField' => 'vendor_name'])->where(['is_active' => 1]);
    #set result values
        $this->set(compact('products', 'vendors'));

    #return
        return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Store Component id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
        #return
        return $this->Crud->execute();
    }

    /**
     * Store Component import method
     *
     */
    public function import()
    {
        #variable declaration
        $importData = [];

        #check request method
        if ($this->request->is('post')) {
            if (file_exists($_FILES['import_file']['tmp_name']) || is_uploaded_file($_FILES['import_file']['tmp_name'])) {
                #import data
                $importData = $this->StoreComponents->importCsv($this->request->data['import_file']['tmp_name'], [
                    'drawing_no',
                    'component_type',
                    'component_name',
                    'component_make',
                    'npm',
                    'buying_frequency',
                ]);
                #variable declaration
                $this->result = [];
                #foreach
                foreach ($importData as $key => $rows) {
                    $importData[$key]['drawing_no']       = (empty($rows['drawing_no']) ? null : $rows['drawing_no']);
                    $importData[$key]['component_type']   = (empty($rows['component_type']) ? null : $rows['component_type']);
                    $importData[$key]['component_name']   = (empty($rows['component_name']) ? null : $rows['component_name']);
                    $importData[$key]['contact_number']   = (empty($rows['contact_number']) ? null : $rows['contact_number']);
                    $importData[$key]['component_make']   = (empty($rows['component_make']) ? null : $rows['component_make']);
                    $importData[$key]['npm']              = (empty($rows['npm']) ? null : $rows['npm']);
                    $importData[$key]['buying_frequency'] = (empty($rows['buying_frequency']) ? 1 : $rows['buying_frequency']);

                }
            }
            #delete cache
            Cache::clear(true);
            #set cache
            Cache::write('importStoreData', $importData);
            return $this->redirect(['action' => 'confirm']);
        }
    }

    /**
     * import store Component  confirm method
     *
     */
    public function confirm()
    {
        #set value
        $importData = Cache::read('importStoreData');
        #check method
        if ($this->request->is('post')) {
            #array each

            $data = [];
            if (!empty($importData)) {

                foreach ($importData as $key => $rows) {
                    #product name
                    if (!empty($rows['component_name'])) {

                        $storeComponent = $this->StoreComponents->find('all')
                            ->where(['component_name' => $rows['component_name'], 'component_type' => $rows['component_type']])
                            ->toArray();
                        if (count($storeComponent) == 0) {
                            $data[$key]['component_name']    = $rows['component_name'];
                            $data[$key]['component_part_no'] = $rows['drawing_no'];
                            $data[$key]['component_type']    = $rows['component_type'];
                            $data[$key]['component_make']    = $rows['component_make'];
                            $data[$key]['npm']               = $rows['npm'];
                            $data[$key]['buying_frequency']  = $rows['buying_frequency'];
                        }

                    }
                }
            }

            if (!empty($data)) {
                $entities = $this->StoreComponents->newEntities($data);
                #pr($entities);exit;
                $saveEntities = $this->StoreComponents->saveMany($entities);

                if ($saveEntities) {

                    #clear cache
                    Cache::clear(true);
                    $this->Flash->success(__('The {0} has been saved.', 'Store Components'));
                    return $this->redirect(['action' => 'index']);

                } else {

                    $this->Flash->error(__('The {0} could not be saved. Please, try again.', 'Store Components'));
                }
            }

        }
        #set time
        $this->set(compact('importData'));
    }
}
