<?php
namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\I18n\Time;
use Cake\Utility\Security;
use Cake\Utility\Text;
use Cake\Routing\Router;
use Cake\Mailer\Email;

/**
 * Users Controller
 *
 * @property \App\Model\Table\UsersTable $Users
 *
 * @method \App\Model\Entity\User[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class UsersController extends AppController
{
	/**
     * initialize method
     *
     * @return \Cake\Http\Response|void
     */
    public function initialize()
    {
	#initialization
        parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'profileEdit' => ['className' => 'Crud.Edit'],
                'changePassword' => ['className' => 'Crud.Edit'],
                'Crud.View',
                'profileView' => ['className' => 'Crud.View'],
                'Crud.Delete',
            ],
            'listeners' => [
                'Crud.Search',
                'Crud.Api'
            ]
        ]);

    #load model
        $this->loadModel('UserRoles');
        $this->loadModel('GroupUsers');
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
    #active status tag
        $activeStatus  = Configure::read('activeStatus');
    #find selection option
        $userRoles = $this->UserRoles->find('list', ['keyField' => 'id', 'valueField' => 'user_role'])->where(['is_active' => 1]);
        $this->set(compact('activeStatus', 'userRoles'));

    #before find
		$this->Crud->on('beforePaginate', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'UserRoles'
			])->order([
                'user_full_name' => 'ASC'
            ]);
		});

    #return data
        return $this->Crud->execute();
    }

    /**
     * View method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
            $event->subject()->query->contain([
                'UserRoles',
            ]);
        });

	#return
        return $this->Crud->execute();
    }

    /**
     * Profile View method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function profileView($id = null)
    {
    #before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
            $event->subject()->query->contain([
                'UserRoles',
            ]);
        });

	#return
        return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
    #get user roles
		$userRoles = $this->UserRoles->find('list', ['keyField' => 'id', 'valueField' => 'user_role'])->where(['is_active' => 1]);
    #set result for view
        $this->set(compact('userRoles'));

	#return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
	#get user roles
		$userRoles = $this->UserRoles->find('list', ['keyField' => 'id', 'valueField' => 'user_role'])->where(['is_active' => 1]);
    #set result for view
        $this->set(compact('userRoles'));

    #after save
        $this->Crud->on('beforeSave', function(\Cake\Event\Event $event) {
        #check condition
            if($event->subject->entity->is_active==0){
                $groupUsers = $this->GroupUsers->find()->where([ 'user_id' => $event->subject->entity->id, 'is_active' => 1 ])->toArray();
            #check empty
                if(!empty($groupUsers)){
                #flash message
                    $this->Flash->error('Inactive failed! user mapped to group!');
                #return
                    return $this->redirect(
                        ['action' => 'edit', $event->subject->entity->id]
                    );
                }
            }
        });

	#return
		return $this->Crud->execute();
    }

    /**
     * update password method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function changePassword($id = null)
    {
    #find user
        $user = $this->Users->get($id);
    #set result for view
        $this->set(compact('user'));

    #Validate User Details and Update Password
        if($this->request->is(['post', 'put'])) {
            $user = $this->Auth->identify();
        #check user
            if(!empty($user)){
            #Update New Password with new expiry Date
                $userEntity = $this->Users->get($user['id']);
                $this->set('user', $userEntity);
                $userEntity = $this->Users
                                ->patchEntity($userEntity, 
                                    [
                                        'password'          => $this->request->data['new_password'],
                                        'new_password'      => $this->request->data['new_password'],
                                        'confirm_password'  => $this->request->data['confirm_password'],
                                        'is_password_reset' => 0,
                                    ], 
                                    [
                                        'validate' => 'resetPassword'
                                    ]
                                );
            #check user entity
                if($this->Users->save($userEntity)) {
                # Update Last Login
                    $userEntity = $this->Users->get($user['id']);
                    $this->Users->save($userEntity);
                #flash message
                    return $this->Flash->success(__('Password changed successfully'));
                }else {
                    return $this->Flash->error(__('Error changing password. Please try again!'));
                }
            }
            else{
                return $this->Flash->error(__('Your username or current password is incorrect.'));
            }
        }
    }

    /**
     * Profile edit method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function ProfileEdit($id = null)
    {
	#get user roles
		$userRoles = $this->UserRoles->find('list', ['keyField' => 'id', 'valueField' => 'user_role'])->where(['is_active' => 1]);
    #set result for view
        $this->set(compact('userRoles'));
    #check method
		$this->Crud->on('afterSave', function(\Cake\Event\Event $event){
        #get last insert id
			$id = $event->subject()->entity->id;
        #get users
            $user = $this->Users->get($id);
        #write/update session
            $this->request->session()->write('Auth.User.user_full_name', $user->user_full_name);
            $this->request->session()->write('Auth.User.user_email', $user->user_email);
            $this->request->session()->write('Auth.User.user_mobile_no', $user->user_mobile_no);
            $this->request->session()->write('Auth.User.photo', $user->photo);
            $this->request->session()->write('Auth.User.dir', $user->dir);

        #return
            return $this->redirect(
                ['action' => 'profileView', $id]
            );
        });

	#return
		return $this->Crud->execute();
    }

    /**
     * reset password method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function adminResetPassword($id = null)
    {
        $this->autoRender = false;
    #get default password
        $defaultPassword  = Configure::read('defaultPassword');
    #check user id
        if($id!=null){
        #get user roles
            $user = $this->Users->get($id);
        #check email address not empty
            if(!empty($user)){
                if(!empty($user["user_email"])){
                #variable declaration
                    $this->data                         = [];
                    $this->data['password']             = $defaultPassword;
                    $this->data['is_password_reset']    = 1;
                #patch entity
                    $entity = $this->Users->patchEntity($user, $this->data);
                #check values updated
                    if($this->Users->save($entity)){
                    #mailer config
                        $mailer = new Email();
                    #message
                        $subject = 'SMW- reset password';
                        $message ='You recently requested to reset your password for your Super Machine Works account. use the below password to login';
                        $message .='<br><br>Password : <b>'.$this->data['password'].'</b>';
                    #mail data
                        $mailData = [
                            'name' => $user['user_full_name'],
                            'data' => $message,
                        ];
                    #send email
                        $mailer
                            ->emailFormat('html')
                            ->template('reset_password')
                            ->setViewVars(['mailData' => $mailData])
                            ->subject($subject)
                            ->to($user["user_email"])
                            ->from(['no-reply@smw.in' => 'SMW Staging'])
                            ->send();
                    }
                }
            }
        }
    #return
        return $this->redirect(
            ['action' => 'index']
        );
    }

    /**
     * Delete method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
    #return
		return $this->Crud->execute();
    }

	/**
     * login method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
	public function login()
    {
    #check method
        if ($this->request->is('post')) {
            $user = $this->Auth->identify();
        #check valid user
            if ($user) {
                if($user['is_active'] == 1){
                #set user session
                    $this->Auth->setUser($user);
                #check user role
                    if($user['is_password_reset']==0){
                    #redirect home page
                        if( ($user['user_role_id']==4) || ($user['user_role_id']==5) || ($user['user_role_id']==6) ){
                            return $this->redirect(['controller' => 'AnnualComponentTrackings', 'action' => 'index']);
                        }
                        else{
                            return $this->redirect(['controller' => 'Users', 'action' => 'index']);
                        }
                    }
                    else{
                    #redirect change password
                        return $this->redirect(['action' => 'changePassword', $user['id']]);
                    }
                }
                else{
                    $this->Flash->error(__('Your account has been temporarily blocked. Please contact administrator.'));
                }
            }
            else{
                $this->Flash->error(__('Invalid user name or password, try again'));
            }
        }
    }

	/**
     * logout method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function logout()
    {
    #session data
        $session = $this->getRequest()->getSession();
        $session->delete('collapseData');
    #logout redirect
        return $this->redirect($this->Auth->logout());
    }

    /**
     * forgot password method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function forgotPassword()
    {
        $this->viewBuilder()->setLayout('forgot-password');
    #checks method
        if($this->request->is(['post', 'put'])) {
        #get user name
            $username   = $this->request->data('username');
             $user      = $this->Users->findByUsername($username);
        #checks empty
            if( !$user->isEmpty() ){
                $user               = $user->first()->toArray();
                $password           = sha1(Text::uuid());
                $password_token     = Security::hash($password, 'sha256', true);
                $hashval            = sha1($user['username'] . rand(0, 100));
                $reset_token_link   = Router::url(['controller' => 'Users', 'action' => 'resetPassword'], TRUE) . '/' . $password_token . '#' . $hashval;
            #get user information
                $user                       = $this->Users->get($user['id']);
                $user->password_reset_token = $password_token;
                $user->hashval              = $hashval;
            #update users
                if($this->Users->save($user)) {
                #mailer config
                    $mailer = new Email();
                #message
                    $subject = 'SMW- Reset your Password';
                    $message = $reset_token_link;
                #mail data
                    $mailData = [
                        'name' => $user['user_full_name'],
                        'data' => $message,
                    ];
                #send email
                    $mailer
                        ->emailFormat('html')
                        ->template('forgot_password', 'default')
                        ->set(['user' => $user, 'reset_token_link' => $reset_token_link])
                        ->setViewVars(['mailData' => $mailData])
                        ->subject($subject)
                        ->to($user["user_email"])
                        ->from(['no-reply@smw.in' => 'SMW Staging'])
                        ->send();
                #flash message
                    $this->Flash->success('Please click on password reset link, sent in your email address to reset password.');
                }else {
                    $this->Flash->error('Error on updating record, please contact support.');
                }
            }
            else{
                $this->Flash->error('Failed! User name does not exist!');
            }
        }
    }

    /**
     * reset password method
     *
     * @param string|null $id User id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function resetPassword($token = null)
    {
        $this->viewBuilder()->setLayout('forgot-password');
    #checks not empty
        if(!empty($token)) {
            $user = $this->Users->findByPasswordResetToken($token)->first();
        #cheks users
            if($user) {
                $this->set('user', $user);
            #check data
                if(!empty($this->request->data)) {
                    $user = $this->Users->patchEntity($user, 
                        [
                            'password'              => $this->request->data['new_password'],
                            'new_password'          => $this->request->data['new_password'],
                            'confirm_password'      => $this->request->data['confirm_password'], 
                        ], 
                        [
                            'validate' => 'resetPassword'
                        ]
                    );
                #checks password
                    $hashval_new = sha1($user['username'] . rand(0, 100));
                    $user->password_reset_token = $hashval_new;
                #checks user data updated
                    if($this->Users->save($user)) {
                        $this->Flash->success(__('Your password has been changed successfully'));
                        return $this->redirect(['action' => 'login']);
                    }else {
                        $this->Flash->error(__('Error changing password. Please try again!'));
                    }
                }
            }
            else{
                $this->Flash->error(__('Sorry your password token has been expired.'));
            }
        }
        else{
            $this->Flash->error(__('Error loading password reset.'));
        }
    }
}
