<?php
namespace App\Controller;

use App\Controller\AppController;

/**
 * VendorComponents Controller
 *
 * @property \App\Model\Table\VendorComponentsTable $VendorComponents
 *
 * @method \App\Model\Entity\VendorComponent[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class VendorComponentsController extends AppController
{

    /**
     * initialize method
     */
    public function initialize()
    {
	#initialization
		parent::initialize();

	#CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
        ]);

    #Datatable configuration
    	$this->DataTables->createConfig('VendorComponents')
            ->queryOptions(['contain' => [
                'Components' => ['fields' => ['component_name' => 'Components.component_name']],
                'Vendors' => ['fields' => ['vendor_name' => 'Vendors.vendor_name']],
            ]])
			->column('Sno.'                         , ['label' => 'Sno.', 'class' => 'col-lg-1 text-center', 'database' => false])
            ->column('Vendors.vendor_name'          , ['label' => 'Vendor Name', 'class' => 'col-lg-2'])
            ->column('Components.component_name'    , ['label' => 'Component Name', 'class' => 'col-lg-5'])
            ->column('VendorComponents.price'       , ['label' => 'Price', 'class' => 'col-lg-1'])
            ->column('VendorComponents.gst'         , ['label' => 'GST', 'class' => 'col-lg-1'])
            ->column('VendorComponents.is_active'   , ['label' => 'Status', 'class' => 'col-lg-1 text-center', 'searchable' => false])
            ->column('VendorComponents.id'          , ['label' => 'Actions', 'class' => 'col-lg-1 text-center', 'searchable' => false]);

    #load model
        $this->loadModel('Vendors');
        $this->loadModel('Components');
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
        $this->DataTables->setViewVars('VendorComponents');
    }

    /**
     * View method
     *
     * @param string|null $id Vendor Component id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Vendors', 'Components'
			]);
		});

	#return
		return $this->Crud->execute();
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $vendors    = $this->Vendors->find('list', ['keyField' => 'id', 'valueField' => 'vendor_name'])->where(['is_active' => 1]);
        $components = $this->Components->find('list', ['keyField' => 'id', 'valueField' => 'component_name'])->where(['is_active' => 1]);
	#set result values
		$this->set(compact('vendors', 'components'));
    #return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Vendor Component id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $vendors    = $this->Vendors->find('list', ['keyField' => 'id', 'valueField' => 'vendor_name'])->where(['is_active' => 1]);
        $components = $this->Components->find('list', ['keyField' => 'id', 'valueField' => 'component_name'])->where(['is_active' => 1]);
	#set result values
		$this->set(compact('vendors', 'components'));
    #return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Vendor Component id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
       $this->Crud->on('afterDelete', function(\Cake\Event\Event $event) {
        if (!$event->getSubject()->success) {
            $this->log("Delete failed for entity " . $event->getSubject()->id);
            }
            return  $this->redirect($this->referer());
        });
    #return
		return $this->Crud->execute();
    }
}
