<?php
namespace App\Controller;

use App\Controller\AppController;

/**
 * VendorStoreComponents Controller
 *
 * @property \App\Model\Table\VendorStoreComponentsTable $VendorStoreComponents
 *
 * @method \App\Model\Entity\VendorStoreComponent[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class VendorStoreComponentsController extends AppController
{

    /**
     * initialize method
     */
    public function initialize()
    {
	//initialization
		parent::initialize();

	//CRUD operations
		$this->loadComponent('Crud.Crud', [
            'actions' => [
                'Crud.Index',
                'add' => ['className' => 'Crud.Add'],
                'Crud.Edit',
                'Crud.View',
                'Crud.Delete',
            ],
        ]);
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|void
     */
    public function index()
    {
        $this->paginate = [
            'contain' => ['Vendors', 'StoreComponents']
        ];
        $vendorStoreComponents = $this->paginate($this->VendorStoreComponents);

        $this->set(compact('vendorStoreComponents'));
    }

    /**
     * View method
     *
     * @param string|null $id Vendor Store Component id.
     * @return \Cake\Http\Response|void
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
	#before find
		$this->Crud->on('beforeFind', function(\Cake\Event\Event $event) {
			$event->subject()->query->contain([
				'Vendors', 'StoreComponents'
			]);
		});

	#return
		return $this->Crud->execute();
    }
    /**
     * Add method
     *
     * @return \Cake\Http\Response|null Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $vendors = $this->VendorStoreComponents->Vendors->find('list', ['limit' => 200]);
        $storeComponents = $this->VendorStoreComponents->StoreComponents->find('list', ['limit' => 200]);
	//set result values
		$this->set(compact('vendors', 'storeComponents'));
    //return
		return $this->Crud->execute();
    }

    /**
     * Edit method
     *
     * @param string|null $id Vendor Store Component id.
     * @return \Cake\Http\Response|null Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $vendors = $this->VendorStoreComponents->Vendors->find('list', ['limit' => 200]);
        $storeComponents = $this->VendorStoreComponents->StoreComponents->find('list', ['limit' => 200]);
	//set result values
		$this->set(compact('vendors', 'storeComponents'));
    //return
		return $this->Crud->execute();
    }

    /**
     * Delete method
     *
     * @param string|null $id Vendor Store Component id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
    //return
		return $this->Crud->execute();
    }
}
