<?php
namespace App\Model\Entity;

use Cake\ORM\Entity;
use Cake\I18n\Time;
/**
 * PurchaseOrder Entity
 *
 * @property int $id
 * @property int $purchase_indent_id
 * @property int $vendor_id
 * @property string|null $purchase_order_no
 * @property \Cake\I18n\FrozenDate|null $expected_delivery_date
 * @property int $tag
 * @property int $is_adhoc
 * @property int $status
 * @property int $is_active
 * @property int|null $created_by
 * @property \Cake\I18n\FrozenTime|null $created
 * @property int|null $modified_by
 * @property \Cake\I18n\FrozenTime|null $modified
 *
 * @property \App\Model\Entity\PurchaseIndent $purchase_indent
 * @property \App\Model\Entity\Vendor $vendor
 * @property \App\Model\Entity\PurchaseOrderComponent[] $purchase_order_components
 */
class PurchaseOrder extends Entity
{

    /**
     * Fields that can be mass assigned using newEntity() or patchEntity().
     *
     * Note that when '*' is set to true, this allows all unspecified fields to
     * be mass assigned. For security purposes, it is advised to set '*' to false
     * (or remove it), and explicitly make individual fields accessible as needed.
     *
     * @var array
     */
    protected $_accessible = [
        'purchase_indent_id' => true,
        'vendor_id' => true,
        'purchase_order_no' => true,
        'expected_delivery_date' => true,
        'tag' => true,
        'is_adhoc' => true,
        'status' => true,
        'is_active' => true,
        'created_by' => true,
        'created' => true,
        'modified_by' => true,
        'modified' => true,
        'purchase_indent' => true,
        'vendor' => true,
        'purchase_order_components' => true,
        'process' => true,
        'component' => true,
        'payment_terms'=>true,
        'terms_of_delivery'=>true,
        'quotation'=>true,
        'quotation_date'=>true,
        'vendors'=>true,
        'job_order_id'=>true,
        'is_store_order'=>true,

    ];


    protected function _setExpectedDeliveryDate($expected_delivery_date)
    {
        $date= Time::parse($expected_delivery_date);
        $date->i18nFormat('yyyy-MM-dd');
        return $date;
    }

    protected function _setQuotationDate($quotation_date)
    {
        $date= Time::parse($quotation_date);
        $date->i18nFormat('yyyy-MM-dd');
        return $date;
    }

    protected function _setCreated($created)
    {
        $date=date("Y-m-d", strtotime($created));
        return $date;
    }
}