<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use Cake\ORM\TableRegistry;
use Cake\ORM\Behavior\CounterCacheBehavior;
use Cake\I18n\Time;

/**
 * AnnualAssemblyTrackings Model
 *
 * @property \App\Model\Table\AnnualQuarterPlansTable|\Cake\ORM\Association\BelongsTo $AnnualQuarterPlans
 * @property |\Cake\ORM\Association\BelongsTo $AnnualProductTrackings
 * @property \App\Model\Table\SubAssembliesTable|\Cake\ORM\Association\BelongsTo $SubAssemblies
 * @property \App\Model\Table\ProductsTable|\Cake\ORM\Association\BelongsTo $Products
 * @property \App\Model\Table\AnnualComponentTrackingsTable|\Cake\ORM\Association\HasMany $AnnualComponentTrackings
 *
 * @method \App\Model\Entity\AnnualAssemblyTracking get($primaryKey, $options = [])
 * @method \App\Model\Entity\AnnualAssemblyTracking newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\AnnualAssemblyTracking[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\AnnualAssemblyTracking|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualAssemblyTracking|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualAssemblyTracking patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualAssemblyTracking[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualAssemblyTracking findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class AnnualAssemblyTrackingsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('annual_assembly_trackings');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('AnnualQuarterPlans', [
            'foreignKey' => 'annual_quarter_plan_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('AnnualProductTrackings', [
            'foreignKey' => 'annual_product_tracking_id'
        ]);
        $this->belongsTo('SubAssemblies', [
            'foreignKey' => 'sub_assembly_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Products', [
            'foreignKey' => 'product_id',
            'joinType' => 'INNER'
        ]);
        $this->hasMany('AnnualComponentTrackings', [
            'foreignKey' => 'annual_assembly_tracking_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->integer('product_number')
            ->requirePresence('product_number', 'create')
            ->allowEmptyString('product_number', false);

        $validator
            ->integer('total_component')
            ->requirePresence('total_component', 'create')
            ->allowEmptyString('total_component', false);

        $validator
            ->integer('completed_component')
            ->requirePresence('completed_component', 'create')
            ->allowEmptyString('completed_component', false);

        $validator
            ->requirePresence('status', 'create')
            ->allowEmptyString('status', false);

        $validator
            ->dateTime('status_updated_on')
            ->allowEmptyDateTime('status_updated_on');

        $validator
            ->scalar('description')
            ->allowEmptyString('description');

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['annual_quarter_plan_id'], 'AnnualQuarterPlans'));
        $rules->add($rules->existsIn(['annual_product_tracking_id'], 'AnnualProductTrackings'));
        $rules->add($rules->existsIn(['sub_assembly_id'], 'SubAssemblies'));
        $rules->add($rules->existsIn(['product_id'], 'Products'));

        return $rules;
    }

    /*
     *after save method
     *after component status updated change status in assembly tracking and product tracking status and count
    */
    public function afterSave($event, $entity, $options)
    {
    #check is new
        if(!$entity->isNew()){
        #check dirty field
            if($entity->dirty('status') || $entity->dirty('completed_assembly')){
            #get annual tracking by id
                $annualProductTrackings    = TableRegistry::get('AnnualProductTrackings');
                $annualProductTracking     = $annualProductTrackings->get($entity->annual_product_tracking_id);
            #get total count of completed components
                $statusCount               = TableRegistry::get('AnnualAssemblyTrackings')->find('all')->where(['status' => 1, 'annual_product_tracking_id' => $entity->annual_product_tracking_id]);
            #total component
                $completed_assembly         = $annualProductTracking['completed_assembly'];
            # find rejected component assembly
                $rejectedStatusCount        = TableRegistry::get('AnnualAssemblyTrackings')->find('all')->where(['status IN' => [3], 'annual_product_tracking_id' => $entity->annual_product_tracking_id]);
            #waiting status count
                $waitingStatusCount         = TableRegistry::get('AnnualAssemblyTrackings')->find('all')->where(['status IN' => [2], 'annual_product_tracking_id' => $entity->annual_product_tracking_id]);
             #fetch result
                $this->result               = [];
                $this->result["completed_assembly"]  = $statusCount->count();
            #check rejected status
                if(!$rejectedStatusCount->isEmpty()){
                    $this->result["status"] = 3;
                }
            #checks waiting status
                elseif(!$waitingStatusCount->isEmpty()){
                    $this->result["status"] = 2;
                }
            #check all component in assembly completed
                elseif($annualProductTracking['total_assembly']==$this->result["completed_assembly"]){
                    $this->result["status"] = 1;
                }
            #check all component in assembly not completed
                elseif($annualProductTracking['total_assembly']!=$this->result["completed_assembly"]){
                    $this->result["status"] = 0;
                }
            #save data
                $entity = $annualProductTrackings->patchEntity($annualProductTracking, $this->result);
                $annualProductTrackings->save($entity);
            }
        }
    }
}
