<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use Cake\ORM\TableRegistry;
use Cake\ORM\Behavior\CounterCacheBehavior;
use Cake\I18n\Time;

/**
 * AnnualComponentTrackings Model
 *
 * @property \App\Model\Table\AnnualQuarterPlansTable|\Cake\ORM\Association\BelongsTo $AnnualQuarterPlans
 * @property \App\Model\Table\ComponentProductsTable|\Cake\ORM\Association\BelongsTo $ComponentProducts
 * @property \App\Model\Table\AnnualAssemblyTrackingsTable|\Cake\ORM\Association\BelongsTo $AnnualAssemblyTrackings
 * @property |\Cake\ORM\Association\HasMany $AnnualComponentStatusTrackings
 * @property \App\Model\Table\ComponentTrackingCommentsTable|\Cake\ORM\Association\HasMany $ComponentTrackingComments
 *
 * @method \App\Model\Entity\AnnualComponentTracking get($primaryKey, $options = [])
 * @method \App\Model\Entity\AnnualComponentTracking newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\AnnualComponentTracking[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\AnnualComponentTracking|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualComponentTracking|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualComponentTracking patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualComponentTracking[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualComponentTracking findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class AnnualComponentTrackingsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('annual_component_trackings');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('AnnualQuarterPlans', [
            'foreignKey' => 'annual_quarter_plan_id'
        ]);
        $this->belongsTo('ComponentProducts', [
            'foreignKey' => 'component_product_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('AnnualAssemblyTrackings', [
            'foreignKey' => 'annual_assembly_tracking_id',
            'joinType' => 'INNER'
        ]);
        $this->hasMany('AnnualComponentStatusTrackings', [
            'foreignKey' => 'annual_component_tracking_id'
        ]);
        $this->hasMany('ComponentTrackingComments', [
            'foreignKey' => 'annual_component_tracking_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->integer('product_number')
            ->allowEmptyString('product_number');

        $validator
            ->integer('total_components')
            ->requirePresence('total_components', 'create')
            ->allowEmptyString('total_components', false);

        $validator
            ->integer('total_completed_components')
            ->requirePresence('total_completed_components', 'create')
            ->allowEmptyString('total_completed_components', false);

        $validator
            ->integer('total_rejected_components')
            ->requirePresence('total_rejected_components', 'create')
            ->allowEmptyString('total_rejected_components', false);

        $validator
            ->requirePresence('status', 'create')
            ->allowEmptyString('status', false);

        $validator
            ->dateTime('status_updated_on')
            ->allowEmptyDateTime('status_updated_on');

        $validator
            ->scalar('description')
            ->allowEmptyString('description');

        $validator
            ->requirePresence('is_eventual_product', 'create')
            ->allowEmptyString('is_eventual_product', false);

        $validator
            ->requirePresence('is_eventual_mapped', 'create')
            ->allowEmptyString('is_eventual_mapped', false);

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['annual_quarter_plan_id'], 'AnnualQuarterPlans'));
        $rules->add($rules->existsIn(['component_product_id'], 'ComponentProducts'));
        $rules->add($rules->existsIn(['annual_assembly_tracking_id'], 'AnnualAssemblyTrackings'));

        return $rules;
    }

    /*
        *after save method
        *after component status updated change status in assembly tracking and product tracking status and count
    */
    public function afterSave($event, $entity, $options)
    {
    #check is new
        if(!$entity->isNew()){
        #check dirty field
            if($entity->dirty('status') || $entity->dirty('completed_component')){
            #get annual assembly by id
                $annualAssemblyTrackings    = TableRegistry::get('AnnualAssemblyTrackings');
                $annualAssemblyTracking     = $annualAssemblyTrackings->get($entity->annual_assembly_tracking_id);
            #get total count of completed components
                $statusCount                = TableRegistry::get('AnnualComponentTrackings')
                                                ->find('all')
                                                ->join([
                                                    'ComponentProducts' => [
                                                        'table'     => 'component_products',
                                                        'type'		=> 'INNER',
                                                        'conditions'=> [ 'ComponentProducts.id=AnnualComponentTrackings.component_product_id' ]
                                                    ],
                                                    'Components' => [
                                                        'table'     => 'components',
                                                        'type'		=> 'INNER',
                                                        'conditions'=> [ 'Components.id=ComponentProducts.component_id', 'Components.is_active' => 1 ]
                                                    ],
                                                ])
                                                ->where([
                                                        'AnnualComponentTrackings.status IN' => [13,14], 
                                                        'AnnualComponentTrackings.annual_assembly_tracking_id' => $entity->annual_assembly_tracking_id
                                                ]);
            # find rejected component assembly
                $rejectedStatusCount        = TableRegistry::get('AnnualComponentTrackings')->find('all')->where(['status IN' => [5,6,11,12], 'annual_assembly_tracking_id' => $entity->annual_assembly_tracking_id]);
            #waiting status count
                $waitingStatusCount         = TableRegistry::get('AnnualComponentTrackings')->find('all')->where(['status IN' => [3,4,9,10], 'annual_assembly_tracking_id' => $entity->annual_assembly_tracking_id]);
            #fetch result
                $this->result                           = [];
                $this->result["completed_component"]    = $statusCount->count();
            #check rejected status
                if(!$rejectedStatusCount->isEmpty()){
                    $this->result["status"] = 3;
                }
            #checks waiting status
                elseif(!$waitingStatusCount->isEmpty()){
                    $this->result["status"] = 2;
                }
            #check all component in assembly completed
                elseif($annualAssemblyTracking['total_component']<=$this->result["completed_component"]){
                    $this->result["status"] = 1;
                }
            #check all component in assembly not completed
                elseif($annualAssemblyTracking['total_component']!=$this->result["completed_component"]){
                    $this->result["status"] = 0;
                }
            #save data
                $assemblyEntity = $annualAssemblyTrackings->patchEntity($annualAssemblyTracking, $this->result);
                $annualAssemblyTrackings->save($assemblyEntity);

            #is undo
                if($entity->is_undo==1){
                #update all
                    $annualComponentStatusTrackings = TableRegistry::get('AnnualComponentStatusTrackings')->updateAll(
                        ['is_undo' => 1],
                        [
                            'annual_component_tracking_id'  => $entity->id,
                            'status'                        => $entity->status,
                        ]
                    );
                }

            #update all
                $annualComponentStatusTrackings = TableRegistry::get('AnnualComponentStatusTrackings')->updateAll(
                    ['is_last_updated' => 0],
                    ['annual_component_tracking_id' => $entity->id]
                );

            #copy data
                $this->result = [];
                $this->result['annual_component_tracking_id']   = $entity->id;
                $this->result['total_components']               = $entity->total_components;
                $this->result['total_completed_components']     = $entity->total_completed_components;
                $this->result['total_rejected_components']      = $entity->total_rejected_components;
                $this->result['status']                         = $entity->status;
                $this->result['description']                    = $entity->description;
                $this->result['is_undo']                        = $entity->is_undo;
                $this->result['is_last_updated']                = 1;
                $this->result['created_by']                     = $entity->created_by;
                $this->result['modified_by']                    = $entity->modified_by;
            #insert data
                $entities 	= TableRegistry::get('AnnualComponentStatusTrackings')->newEntity($this->result);
                $result		= TableRegistry::get('AnnualComponentStatusTrackings')->save($entities);
            }
        }
    }
}
