<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\ORM\TableRegistry;
use Cake\Validation\Validator;

/**
 * AnnualPlannedCapacities Model
 *
 * @property \App\Model\Table\AnnualPlansTable|\Cake\ORM\Association\BelongsTo $AnnualPlans
 * @property \App\Model\Table\ProductsTable|\Cake\ORM\Association\BelongsTo $Products
 * @property \App\Model\Table\ComponentsTable|\Cake\ORM\Association\BelongsTo $Components
 * @property \App\Model\Table\MachinesTable|\Cake\ORM\Association\BelongsTo $Machines
 * @property \App\Model\Table\ProcessesTable|\Cake\ORM\Association\BelongsTo $Processes
 *
 * @method \App\Model\Entity\AnnualPlannedCapacity get($primaryKey, $options = [])
 * @method \App\Model\Entity\AnnualPlannedCapacity newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\AnnualPlannedCapacity[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\AnnualPlannedCapacity|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualPlannedCapacity|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualPlannedCapacity patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualPlannedCapacity[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualPlannedCapacity findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class AnnualPlannedCapacitiesTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('annual_planned_capacities');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('AnnualPlans', [
            'foreignKey' => 'annual_plan_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Products', [
            'foreignKey' => 'product_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Components', [
            'foreignKey' => 'component_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Machines', [
            'foreignKey' => 'machine_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Processes', [
            'foreignKey' => 'process_id',
            'joinType' => 'INNER'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->integer('total_components')
            ->allowEmptyString('total_components');

        $validator
            ->numeric('total_process_time')
            ->allowEmptyString('total_process_time');

        $validator
            ->numeric('total_batches')
            ->allowEmptyString('total_batches');

        $validator
            ->numeric('total_batch_set_time')
            ->requirePresence('total_batch_set_time', 'create')
            ->allowEmptyString('total_batch_set_time', false);

        $validator
            ->numeric('total_time')
            ->allowEmptyString('total_time');

        $validator
            ->integer('in_components')
            ->requirePresence('in_components', 'create')
            ->allowEmptyString('in_components', false);

        $validator
            ->integer('out_components')
            ->requirePresence('out_components', 'create')
            ->allowEmptyString('out_components', false);

        $validator
            ->allowEmptyString('tag');

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['annual_plan_id'], 'AnnualPlans'));
        $rules->add($rules->existsIn(['product_id'], 'Products'));
        $rules->add($rules->existsIn(['component_id'], 'Components'));
        $rules->add($rules->existsIn(['machine_id'], 'Machines'));
        $rules->add($rules->existsIn(['process_id'], 'Processes'));

        return $rules;
    }

    /**
     * inout component quantity update
     */
    public function afterSave($event, $entity, $options)
    {
        if (!$entity->isNew()) {
            $tag = 0;
            if ($entity->tag == 2) {
                $this->updateAll(
                    ['in_components' => $entity->in_components, 'out_components' => $entity->out_components],
                    ['annual_plan_id' => $entity->annual_plan_id, 'product_id' => $entity->product_id, 'component_id' => $entity->component_id]
                );
            }
        }
    }
}
