<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * AnnualPlannedMachines Model
 *
 * @property \App\Model\Table\AnnualPlansTable|\Cake\ORM\Association\BelongsTo $AnnualPlans
 * @property \App\Model\Table\MachinesTable|\Cake\ORM\Association\BelongsTo $Machines
 *
 * @method \App\Model\Entity\AnnualPlannedMachine get($primaryKey, $options = [])
 * @method \App\Model\Entity\AnnualPlannedMachine newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\AnnualPlannedMachine[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\AnnualPlannedMachine|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualPlannedMachine|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualPlannedMachine patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualPlannedMachine[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualPlannedMachine findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class AnnualPlannedMachinesTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('annual_planned_machines');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('AnnualPlans', [
            'foreignKey' => 'annual_plan_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Machines', [
            'foreignKey' => 'machine_id',
            'joinType' => 'INNER'
        ]);

	//foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	//audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->integer('hours_per_shift')
            ->requirePresence('hours_per_shift', 'create')
            ->allowEmptyString('hours_per_shift', false);

        $validator
            ->integer('shifts_per_day')
            ->requirePresence('shifts_per_day', 'create')
            ->allowEmptyString('shifts_per_day', false);

        $validator
            ->integer('down_time_per_month')
            ->requirePresence('down_time_per_month', 'create')
            ->allowEmptyString('down_time_per_month', false);

        $validator
            ->integer('annual_working_hour')
            ->requirePresence('annual_working_hour', 'create')
            ->allowEmptyString('annual_working_hour', false);

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['annual_plan_id'], 'AnnualPlans'));
        $rules->add($rules->existsIn(['machine_id'], 'Machines'));

        return $rules;
    }
}
