<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use Cake\ORM\TableRegistry;
use Cake\ORM\Behavior\CounterCacheBehavior;
use Cake\I18n\Time;

/**
 * AnnualPlannedProducts Model
 *
 * @property \App\Model\Table\AnnualPlansTable|\Cake\ORM\Association\BelongsTo $AnnualPlans
 * @property \App\Model\Table\ProductsTable|\Cake\ORM\Association\BelongsTo $Products
 *
 * @method \App\Model\Entity\AnnualPlannedProduct get($primaryKey, $options = [])
 * @method \App\Model\Entity\AnnualPlannedProduct newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\AnnualPlannedProduct[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\AnnualPlannedProduct|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualPlannedProduct|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualPlannedProduct patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualPlannedProduct[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualPlannedProduct findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class AnnualPlannedProductsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('annual_planned_products');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('AnnualPlans', [
            'foreignKey' => 'annual_plan_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Products', [
            'foreignKey' => 'product_id',
            'joinType' => 'INNER'
        ]);

	//foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	//audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->integer('product_quantity')
            ->requirePresence('product_quantity', 'create')
            ->allowEmptyString('product_quantity', false);

        $validator
            ->integer('product_buffer_quantity')
            ->requirePresence('product_buffer_quantity', 'create')
            ->allowEmptyString('product_buffer_quantity', false);

        $validator
            ->integer('product_requested_quantity')
            ->requirePresence('product_requested_quantity', 'create')
            ->allowEmptyString('product_requested_quantity', false);

        $validator
            ->integer('product_total_quantity')
            ->requirePresence('product_total_quantity', 'create')
            ->allowEmptyString('product_total_quantity', false);

        $validator
            ->integer('last_product_no')
            ->requirePresence('last_product_no', 'create')
            ->allowEmptyString('last_product_no', false);

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['annual_plan_id'], 'AnnualPlans'));
        $rules->add($rules->existsIn(['product_id'], 'Products'));

        return $rules;
    }

    /*
     *before save method
     *before annual product count update
    */
    public function beforeSave($event, $entity, $options)
    {
    #check is new
        if(!$entity->isNew()){
        #check dirty field
            if($entity->dirty('product_quantity')){
            #get
                $annualPlannedProducts  = TableRegistry::get('AnnualPlannedProducts');
                $annualPlannedProduct   = $annualPlannedProducts->get($entity->id);
            #prepare data
                $this->data = [];
                $this->data['annual_plan_id']               = $annualPlannedProduct['annual_plan_id'];
                $this->data['product_id']                   = $annualPlannedProduct['product_id'];
                $this->data['product_quantity']             = $annualPlannedProduct['product_quantity'];
                $this->data['product_buffer_quantity']      = $annualPlannedProduct['product_buffer_quantity'];
                $this->data['product_requested_quantity']   = $annualPlannedProduct['product_requested_quantity'];
                $this->data['product_total_quantity']       = $annualPlannedProduct['product_total_quantity'];
                $this->data['last_product_no']              = $annualPlannedProduct['last_product_no'];
                $this->data['is_active']                    = $annualPlannedProduct['is_active'];
                $this->data['created_by']                   = $annualPlannedProduct['created_by'];
                $this->data['created']                      = $annualPlannedProduct['created'];
                $this->data['modified_by']                  = $annualPlannedProduct['modified_by'];
                $this->data['modified']                     = $annualPlannedProduct['modified'];
            #insert copy of the data
                $entities 	= TableRegistry::get('AnnualPlannedProductTrackings')->newEntity($this->data);
                $result		= TableRegistry::get('AnnualPlannedProductTrackings')->save($entities);
            }
        }
    }
}
