<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;

/**
 * AnnualProductTrackings Model
 *
 * @property \App\Model\Table\AnnualQuarterPlansTable|\Cake\ORM\Association\BelongsTo $AnnualQuarterPlans
 * @property \App\Model\Table\ProductsTable|\Cake\ORM\Association\BelongsTo $Products
 * @property \App\Model\Table\AnnualAssemblyTrackingsTable|\Cake\ORM\Association\HasMany $AnnualAssemblyTrackings
 *
 * @method \App\Model\Entity\AnnualProductTracking get($primaryKey, $options = [])
 * @method \App\Model\Entity\AnnualProductTracking newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\AnnualProductTracking[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\AnnualProductTracking|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualProductTracking|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualProductTracking patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualProductTracking[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualProductTracking findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class AnnualProductTrackingsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('annual_product_trackings');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('AnnualQuarterPlans', [
            'foreignKey' => 'annual_quarter_plan_id'
        ]);
        $this->belongsTo('Products', [
            'foreignKey' => 'product_id',
            'joinType' => 'INNER'
        ]);
        $this->hasMany('AnnualAssemblyTrackings', [
            'foreignKey' => 'annual_product_tracking_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    
    #Add search behavior
        $this->addBehavior('Search.Search');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->integer('product_number')
            ->allowEmptyString('product_number');

        $validator
            ->requirePresence('status', 'create')
            ->allowEmptyString('status', false);

        $validator
            ->dateTime('status_updated_on')
            ->allowEmptyDateTime('status_updated_on');

        $validator
            ->scalar('description')
            ->allowEmptyString('description');

        $validator
            ->integer('total_assembly')
            ->requirePresence('total_assembly', 'create')
            ->allowEmptyString('total_assembly', false);

        $validator
            ->integer('completed_assembly')
            ->requirePresence('completed_assembly', 'create')
            ->allowEmptyString('completed_assembly', false);

        $validator
            ->requirePresence('is_eventual_product', 'create')
            ->allowEmptyString('is_eventual_product', false);

        $validator
            ->requirePresence('is_eventual_mapped', 'create')
            ->allowEmptyString('is_eventual_mapped', false);

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['annual_quarter_plan_id'], 'AnnualQuarterPlans'));
        $rules->add($rules->existsIn(['product_id'], 'Products'));

        return $rules;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('product_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){
            #checks empty
                if(!empty($args['product_id'])){
                    return $query->where(['product_id' => $args['product_id']]);
                }
                return $query;
            }
        ])
        ->add('product_number', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){
            #checks empty
                if(!empty($args['product_number'])){
                    return $query->where(['product_number' => $args['product_number']]);
                }
                return $query;
            }
        ])
        ->add('status', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){
            #checks empty/null value
                if( $args['status']!="" && $args['status']!=null ){
                    return $query->where(['annual_product_trackings.status' => $args['status']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
