<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * AnnualQuarterPlannedProductTrackings Model
 *
 * @property \App\Model\Table\AnnualQuarterPlansTable|\Cake\ORM\Association\BelongsTo $AnnualQuarterPlans
 * @property \App\Model\Table\ProductsTable|\Cake\ORM\Association\BelongsTo $Products
 *
 * @method \App\Model\Entity\AnnualQuarterPlannedProductTracking get($primaryKey, $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProductTracking newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProductTracking[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProductTracking|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProductTracking|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProductTracking patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProductTracking[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProductTracking findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class AnnualQuarterPlannedProductTrackingsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('annual_quarter_planned_product_trackings');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('AnnualQuarterPlans', [
            'foreignKey' => 'annual_quarter_plan_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Products', [
            'foreignKey' => 'product_id',
            'joinType' => 'INNER'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->integer('product_quantity')
            ->requirePresence('product_quantity', 'create')
            ->allowEmptyString('product_quantity', false);

        $validator
            ->integer('product_buffer_quantity')
            ->requirePresence('product_buffer_quantity', 'create')
            ->allowEmptyString('product_buffer_quantity', false);

        $validator
            ->integer('product_requested_quantity')
            ->requirePresence('product_requested_quantity', 'create')
            ->allowEmptyString('product_requested_quantity', false);

        $validator
            ->integer('product_total_quantity')
            ->requirePresence('product_total_quantity', 'create')
            ->allowEmptyString('product_total_quantity', false);

        $validator
            ->integer('last_product_no')
            ->requirePresence('last_product_no', 'create')
            ->allowEmptyString('last_product_no', false);

        $validator
            ->requirePresence('is_active', 'create')
            ->allowEmptyString('is_active', false);

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['annual_quarter_plan_id'], 'AnnualQuarterPlans'));
        $rules->add($rules->existsIn(['product_id'], 'Products'));

        return $rules;
    }
}