<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use Cake\ORM\TableRegistry;
use Cake\ORM\Behavior\CounterCacheBehavior;
use Cake\I18n\Time;

/**
 * AnnualQuarterPlannedProducts Model
 *
 * @property \App\Model\Table\AnnualQuarterPlansTable|\Cake\ORM\Association\BelongsTo $AnnualQuarterPlans
 * @property \App\Model\Table\ProductsTable|\Cake\ORM\Association\BelongsTo $Products
 *
 * @method \App\Model\Entity\AnnualQuarterPlannedProduct get($primaryKey, $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProduct newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProduct[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProduct|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProduct|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProduct patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProduct[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\AnnualQuarterPlannedProduct findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class AnnualQuarterPlannedProductsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('annual_quarter_planned_products');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('AnnualQuarterPlans', [
            'foreignKey' => 'annual_quarter_plan_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Products', [
            'foreignKey' => 'product_id',
            'joinType' => 'INNER'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->integer('product_quantity')
            ->requirePresence('product_quantity', 'create')
            ->allowEmptyString('product_quantity', false);

        $validator
            ->integer('product_buffer_quantity')
            ->requirePresence('product_buffer_quantity', 'create')
            ->allowEmptyString('product_buffer_quantity', false);

        $validator
            ->integer('product_requested_quantity')
            ->requirePresence('product_requested_quantity', 'create')
            ->allowEmptyString('product_requested_quantity', false);

        $validator
            ->integer('product_total_quantity')
            ->requirePresence('product_total_quantity', 'create')
            ->allowEmptyString('product_total_quantity', false);

        $validator
            ->integer('last_product_no')
            ->requirePresence('last_product_no', 'create')
            ->allowEmptyString('last_product_no', false);

        $validator
            ->requirePresence('is_active', 'create')
            ->allowEmptyString('is_active', false);

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['annual_quarter_plan_id'], 'AnnualQuarterPlans'));
        $rules->add($rules->existsIn(['product_id'], 'Products'));

        return $rules;
    }

    /*
     *before save method
     *before annual product count update
    */
    public function beforeSave($event, $entity, $options)
    {
    #check is new
        if(!$entity->isNew()){
        #check dirty field
            if($entity->dirty('product_quantity')){
            #get
                $annualQuarterPlannedProducts   = TableRegistry::get('AnnualQuarterPlannedProducts');
                $annualQuarterPlannedProduct    = $annualQuarterPlannedProducts->get($entity->id);
            #prepare data
                $this->data = [];
                $this->data['annual_quarter_plan_id']       = $annualQuarterPlannedProduct['annual_quarter_plan_id'];
                $this->data['product_id']                   = $annualQuarterPlannedProduct['product_id'];
                $this->data['product_quantity']             = $annualQuarterPlannedProduct['product_quantity'];
                $this->data['product_buffer_quantity']      = $annualQuarterPlannedProduct['product_buffer_quantity'];
                $this->data['product_requested_quantity']   = $annualQuarterPlannedProduct['product_requested_quantity'];
                $this->data['product_total_quantity']       = $annualQuarterPlannedProduct['product_total_quantity'];
                $this->data['last_product_no']              = $annualQuarterPlannedProduct['last_product_no'];
                $this->data['is_active']                    = $annualQuarterPlannedProduct['is_active'];
                $this->data['created_by']                   = $annualQuarterPlannedProduct['created_by'];
                $this->data['created']                      = $annualQuarterPlannedProduct['created'];
                $this->data['modified_by']                  = $annualQuarterPlannedProduct['modified_by'];
                $this->data['modified']                     = $annualQuarterPlannedProduct['modified'];
            #insert copy of the data
                $entities 	= TableRegistry::get('AnnualQuarterPlannedProductTrackings')->newEntity($this->data);
                $result		= TableRegistry::get('AnnualQuarterPlannedProductTrackings')->save($entities);
            }
        }
    }
}
