<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * ComponentProducts Model
 *
 * @property \App\Model\Table\ComponentsTable|\Cake\ORM\Association\BelongsTo $Components
 * @property \App\Model\Table\ProductsTable|\Cake\ORM\Association\BelongsTo $Products
 * @property \App\Model\Table\AnnualComponentTrackingsTable|\Cake\ORM\Association\HasMany $AnnualComponentTrackings
 *
 * @method \App\Model\Entity\ComponentProduct get($primaryKey, $options = [])
 * @method \App\Model\Entity\ComponentProduct newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\ComponentProduct[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\ComponentProduct|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\ComponentProduct|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\ComponentProduct patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\ComponentProduct[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\ComponentProduct findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class ComponentProductsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('component_products');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('Components', [
            'foreignKey' => 'component_id'
        ]);
        $this->belongsTo('Products', [
            'foreignKey' => 'product_id'
        ]);
        $this->hasMany('AnnualComponentTrackings', [
            'foreignKey' => 'component_product_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        /* $validator
            ->integer('spare_count')
            // ->requirePresence('spare_count', 'create')
            ->allowEmptyString('spare_count');

        $validator
            ->integer('spare_required')
            // ->requirePresence('spare_required', 'create')
            ->allowEmptyString('spare_required');

        $validator
            ->integer('spares_taken')
            // ->requirePresence('spares_taken', 'create')
            ->allowEmptyString('spares_taken');

        $validator
            ->integer('spare_remaning')
            // ->requirePresence('spare_remaning', 'create')
            ->allowEmptyString('spare_remaning');

        $validator
            ->integer('start_product_no')
            // ->requirePresence('start_product_no', 'create')
            ->allowEmptyString('start_product_no');

        $validator
            ->integer('end_product_no')
            // ->requirePresence('end_product_no', 'create')
            ->allowEmptyString('end_product_no');

        $validator
            ->integer('total_completed_components')
            // ->requirePresence('total_completed_components', 'create')
            ->allowEmptyString('total_completed_components');

        $validator
            ->integer('total_rejected_components')
            // ->requirePresence('total_rejected_components', 'create')
            ->allowEmptyString('total_rejected_components');

        $validator
            // ->requirePresence('status', 'create')
            ->allowEmptyString('status'); */

        $validator
            ->dateTime('status_on')
            ->allowEmptyDateTime('status_on');

        $validator
            ->scalar('description')
            ->allowEmptyString('description');

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['component_id'], 'Components'));
        $rules->add($rules->existsIn(['product_id'], 'Products'));

        return $rules;
    }
}