<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;
/**
 * ComponentRawMaterials Model
 *
 * @property \App\Model\Table\ComponentsTable|\Cake\ORM\Association\BelongsTo $Components
 * @property \App\Model\Table\RawMaterialsTable|\Cake\ORM\Association\BelongsTo $RawMaterials
 *
 * @method \App\Model\Entity\ComponentRawMaterial get($primaryKey, $options = [])
 * @method \App\Model\Entity\ComponentRawMaterial newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\ComponentRawMaterial[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\ComponentRawMaterial|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\ComponentRawMaterial|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\ComponentRawMaterial patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\ComponentRawMaterial[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\ComponentRawMaterial findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class ComponentRawMaterialsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('component_raw_materials');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('Components', [
            'foreignKey' => 'component_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('RawMaterials', [
            'foreignKey' => 'raw_material_id',
            'joinType' => 'INNER'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');

    #Add search behavior
        $this->addBehavior('Search.Search');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->numeric('component_id')
            ->allowEmptyString('component_id', false);

        $validator
            ->numeric('raw_material_id')
            ->allowEmptyString('raw_material_id', false);

        $validator
            ->scalar('raw_material_shape')
            ->maxLength('raw_material_shape', 100)
            ->requirePresence('raw_material_shape', 'create')
            ->allowEmptyString('raw_material_shape', false);

        $validator
            ->numeric('min_cut_length')
            ->requirePresence('min_cut_length', 'create')
            ->allowEmptyString('min_cut_length', false);

        $validator
            ->numeric('outer_dia')
            ->requirePresence('outer_dia', 'create')
            ->allowEmptyString('outer_dia', false);

        $validator
            ->numeric('inner_dia')
            ->requirePresence('inner_dia', 'create')
            ->allowEmptyString('inner_dia', false);

        $validator
            ->numeric('raw_material_thickness')
            ->requirePresence('raw_material_thickness', 'create')
            ->allowEmptyString('raw_material_thickness', false);

        $validator
            ->numeric('raw_material_width')
            ->requirePresence('raw_material_width', 'create')
            ->allowEmptyString('raw_material_width', false);

        $validator
            ->numeric('raw_material_length')
            ->requirePresence('raw_material_length', 'create')
            ->allowEmptyString('raw_material_length', false);

        $validator
            ->scalar('unit_of_measurement')
            ->maxLength('unit_of_measurement', 50)
            ->requirePresence('unit_of_measurement', 'create')
            ->allowEmptyString('unit_of_measurement', false);

        $validator
            ->numeric('gst')
            ->requirePresence('gst', 'create')
            ->allowEmptyString('gst', false);

        $validator
            ->numeric('buy_frequency')
            ->requirePresence('buy_frequency', 'create')
            ->allowEmptyString('buy_frequency', false);

        $validator
            ->numeric('quantity_per_component')
            ->requirePresence('quantity_per_component', 'create')
            ->allowEmptyString('quantity_per_component', false);

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['component_id'], 'Components'));
        $rules->add($rules->existsIn(['raw_material_id'], 'RawMaterials'));
        $rules->add($rules->isUnique(
            ['component_id', 'raw_material_id'],
            'Component Name and Raw Material Name already exist.'
        ));

        return $rules;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('user_full_name', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['Users.user_full_name']
        ])
        ->add('component_name', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if(!empty($args['component_name'])){
                    return $query
                        ->matching('Components', function (Query $query) use ($args) {
                            return $query
                            ->where(function (QueryExpression $exp, Query $q) use($args) {
                                return $exp->in('Components.component_name', $args['component_name']);
                            });
                        });
                }
                return $query;
            }
        ])
        ->add('material_name', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if(!empty($args['material_name'])){
                    return $query
                        ->matching('RawMaterials', function (Query $query) use ($args) {
                            return $query
                            ->where(function (QueryExpression $exp, Query $q) use($args) {
                                return $exp->in('RawMaterials.material_name', $args['material_name']);
                            });
                        });
                }
                return $query;
            }
        ])
        ->add('is_active', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['is_active']!=null){
                    return $query->where(['RawMaterials.is_active' => $args['is_active']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
