<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * JobOrderComponents Model
 *
 * @property \App\Model\Table\JobOrdersTable|\Cake\ORM\Association\BelongsTo $JobOrders
 * @property \App\Model\Table\ComponentsTable|\Cake\ORM\Association\BelongsTo $Components
 * @property \App\Model\Table\ProcessesTable|\Cake\ORM\Association\BelongsTo $Processes
 *
 * @method \App\Model\Entity\JobOrderComponent get($primaryKey, $options = [])
 * @method \App\Model\Entity\JobOrderComponent newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\JobOrderComponent[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\JobOrderComponent|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\JobOrderComponent|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\JobOrderComponent patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\JobOrderComponent[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\JobOrderComponent findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class JobOrderComponentsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('job_order_components');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('JobOrders', [
            'foreignKey' => 'job_order_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Components', [
            'foreignKey' => 'component_id',
            'joinType' => 'INNER'
        ]);

        $this->belongsTo('Processes', [
            'foreignKey' => 'process_id',
            'joinType' => 'INNER'
        ]);

         $this->belongsTo('VendorProcesses', [
            'foreignKey' => false,
            'conditions'=>[
                'VendorProcesses.component_id = JobOrderComponents.component_id',
                'VendorProcesses.process_id = JobOrderComponents.process_id',
            ],
            'joinType' => 'INNER'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->numeric('component_qty')
            ->requirePresence('component_qty', 'create')
            ->allowEmptyString('component_qty', false);

        $validator
            ->requirePresence('expected_date_delivery', 'create')
            ->allowEmptyDate('expected_date_delivery', false);


        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['job_order_id'], 'JobOrders'));
        $rules->add($rules->existsIn(['component_id'], 'Components'));
        $rules->add($rules->existsIn(['process_id'], 'Processes'));

        return $rules;
    }
}