<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;

/**
 * Machines Model
 *
 * @property \App\Model\Table\MachineTypesTable|\Cake\ORM\Association\BelongsTo $MachineTypes
 * @property \App\Model\Table\GroupsTable|\Cake\ORM\Association\BelongsTo $Groups
 * @property \App\Model\Table\AnnualPlannedCapacitiesTable|\Cake\ORM\Association\HasMany $AnnualPlannedCapacities
 * @property \App\Model\Table\AnnualPlannedMachinesTable|\Cake\ORM\Association\HasMany $AnnualPlannedMachines
 * @property \App\Model\Table\ComponentProcessesTable|\Cake\ORM\Association\HasMany $ComponentProcesses
 * @property \App\Model\Table\MachineProcessesTable|\Cake\ORM\Association\HasMany $MachineProcesses
 *
 * @method \App\Model\Entity\Machine get($primaryKey, $options = [])
 * @method \App\Model\Entity\Machine newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\Machine[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\Machine|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Machine|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Machine patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\Machine[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\Machine findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class MachinesTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('machines');
        $this->setDisplayField('machine_name');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('MachineTypes', [
            'foreignKey' => 'machine_type_id'
        ]);
        $this->belongsTo('Groups', [
            'foreignKey' => 'group_id'
        ]);
        $this->hasMany('AnnualPlannedCapacities', [
            'foreignKey' => 'machine_id'
        ]);
        $this->hasMany('AnnualPlannedMachines', [
            'foreignKey' => 'machine_id'
        ]);
        $this->hasMany('ComponentProcesses', [
            'foreignKey' => 'machine_id'
        ]);
        $this->hasMany('MachineProcesses', [
            'foreignKey' => 'machine_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');

    #Add search behavior
        $this->addBehavior('Search.Search');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('machine_type_id')
            ->maxLength('machine_type_id', 100)
            ->requirePresence('machine_type_id', 'create')
            ->allowEmptyString('machine_type_id', false);

        $validator
            ->scalar('group_id')
            ->maxLength('group_id', 100)
            ->requirePresence('group_id', 'create')
            ->allowEmptyString('group_id', false);

        $validator
            ->scalar('machine_name')
            ->maxLength('machine_name', 100)
            ->requirePresence('machine_name', 'create')
            ->allowEmptyString('machine_name', false);

        $validator
            ->scalar('machine_code')
            ->maxLength('machine_code', 100)
            ->requirePresence('machine_code', 'create')
            ->allowEmptyString('machine_code', false);

        $validator
            ->integer('machine_hours_per_shift')
            ->requirePresence('machine_hours_per_shift', 'create')
            ->allowEmptyString('machine_hours_per_shift', false);

        # $validator
        #     ->integer('machine_shift_per_day')
        #     ->requirePresence('machine_shift_per_day', 'create')
        #     ->allowEmptyString('machine_shift_per_day', false);

        $validator
            ->integer('machine_downtime_per_month')
            ->requirePresence('machine_downtime_per_month', 'create')
            ->allowEmptyString('machine_downtime_per_month', false);

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['machine_type_id'], 'MachineTypes'));
        $rules->add($rules->existsIn(['group_id'], 'Groups'));
        $rules->add($rules->isUnique(
            ['machine_name', 'machine_type_id', 'group_id'],
            'Same Machine Name and Machine type and Associated Group already exist.'
        ));

        return $rules;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('machine_name', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['Machines.machine_name']
        ])
        ->add('machine_code', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['Machines.machine_code']
        ])
        ->add('machine_type_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if(!empty($args['machine_type_id'])){
                    return $query
                        ->matching('MachineTypes', function (Query $query) use ($args) {
                            return $query
                            ->where(function (QueryExpression $exp, Query $q) use($args) {
                                return $exp->in('MachineTypes.id', $args['machine_type_id']);
                            });
                        });
                }
                return $query;
            }
        ])
        ->add('group_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if(!empty($args['group_id'])){
                    return $query
                        ->matching('Groups', function (Query $query) use ($args) {
                            return $query
                            ->where(function (QueryExpression $exp, Query $q) use($args) {
                                return $exp->in('Groups.id', $args['group_id']);
                            });
                        });
                }
                return $query;
            }
        ])
        ->add('is_active', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['is_active']!=null){
                    return $query->where(['Machines.is_active' => $args['is_active']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
