<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * PurchaseIndents Model
 *
 * @property \App\Model\Table\AnnualQuarterPlansTable|\Cake\ORM\Association\BelongsTo $AnnualQuarterPlans
 * @property \App\Model\Table\ProductsTable|\Cake\ORM\Association\BelongsTo $Products
 * @property \App\Model\Table\GroupsTable|\Cake\ORM\Association\BelongsTo $Groups
 * @property \App\Model\Table\PurchaseIndentCommentsTable|\Cake\ORM\Association\HasMany $PurchaseIndentComments
 * @property \App\Model\Table\PurchaseIndentComponentsTable|\Cake\ORM\Association\HasMany $PurchaseIndentComponents
 *
 * @method \App\Model\Entity\PurchaseIndent get($primaryKey, $options = [])
 * @method \App\Model\Entity\PurchaseIndent newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\PurchaseIndent[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseIndent|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\PurchaseIndent|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\PurchaseIndent patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseIndent[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseIndent findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class PurchaseIndentsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('purchase_indents');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('AnnualQuarterPlans', [
            'foreignKey' => 'annual_quarter_plan_id'
        ]);
        $this->belongsTo('Products', [
            'foreignKey' => 'product_id',
            'joinType' => 'INNER'
        ]);
        $this->belongsTo('Groups', [
            'foreignKey' => 'group_id'
        ]);
        $this->hasMany('PurchaseIndentComments', [
            'foreignKey' => 'purchase_indent_id'
        ]);
        $this->hasMany('PurchaseIndentComponents', [
            'foreignKey' => 'purchase_indent_id'
        ]);


        $this->hasMany('PurchaseOrders', [
            'foreignKey' => 'purchase_indent_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('purchase_intent_no')
            ->maxLength('purchase_intent_no', 100)
            ->allowEmptyString('purchase_intent_no');

        // $validator
        //     ->requirePresence('status', 'create')
        //     ->allowEmptyString('status', false);

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['annual_quarter_plan_id'], 'AnnualQuarterPlans'));
        $rules->add($rules->existsIn(['product_id'], 'Products'));
        $rules->add($rules->existsIn(['group_id'], 'Groups'));

        return $rules;
    }
}