<?php
namespace App\Model\Table;

use Cake\Mailer\Email;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\ORM\TableRegistry;
use Cake\Routing\Router;
use Cake\Validation\Validator;

/**
 * PurchaseOrders Model
 *
 * @property \App\Model\Table\PurchaseIndentsTable|\Cake\ORM\Association\BelongsTo $PurchaseIndents
 * @property \App\Model\Table\VendorsTable|\Cake\ORM\Association\BelongsTo $Vendors
 * @property \App\Model\Table\PurchaseOrderComponentsTable|\Cake\ORM\Association\HasMany $PurchaseOrderComponents
 *
 * @method \App\Model\Entity\PurchaseOrder get($primaryKey, $options = [])
 * @method \App\Model\Entity\PurchaseOrder newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\PurchaseOrder[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseOrder|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\PurchaseOrder|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\PurchaseOrder patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseOrder[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\PurchaseOrder findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class PurchaseOrdersTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('purchase_orders');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('PurchaseIndents', [
            'foreignKey' => 'purchase_indent_id',
            'joinType'   => 'INNER',
        ]);
        $this->belongsTo('Vendors', [
            'foreignKey' => 'vendor_id',
            'joinType'   => 'INNER',
        ]);
        $this->hasMany('PurchaseOrderComponents', [
            'foreignKey' => 'purchase_order_id',
        ]);

        $this->hasMany('GoodsReceivedNotes', [
            'foreignKey' => 'purchase_order_id',
        ]);

        $this->belongsTo('Users', [
            'foreignKey' => 'created_by',
            'joinType'   => 'INNER',
        ]);

        $this->belongsTo('JobOrders', [
            'foreignKey' => 'job_order_id',
            'joinType'   => 'INNER',
        ]);

        #foot print
        $this->addBehavior('Muffin/Footprint.Footprint');

        #audit log
        $this->addBehavior('AuditStash.AuditLog');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('purchase_order_no')
            ->maxLength('purchase_order_no', 100)
            ->allowEmptyString('purchase_order_no');

        $validator
            ->allowEmpty('expected_delivery_date', false);
        //->allowEmptyDate('expected_delivery_date');

        // $validator
        //     ->requirePresence('tag', 'create')
        //     ->allowEmptyString('tag', false);

        // $validator
        //     ->requirePresence('is_adhoc', 'create')
        //     ->allowEmptyString('is_adhoc', false);

        // $validator
        //     ->requirePresence('status', 'create')
        //     ->allowEmptyString('status', false);

        // $validator
        //     ->requirePresence('is_active', 'create')
        //     ->allowEmptyString('is_active', false);

        // $validator
        //     ->integer('created_by')
        //     ->allowEmptyString('created_by');

        // $validator
        //     ->integer('modified_by')
        //     ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['purchase_indent_id'], 'PurchaseIndents'));
        //$rules->add($rules->existsIn(['vendor_id'], 'Vendors'));

        return $rules;
    }

    public function beforeSave($event, $entity, $options)
    {
        if ($entity->status == 1) {
            $entity->created           = date('Y-m-d');
            $entity->purchase_order_no = 'PO' . date("Yds") . mt_rand(10, 100);
        }

    }

    public function afterSave($event, $entity, $options)
    {

        //echo "string";pr($entity);exit;

        $PurchaseOrdersTable = TableRegistry::get('PurchaseOrders');
        $purchase_order      = $PurchaseOrdersTable
            ->find('all')
            ->contain(['Vendors'])
            ->where(['PurchaseOrders.id' => $entity->id])
            ->enableAutoFields(true)
            ->first();
        #add purchase order number
        //$purchase_order->purchase_order_no = 'PO' . date("Y") . '00' . $entity->id;
        //$PurchaseOrdersTable->save($purchase_order);

        #BOF Get management User Data
        $userTable = TableRegistry::get('Users');
        $md_user   = $userTable->find('all')->select([
            'user_full_name',
            'user_email',
            'user_mobile_no',
        ])->where(['Users.user_role_id' => 1])->first();

        $po_user = $userTable->find('all')->select([
            'user_full_name',
            'user_email',
            'user_mobile_no',
        ])->where(['Users.user_role_id' => 7])->first();
        #END of management user data

        #Create PDF when purchase order approved

        if ($purchase_order->status == 2) {
            #add pdf attachment to mail

            $purchase_order_data = $PurchaseOrdersTable->find()
                ->select([
                    'Users.username',
                    'Users.user_full_name',
                    'Users.user_mobile_no',
                ])
                ->contain([
                    'Vendors'
                    , 'PurchaseOrderComponents' => [
                        'Processes', 'Components', 'RawMaterials','StoreComponents'
                    ],
                ])
                ->join([
                    'Users' => [
                        'table'      => 'users',
                        'type'       => 'left',
                        'conditions' => [
                            'Users.id = PurchaseOrders.created_by',
                        ]],
                ])
                ->enableAutoFields(true)
                ->where(['PurchaseOrders.id' => $entity->id])
                ->first()
                ->toArray();

            $CakePdf = new \CakePdf\Pdf\CakePdf();
            $CakePdf->template('purchase_order', 'default');
            $CakePdf->viewVars(['purchase_order' => $purchase_order_data]);
            //$pdf = $CakePdf->output();
            $pdf = $CakePdf->write(WWW_ROOT . 'files/purchase_orders' . DS . $purchase_order->purchase_order_no . '.pdf');
            #send Email to vendor

            if (file_exists(WWW_ROOT . 'files/purchase_orders' . DS . $purchase_order->purchase_order_no . '.pdf')) {

            }

            //echo "email<pre>";print_r($pdf);exit;
        }



        #mail for when purchase order Rejected
        if ($purchase_order->status == 5) {
            #Mail Content
            $subject = 'New Purchase Order Created for Approval';
            $name    = $po_user->user_full_name;

            $message = 'Purchase Order with PO number ' . $entity->purchase_order_no . ' is Rejected By management User.<br> Please discuss the same with management and update purchase order.<br>Thank You.';

            $mailData = [
                    'name' => $name,
                    'data' => $message,
                ];

            #Mail Sent to Md user
            $to = $md_user->user_email;
            $cc = $po_user->user_email;

            $this->_sendEmail($to, $cc, $subject, $mailData, null);
        }


        $purchase_indent_table = TableRegistry::get('PurchaseIndents');
        $job_order_table       = TableRegistry::get('JobOrders');

        #BOF Po Supervisor Mail Content
        $name    = '';
        $po_type = '';
        $subject = '';
        $message = '';
        $group   = '';
        $vendor  = '';
        $link    = '';
        $to      = '';
        $cc      = '';

        #EOF Po supervisor Mail

        #update purchase Indent table
        if (!empty($purchase_order->purchase_indent_id)) {
            $purchase_indent = $purchase_indent_table
                ->find('all')
                ->contain(['Groups', 'Products'])
                ->where(['PurchaseIndents.id' => $entity->purchase_indent_id])
                ->first();

            $group  = $purchase_indent->group->group_name;
            $vendor = $purchase_order->vendor->vendor_name;

            if ($entity->isNew()) {
                $purchase_indent->status = 4;

                if ($entity->tag == 0) {
                    $po_type = 'Raw Materials';
                } else {
                    $po_type = 'Out Components';
                }

                #Mail Content

                $subject = 'New Purchase Order Created for Approval';
                $name    = $md_user->user_full_name;

                $link = Router::url([
                    'controller' => 'purchase-orders',
                    'action'     => 'view',
                    $entity->id,

                ], true);
                // $link    = 'http://ang.qms.local/Projects/SuperMachines/purchase-orders/view/' . $entity->id;
                $message = 'Store supervisor ' . $po_user->user_full_name . ' raised a Purchase order for a ' . $po_type . ' with ' . $group . ' and vendor ' . $vendor . '.<br> Please click <a href="' . $link . '">APPROVE </a> to view and approve the Purchase order. <br>Thank You';

                $mailData = [
                    'name' => $name,
                    'data' => $message,
                ];

                #Mail Sent to Md user
                $to = $md_user->user_email;
                $cc = $po_user->user_email;

                $this->_sendEmail($to, $cc, $subject, $mailData, null);

                #Mail Sent to PO user
                $to       = $po_user->user_email;
                $cc       = $md_user->user_email;

                $name    = $po_user->user_full_name;
                $message ='New Purchase order raised for a '.$po_type.' with '.$group.'. <br> Once approved  Purchase order will be shared to '.$vendor.'through email with PDF. <br> Thank You';

                $mailData = [
                    'name' => $name,
                    'data' => $message,
                ];
                $this->_sendEmail($to, $cc, $subject, $mailData, null);

            } else {
                if ($purchase_order->status == 2) {
                    #change purchase indent to inprogress
                    $purchase_indent->status = 6;

                    #Mail for Md User
                    $subject = 'New Purchase Order Raised';
                    $name    = $md_user->user_full_name;
                    $link    = Router::url([
                        'controller' => 'purchase-orders',
                        'action'     => 'view',
                        $entity->id,

                    ], true);
                    $message = 'New Purchase order raised on ' . $vendor . ' with PO number ' . $entity->purchase_order_no . ' for ' . $group . '.<br>Please find the auto generated purchase order as a attachment Thank You';

                    $mailData = [
                        'name' => $name,
                        'data' => $message,
                    ];
                    $this->_sendEmail($to, $cc, $subject, $mailData, $purchase_order->purchase_order_no);

                    #Mail for PO User
                    $name = $po_user->user_full_name;

                    $mailData = [
                        'name' => $name,
                        'data' => $message,
                    ];
                    $this->_sendEmail($to, $cc, $subject, $mailData, $purchase_order->purchase_order_no);

                    #Mail  For Vendor
                    $subject = 'Super Machine Works New Purchase Order Raised';
                    $name    = $vendor;
                    $message = 'New Purchase order received from SMW with PO number ' . $entity->purchase_order_no . '.<br> Please find the auto generated purchase order attached.<br> kindly contact po officer for any clarifications. <br>Thank You,<br> Regards<br> Super Machine Works';

                    $mailData = [
                        'name' => $name,
                        'data' => $message,
                    ];
                    $this->_sendEmail($to, $cc, $subject, $mailData, null);

                } else if ($purchase_order->status == 3) {

                    #change purchase indent to delivered
                    $purchase_indent->status = 7;

                } else if ($purchase_order->status == 4) {
                    #change purchase indent to hold

                    $purchase_indent->status = 8;
                } else if ($purchase_order->status == 5) {
                    #change purchase indent to Rejected
                    $purchase_indent->status = 9;
                }
            }

            $purchase_indent_table->save($purchase_indent);
        } else {
            #Update Job Order Status
            if (!empty($purchase_order->job_order_id) && $purchase_order->job_order_id != '') {

                $job_order = $job_order_table
                    ->find('all')
                    ->contain(['Groups', 'Products', 'Vendors'])
                    ->where(['JobOrders.id' => $entity->job_order_id])
                    ->first();

                $group  = $job_order->group->group_name;
                $vendor = $purchase_order->vendor->vendor_name;
                if ($entity->isNew()) {
                    $job_order->status = 2;

                    #Mail Content

                    #mail for md user
                    $po_type = 'Job Order';
                    $subject = 'New Purchase Order Created for Approval';
                    $name    = $md_user->user_full_name;
                    $link    = Router::url([
                        'controller' => 'purchase-orders',
                        'action'     => 'view-job-orders',
                        $entity->id,

                    ], true);
                    $message = 'Store supervisor ' . $po_user->user_full_name . ' raised a Purchase order for a ' . $po_type . ' with ' . $group . ' and vendor ' . $vendor . '.<br> Please click <a href="' . $link . '">APPROVE</a> to view and approve the Purchase order. <br>Thank You';

                    $mailData = [
                        'name' => $name,
                        'data' => $message,
                    ];
                    $this->_sendEmail($to, $cc, $subject, $mailData, null);


                     #Mail Content

                    #mail for po user
                    $po_type = 'Job Order';
                    $subject = 'New Purchase Order Created for Approval';
                    $name    = $po_user->user_full_name;
                    $link    = Router::url([
                        'controller' => 'purchase-orders',
                        'action'     => 'view-job-orders',
                        $entity->id,

                    ], true);
                    

                    $message ='New Purchase order raised for a '.$po_type.' with '.$group.'. <br> Once approved  Purchase order will be shared to '.$vendor.'through email with PDF. <br> Thank You';

                    $mailData = [
                        'name' => $name,
                        'data' => $message,
                    ];
                    $this->_sendEmail($to, $cc, $subject, $mailData, null);


                    

                } else {
                    if ($purchase_order->status == 2) {
                        #change Job Order to inprogress
                        $job_order->status = 3;

                        #Mail for Md User
                        $subject = 'New Purchase Order Raised';
                        $name    = $md_user->user_full_name;
                        $link    = Router::url([
                            'controller' => 'purchase-orders',
                            'action'     => 'view-job-orders',
                            $entity->id,
                        ], true);
                        $message = 'New Purchase order raised on ' . $vendor . ' with PO number ' . $entity->purchase_order_no . ' for ' . $group . '.<br>Please find the auto generated purchase order as a attachment Thank You';

                        $mailData = [
                            'name' => $name,
                            'data' => $message,
                        ];
                        $this->_sendEmail($to, $cc, $subject, $mailData, $purchase_order->purchase_order_no);

                        #Mail for PO User
                        $name = $po_user->user_full_name;

                        $mailData = [
                            'name' => $name,
                            'data' => $message,
                        ];
                        $this->_sendEmail($to, $cc, $subject, $mailData, $purchase_order->purchase_order_no);

                        #Mail  For Vendor
                        $subject = 'Super Machine Works New Purchase Order Raised';
                        $name    = $vendor;
                        $message = 'New Purchase order received from SMW with PO number ' . $entity->purchase_order_no . '.<br> Please find the auto generated purchase order attached.<br> kindly contact po officer for any clarifications. <br>Thank You,<br> Regards<br> Super Machine Works';

                        $mailData = [
                            'name' => $name,
                            'data' => $message,
                        ];
                        $this->_sendEmail($to, $cc, $subject, $mailData, null);

                    } else if ($purchase_order->status == 3) {
                        #change Job Order to delivered
                        $job_order->status = 4;
                    } else if ($purchase_order->status == 4) {
                        #change Job Order to hold
                        $job_order->status = 5;
                    } else if ($purchase_order->status == 5) {
                        #change Job Order to Rejected
                        $job_order->status = 6;
                    }
                }
                $job_order_table->save($job_order);

            }



            #check for store purchase order 

            if($purchase_order->is_store_order == 1){
                $vendor = $purchase_order->vendor->vendor_name;

                if ($entity->isNew()) {
                    
                    #Mail Content
                    #mail for md user
                    $po_type = 'Store Component ';
                    $subject = 'New Purchase Order Created for Approval';
                    $name    = $md_user->user_full_name;
                    $link    = Router::url([
                        'controller' => 'purchase-orders',
                        'action'     => 'view-store-components',
                        $entity->id,

                    ], true);
                    $message = 'Store supervisor ' . $po_user->user_full_name . ' raised a Purchase order for a ' . $po_type . ' and vendor ' . $vendor . '.<br> Please click <a href="' . $link . '">APPROVE</a> to view and approve the Purchase order. <br>Thank You';

                    $mailData = [
                        'name' => $name,
                        'data' => $message,
                    ];
                    $this->_sendEmail($to, $cc, $subject, $mailData, null);


                     #Mail Content

                    #mail for po user
                    $po_type = 'Store Component';
                    $subject = 'New Purchase Order Created for Approval';
                    $name    = $po_user->user_full_name;
                    $link    = Router::url([
                        'controller' => 'purchase-orders',
                        'action'     => 'view-store-components',
                        $entity->id,

                    ], true);
                    

                    $message ='New Purchase order raised for a '.$po_type.' <br> Once approved  Purchase order will be shared to '.$vendor.' through email with PDF. <br> Thank You';

                    $mailData = [
                        'name' => $name,
                        'data' => $message,
                    ];
                    $this->_sendEmail($to, $cc, $subject, $mailData, null);


                    

                } else {
                    if ($purchase_order->status == 2) {
                        #change Job Order to inprogress
                        
                        #Mail for Md User
                        $subject = 'New Purchase Order Raised';
                        $name    = $md_user->user_full_name;
                        $link    = Router::url([
                            'controller' => 'purchase-orders',
                            'action'     => 'view-store-components',
                            $entity->id,
                        ], true);
                        $message = 'New Purchase order raised on ' . $vendor . ' with PO number ' . $entity->purchase_order_no  . '.<br>Please find the auto generated purchase order as a attachment Thank You';

                        $mailData = [
                            'name' => $name,
                            'data' => $message,
                        ];
                        $this->_sendEmail($to, $cc, $subject, $mailData, $purchase_order->purchase_order_no);

                        #Mail for PO User
                        $name = $po_user->user_full_name;

                        $mailData = [
                            'name' => $name,
                            'data' => $message,
                        ];
                        $this->_sendEmail($to, $cc, $subject, $mailData, $purchase_order->purchase_order_no);

                        #Mail  For Vendor
                        $subject = 'Super Machine Works New Purchase Order Raised';
                        $name    = $vendor;
                        $message = 'New Purchase order received from SMW with PO number ' . $entity->purchase_order_no . '.<br> Please find the auto generated purchase order attached.<br> kindly contact po officer for any clarifications. <br>Thank You,<br> Regards<br> Super Machine Works';

                        $mailData = [
                            'name' => $name,
                            'data' => $message,
                        ];
                        $this->_sendEmail($to, $cc, $subject, $mailData, null);

                    } 
                }
                
            }


        }

        #Update Purchase order quantity in purchase indent components
        if (!empty($purchase_order->purchase_indent_id)) {
            $purchase_indent_components_table = TableRegistry::get('PurchaseIndentComponents');
            if (!empty($entity->purchase_order_components)) {
                foreach ($entity->purchase_order_components as $key => $value) {

                    $purchase_indent_components = $purchase_indent_components_table->find('all')->where(['PurchaseIndentComponents.purchase_indent_id' => $entity->purchase_indent_id, 'PurchaseIndentComponents.component_id' => $value->component_id])->first();

                    $purchase_indent_components->po_quantity = $purchase_indent_components->po_quantity + $value->order_quantity;

                    $is_saved = $purchase_indent_components_table->save($purchase_indent_components);

                }
            }
        }

    }

    public function _sendSms($numbers = null, $sender = null, $message = null)
    {
        $apiKey = urlencode('mRko+RHZ2xo-8vGpQ7yPpJ5zwbnEED57MJMYJYnt54');

        // Message details
        $numbers = array(918489855454, 918072617697);
        $sender  = urlencode('TXTLCL');
        $message = rawurlencode('New PO created from super machine works');
        $numbers = implode(',', $numbers);
        // Prepare data for POST request
        $data = array('apikey' => $apiKey, 'numbers' => $numbers, "sender" => $sender, "message" => $message);

        // Send the POST request with cURL
        $ch = curl_init('https://api.textlocal.in/send/');
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);
        curl_close($ch);

        return $response;

    }

    public function _sendEmail($to = null, $cc = null, $subject = null, $mailData = null, $attachments = null)
    {

        // pr($mailData);exit;

        $mailer = new Email();

        if ($attachments == null) {
            $mailer
                ->emailFormat('html')
                ->template('purchase_order')
                ->setViewVars(['mailData' => $mailData])
                ->subject($subject)
                ->to('kathiresan@m07.ovh')
                ->from(['no-reply@smw.in' => 'SMW Staging'])
                ->send();
        } else {
            $mailer
                ->attachments(WWW_ROOT . 'files/purchase_orders' . DS . $attachments . '.pdf')
                ->emailFormat('html')
                ->template('purchase_order')
                ->setViewVars(['mailData' => $mailData])
                ->subject($subject)
                ->to('raghu@mailinator.com')
                ->from(['no-reply@smw.in' => 'SMW Staging'])
                ->send();
        }

    }

}

