<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;

/**
 * RawMaterials Model
 *
 * @property \App\Model\Table\ComponentRawMaterialsTable|\Cake\ORM\Association\HasMany $ComponentRawMaterials
 * @property \App\Model\Table\PurchaseIndentComponentsTable|\Cake\ORM\Association\HasMany $PurchaseIndentComponents
 * @property \App\Model\Table\PurchaseOrderComponentsTable|\Cake\ORM\Association\HasMany $PurchaseOrderComponents
 * @property \App\Model\Table\RawMaterialFormulasTable|\Cake\ORM\Association\HasMany $RawMaterialFormulas
 * @property \App\Model\Table\VendorRawMaterialsTable|\Cake\ORM\Association\HasMany $VendorRawMaterials
 *
 * @method \App\Model\Entity\RawMaterial get($primaryKey, $options = [])
 * @method \App\Model\Entity\RawMaterial newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\RawMaterial[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\RawMaterial|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\RawMaterial|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\RawMaterial patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\RawMaterial[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\RawMaterial findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class RawMaterialsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('raw_materials');
        $this->setDisplayField('material_name');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->hasMany('ComponentRawMaterials', [
            'foreignKey' => 'raw_material_id'
        ]);
        $this->hasMany('PurchaseIndentComponents', [
            'foreignKey' => 'raw_material_id'
        ]);
        $this->hasMany('PurchaseOrderComponents', [
            'foreignKey' => 'raw_material_id'
        ]);
        $this->hasMany('RawMaterialFormulas', [
            'foreignKey' => 'raw_material_id'
        ]);
        $this->hasMany('VendorRawMaterials', [
            'foreignKey' => 'raw_material_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');

    #Add search behavior
        $this->addBehavior('Search.Search');

    #Import CSV Plugin
        $options = array(
        #Refer to php.net fgetcsv for more information
            'length'    => 0,
            'delimiter' => ',',
            'enclosure' => '"',
            'escape'    => '\\',
        #Generates a Model.field headings row from the csv file
            'headers'   => true,
        #If true, String $content is the data, not a path to the file
            'text'      => false,
        );
        $this->addBehavior('CakePHPCSV.Csv', $options);
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('material_name')
            ->maxLength('material_name', 100)
            ->requirePresence('material_name', 'create')
            ->allowEmptyString('material_name', false)
            ->add('material_name', 'unique', ['rule' => 'validateUnique', 'provider' => 'table', 'message' => 'Material name already exists.']);

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    public function buildRules(RulesChecker $rules) {
        $rules->add($rules->isUnique(
            ['material_name'],
            'Raw material name already exist'
        ));
        return $rules;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('material_name', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['RawMaterials.material_name']
        ])
        ->add('is_active', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['is_active']!=null){
                    return $query->where(['RawMaterials.is_active' => $args['is_active']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
