<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;

/**
 * SubAssemblies Model
 *
 * @property \App\Model\Table\AnnualAssemblyTrackingsTable|\Cake\ORM\Association\HasMany $AnnualAssemblyTrackings
 * @property \App\Model\Table\ComponentAssembliesTable|\Cake\ORM\Association\HasMany $ComponentAssemblies
 * @property \App\Model\Table\SubAssemblyProductsTable|\Cake\ORM\Association\HasMany $SubAssemblyProducts
 *
 * @method \App\Model\Entity\SubAssembly get($primaryKey, $options = [])
 * @method \App\Model\Entity\SubAssembly newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\SubAssembly[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\SubAssembly|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\SubAssembly|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\SubAssembly patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\SubAssembly[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\SubAssembly findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class SubAssembliesTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('sub_assemblies');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->hasMany('AnnualAssemblyTrackings', [
            'foreignKey' => 'sub_assembly_id'
        ]);
        $this->hasMany('ComponentAssemblies', [
            'foreignKey' => 'sub_assembly_id'
        ]);
        $this->hasMany('SubAssemblyProducts', [
            'foreignKey' => 'sub_assembly_id'
        ]);
        
        $this->belongsToMany('products', [
            'foreignKey'        => 'sub_assembly_id',
            'targetForiegnKey'  => 'product_id',
            'joinTable'         => 'sub_assembly_products'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');

    #Add search behavior
        $this->addBehavior('Search.Search');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        /* $validator
            ->scalar('sub_assembly_name')
            ->maxLength('sub_assembly_name', 100)
            ->requirePresence('sub_assembly_name', 'create')
            ->allowEmptyString('sub_assembly_name', false);*/

        $validator
            ->allowEmptyString('sub_assembly_name', true)
            ->add('sub_assembly_name', 'unique', ['rule' => 'validateUnique', 'provider' => 'table', 'message' => 'Sub assembly name already exists.']);

        /*$validator
            ->integer('sub_assembly_code')
            ->requirePresence('sub_assembly_code', 'create')
            ->allowEmptyString('sub_assembly_code', false); */

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }


    public function buildRules(RulesChecker $rules) {
        $rules->add($rules->isUnique(
            ['sub_assembly_name', 'sub_assembly_code'],
            'Sub Assembly Name And Sub Assembly Code already exist.'
        ));
        return $rules;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('sub_assembly_name', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['SubAssemblies.sub_assembly_name']
        ])
        ->add('sub_assembly_code', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['SubAssemblies.sub_assembly_code']
        ])
        ->add('is_active', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['is_active']!=null){
                    return $query->where(['SubAssemblies.is_active' => $args['is_active']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
