<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use Cake\Auth\DefaultPasswordHasher;

use Cake\Event\Event;
use ArrayObject;

use Cake\Database\Expression\QueryExpression;
use Cake\ORM\TableRegistry;
/**
 * Users Model
 *
 * @property \App\Model\Table\UserRolesTable|\Cake\ORM\Association\BelongsTo $UserRoles
 * @property \App\Model\Table\AnnualAssemblyTrackingsTable|\Cake\ORM\Association\HasMany $AnnualAssemblyTrackings
 * @property \App\Model\Table\ComponentTrackingCommentsTable|\Cake\ORM\Association\HasMany $ComponentTrackingComments
 * @property \App\Model\Table\GroupUsersTable|\Cake\ORM\Association\HasMany $GroupUsers
 *
 * @method \App\Model\Entity\User get($primaryKey, $options = [])
 * @method \App\Model\Entity\User newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\User[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\User|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\User|bool saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\User patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\User[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\User findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class UsersTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('users');
        $this->setDisplayField('user_full_name');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('UserRoles', [
            'foreignKey' => 'user_role_id',
            'joinType' => 'INNER'
        ]);
        $this->hasMany('AnnualAssemblyTrackings', [
            'foreignKey' => 'user_id'
        ]);
        $this->hasMany('ComponentTrackingComments', [
            'foreignKey' => 'user_id'
        ]);
        $this->hasMany('GroupUsers', [
            'foreignKey' => 'user_id'
        ]);

	#foot print
		$this->addBehavior('Muffin/Footprint.Footprint');

	#audit log
		$this->addBehavior('AuditStash.AuditLog');

    #START: IMPORTANT PART HERE
        $this->addBehavior('Josegonzalez/Upload.Upload', [
            'photo',
        ]);
    #END: IMPORTANT PART ABOVE

    #Add the behaviour to your table
        $this->addBehavior('Search.Search');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('user_full_name')
            ->maxLength('user_full_name', 100)
            ->requirePresence('user_full_name', 'create')
            ->allowEmptyString('user_full_name', false);

        $validator
            ->scalar('username')
            ->maxLength('username', 100)
            ->requirePresence('username', 'create')
            ->allowEmptyString('username', false)
            ->add('username', 'unique', ['rule' => 'validateUnique', 'provider' => 'table', 'message' => 'User Name already exists.']);

        $validator
            ->email('user_email')
            ->maxLength('user_email', 100)
            ->requirePresence('user_email', 'create')
            ->notEmpty('user_email')
            ->add('user_email', 'unique', ['rule' => 'validateUnique', 'provider' => 'table', 'message' => 'Email already exists.']);

        $validator
            ->requirePresence('user_mobile_no', 'create')
            ->allowEmptyString('user_mobile_no', false)
            ->numeric('user_mobile_no')
            ->add('user_mobile_no', 'unique', ['rule' => 'validateUnique', 'provider' => 'table', 'message' => 'Mobile No. already exists.'])
            ->add('user_mobile_no', 'validFormat', 
                [
                    'rule' => ['custom', '/^\d{10}$/'], 
                    'message' => __('Minimum 10 character'), 
                    'allowEmpty' => true
                ]
            );

        $validator
            ->allowEmpty('photo', true)
            ->add('photo', 'file', [
                'rule' => ['uploadedFile', ['types' => ['image/png', 'image/jpeg', 'image/jpg']]],
                'message' => __("authorized extensions: jpeg, jpg, png")
            ]);

        $validator
            ->scalar('password')
            ->maxLength('password', 255)
            ->requirePresence('password', 'create')
            ->allowEmptyString('password', false);

        $validator
            ->allowEmptyString('is_password_reset');

        $validator
            ->allowEmptyString('is_active');

        $validator
            ->integer('created_by')
            ->allowEmptyString('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmptyString('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->isUnique(['username']));
        $rules->add($rules->existsIn(['user_role_id'], 'UserRoles'));

        return $rules;
    }

    /**
     * Reset Password Validation Function
     *
     */
    public function validationResetPassword(Validator $validator)
    {
        $validator
            /* ->add('new_password', 'validFormat', 
                [
                    'rule' => ['custom', '/^(?=.*[!@$])(?=.*[a-z])(?=.*[A-Z])(?=.*[0-9]).{6,16}$/i'], 
                    'message' => __('Minimum 6 character & alphanumeric with one symbol(!@$) is Required.'), 
                    'allowEmpty' => true
                ]
            ) */
            ->add('new_password',[
                'match' => [
                    'rule' => ['compareWith','confirm_password'],
                    'message' => 'Password dose not match.'
                ]
            ])
            ->add('new_password', 'custom', [
                'rule' => function($value, $context){
                    $user = $this->get($context['data']['id']);

                    if ($user) {
                        if (!(new DefaultPasswordHasher)->check($value, $user->password)) {
                            return true;
                        }
                    }
                    return false;
                }, 
                'message' => 'You cannot use your previous password'
            ])
            ->notEmpty('new_password');
        
        $validator
            ->add('confirm_password',[
                'match' => [
                    'rule' => ['compareWith','new_password'],
                    'message' => 'Password dose not match.'
                ]
            ])
            ->notEmpty('confirm_password');

        return $validator;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {
        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager 
        ->add('user_full_name', 'Search.Like', [
            'before' => true,
            'after' => true,
            'fieldMode' => 'OR',
            'comparison' => 'LIKE',
            'wildcardAny' => '*',
            'wildcardOne' => '?',
            'field' => ['Users.user_full_name']
        ])
        ->add('user_role_id', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if(!empty($args['user_role_id'])){
                    return $query
                        ->matching('UserRoles', function (Query $query) use ($args) {
                            return $query
                            ->where(function (QueryExpression $exp, Query $q) use($args) {
                                return $exp->in('UserRoles.id', $args['user_role_id']);
                            });
                        });
                }
                return $query;
            }
        ])
        ->add('is_active', 'Search.Callback', [
            'callback' => function ($query, $args, $filter){                
                if($args['is_active']!=null){
                    return $query->where(['Users.is_active' => $args['is_active']]);
                }
                return $query;
            }
        ]);

        return $searchManager;
    }
}
