<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * BomPcrTemp Model
 *
 * @property \App\Model\Table\BillOfMaterialsTable|\Cake\ORM\Association\BelongsTo $BillOfMaterials
 *
 * @method \App\Model\Entity\BomPcrTemp get($primaryKey, $options = [])
 * @method \App\Model\Entity\BomPcrTemp newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\BomPcrTemp[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\BomPcrTemp|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\BomPcrTemp patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\BomPcrTemp[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\BomPcrTemp findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class BomPcrTempTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('bom_pcr_temp');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('BillOfMaterials', [
            'foreignKey' => 'bill_of_material_id',
            'joinType' => 'INNER'
        ]);
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmpty('id', 'create');

        $validator
            ->integer('request_no')
            ->requirePresence('request_no', 'create')
            ->notEmpty('request_no');

        $validator
            ->decimal('previous_rate')
            ->requirePresence('previous_rate', 'create')
            ->notEmpty('previous_rate');

        $validator
            ->decimal('initial_negotiation_rate')
            ->allowEmpty('initial_negotiation_rate');

        $validator
            ->decimal('current_rate')
            ->requirePresence('current_rate', 'create')
            ->notEmpty('current_rate');

        $validator
            ->integer('status')
            ->requirePresence('status', 'create')
            ->notEmpty('status');

        $validator
            ->scalar('requested_by')
            ->maxLength('requested_by', 15)
            ->requirePresence('requested_by', 'create')
            ->notEmpty('requested_by');

        $validator
            ->scalar('comments')
            ->allowEmpty('comments');

        $validator
            ->integer('created_by')
            ->allowEmpty('created_by');

        $validator
            ->integer('modified_by')
            ->allowEmpty('modified_by');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->existsIn(['bill_of_material_id'], 'BillOfMaterials'));

        return $rules;
    }
}
